/* @copyright Itential, LLC 2019 (pre-modifications) */

// Set globals
/* global describe it log pronghornProps */
/* eslint global-require: warn */
/* eslint no-unused-vars: warn */
/* eslint import/no-dynamic-require:warn */

// include required items for testing & logging
const assert = require('assert');
const path = require('path');
const util = require('util');
const execute = require('child_process').execSync;
const fs = require('fs-extra');
const mocha = require('mocha');
const winston = require('winston');
const { expect } = require('chai');
const { use } = require('chai');
const td = require('testdouble');
const Ajv = require('ajv');

const ajv = new Ajv({ strictSchema: false, allErrors: true, allowUnionTypes: true });
const anything = td.matchers.anything();
let logLevel = 'none';
const isRapidFail = false;

// read in the properties from the sampleProperties files
let adaptdir = __dirname;
if (adaptdir.endsWith('/test/integration')) {
  adaptdir = adaptdir.substring(0, adaptdir.length - 17);
} else if (adaptdir.endsWith('/test/unit')) {
  adaptdir = adaptdir.substring(0, adaptdir.length - 10);
}
const samProps = require(`${adaptdir}/sampleProperties.json`).properties;

// these variables can be changed to run in integrated mode so easier to set them here
// always check these in with bogus data!!!
samProps.stub = true;
samProps.host = 'replace.hostorip.here';
samProps.authentication.username = 'username';
samProps.authentication.password = 'password';
samProps.protocol = 'http';
samProps.port = 80;
samProps.ssl.enabled = false;
samProps.ssl.accept_invalid_cert = false;
samProps.request.attempt_timeout = 1200000;
const attemptTimeout = samProps.request.attempt_timeout;
const { stub } = samProps;

// these are the adapter properties. You generally should not need to alter
// any of these after they are initially set up
global.pronghornProps = {
  pathProps: {
    encrypted: false
  },
  adapterProps: {
    adapters: [{
      id: 'Test-128technology',
      type: 'Technology',
      properties: samProps
    }]
  }
};

global.$HOME = `${__dirname}/../..`;

// set the log levels that Pronghorn uses, spam and trace are not defaulted in so without
// this you may error on log.trace calls.
const myCustomLevels = {
  levels: {
    spam: 6,
    trace: 5,
    debug: 4,
    info: 3,
    warn: 2,
    error: 1,
    none: 0
  }
};

// need to see if there is a log level passed in
process.argv.forEach((val) => {
  // is there a log level defined to be passed in?
  if (val.indexOf('--LOG') === 0) {
    // get the desired log level
    const inputVal = val.split('=')[1];

    // validate the log level is supported, if so set it
    if (Object.hasOwnProperty.call(myCustomLevels.levels, inputVal)) {
      logLevel = inputVal;
    }
  }
});

// need to set global logging
global.log = winston.createLogger({
  level: logLevel,
  levels: myCustomLevels.levels,
  transports: [
    new winston.transports.Console()
  ]
});

/**
 * Runs the error asserts for the test
 */
function runErrorAsserts(data, error, code, origin, displayStr) {
  assert.equal(null, data);
  assert.notEqual(undefined, error);
  assert.notEqual(null, error);
  assert.notEqual(undefined, error.IAPerror);
  assert.notEqual(null, error.IAPerror);
  assert.notEqual(undefined, error.IAPerror.displayString);
  assert.notEqual(null, error.IAPerror.displayString);
  assert.equal(code, error.icode);
  assert.equal(origin, error.IAPerror.origin);
  assert.equal(displayStr, error.IAPerror.displayString);
}

// require the adapter that we are going to be using
const Technology = require('../../adapter');

// delete the .DS_Store directory in entities -- otherwise this will cause errors
const dirPath = path.join(__dirname, '../../entities/.DS_Store');
if (fs.existsSync(dirPath)) {
  try {
    fs.removeSync(dirPath);
    console.log('.DS_Store deleted');
  } catch (e) {
    console.log('Error when deleting .DS_Store:', e);
  }
}

// begin the testing - these should be pretty well defined between the describe and the it!
describe('[unit] 128technology Adapter Test', () => {
  describe('Technology Class Tests', () => {
    const a = new Technology(
      pronghornProps.adapterProps.adapters[0].id,
      pronghornProps.adapterProps.adapters[0].properties
    );

    if (isRapidFail) {
      const state = {};
      state.passed = true;

      mocha.afterEach(function x() {
        state.passed = state.passed
        && (this.currentTest.state === 'passed');
      });
      mocha.beforeEach(function x() {
        if (!state.passed) {
          return this.currentTest.skip();
        }
        return true;
      });
    }

    describe('#class instance created', () => {
      it('should be a class with properties', (done) => {
        try {
          assert.notEqual(null, a);
          assert.notEqual(undefined, a);
          const checkId = global.pronghornProps.adapterProps.adapters[0].id;
          assert.equal(checkId, a.id);
          assert.notEqual(null, a.allProps);
          const check = global.pronghornProps.adapterProps.adapters[0].properties.healthcheck.type;
          assert.equal(check, a.healthcheckType);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('adapterBase.js', () => {
      it('should have an adapterBase.js', (done) => {
        try {
          fs.exists('adapterBase.js', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    let wffunctions = [];
    describe('#iapGetAdapterWorkflowFunctions', () => {
      it('should retrieve workflow functions', (done) => {
        try {
          wffunctions = a.iapGetAdapterWorkflowFunctions([]);

          try {
            assert.notEqual(0, wffunctions.length);
            done();
          } catch (err) {
            log.error(`Test Failure: ${err}`);
            done(err);
          }
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('package.json', () => {
      it('should have a package.json', (done) => {
        try {
          fs.exists('package.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json should be validated', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          // Define the JSON schema for package.json
          const packageJsonSchema = {
            type: 'object',
            properties: {
              name: { type: 'string' },
              version: { type: 'string' }
              // May need to add more properties as needed
            },
            required: ['name', 'version']
          };
          const validate = ajv.compile(packageJsonSchema);
          const isValid = validate(packageDotJson);

          if (isValid === false) {
            log.error('The package.json contains errors');
            assert.equal(true, isValid);
          } else {
            assert.equal(true, isValid);
          }

          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json standard fields should be customized', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          assert.notEqual(-1, packageDotJson.name.indexOf('128technology'));
          assert.notEqual(undefined, packageDotJson.version);
          assert.notEqual(null, packageDotJson.version);
          assert.notEqual('', packageDotJson.version);
          assert.notEqual(undefined, packageDotJson.description);
          assert.notEqual(null, packageDotJson.description);
          assert.notEqual('', packageDotJson.description);
          assert.equal('adapter.js', packageDotJson.main);
          assert.notEqual(undefined, packageDotJson.wizardVersion);
          assert.notEqual(null, packageDotJson.wizardVersion);
          assert.notEqual('', packageDotJson.wizardVersion);
          assert.notEqual(undefined, packageDotJson.engineVersion);
          assert.notEqual(null, packageDotJson.engineVersion);
          assert.notEqual('', packageDotJson.engineVersion);
          assert.equal('http', packageDotJson.adapterType);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json proper scripts should be provided', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          assert.notEqual(undefined, packageDotJson.scripts);
          assert.notEqual(null, packageDotJson.scripts);
          assert.notEqual('', packageDotJson.scripts);
          assert.equal('node utils/setup.js', packageDotJson.scripts.preinstall);
          assert.equal('node --max_old_space_size=4096 ./node_modules/eslint/bin/eslint.js . --ext .json --ext .js', packageDotJson.scripts.lint);
          assert.equal('node --max_old_space_size=4096 ./node_modules/eslint/bin/eslint.js . --ext .json --ext .js --quiet', packageDotJson.scripts['lint:errors']);
          assert.equal('mocha test/unit/adapterBaseTestUnit.js --LOG=error', packageDotJson.scripts['test:baseunit']);
          assert.equal('mocha test/unit/adapterTestUnit.js --LOG=error', packageDotJson.scripts['test:unit']);
          assert.equal('mocha test/integration/adapterTestIntegration.js --LOG=error', packageDotJson.scripts['test:integration']);
          assert.equal('nyc --reporter html --reporter text mocha --reporter dot test/*', packageDotJson.scripts['test:cover']);
          assert.equal('npm run test:baseunit && npm run test:unit && npm run test:integration', packageDotJson.scripts.test);
          assert.equal('npm publish --registry=https://registry.npmjs.org --access=public', packageDotJson.scripts.deploy);
          assert.equal('npm run deploy', packageDotJson.scripts.build);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json proper directories should be provided', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          assert.notEqual(undefined, packageDotJson.repository);
          assert.notEqual(null, packageDotJson.repository);
          assert.notEqual('', packageDotJson.repository);
          assert.equal('git', packageDotJson.repository.type);
          assert.equal('git@gitlab.com:itentialopensource/adapters/', packageDotJson.repository.url.substring(0, 43));
          assert.equal('https://gitlab.com/itentialopensource/adapters/', packageDotJson.homepage.substring(0, 47));
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json proper dependencies should be provided', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          assert.notEqual(undefined, packageDotJson.dependencies);
          assert.notEqual(null, packageDotJson.dependencies);
          assert.notEqual('', packageDotJson.dependencies);
          assert.equal('^8.12.0', packageDotJson.dependencies.ajv);
          assert.equal('^1.6.7', packageDotJson.dependencies.axios);
          assert.equal('^11.0.0', packageDotJson.dependencies.commander);
          assert.equal('^11.1.1', packageDotJson.dependencies['fs-extra']);
          assert.equal('^10.3.0', packageDotJson.dependencies.mocha);
          assert.equal('^2.0.1', packageDotJson.dependencies['mocha-param']);
          assert.equal('^15.1.0', packageDotJson.dependencies.nyc);
          assert.equal('^0.4.4', packageDotJson.dependencies.ping);
          assert.equal('^1.4.10', packageDotJson.dependencies['readline-sync']);
          assert.equal('^7.5.3', packageDotJson.dependencies.semver);
          assert.equal('^3.9.0', packageDotJson.dependencies.winston);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('package.json proper dev dependencies should be provided', (done) => {
        try {
          const packageDotJson = require('../../package.json');
          assert.notEqual(undefined, packageDotJson.devDependencies);
          assert.notEqual(null, packageDotJson.devDependencies);
          assert.notEqual('', packageDotJson.devDependencies);
          assert.equal('^4.3.7', packageDotJson.devDependencies.chai);
          assert.equal('^8.44.0', packageDotJson.devDependencies.eslint);
          assert.equal('^15.0.0', packageDotJson.devDependencies['eslint-config-airbnb-base']);
          assert.equal('^2.27.5', packageDotJson.devDependencies['eslint-plugin-import']);
          assert.equal('^3.1.0', packageDotJson.devDependencies['eslint-plugin-json']);
          assert.equal('^3.18.0', packageDotJson.devDependencies.testdouble);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('pronghorn.json', () => {
      it('should have a pronghorn.json', (done) => {
        try {
          fs.exists('pronghorn.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('pronghorn.json should be customized', (done) => {
        try {
          const pronghornDotJson = require('../../pronghorn.json');
          assert.notEqual(-1, pronghornDotJson.id.indexOf('128technology'));
          assert.equal('Adapter', pronghornDotJson.type);
          assert.equal('Technology', pronghornDotJson.export);
          assert.equal('128technology', pronghornDotJson.title);
          assert.equal('adapter.js', pronghornDotJson.src);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('pronghorn.json should contain generic adapter methods', (done) => {
        try {
          const pronghornDotJson = require('../../pronghorn.json');
          assert.notEqual(undefined, pronghornDotJson.methods);
          assert.notEqual(null, pronghornDotJson.methods);
          assert.notEqual('', pronghornDotJson.methods);
          assert.equal(true, Array.isArray(pronghornDotJson.methods));
          assert.notEqual(0, pronghornDotJson.methods.length);
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapUpdateAdapterConfiguration'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapSuspendAdapter'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapUnsuspendAdapter'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapGetAdapterQueue'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapFindAdapterPath'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapTroubleshootAdapter'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRunAdapterHealthcheck'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRunAdapterConnectivity'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRunAdapterBasicGet'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapMoveAdapterEntitiesToDB'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapDeactivateTasks'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapActivateTasks'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapPopulateEntityCache'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRetrieveEntitiesCache'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'getDevice'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'getDevicesFiltered'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'isAlive'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'getConfig'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapGetDeviceCount'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapExpandedGenericAdapterRequest'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'genericAdapterRequest'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'genericAdapterRequestNoBasePath'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRunAdapterLint'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapRunAdapterTests'));
          assert.notEqual(undefined, pronghornDotJson.methods.find((e) => e.name === 'iapGetAdapterInventory'));
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('pronghorn.json should only expose workflow functions', (done) => {
        try {
          const pronghornDotJson = require('../../pronghorn.json');

          for (let m = 0; m < pronghornDotJson.methods.length; m += 1) {
            let found = false;
            let paramissue = false;

            for (let w = 0; w < wffunctions.length; w += 1) {
              if (pronghornDotJson.methods[m].name === wffunctions[w]) {
                found = true;
                const methLine = execute(`grep "  ${wffunctions[w]}(" adapter.js | grep "callback) {"`).toString();
                let wfparams = [];

                if (methLine && methLine.indexOf('(') >= 0 && methLine.indexOf(')') >= 0) {
                  const temp = methLine.substring(methLine.indexOf('(') + 1, methLine.lastIndexOf(')'));
                  wfparams = temp.split(',');

                  for (let t = 0; t < wfparams.length; t += 1) {
                    // remove default value from the parameter name
                    wfparams[t] = wfparams[t].substring(0, wfparams[t].search(/=/) > 0 ? wfparams[t].search(/#|\?|=/) : wfparams[t].length);
                    // remove spaces
                    wfparams[t] = wfparams[t].trim();

                    if (wfparams[t] === 'callback') {
                      wfparams.splice(t, 1);
                    }
                  }
                }

                // if there are inputs defined but not on the method line
                if (wfparams.length === 0 && (pronghornDotJson.methods[m].input
                    && pronghornDotJson.methods[m].input.length > 0)) {
                  paramissue = true;
                } else if (wfparams.length > 0 && (!pronghornDotJson.methods[m].input
                    || pronghornDotJson.methods[m].input.length === 0)) {
                  // if there are no inputs defined but there are on the method line
                  paramissue = true;
                } else {
                  for (let p = 0; p < pronghornDotJson.methods[m].input.length; p += 1) {
                    let pfound = false;
                    for (let wfp = 0; wfp < wfparams.length; wfp += 1) {
                      if (pronghornDotJson.methods[m].input[p].name.toUpperCase() === wfparams[wfp].toUpperCase()) {
                        pfound = true;
                      }
                    }

                    if (!pfound) {
                      paramissue = true;
                    }
                  }
                  for (let wfp = 0; wfp < wfparams.length; wfp += 1) {
                    let pfound = false;
                    for (let p = 0; p < pronghornDotJson.methods[m].input.length; p += 1) {
                      if (pronghornDotJson.methods[m].input[p].name.toUpperCase() === wfparams[wfp].toUpperCase()) {
                        pfound = true;
                      }
                    }

                    if (!pfound) {
                      paramissue = true;
                    }
                  }
                }

                break;
              }
            }

            if (!found) {
              // this is the reason to go through both loops - log which ones are not found so
              // they can be worked
              log.error(`${pronghornDotJson.methods[m].name} not found in workflow functions`);
            }
            if (paramissue) {
              // this is the reason to go through both loops - log which ones are not found so
              // they can be worked
              log.error(`${pronghornDotJson.methods[m].name} has a parameter mismatch`);
            }
            assert.equal(true, found);
            assert.equal(false, paramissue);
          }
          done();
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('pronghorn.json should expose all workflow functions', (done) => {
        try {
          const pronghornDotJson = require('../../pronghorn.json');
          for (let w = 0; w < wffunctions.length; w += 1) {
            let found = false;

            for (let m = 0; m < pronghornDotJson.methods.length; m += 1) {
              if (pronghornDotJson.methods[m].name === wffunctions[w]) {
                found = true;
                break;
              }
            }

            if (!found) {
              // this is the reason to go through both loops - log which ones are not found so
              // they can be worked
              log.error(`${wffunctions[w]} not found in pronghorn.json`);
            }
            assert.equal(true, found);
          }
          done();
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      });
      it('pronghorn.json verify input/output schema objects', (done) => {
        const verifySchema = (methodName, schema) => {
          try {
            ajv.compile(schema);
          } catch (error) {
            const errorMessage = `Invalid schema found in '${methodName}' method.
          Schema => ${JSON.stringify(schema)}.
          Details => ${error.message}`;
            throw new Error(errorMessage);
          }
        };

        try {
          const pronghornDotJson = require('../../pronghorn.json');
          const { methods } = pronghornDotJson;
          for (let i = 0; i < methods.length; i += 1) {
            for (let j = 0; j < methods[i].input.length; j += 1) {
              const inputSchema = methods[i].input[j].schema;
              if (inputSchema) {
                verifySchema(methods[i].name, inputSchema);
              }
            }
            const outputSchema = methods[i].output.schema;
            if (outputSchema) {
              verifySchema(methods[i].name, outputSchema);
            }
          }
          done();
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      });
    });

    describe('propertiesSchema.json', () => {
      it('should have a propertiesSchema.json', (done) => {
        try {
          fs.exists('propertiesSchema.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('propertiesSchema.json should be customized', (done) => {
        try {
          const propertiesDotJson = require('../../propertiesSchema.json');
          assert.equal('adapter-128technology', propertiesDotJson.$id);
          assert.equal('object', propertiesDotJson.type);
          assert.equal('http://json-schema.org/draft-07/schema#', propertiesDotJson.$schema);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('propertiesSchema.json should contain generic adapter properties', (done) => {
        try {
          const propertiesDotJson = require('../../propertiesSchema.json');
          assert.notEqual(undefined, propertiesDotJson.properties);
          assert.notEqual(null, propertiesDotJson.properties);
          assert.notEqual('', propertiesDotJson.properties);
          assert.equal('string', propertiesDotJson.properties.host.type);
          assert.equal('integer', propertiesDotJson.properties.port.type);
          assert.equal('boolean', propertiesDotJson.properties.stub.type);
          assert.equal('string', propertiesDotJson.properties.protocol.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.authentication);
          assert.notEqual(null, propertiesDotJson.definitions.authentication);
          assert.notEqual('', propertiesDotJson.definitions.authentication);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.auth_method.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.username.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.password.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.token.type);
          assert.equal('integer', propertiesDotJson.definitions.authentication.properties.invalid_token_error.type);
          assert.equal('integer', propertiesDotJson.definitions.authentication.properties.token_timeout.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.token_cache.type);
          assert.equal(true, Array.isArray(propertiesDotJson.definitions.authentication.properties.auth_field.type));
          assert.equal(true, Array.isArray(propertiesDotJson.definitions.authentication.properties.auth_field_format.type));
          assert.equal('boolean', propertiesDotJson.definitions.authentication.properties.auth_logging.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.client_id.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.client_secret.type);
          assert.equal('string', propertiesDotJson.definitions.authentication.properties.grant_type.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.ssl);
          assert.notEqual(null, propertiesDotJson.definitions.ssl);
          assert.notEqual('', propertiesDotJson.definitions.ssl);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.ecdhCurve.type);
          assert.equal('boolean', propertiesDotJson.definitions.ssl.properties.enabled.type);
          assert.equal('boolean', propertiesDotJson.definitions.ssl.properties.accept_invalid_cert.type);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.ca_file.type);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.key_file.type);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.cert_file.type);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.secure_protocol.type);
          assert.equal('string', propertiesDotJson.definitions.ssl.properties.ciphers.type);
          assert.equal('string', propertiesDotJson.properties.base_path.type);
          assert.equal('string', propertiesDotJson.properties.version.type);
          assert.equal('string', propertiesDotJson.properties.cache_location.type);
          assert.equal('boolean', propertiesDotJson.properties.encode_pathvars.type);
          assert.equal('boolean', propertiesDotJson.properties.encode_queryvars.type);
          assert.equal(true, Array.isArray(propertiesDotJson.properties.save_metric.type));
          assert.notEqual(undefined, propertiesDotJson.definitions);
          assert.notEqual(null, propertiesDotJson.definitions);
          assert.notEqual('', propertiesDotJson.definitions);
          assert.notEqual(undefined, propertiesDotJson.definitions.healthcheck);
          assert.notEqual(null, propertiesDotJson.definitions.healthcheck);
          assert.notEqual('', propertiesDotJson.definitions.healthcheck);
          assert.equal('string', propertiesDotJson.definitions.healthcheck.properties.type.type);
          assert.equal('integer', propertiesDotJson.definitions.healthcheck.properties.frequency.type);
          assert.equal('object', propertiesDotJson.definitions.healthcheck.properties.query_object.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.throttle);
          assert.notEqual(null, propertiesDotJson.definitions.throttle);
          assert.notEqual('', propertiesDotJson.definitions.throttle);
          assert.equal('boolean', propertiesDotJson.definitions.throttle.properties.throttle_enabled.type);
          assert.equal('integer', propertiesDotJson.definitions.throttle.properties.number_pronghorns.type);
          assert.equal('string', propertiesDotJson.definitions.throttle.properties.sync_async.type);
          assert.equal('integer', propertiesDotJson.definitions.throttle.properties.max_in_queue.type);
          assert.equal('integer', propertiesDotJson.definitions.throttle.properties.concurrent_max.type);
          assert.equal('integer', propertiesDotJson.definitions.throttle.properties.expire_timeout.type);
          assert.equal('integer', propertiesDotJson.definitions.throttle.properties.avg_runtime.type);
          assert.equal('array', propertiesDotJson.definitions.throttle.properties.priorities.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.request);
          assert.notEqual(null, propertiesDotJson.definitions.request);
          assert.notEqual('', propertiesDotJson.definitions.request);
          assert.equal('integer', propertiesDotJson.definitions.request.properties.number_redirects.type);
          assert.equal('integer', propertiesDotJson.definitions.request.properties.number_retries.type);
          assert.equal(true, Array.isArray(propertiesDotJson.definitions.request.properties.limit_retry_error.type));
          assert.equal('array', propertiesDotJson.definitions.request.properties.failover_codes.type);
          assert.equal('integer', propertiesDotJson.definitions.request.properties.attempt_timeout.type);
          assert.equal('object', propertiesDotJson.definitions.request.properties.global_request.type);
          assert.equal('object', propertiesDotJson.definitions.request.properties.global_request.properties.payload.type);
          assert.equal('object', propertiesDotJson.definitions.request.properties.global_request.properties.uriOptions.type);
          assert.equal('object', propertiesDotJson.definitions.request.properties.global_request.properties.addlHeaders.type);
          assert.equal('object', propertiesDotJson.definitions.request.properties.global_request.properties.authData.type);
          assert.equal('boolean', propertiesDotJson.definitions.request.properties.healthcheck_on_timeout.type);
          assert.equal('boolean', propertiesDotJson.definitions.request.properties.return_raw.type);
          assert.equal('boolean', propertiesDotJson.definitions.request.properties.archiving.type);
          assert.equal('boolean', propertiesDotJson.definitions.request.properties.return_request.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.proxy);
          assert.notEqual(null, propertiesDotJson.definitions.proxy);
          assert.notEqual('', propertiesDotJson.definitions.proxy);
          assert.equal('boolean', propertiesDotJson.definitions.proxy.properties.enabled.type);
          assert.equal('string', propertiesDotJson.definitions.proxy.properties.host.type);
          assert.equal('integer', propertiesDotJson.definitions.proxy.properties.port.type);
          assert.equal('string', propertiesDotJson.definitions.proxy.properties.protocol.type);
          assert.equal('string', propertiesDotJson.definitions.proxy.properties.username.type);
          assert.equal('string', propertiesDotJson.definitions.proxy.properties.password.type);
          assert.notEqual(undefined, propertiesDotJson.definitions.mongo);
          assert.notEqual(null, propertiesDotJson.definitions.mongo);
          assert.notEqual('', propertiesDotJson.definitions.mongo);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.host.type);
          assert.equal('integer', propertiesDotJson.definitions.mongo.properties.port.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.database.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.username.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.password.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.replSet.type);
          assert.equal('object', propertiesDotJson.definitions.mongo.properties.db_ssl.type);
          assert.equal('boolean', propertiesDotJson.definitions.mongo.properties.db_ssl.properties.enabled.type);
          assert.equal('boolean', propertiesDotJson.definitions.mongo.properties.db_ssl.properties.accept_invalid_cert.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.db_ssl.properties.ca_file.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.db_ssl.properties.key_file.type);
          assert.equal('string', propertiesDotJson.definitions.mongo.properties.db_ssl.properties.cert_file.type);
          assert.notEqual('', propertiesDotJson.definitions.devicebroker);
          assert.equal('array', propertiesDotJson.definitions.devicebroker.properties.getDevice.type);
          assert.equal('array', propertiesDotJson.definitions.devicebroker.properties.getDevicesFiltered.type);
          assert.equal('array', propertiesDotJson.definitions.devicebroker.properties.isAlive.type);
          assert.equal('array', propertiesDotJson.definitions.devicebroker.properties.getConfig.type);
          assert.equal('array', propertiesDotJson.definitions.devicebroker.properties.getCount.type);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('error.json', () => {
      it('should have an error.json', (done) => {
        try {
          fs.exists('error.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('error.json should have standard adapter errors', (done) => {
        try {
          const errorDotJson = require('../../error.json');
          assert.notEqual(undefined, errorDotJson.errors);
          assert.notEqual(null, errorDotJson.errors);
          assert.notEqual('', errorDotJson.errors);
          assert.equal(true, Array.isArray(errorDotJson.errors));
          assert.notEqual(0, errorDotJson.errors.length);
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.100'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.101'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.102'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.110'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.111'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.112'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.113'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.114'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.115'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.116'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.300'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.301'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.302'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.303'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.304'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.305'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.310'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.311'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.312'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.320'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.321'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.400'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.401'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.402'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.500'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.501'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.502'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.503'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.600'));
          assert.notEqual(undefined, errorDotJson.errors.find((e) => e.icode === 'AD.900'));
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('sampleProperties.json', () => {
      it('should have a sampleProperties.json', (done) => {
        try {
          fs.exists('sampleProperties.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('sampleProperties.json should contain generic adapter properties', (done) => {
        try {
          const sampleDotJson = require('../../sampleProperties.json');
          assert.notEqual(-1, sampleDotJson.id.indexOf('128technology'));
          assert.equal('Technology', sampleDotJson.type);
          assert.notEqual(undefined, sampleDotJson.properties);
          assert.notEqual(null, sampleDotJson.properties);
          assert.notEqual('', sampleDotJson.properties);
          assert.notEqual(undefined, sampleDotJson.properties.host);
          assert.notEqual(undefined, sampleDotJson.properties.port);
          assert.notEqual(undefined, sampleDotJson.properties.stub);
          assert.notEqual(undefined, sampleDotJson.properties.protocol);
          assert.notEqual(undefined, sampleDotJson.properties.authentication);
          assert.notEqual(null, sampleDotJson.properties.authentication);
          assert.notEqual('', sampleDotJson.properties.authentication);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.auth_method);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.username);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.password);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.token);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.invalid_token_error);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.token_timeout);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.token_cache);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.auth_field);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.auth_field_format);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.auth_logging);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.client_id);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.client_secret);
          assert.notEqual(undefined, sampleDotJson.properties.authentication.grant_type);
          assert.notEqual(undefined, sampleDotJson.properties.ssl);
          assert.notEqual(null, sampleDotJson.properties.ssl);
          assert.notEqual('', sampleDotJson.properties.ssl);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.ecdhCurve);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.enabled);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.accept_invalid_cert);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.ca_file);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.key_file);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.cert_file);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.secure_protocol);
          assert.notEqual(undefined, sampleDotJson.properties.ssl.ciphers);
          assert.notEqual(undefined, sampleDotJson.properties.base_path);
          assert.notEqual(undefined, sampleDotJson.properties.version);
          assert.notEqual(undefined, sampleDotJson.properties.cache_location);
          assert.notEqual(undefined, sampleDotJson.properties.encode_pathvars);
          assert.notEqual(undefined, sampleDotJson.properties.encode_queryvars);
          assert.notEqual(undefined, sampleDotJson.properties.save_metric);
          assert.notEqual(undefined, sampleDotJson.properties.healthcheck);
          assert.notEqual(null, sampleDotJson.properties.healthcheck);
          assert.notEqual('', sampleDotJson.properties.healthcheck);
          assert.notEqual(undefined, sampleDotJson.properties.healthcheck.type);
          assert.notEqual(undefined, sampleDotJson.properties.healthcheck.frequency);
          assert.notEqual(undefined, sampleDotJson.properties.healthcheck.query_object);
          assert.notEqual(undefined, sampleDotJson.properties.throttle);
          assert.notEqual(null, sampleDotJson.properties.throttle);
          assert.notEqual('', sampleDotJson.properties.throttle);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.throttle_enabled);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.number_pronghorns);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.sync_async);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.max_in_queue);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.concurrent_max);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.expire_timeout);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.avg_runtime);
          assert.notEqual(undefined, sampleDotJson.properties.throttle.priorities);
          assert.notEqual(undefined, sampleDotJson.properties.request);
          assert.notEqual(null, sampleDotJson.properties.request);
          assert.notEqual('', sampleDotJson.properties.request);
          assert.notEqual(undefined, sampleDotJson.properties.request.number_redirects);
          assert.notEqual(undefined, sampleDotJson.properties.request.number_retries);
          assert.notEqual(undefined, sampleDotJson.properties.request.limit_retry_error);
          assert.notEqual(undefined, sampleDotJson.properties.request.failover_codes);
          assert.notEqual(undefined, sampleDotJson.properties.request.attempt_timeout);
          assert.notEqual(undefined, sampleDotJson.properties.request.global_request);
          assert.notEqual(undefined, sampleDotJson.properties.request.global_request.payload);
          assert.notEqual(undefined, sampleDotJson.properties.request.global_request.uriOptions);
          assert.notEqual(undefined, sampleDotJson.properties.request.global_request.addlHeaders);
          assert.notEqual(undefined, sampleDotJson.properties.request.global_request.authData);
          assert.notEqual(undefined, sampleDotJson.properties.request.healthcheck_on_timeout);
          assert.notEqual(undefined, sampleDotJson.properties.request.return_raw);
          assert.notEqual(undefined, sampleDotJson.properties.request.archiving);
          assert.notEqual(undefined, sampleDotJson.properties.request.return_request);
          assert.notEqual(undefined, sampleDotJson.properties.proxy);
          assert.notEqual(null, sampleDotJson.properties.proxy);
          assert.notEqual('', sampleDotJson.properties.proxy);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.enabled);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.host);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.port);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.protocol);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.username);
          assert.notEqual(undefined, sampleDotJson.properties.proxy.password);
          assert.notEqual(undefined, sampleDotJson.properties.mongo);
          assert.notEqual(null, sampleDotJson.properties.mongo);
          assert.notEqual('', sampleDotJson.properties.mongo);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.host);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.port);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.database);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.username);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.password);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.replSet);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl.enabled);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl.accept_invalid_cert);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl.ca_file);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl.key_file);
          assert.notEqual(undefined, sampleDotJson.properties.mongo.db_ssl.cert_file);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker.getDevice);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker.getDevicesFiltered);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker.isAlive);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker.getConfig);
          assert.notEqual(undefined, sampleDotJson.properties.devicebroker.getCount);
          assert.notEqual(undefined, sampleDotJson.properties.cache);
          assert.notEqual(undefined, sampleDotJson.properties.cache.entities);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#checkProperties', () => {
      it('should have a checkProperties function', (done) => {
        try {
          assert.equal(true, typeof a.checkProperties === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('the sample properties should be good - if failure change the log level', (done) => {
        try {
          const samplePropsJson = require('../../sampleProperties.json');
          const clean = a.checkProperties(samplePropsJson.properties);

          try {
            assert.notEqual(0, Object.keys(clean));
            assert.equal(undefined, clean.exception);
            assert.notEqual(undefined, clean.host);
            assert.notEqual(null, clean.host);
            assert.notEqual('', clean.host);
            done();
          } catch (err) {
            log.error(`Test Failure: ${err}`);
            done(err);
          }
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('README.md', () => {
      it('should have a README', (done) => {
        try {
          fs.exists('README.md', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('README.md should be customized', (done) => {
        try {
          fs.readFile('README.md', 'utf8', (err, data) => {
            assert.equal(-1, data.indexOf('[System]'));
            assert.equal(-1, data.indexOf('[system]'));
            assert.equal(-1, data.indexOf('[version]'));
            assert.equal(-1, data.indexOf('[namespace]'));
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#connect', () => {
      it('should have a connect function', (done) => {
        try {
          assert.equal(true, typeof a.connect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#healthCheck', () => {
      it('should have a healthCheck function', (done) => {
        try {
          assert.equal(true, typeof a.healthCheck === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapUpdateAdapterConfiguration', () => {
      it('should have a iapUpdateAdapterConfiguration function', (done) => {
        try {
          assert.equal(true, typeof a.iapUpdateAdapterConfiguration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapSuspendAdapter', () => {
      it('should have a iapSuspendAdapter function', (done) => {
        try {
          assert.equal(true, typeof a.iapSuspendAdapter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapUnsuspendAdapter', () => {
      it('should have a iapUnsuspendAdapter function', (done) => {
        try {
          assert.equal(true, typeof a.iapUnsuspendAdapter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapGetAdapterQueue', () => {
      it('should have a iapGetAdapterQueue function', (done) => {
        try {
          assert.equal(true, typeof a.iapGetAdapterQueue === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapFindAdapterPath', () => {
      it('should have a iapFindAdapterPath function', (done) => {
        try {
          assert.equal(true, typeof a.iapFindAdapterPath === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('iapFindAdapterPath should find atleast one path that matches', (done) => {
        try {
          a.iapFindAdapterPath('{base_path}/{version}', (data, error) => {
            try {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.equal(true, data.found);
              assert.notEqual(undefined, data.foundIn);
              assert.notEqual(null, data.foundIn);
              assert.notEqual(0, data.foundIn.length);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#iapTroubleshootAdapter', () => {
      it('should have a iapTroubleshootAdapter function', (done) => {
        try {
          assert.equal(true, typeof a.iapTroubleshootAdapter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapRunAdapterHealthcheck', () => {
      it('should have a iapRunAdapterHealthcheck function', (done) => {
        try {
          assert.equal(true, typeof a.iapRunAdapterHealthcheck === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapRunAdapterConnectivity', () => {
      it('should have a iapRunAdapterConnectivity function', (done) => {
        try {
          assert.equal(true, typeof a.iapRunAdapterConnectivity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapRunAdapterBasicGet', () => {
      it('should have a iapRunAdapterBasicGet function', (done) => {
        try {
          assert.equal(true, typeof a.iapRunAdapterBasicGet === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapMoveAdapterEntitiesToDB', () => {
      it('should have a iapMoveAdapterEntitiesToDB function', (done) => {
        try {
          assert.equal(true, typeof a.iapMoveAdapterEntitiesToDB === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#checkActionFiles', () => {
      it('should have a checkActionFiles function', (done) => {
        try {
          assert.equal(true, typeof a.checkActionFiles === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('the action files should be good - if failure change the log level as most issues are warnings', (done) => {
        try {
          const clean = a.checkActionFiles();

          try {
            for (let c = 0; c < clean.length; c += 1) {
              log.error(clean[c]);
            }
            assert.equal(0, clean.length);
            done();
          } catch (err) {
            log.error(`Test Failure: ${err}`);
            done(err);
          }
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#encryptProperty', () => {
      it('should have a encryptProperty function', (done) => {
        try {
          assert.equal(true, typeof a.encryptProperty === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('should get base64 encoded property', (done) => {
        try {
          a.encryptProperty('testing', 'base64', (data, error) => {
            try {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(undefined, data.response);
              assert.notEqual(null, data.response);
              assert.equal(0, data.response.indexOf('{code}'));
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should get encrypted property', (done) => {
        try {
          a.encryptProperty('testing', 'encrypt', (data, error) => {
            try {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(undefined, data.response);
              assert.notEqual(null, data.response);
              assert.equal(0, data.response.indexOf('{crypt}'));
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#iapDeactivateTasks', () => {
      it('should have a iapDeactivateTasks function', (done) => {
        try {
          assert.equal(true, typeof a.iapDeactivateTasks === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapActivateTasks', () => {
      it('should have a iapActivateTasks function', (done) => {
        try {
          assert.equal(true, typeof a.iapActivateTasks === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapPopulateEntityCache', () => {
      it('should have a iapPopulateEntityCache function', (done) => {
        try {
          assert.equal(true, typeof a.iapPopulateEntityCache === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapRetrieveEntitiesCache', () => {
      it('should have a iapRetrieveEntitiesCache function', (done) => {
        try {
          assert.equal(true, typeof a.iapRetrieveEntitiesCache === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#hasEntities', () => {
      it('should have a hasEntities function', (done) => {
        try {
          assert.equal(true, typeof a.hasEntities === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#getDevice', () => {
      it('should have a getDevice function', (done) => {
        try {
          assert.equal(true, typeof a.getDevice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#getDevicesFiltered', () => {
      it('should have a getDevicesFiltered function', (done) => {
        try {
          assert.equal(true, typeof a.getDevicesFiltered === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#isAlive', () => {
      it('should have a isAlive function', (done) => {
        try {
          assert.equal(true, typeof a.isAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#getConfig', () => {
      it('should have a getConfig function', (done) => {
        try {
          assert.equal(true, typeof a.getConfig === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapGetDeviceCount', () => {
      it('should have a iapGetDeviceCount function', (done) => {
        try {
          assert.equal(true, typeof a.iapGetDeviceCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapExpandedGenericAdapterRequest', () => {
      it('should have a iapExpandedGenericAdapterRequest function', (done) => {
        try {
          assert.equal(true, typeof a.iapExpandedGenericAdapterRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#genericAdapterRequest', () => {
      it('should have a genericAdapterRequest function', (done) => {
        try {
          assert.equal(true, typeof a.genericAdapterRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#genericAdapterRequestNoBasePath', () => {
      it('should have a genericAdapterRequestNoBasePath function', (done) => {
        try {
          assert.equal(true, typeof a.genericAdapterRequestNoBasePath === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapRunAdapterLint', () => {
      it('should have a iapRunAdapterLint function', (done) => {
        try {
          assert.equal(true, typeof a.iapRunAdapterLint === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('retrieve the lint results', (done) => {
        try {
          a.iapRunAdapterLint((data, error) => {
            try {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(undefined, data.status);
              assert.notEqual(null, data.status);
              assert.equal('SUCCESS', data.status);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#iapRunAdapterTests', () => {
      it('should have a iapRunAdapterTests function', (done) => {
        try {
          assert.equal(true, typeof a.iapRunAdapterTests === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });

    describe('#iapGetAdapterInventory', () => {
      it('should have a iapGetAdapterInventory function', (done) => {
        try {
          assert.equal(true, typeof a.iapGetAdapterInventory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('retrieve the inventory', (done) => {
        try {
          a.iapGetAdapterInventory((data, error) => {
            try {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });
    describe('metadata.json', () => {
      it('should have a metadata.json', (done) => {
        try {
          fs.exists('metadata.json', (val) => {
            assert.equal(true, val);
            done();
          });
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('metadata.json is customized', (done) => {
        try {
          const metadataDotJson = require('../../metadata.json');
          assert.equal('adapter-128technology', metadataDotJson.name);
          assert.notEqual(undefined, metadataDotJson.webName);
          assert.notEqual(null, metadataDotJson.webName);
          assert.notEqual('', metadataDotJson.webName);
          assert.equal('Adapter', metadataDotJson.type);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('metadata.json contains accurate documentation', (done) => {
        try {
          const metadataDotJson = require('../../metadata.json');
          assert.notEqual(undefined, metadataDotJson.documentation);
          assert.equal('https://www.npmjs.com/package/@itentialopensource/adapter-128technology', metadataDotJson.documentation.npmLink);
          assert.equal('https://docs.itential.com/opensource/docs/troubleshooting-an-adapter', metadataDotJson.documentation.faqLink);
          assert.equal('https://gitlab.com/itentialopensource/adapters/contributing-guide', metadataDotJson.documentation.contributeLink);
          assert.equal('https://itential.atlassian.net/servicedesk/customer/portals', metadataDotJson.documentation.issueLink);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
      it('metadata.json has related items', (done) => {
        try {
          const metadataDotJson = require('../../metadata.json');
          assert.notEqual(undefined, metadataDotJson.relatedItems);
          assert.notEqual(undefined, metadataDotJson.relatedItems.adapters);
          assert.notEqual(undefined, metadataDotJson.relatedItems.integrations);
          assert.notEqual(undefined, metadataDotJson.relatedItems.ecosystemApplications);
          assert.notEqual(undefined, metadataDotJson.relatedItems.workflowProjects);
          assert.notEqual(undefined, metadataDotJson.relatedItems.transformationProjects);
          assert.notEqual(undefined, metadataDotJson.relatedItems.exampleProjects);
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      });
    });
    /*
    -----------------------------------------------------------------------
    -----------------------------------------------------------------------
    *** All code above this comment will be replaced during a migration ***
    ******************* DO NOT REMOVE THIS COMMENT BLOCK ******************
    -----------------------------------------------------------------------
    -----------------------------------------------------------------------
    */

    describe('#getAlarm - errors', () => {
      it('should have a getAlarm function', (done) => {
        try {
          assert.equal(true, typeof a.getAlarm === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterAlarm - errors', () => {
      it('should have a getRouterAlarm function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterAlarm === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterAlarm(null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterAlarm', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeAlarm - errors', () => {
      it('should have a getRouterNodeAlarm function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeAlarm === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeAlarm(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeAlarm', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeAlarm('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeAlarm', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postAnalyticsRunningTransform - errors', () => {
      it('should have a postAnalyticsRunningTransform function', (done) => {
        try {
          assert.equal(true, typeof a.postAnalyticsRunningTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postAnalyticsRunningTransform(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postAnalyticsRunningTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postAnalyticsTransform - errors', () => {
      it('should have a postAnalyticsTransform function', (done) => {
        try {
          assert.equal(true, typeof a.postAnalyticsTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postAnalyticsTransform(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postAnalyticsTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#putAssetIdControl - errors', () => {
      it('should have a putAssetIdControl function', (done) => {
        try {
          assert.equal(true, typeof a.putAssetIdControl === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing assetId', (done) => {
        try {
          a.putAssetIdControl(null, null, (data, error) => {
            try {
              const displayE = 'assetId is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-putAssetIdControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.putAssetIdControl('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-putAssetIdControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getAudit - errors', () => {
      it('should have a getAudit function', (done) => {
        try {
          assert.equal(true, typeof a.getAudit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getAuditDownload - errors', () => {
      it('should have a getAuditDownload function', (done) => {
        try {
          assert.equal(true, typeof a.getAuditDownload === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigCommit - errors', () => {
      it('should have a postConfigCommit function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigCommit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigExport - errors', () => {
      it('should have a postConfigExport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigExport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigExport(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigExport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigExport - errors', () => {
      it('should have a getConfigExport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigExport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigGetJson - errors', () => {
      it('should have a getConfigGetJson function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigGetJson === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing source', (done) => {
        try {
          a.getConfigGetJson(null, (data, error) => {
            try {
              const displayE = 'source is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigGetJson', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigImport - errors', () => {
      it('should have a postConfigImport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigImport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigImport(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigImport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRevertToRunning - errors', () => {
      it('should have a postConfigRevertToRunning function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRevertToRunning === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigValidate - errors', () => {
      it('should have a postConfigValidate function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigValidate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigVersion - errors', () => {
      it('should have a getConfigVersion function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigVersion === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStore - errors', () => {
      it('should have a getConfigStore function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStore === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStore(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStore', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStore - errors', () => {
      it('should have a patchConfigStore function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStore === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStore(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStore', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStore('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStore', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthority - errors', () => {
      it('should have a getConfigStoreAuthority function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthority === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthority(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthority', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthority - errors', () => {
      it('should have a patchConfigStoreAuthority function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthority === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthority(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthority', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthority('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthority', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityDistrict - errors', () => {
      it('should have a getConfigStoreAuthorityDistrict function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityDistrict === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityDistrict(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityDistrict - errors', () => {
      it('should have a postConfigStoreAuthorityDistrict function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityDistrict === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityDistrict(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityDistrict('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityDistrictClone - errors', () => {
      it('should have a postConfigStoreAuthorityDistrictClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityDistrictClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityDistrictClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDistrictClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityDistrictClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDistrictClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityDistrict2 - errors', () => {
      it('should have a getConfigStoreAuthorityDistrict2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityDistrict2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityDistrict2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDistrict2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing district', (done) => {
        try {
          a.getConfigStoreAuthorityDistrict2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'district is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDistrict2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityDistrict - errors', () => {
      it('should have a patchConfigStoreAuthorityDistrict function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityDistrict === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityDistrict(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing district', (done) => {
        try {
          a.patchConfigStoreAuthorityDistrict('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'district is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityDistrict('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityDistrict - errors', () => {
      it('should have a deleteConfigStoreAuthorityDistrict function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityDistrict === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityDistrict(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing district', (done) => {
        try {
          a.deleteConfigStoreAuthorityDistrict('fakeparam', null, (data, error) => {
            try {
              const displayE = 'district is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityDistrict', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityDscpMap - errors', () => {
      it('should have a getConfigStoreAuthorityDscpMap function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityDscpMap === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityDscpMap(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDscpMap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityDscpMap - errors', () => {
      it('should have a postConfigStoreAuthorityDscpMap function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityDscpMap === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityDscpMap(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDscpMap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityDscpMap('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDscpMap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityDscpMapClone - errors', () => {
      it('should have a postConfigStoreAuthorityDscpMapClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityDscpMapClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityDscpMapClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDscpMapClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityDscpMapClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityDscpMapClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityDscpMapDscpmap - errors', () => {
      it('should have a getConfigStoreAuthorityDscpMapDscpmap function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityDscpMapDscpmap === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityDscpMapDscpmap(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigStoreAuthorityDscpMapDscpmap('fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityDscpMapDscpmap - errors', () => {
      it('should have a patchConfigStoreAuthorityDscpMapDscpmap function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityDscpMapDscpmap === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityDscpMapDscpmap(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.patchConfigStoreAuthorityDscpMapDscpmap('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityDscpMapDscpmap('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityDscpMapDscpmap - errors', () => {
      it('should have a deleteConfigStoreAuthorityDscpMapDscpmap function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityDscpMapDscpmap === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityDscpMapDscpmap(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.deleteConfigStoreAuthorityDscpMapDscpmap('fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityDscpMapDscpmap', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDscpMapDscpmapPrioritization - errors', () => {
      it('should have a getConfigDscpMapDscpmapPrioritization function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDscpMapDscpmapPrioritization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDscpMapDscpmapPrioritization(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpMapDscpmapPrioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigDscpMapDscpmapPrioritization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpMapDscpmapPrioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDscpMapDscpmapPrioritization - errors', () => {
      it('should have a postConfigDscpMapDscpmapPrioritization function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDscpMapDscpmapPrioritization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDscpMapDscpmapPrioritization(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpMapDscpmapPrioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigDscpMapDscpmapPrioritization('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpMapDscpmapPrioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDscpMapDscpmapPrioritization('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpMapDscpmapPrioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMapDscpmapDscpPrioritizationClone - errors', () => {
      it('should have a postConfigMapDscpmapDscpPrioritizationClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMapDscpmapDscpPrioritizationClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMapDscpmapDscpPrioritizationClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpPrioritizationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigMapDscpmapDscpPrioritizationClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpPrioritizationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMapDscpmapDscpPrioritizationClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpPrioritizationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMapDscpmapDscpPrioritizationDscpprioritization - errors', () => {
      it('should have a getConfigMapDscpmapDscpPrioritizationDscpprioritization function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMapDscpmapDscpPrioritizationDscpprioritization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMapDscpmapDscpPrioritizationDscpprioritization(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.getConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigMapDscpmapDscpPrioritizationDscpprioritization - errors', () => {
      it('should have a patchConfigMapDscpmapDscpPrioritizationDscpprioritization function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigMapDscpmapDscpPrioritizationDscpprioritization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigMapDscpmapDscpPrioritizationDscpprioritization(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.patchConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.patchConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigMapDscpmapDscpPrioritizationDscpprioritization - errors', () => {
      it('should have a deleteConfigMapDscpmapDscpPrioritizationDscpprioritization function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigMapDscpmapDscpPrioritizationDscpprioritization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigMapDscpmapDscpPrioritizationDscpprioritization(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.deleteConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.deleteConfigMapDscpmapDscpPrioritizationDscpprioritization('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMapDscpmapDscpPrioritizationDscpprioritization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDscpPrioritizationDscpprioritizationRange - errors', () => {
      it('should have a getConfigDscpPrioritizationDscpprioritizationRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDscpPrioritizationDscpprioritizationRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDscpPrioritizationDscpprioritizationRange(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigDscpPrioritizationDscpprioritizationRange('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.getConfigDscpPrioritizationDscpprioritizationRange('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDscpPrioritizationDscpprioritizationRange - errors', () => {
      it('should have a postConfigDscpPrioritizationDscpprioritizationRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDscpPrioritizationDscpprioritizationRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDscpPrioritizationDscpprioritizationRange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigDscpPrioritizationDscpprioritizationRange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.postConfigDscpPrioritizationDscpprioritizationRange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDscpPrioritizationDscpprioritizationRange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpPrioritizationDscpprioritizationRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPrioritizationDscpprioritizationDscpRangeClone - errors', () => {
      it('should have a postConfigPrioritizationDscpprioritizationDscpRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPrioritizationDscpprioritizationDscpRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPrioritizationDscpprioritizationDscpRangeClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPrioritizationDscpprioritizationDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigPrioritizationDscpprioritizationDscpRangeClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPrioritizationDscpprioritizationDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.postConfigPrioritizationDscpprioritizationDscpRangeClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPrioritizationDscpprioritizationDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPrioritizationDscpprioritizationDscpRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPrioritizationDscpprioritizationDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPrioritizationDscpprioritizationDscpRangeDscprange - errors', () => {
      it('should have a getConfigPrioritizationDscpprioritizationDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPrioritizationDscpprioritizationDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPrioritizationDscpprioritizationDscpRangeDscprange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.getConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.getConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPrioritizationDscpprioritizationDscpRangeDscprange - errors', () => {
      it('should have a patchConfigPrioritizationDscpprioritizationDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange - errors', () => {
      it('should have a deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpprioritization', (done) => {
        try {
          a.deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpprioritization is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPrioritizationDscpprioritizationDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMapDscpmapDscpTrafficClass - errors', () => {
      it('should have a getConfigMapDscpmapDscpTrafficClass function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMapDscpmapDscpTrafficClass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMapDscpmapDscpTrafficClass(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMapDscpmapDscpTrafficClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigMapDscpmapDscpTrafficClass('fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMapDscpmapDscpTrafficClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMapDscpmapDscpTrafficClass - errors', () => {
      it('should have a postConfigMapDscpmapDscpTrafficClass function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMapDscpmapDscpTrafficClass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMapDscpmapDscpTrafficClass(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpTrafficClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigMapDscpmapDscpTrafficClass('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpTrafficClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMapDscpmapDscpTrafficClass('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMapDscpmapDscpTrafficClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDscpmapDscpTrafficClassClone - errors', () => {
      it('should have a postConfigDscpmapDscpTrafficClassClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDscpmapDscpTrafficClassClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDscpmapDscpTrafficClassClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpmapDscpTrafficClassClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigDscpmapDscpTrafficClassClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpmapDscpTrafficClassClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDscpmapDscpTrafficClassClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDscpmapDscpTrafficClassClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDscpmapDscpTrafficClassDscptrafficclass - errors', () => {
      it('should have a getConfigDscpmapDscpTrafficClassDscptrafficclass function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDscpmapDscpTrafficClassDscptrafficclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDscpmapDscpTrafficClassDscptrafficclass(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.getConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigDscpmapDscpTrafficClassDscptrafficclass - errors', () => {
      it('should have a patchConfigDscpmapDscpTrafficClassDscptrafficclass function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigDscpmapDscpTrafficClassDscptrafficclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigDscpmapDscpTrafficClassDscptrafficclass(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.patchConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.patchConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigDscpmapDscpTrafficClassDscptrafficclass - errors', () => {
      it('should have a deleteConfigDscpmapDscpTrafficClassDscptrafficclass function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigDscpmapDscpTrafficClassDscptrafficclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigDscpmapDscpTrafficClassDscptrafficclass(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.deleteConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.deleteConfigDscpmapDscpTrafficClassDscptrafficclass('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigDscpmapDscpTrafficClassDscptrafficclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTrafficClassDscptrafficclassDscpRange - errors', () => {
      it('should have a getConfigTrafficClassDscptrafficclassDscpRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTrafficClassDscptrafficclassDscpRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTrafficClassDscptrafficclassDscpRange(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigTrafficClassDscptrafficclassDscpRange('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.getConfigTrafficClassDscptrafficclassDscpRange('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTrafficClassDscptrafficclassDscpRange - errors', () => {
      it('should have a postConfigTrafficClassDscptrafficclassDscpRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTrafficClassDscptrafficclassDscpRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTrafficClassDscptrafficclassDscpRange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigTrafficClassDscptrafficclassDscpRange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.postConfigTrafficClassDscptrafficclassDscpRange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTrafficClassDscptrafficclassDscpRange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTrafficClassDscptrafficclassDscpRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigClassDscptrafficclassDscpRangeClone - errors', () => {
      it('should have a postConfigClassDscptrafficclassDscpRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigClassDscptrafficclassDscpRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigClassDscptrafficclassDscpRangeClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigClassDscptrafficclassDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.postConfigClassDscptrafficclassDscpRangeClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigClassDscptrafficclassDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.postConfigClassDscptrafficclassDscpRangeClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigClassDscptrafficclassDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigClassDscptrafficclassDscpRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigClassDscptrafficclassDscpRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigClassDscptrafficclassDscpRangeDscprange - errors', () => {
      it('should have a getConfigClassDscptrafficclassDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigClassDscptrafficclassDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigClassDscptrafficclassDscpRangeDscprange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.getConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.getConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.getConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigClassDscptrafficclassDscpRangeDscprange - errors', () => {
      it('should have a patchConfigClassDscptrafficclassDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigClassDscptrafficclassDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigClassDscptrafficclassDscpRangeDscprange(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.patchConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.patchConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.patchConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigClassDscptrafficclassDscpRangeDscprange - errors', () => {
      it('should have a deleteConfigClassDscptrafficclassDscpRangeDscprange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigClassDscptrafficclassDscpRangeDscprange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigClassDscptrafficclassDscpRangeDscprange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscpmap', (done) => {
        try {
          a.deleteConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'dscpmap is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscptrafficclass', (done) => {
        try {
          a.deleteConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dscptrafficclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dscprange', (done) => {
        try {
          a.deleteConfigClassDscptrafficclassDscpRangeDscprange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dscprange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigClassDscptrafficclassDscpRangeDscprange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityIpfixCollector - errors', () => {
      it('should have a getConfigStoreAuthorityIpfixCollector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityIpfixCollector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityIpfixCollector(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityIpfixCollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityIpfixCollector - errors', () => {
      it('should have a postConfigStoreAuthorityIpfixCollector function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityIpfixCollector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityIpfixCollector(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityIpfixCollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityIpfixCollector('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityIpfixCollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityIpfixCollectorClone - errors', () => {
      it('should have a postConfigStoreAuthorityIpfixCollectorClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityIpfixCollectorClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityIpfixCollectorClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityIpfixCollectorClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityIpfixCollectorClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityIpfixCollectorClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityIpfixCollectorIpfixcollector - errors', () => {
      it('should have a getConfigStoreAuthorityIpfixCollectorIpfixcollector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityIpfixCollectorIpfixcollector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityIpfixCollectorIpfixcollector(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ipfixcollector', (done) => {
        try {
          a.getConfigStoreAuthorityIpfixCollectorIpfixcollector('fakeparam', null, (data, error) => {
            try {
              const displayE = 'ipfixcollector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityIpfixCollectorIpfixcollector - errors', () => {
      it('should have a patchConfigStoreAuthorityIpfixCollectorIpfixcollector function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityIpfixCollectorIpfixcollector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityIpfixCollectorIpfixcollector(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ipfixcollector', (done) => {
        try {
          a.patchConfigStoreAuthorityIpfixCollectorIpfixcollector('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ipfixcollector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityIpfixCollectorIpfixcollector('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityIpfixCollectorIpfixcollector - errors', () => {
      it('should have a deleteConfigStoreAuthorityIpfixCollectorIpfixcollector function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityIpfixCollectorIpfixcollector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityIpfixCollectorIpfixcollector(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ipfixcollector', (done) => {
        try {
          a.deleteConfigStoreAuthorityIpfixCollectorIpfixcollector('fakeparam', null, (data, error) => {
            try {
              const displayE = 'ipfixcollector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityIpfixCollectorIpfixcollector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityLdapServer - errors', () => {
      it('should have a getConfigStoreAuthorityLdapServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityLdapServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityLdapServer(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityLdapServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityLdapServer - errors', () => {
      it('should have a postConfigStoreAuthorityLdapServer function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityLdapServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityLdapServer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityLdapServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityLdapServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityLdapServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityLdapServerClone - errors', () => {
      it('should have a postConfigStoreAuthorityLdapServerClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityLdapServerClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityLdapServerClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityLdapServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityLdapServerClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityLdapServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityLdapServerLdapserver - errors', () => {
      it('should have a getConfigStoreAuthorityLdapServerLdapserver function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityLdapServerLdapserver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityLdapServerLdapserver(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ldapserver', (done) => {
        try {
          a.getConfigStoreAuthorityLdapServerLdapserver('fakeparam', null, (data, error) => {
            try {
              const displayE = 'ldapserver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityLdapServerLdapserver - errors', () => {
      it('should have a patchConfigStoreAuthorityLdapServerLdapserver function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityLdapServerLdapserver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityLdapServerLdapserver(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ldapserver', (done) => {
        try {
          a.patchConfigStoreAuthorityLdapServerLdapserver('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ldapserver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityLdapServerLdapserver('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityLdapServerLdapserver - errors', () => {
      it('should have a deleteConfigStoreAuthorityLdapServerLdapserver function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityLdapServerLdapserver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityLdapServerLdapserver(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ldapserver', (done) => {
        try {
          a.deleteConfigStoreAuthorityLdapServerLdapserver('fakeparam', null, (data, error) => {
            try {
              const displayE = 'ldapserver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityLdapServerLdapserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityManagementServiceGeneration - errors', () => {
      it('should have a getConfigStoreAuthorityManagementServiceGeneration function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityManagementServiceGeneration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityManagementServiceGeneration(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityManagementServiceGeneration - errors', () => {
      it('should have a patchConfigStoreAuthorityManagementServiceGeneration function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityManagementServiceGeneration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityManagementServiceGeneration(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityManagementServiceGeneration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRemoteLogin - errors', () => {
      it('should have a getConfigStoreAuthorityRemoteLogin function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRemoteLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRemoteLogin(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityRemoteLogin - errors', () => {
      it('should have a patchConfigStoreAuthorityRemoteLogin function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityRemoteLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityRemoteLogin(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityRemoteLogin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRouter - errors', () => {
      it('should have a getConfigStoreAuthorityRouter function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRouter(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityRouter - errors', () => {
      it('should have a postConfigStoreAuthorityRouter function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityRouter(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityRouter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityRouterClone - errors', () => {
      it('should have a postConfigStoreAuthorityRouterClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityRouterClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityRouterClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRouterClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityRouterClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRouterClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRouter2 - errors', () => {
      it('should have a getConfigStoreAuthorityRouter2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRouter2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRouter2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRouter2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStoreAuthorityRouter2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRouter2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityRouter - errors', () => {
      it('should have a patchConfigStoreAuthorityRouter function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityRouter(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStoreAuthorityRouter('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityRouter('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityRouter - errors', () => {
      it('should have a deleteConfigStoreAuthorityRouter function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityRouter(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStoreAuthorityRouter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterApplicationIdentification - errors', () => {
      it('should have a getConfigAuthorityRouterApplicationIdentification function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterApplicationIdentification === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterApplicationIdentification(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterApplicationIdentification', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterApplicationIdentification('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterApplicationIdentification', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRouterApplicationIdentification - errors', () => {
      it('should have a patchConfigAuthorityRouterApplicationIdentification function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRouterApplicationIdentification === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRouterApplicationIdentification(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterApplicationIdentification', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAuthorityRouterApplicationIdentification('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterApplicationIdentification', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRouterApplicationIdentification('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterApplicationIdentification', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterBfd - errors', () => {
      it('should have a getConfigAuthorityRouterBfd function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterBfd(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterBfd('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRouterBfd - errors', () => {
      it('should have a patchConfigAuthorityRouterBfd function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRouterBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRouterBfd(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAuthorityRouterBfd('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRouterBfd('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterDistrictSettings - errors', () => {
      it('should have a getConfigAuthorityRouterDistrictSettings function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterDistrictSettings === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterDistrictSettings(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterDistrictSettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterDistrictSettings('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterDistrictSettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterDistrictSettings - errors', () => {
      it('should have a postConfigAuthorityRouterDistrictSettings function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterDistrictSettings === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterDistrictSettings(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDistrictSettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterDistrictSettings('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDistrictSettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterDistrictSettings('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDistrictSettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterDistrictSettingsClone - errors', () => {
      it('should have a postConfigRouterDistrictSettingsClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterDistrictSettingsClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterDistrictSettingsClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDistrictSettingsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterDistrictSettingsClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDistrictSettingsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterDistrictSettingsClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDistrictSettingsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterDistrictSettingsDistrictsettings - errors', () => {
      it('should have a getConfigRouterDistrictSettingsDistrictsettings function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterDistrictSettingsDistrictsettings === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterDistrictSettingsDistrictsettings(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterDistrictSettingsDistrictsettings('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing districtsettings', (done) => {
        try {
          a.getConfigRouterDistrictSettingsDistrictsettings('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'districtsettings is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterDistrictSettingsDistrictsettings - errors', () => {
      it('should have a patchConfigRouterDistrictSettingsDistrictsettings function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterDistrictSettingsDistrictsettings === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterDistrictSettingsDistrictsettings(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterDistrictSettingsDistrictsettings('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing districtsettings', (done) => {
        try {
          a.patchConfigRouterDistrictSettingsDistrictsettings('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'districtsettings is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterDistrictSettingsDistrictsettings('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterDistrictSettingsDistrictsettings - errors', () => {
      it('should have a deleteConfigRouterDistrictSettingsDistrictsettings function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterDistrictSettingsDistrictsettings === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterDistrictSettingsDistrictsettings(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterDistrictSettingsDistrictsettings('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing districtsettings', (done) => {
        try {
          a.deleteConfigRouterDistrictSettingsDistrictsettings('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'districtsettings is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDistrictSettingsDistrictsettings', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPeerPathSlaMetricsAdvertisement - errors', () => {
      it('should have a getConfigPeerPathSlaMetricsAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPeerPathSlaMetricsAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPeerPathSlaMetricsAdvertisement(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPeerPathSlaMetricsAdvertisement('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing districtsettings', (done) => {
        try {
          a.getConfigPeerPathSlaMetricsAdvertisement('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'districtsettings is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPeerPathSlaMetricsAdvertisement - errors', () => {
      it('should have a patchConfigPeerPathSlaMetricsAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPeerPathSlaMetricsAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPeerPathSlaMetricsAdvertisement(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPeerPathSlaMetricsAdvertisement('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing districtsettings', (done) => {
        try {
          a.patchConfigPeerPathSlaMetricsAdvertisement('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'districtsettings is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPeerPathSlaMetricsAdvertisement('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathSlaMetricsAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterDns - errors', () => {
      it('should have a getConfigAuthorityRouterDns function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterDns === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterDns(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterDns', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterDns('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterDns', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterDns - errors', () => {
      it('should have a postConfigAuthorityRouterDns function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterDns === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterDns(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDns', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterDns('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDns', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterDns('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterDns', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterDnsClone - errors', () => {
      it('should have a postConfigRouterDnsClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterDnsClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterDnsClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDnsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterDnsClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDnsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterDnsClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterDnsClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterDnsDnsconfig - errors', () => {
      it('should have a getConfigRouterDnsDnsconfig function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterDnsDnsconfig === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterDnsDnsconfig(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterDnsDnsconfig('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dnsconfig', (done) => {
        try {
          a.getConfigRouterDnsDnsconfig('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dnsconfig is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterDnsDnsconfig - errors', () => {
      it('should have a patchConfigRouterDnsDnsconfig function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterDnsDnsconfig === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterDnsDnsconfig(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterDnsDnsconfig('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dnsconfig', (done) => {
        try {
          a.patchConfigRouterDnsDnsconfig('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'dnsconfig is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterDnsDnsconfig('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterDnsDnsconfig - errors', () => {
      it('should have a deleteConfigRouterDnsDnsconfig function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterDnsDnsconfig === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterDnsDnsconfig(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterDnsDnsconfig('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing dnsconfig', (done) => {
        try {
          a.deleteConfigRouterDnsDnsconfig('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'dnsconfig is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterDnsDnsconfig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterEntitlement - errors', () => {
      it('should have a getConfigAuthorityRouterEntitlement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterEntitlement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterEntitlement(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterEntitlement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRouterEntitlement - errors', () => {
      it('should have a patchConfigAuthorityRouterEntitlement function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRouterEntitlement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRouterEntitlement(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAuthorityRouterEntitlement('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRouterEntitlement('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterManagementServiceGeneration - errors', () => {
      it('should have a getConfigRouterManagementServiceGeneration function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterManagementServiceGeneration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterManagementServiceGeneration(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterManagementServiceGeneration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterManagementServiceGeneration - errors', () => {
      it('should have a patchConfigRouterManagementServiceGeneration function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterManagementServiceGeneration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterManagementServiceGeneration(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterManagementServiceGeneration('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterManagementServiceGeneration('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterManagementServiceGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterNatPool - errors', () => {
      it('should have a getConfigAuthorityRouterNatPool function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterNatPool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterNatPool(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterNatPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterNatPool('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterNatPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterNatPool - errors', () => {
      it('should have a postConfigAuthorityRouterNatPool function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterNatPool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterNatPool(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNatPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterNatPool('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNatPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterNatPool('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNatPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterNatPoolClone - errors', () => {
      it('should have a postConfigRouterNatPoolClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterNatPoolClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterNatPoolClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNatPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterNatPoolClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNatPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterNatPoolClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNatPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterNatPoolNatpool - errors', () => {
      it('should have a getConfigRouterNatPoolNatpool function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterNatPoolNatpool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterNatPoolNatpool(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterNatPoolNatpool('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.getConfigRouterNatPoolNatpool('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterNatPoolNatpool - errors', () => {
      it('should have a patchConfigRouterNatPoolNatpool function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterNatPoolNatpool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterNatPoolNatpool(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterNatPoolNatpool('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.patchConfigRouterNatPoolNatpool('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterNatPoolNatpool('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterNatPoolNatpool - errors', () => {
      it('should have a deleteConfigRouterNatPoolNatpool function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterNatPoolNatpool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterNatPoolNatpool(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterNatPoolNatpool('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.deleteConfigRouterNatPoolNatpool('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNatPoolNatpool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNatPoolNatpoolAddress - errors', () => {
      it('should have a getConfigNatPoolNatpoolAddress function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNatPoolNatpoolAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNatPoolNatpoolAddress(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNatPoolNatpoolAddress('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.getConfigNatPoolNatpoolAddress('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNatPoolNatpoolAddress - errors', () => {
      it('should have a postConfigNatPoolNatpoolAddress function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNatPoolNatpoolAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNatPoolNatpoolAddress(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNatPoolNatpoolAddress('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.postConfigNatPoolNatpoolAddress('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNatPoolNatpoolAddress('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNatPoolNatpoolAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPoolNatpoolAddressClone - errors', () => {
      it('should have a postConfigPoolNatpoolAddressClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPoolNatpoolAddressClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPoolNatpoolAddressClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigPoolNatpoolAddressClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.postConfigPoolNatpoolAddressClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPoolNatpoolAddressClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPoolNatpoolAddressMove - errors', () => {
      it('should have a postConfigPoolNatpoolAddressMove function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPoolNatpoolAddressMove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPoolNatpoolAddressMove(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigPoolNatpoolAddressMove('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.postConfigPoolNatpoolAddressMove('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPoolNatpoolAddressMove('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolNatpoolAddressMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPoolNatpoolAddressAddresspool - errors', () => {
      it('should have a getConfigPoolNatpoolAddressAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPoolNatpoolAddressAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPoolNatpoolAddressAddresspool(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPoolNatpoolAddressAddresspool('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.getConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPoolNatpoolAddressAddresspool - errors', () => {
      it('should have a patchConfigPoolNatpoolAddressAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPoolNatpoolAddressAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPoolNatpoolAddressAddresspool(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPoolNatpoolAddressAddresspool('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.patchConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPoolNatpoolAddressAddresspool - errors', () => {
      it('should have a deleteConfigPoolNatpoolAddressAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPoolNatpoolAddressAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPoolNatpoolAddressAddresspool(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigPoolNatpoolAddressAddresspool('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing natpool', (done) => {
        try {
          a.deleteConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'natpool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigPoolNatpoolAddressAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolNatpoolAddressAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterNode - errors', () => {
      it('should have a getConfigAuthorityRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterNode(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterNode('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterNode - errors', () => {
      it('should have a postConfigAuthorityRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterNode(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterNode('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterNode('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterNodeClone - errors', () => {
      it('should have a postConfigRouterNodeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterNodeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterNodeClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterNodeClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterNodeClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterNode - errors', () => {
      it('should have a getConfigRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterNode(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterNode('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigRouterNode('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterNode - errors', () => {
      it('should have a patchConfigRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterNode(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterNode('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigRouterNode('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterNode('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterNode - errors', () => {
      it('should have a deleteConfigRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterNode(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterNode('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigRouterNode('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterNodeDeviceInterface - errors', () => {
      it('should have a getConfigRouterNodeDeviceInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterNodeDeviceInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterNodeDeviceInterface(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterNodeDeviceInterface('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigRouterNodeDeviceInterface('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterNodeDeviceInterface - errors', () => {
      it('should have a postConfigRouterNodeDeviceInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterNodeDeviceInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterNodeDeviceInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterNodeDeviceInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigRouterNodeDeviceInterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterNodeDeviceInterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNodeDeviceInterfaceClone - errors', () => {
      it('should have a postConfigNodeDeviceInterfaceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNodeDeviceInterfaceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNodeDeviceInterfaceClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNodeDeviceInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNodeDeviceInterfaceClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNodeDeviceInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNodeDeviceInterfaceClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNodeDeviceInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNodeDeviceInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNodeDeviceInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNodeDeviceInterfaceDeviceinterface - errors', () => {
      it('should have a getConfigNodeDeviceInterfaceDeviceinterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNodeDeviceInterfaceDeviceinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNodeDeviceInterfaceDeviceinterface - errors', () => {
      it('should have a patchConfigNodeDeviceInterfaceDeviceinterface function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNodeDeviceInterfaceDeviceinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNodeDeviceInterfaceDeviceinterface - errors', () => {
      it('should have a deleteConfigNodeDeviceInterfaceDeviceinterface function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNodeDeviceInterfaceDeviceinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNodeDeviceInterfaceDeviceinterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNodeDeviceInterfaceDeviceinterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNodeDeviceInterfaceDeviceinterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNodeDeviceInterfaceDeviceinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceInterfaceDeviceinterfaceLoadBalancing - errors', () => {
      it('should have a getConfigDeviceInterfaceDeviceinterfaceLoadBalancing function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceInterfaceDeviceinterfaceLoadBalancing === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLoadBalancing(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing - errors', () => {
      it('should have a patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLoadBalancing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNodeDeviceInterfaceDeviceinterfaceLte - errors', () => {
      it('should have a getConfigNodeDeviceInterfaceDeviceinterfaceLte function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNodeDeviceInterfaceDeviceinterfaceLte === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfaceLte(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNodeDeviceInterfaceDeviceinterfaceLte - errors', () => {
      it('should have a patchConfigNodeDeviceInterfaceDeviceinterfaceLte function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfaceLte('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfaceLte', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceInterfaceDeviceinterfaceLteAuthentication - errors', () => {
      it('should have a getConfigDeviceInterfaceDeviceinterfaceLteAuthentication function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceInterfaceDeviceinterfaceLteAuthentication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLteAuthentication(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication - errors', () => {
      it('should have a patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceLteAuthentication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceInterfaceDeviceinterfaceNetwork - errors', () => {
      it('should have a getConfigDeviceInterfaceDeviceinterfaceNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceInterfaceDeviceinterfaceNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceNetwork(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDeviceInterfaceDeviceinterfaceNetwork - errors', () => {
      it('should have a postConfigDeviceInterfaceDeviceinterfaceNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDeviceInterfaceDeviceinterfaceNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDeviceInterfaceDeviceinterfaceNetwork(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDeviceInterfaceDeviceinterfaceNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceInterfaceDeviceinterfaceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigInterfaceDeviceinterfaceNetworkClone - errors', () => {
      it('should have a postConfigInterfaceDeviceinterfaceNetworkClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigInterfaceDeviceinterfaceNetworkClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigInterfaceDeviceinterfaceNetworkClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceDeviceinterfaceNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigInterfaceDeviceinterfaceNetworkClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceDeviceinterfaceNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigInterfaceDeviceinterfaceNetworkClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceDeviceinterfaceNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigInterfaceDeviceinterfaceNetworkClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceDeviceinterfaceNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigInterfaceDeviceinterfaceNetworkClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceDeviceinterfaceNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigInterfaceDeviceinterfaceNetworkNetworkinterface - errors', () => {
      it('should have a getConfigInterfaceDeviceinterfaceNetworkNetworkinterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface - errors', () => {
      it('should have a patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface - errors', () => {
      it('should have a deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceDeviceinterfaceNetworkNetworkinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress - errors', () => {
      it('should have a getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress - errors', () => {
      it('should have a postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceAddressClone - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceAddressClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceAddressClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAddressClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceAddress - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceAddress function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkInterfaceNetworkinterfaceAddress - errors', () => {
      it('should have a patchConfigNetworkInterfaceNetworkinterfaceAddress function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkInterfaceNetworkinterfaceAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNetworkInterfaceNetworkinterfaceAddress - errors', () => {
      it('should have a deleteConfigNetworkInterfaceNetworkinterfaceAddress function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNetworkInterfaceNetworkinterfaceAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceAddressHostService - errors', () => {
      it('should have a getConfigNetworkinterfaceAddressHostService function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceAddressHostService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkinterfaceAddressHostService - errors', () => {
      it('should have a postConfigNetworkinterfaceAddressHostService function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkinterfaceAddressHostService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkinterfaceAddressHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAddressHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressHostServiceClone - errors', () => {
      it('should have a postConfigAddressHostServiceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressHostServiceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressHostServiceClone(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressHostServiceHostservice - errors', () => {
      it('should have a getConfigAddressHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressHostServiceHostservice - errors', () => {
      it('should have a patchConfigAddressHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAddressHostServiceHostservice - errors', () => {
      it('should have a deleteConfigAddressHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAddressHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigAddressHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceAccessPolicy - errors', () => {
      it('should have a getConfigHostServiceHostserviceAccessPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceAccessPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceAccessPolicy - errors', () => {
      it('should have a postConfigHostServiceHostserviceAccessPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceAccessPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceHostserviceAccessPolicyClone - errors', () => {
      it('should have a postConfigServiceHostserviceAccessPolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceHostserviceAccessPolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceHostserviceAccessPolicyAccesspolicy - errors', () => {
      it('should have a getConfigServiceHostserviceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceHostserviceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceHostserviceAccessPolicyAccesspolicy - errors', () => {
      it('should have a patchConfigServiceHostserviceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceHostserviceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceHostserviceAccessPolicyAccesspolicy - errors', () => {
      it('should have a deleteConfigServiceHostserviceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceAddressPool - errors', () => {
      it('should have a getConfigHostServiceHostserviceAddressPool function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceAddressPool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceAddressPool - errors', () => {
      it('should have a postConfigHostServiceHostserviceAddressPool function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceAddressPool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceAddressPool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAddressPool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceHostserviceAddressPoolClone - errors', () => {
      it('should have a postConfigServiceHostserviceAddressPoolClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceHostserviceAddressPoolClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceHostserviceAddressPoolClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAddressPoolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceHostserviceAddressPoolAddresspool - errors', () => {
      it('should have a getConfigServiceHostserviceAddressPoolAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceHostserviceAddressPoolAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceHostserviceAddressPoolAddresspool - errors', () => {
      it('should have a patchConfigServiceHostserviceAddressPoolAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceHostserviceAddressPoolAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceHostserviceAddressPoolAddresspool - errors', () => {
      it('should have a deleteConfigServiceHostserviceAddressPoolAddresspool function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceHostserviceAddressPoolAddresspool === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigServiceHostserviceAddressPoolAddresspool('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAddressPoolAddresspool', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostserviceAddressPoolAddresspoolCustom - errors', () => {
      it('should have a getConfigHostserviceAddressPoolAddresspoolCustom function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostserviceAddressPoolAddresspoolCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostserviceAddressPoolAddresspoolCustom - errors', () => {
      it('should have a postConfigHostserviceAddressPoolAddresspoolCustom function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostserviceAddressPoolAddresspoolCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostserviceAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressPoolAddresspoolCustomClone - errors', () => {
      it('should have a postConfigAddressPoolAddresspoolCustomClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressPoolAddresspoolCustomClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressPoolAddresspoolCustom - errors', () => {
      it('should have a getConfigAddressPoolAddresspoolCustom function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressPoolAddresspoolCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressPoolAddresspoolCustom - errors', () => {
      it('should have a patchConfigAddressPoolAddresspoolCustom function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressPoolAddresspoolCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAddressPoolAddresspoolCustom - errors', () => {
      it('should have a deleteConfigAddressPoolAddresspoolCustom function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAddressPoolAddresspoolCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.deleteConfigAddressPoolAddresspoolCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressPoolAddresspoolCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressPoolAddresspoolStaticAssignment - errors', () => {
      it('should have a getConfigAddressPoolAddresspoolStaticAssignment function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressPoolAddresspoolStaticAssignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressPoolAddresspoolStaticAssignment - errors', () => {
      it('should have a postConfigAddressPoolAddresspoolStaticAssignment function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressPoolAddresspoolStaticAssignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPoolAddresspoolStaticAssignmentClone - errors', () => {
      it('should have a postConfigPoolAddresspoolStaticAssignmentClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPoolAddresspoolStaticAssignmentClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPoolAddresspoolStaticAssignmentStaticassignment - errors', () => {
      it('should have a getConfigPoolAddresspoolStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPoolAddresspoolStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPoolAddresspoolStaticAssignmentStaticassignment - errors', () => {
      it('should have a patchConfigPoolAddresspoolStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPoolAddresspoolStaticAssignmentStaticassignment - errors', () => {
      it('should have a deleteConfigPoolAddresspoolStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddresspoolStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a getConfigAddresspoolStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddresspoolStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a postConfigAddresspoolStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddresspoolStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticAssignmentStaticassignmentCustomClone - errors', () => {
      it('should have a postConfigStaticAssignmentStaticassignmentCustomClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticAssignmentStaticassignmentCustomClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a getConfigStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a patchConfigStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a deleteConfigStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticAssignmentStaticassignmentRoute - errors', () => {
      it('should have a getConfigStaticAssignmentStaticassignmentRoute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticAssignmentStaticassignmentRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticAssignmentStaticassignmentRoute - errors', () => {
      it('should have a postConfigStaticAssignmentStaticassignmentRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticAssignmentStaticassignmentRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAssignmentStaticassignmentStaticRouteClone - errors', () => {
      it('should have a postConfigAssignmentStaticassignmentStaticRouteClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAssignmentStaticassignmentStaticRouteClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAssignmentStaticassignmentStaticRouteStaticroute - errors', () => {
      it('should have a getConfigAssignmentStaticassignmentStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAssignmentStaticassignmentStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAssignmentStaticassignmentStaticRouteStaticroute - errors', () => {
      it('should have a patchConfigAssignmentStaticassignmentStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAssignmentStaticassignmentStaticRouteStaticroute - errors', () => {
      it('should have a deleteConfigAssignmentStaticassignmentStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigVendorIdentifyingSpecificInformation - errors', () => {
      it('should have a getConfigVendorIdentifyingSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigVendorIdentifyingSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigVendorIdentifyingSpecificInformation - errors', () => {
      it('should have a postConfigVendorIdentifyingSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigVendorIdentifyingSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigIdentifyingVendorSpecificInformationClone - errors', () => {
      it('should have a postConfigIdentifyingVendorSpecificInformationClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigIdentifyingVendorSpecificInformationClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation - errors', () => {
      it('should have a getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation - errors', () => {
      it('should have a patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation(null, null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation - errors', () => {
      it('should have a deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAssignmentStaticassignmentVendorSpecificInformation - errors', () => {
      it('should have a getConfigAssignmentStaticassignmentVendorSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAssignmentStaticassignmentVendorSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAssignmentStaticassignmentVendorSpecificInformation - errors', () => {
      it('should have a postConfigAssignmentStaticassignmentVendorSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAssignmentStaticassignmentVendorSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticassignmentVendorSpecificInformationClone - errors', () => {
      it('should have a postConfigStaticassignmentVendorSpecificInformationClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticassignmentVendorSpecificInformationClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressPoolAddresspoolStaticRoute - errors', () => {
      it('should have a getConfigAddressPoolAddresspoolStaticRoute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressPoolAddresspoolStaticRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressPoolAddresspoolStaticRoute - errors', () => {
      it('should have a postConfigAddressPoolAddresspoolStaticRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressPoolAddresspoolStaticRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressPoolAddresspoolStaticRoute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressPoolAddresspoolStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPoolAddresspoolStaticRouteClone - errors', () => {
      it('should have a postConfigPoolAddresspoolStaticRouteClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPoolAddresspoolStaticRouteClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPoolAddresspoolStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPoolAddresspoolStaticRouteStaticroute - errors', () => {
      it('should have a getConfigPoolAddresspoolStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPoolAddresspoolStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPoolAddresspoolStaticRouteStaticroute - errors', () => {
      it('should have a patchConfigPoolAddresspoolStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPoolAddresspoolStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPoolAddresspoolStaticRouteStaticroute - errors', () => {
      it('should have a deleteConfigPoolAddresspoolStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPoolAddresspoolStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.deleteConfigPoolAddresspoolStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPoolAddresspoolStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigVendorIdentifyingSpecificInformation2 - errors', () => {
      it('should have a getConfigVendorIdentifyingSpecificInformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigVendorIdentifyingSpecificInformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigVendorIdentifyingSpecificInformation2 - errors', () => {
      it('should have a postConfigVendorIdentifyingSpecificInformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigVendorIdentifyingSpecificInformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigIdentifyingVendorSpecificInformationClone2 - errors', () => {
      it('should have a postConfigIdentifyingVendorSpecificInformationClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigIdentifyingVendorSpecificInformationClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 - errors', () => {
      it('should have a getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 - errors', () => {
      it('should have a patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 - errors', () => {
      it('should have a deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPoolAddresspoolVendorSpecificInformation - errors', () => {
      it('should have a getConfigPoolAddresspoolVendorSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPoolAddresspoolVendorSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPoolAddresspoolVendorSpecificInformation - errors', () => {
      it('should have a postConfigPoolAddresspoolVendorSpecificInformation function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPoolAddresspoolVendorSpecificInformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPoolAddresspoolVendorSpecificInformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPoolAddresspoolVendorSpecificInformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddresspoolVendorSpecificInformationClone - errors', () => {
      it('should have a postConfigAddresspoolVendorSpecificInformationClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddresspoolVendorSpecificInformationClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddresspoolVendorSpecificInformationClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddresspoolVendorSpecificInformationClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddresspoolVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a getConfigAddresspoolVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.getConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation - errors', () => {
      it('should have a deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addresspool', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addresspool is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddresspoolVendorSpecificInformationVendorspecificinformation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceStaticAssignment - errors', () => {
      it('should have a getConfigHostServiceHostserviceStaticAssignment function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceStaticAssignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceStaticAssignment - errors', () => {
      it('should have a postConfigHostServiceHostserviceStaticAssignment function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceStaticAssignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceStaticAssignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceStaticAssignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceHostserviceStaticAssignmentClone - errors', () => {
      it('should have a postConfigServiceHostserviceStaticAssignmentClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceHostserviceStaticAssignmentClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceHostserviceStaticAssignmentClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceStaticAssignmentClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceHostserviceStaticAssignmentStaticassignment - errors', () => {
      it('should have a getConfigServiceHostserviceStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceHostserviceStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceHostserviceStaticAssignmentStaticassignment - errors', () => {
      it('should have a patchConfigServiceHostserviceStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceHostserviceStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceHostserviceStaticAssignmentStaticassignment - errors', () => {
      it('should have a deleteConfigServiceHostserviceStaticAssignmentStaticassignment function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigServiceHostserviceStaticAssignmentStaticassignment('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceStaticAssignmentStaticassignment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostserviceStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a getConfigHostserviceStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostserviceStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostserviceStaticAssignmentStaticassignmentCustom - errors', () => {
      it('should have a postConfigHostserviceStaticAssignmentStaticassignmentCustom function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostserviceStaticAssignmentStaticassignmentCustom === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostserviceStaticAssignmentStaticassignmentCustom('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceStaticAssignmentStaticassignmentCustom', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticAssignmentStaticassignmentCustomClone2 - errors', () => {
      it('should have a postConfigStaticAssignmentStaticassignmentCustomClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticAssignmentStaticassignmentCustomClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentCustomClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentCustomClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticAssignmentStaticassignmentCustom2 - errors', () => {
      it('should have a getConfigStaticAssignmentStaticassignmentCustom2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticAssignmentStaticassignmentCustom2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStaticAssignmentStaticassignmentCustom2 - errors', () => {
      it('should have a patchConfigStaticAssignmentStaticassignmentCustom2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStaticAssignmentStaticassignmentCustom2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStaticAssignmentStaticassignmentCustom2 - errors', () => {
      it('should have a deleteConfigStaticAssignmentStaticassignmentCustom2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStaticAssignmentStaticassignmentCustom2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing custom', (done) => {
        try {
          a.deleteConfigStaticAssignmentStaticassignmentCustom2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'custom is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticAssignmentStaticassignmentCustom2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticAssignmentStaticassignmentRoute2 - errors', () => {
      it('should have a getConfigStaticAssignmentStaticassignmentRoute2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticAssignmentStaticassignmentRoute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticAssignmentStaticassignmentRoute2 - errors', () => {
      it('should have a postConfigStaticAssignmentStaticassignmentRoute2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticAssignmentStaticassignmentRoute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticAssignmentStaticassignmentRoute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticAssignmentStaticassignmentRoute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAssignmentStaticassignmentStaticRouteClone2 - errors', () => {
      it('should have a postConfigAssignmentStaticassignmentStaticRouteClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAssignmentStaticassignmentStaticRouteClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentStaticRouteClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentStaticRouteClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAssignmentStaticassignmentStaticRouteStaticroute2 - errors', () => {
      it('should have a getConfigAssignmentStaticassignmentStaticRouteStaticroute2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAssignmentStaticassignmentStaticRouteStaticroute2 - errors', () => {
      it('should have a patchConfigAssignmentStaticassignmentStaticRouteStaticroute2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2 - errors', () => {
      it('should have a deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAssignmentStaticassignmentStaticRouteStaticroute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigVendorIdentifyingSpecificInformation3 - errors', () => {
      it('should have a getConfigVendorIdentifyingSpecificInformation3 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigVendorIdentifyingSpecificInformation3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigVendorIdentifyingSpecificInformation3 - errors', () => {
      it('should have a postConfigVendorIdentifyingSpecificInformation3 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigVendorIdentifyingSpecificInformation3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigVendorIdentifyingSpecificInformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigVendorIdentifyingSpecificInformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigIdentifyingVendorSpecificInformationClone3 - errors', () => {
      it('should have a postConfigIdentifyingVendorSpecificInformationClone3 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigIdentifyingVendorSpecificInformationClone3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigIdentifyingVendorSpecificInformationClone3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigIdentifyingVendorSpecificInformationClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 - errors', () => {
      it('should have a getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 - errors', () => {
      it('should have a patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 - errors', () => {
      it('should have a deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendoridentifyingvendorspecificinformation', (done) => {
        try {
          a.deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendoridentifyingvendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigIdentifyingVendorSpecificInformationVendoridentifyingvendorspecificinformation3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAssignmentStaticassignmentVendorSpecificInformation2 - errors', () => {
      it('should have a getConfigAssignmentStaticassignmentVendorSpecificInformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAssignmentStaticassignmentVendorSpecificInformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAssignmentStaticassignmentVendorSpecificInformation2 - errors', () => {
      it('should have a postConfigAssignmentStaticassignmentVendorSpecificInformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAssignmentStaticassignmentVendorSpecificInformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAssignmentStaticassignmentVendorSpecificInformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAssignmentStaticassignmentVendorSpecificInformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticassignmentVendorSpecificInformationClone2 - errors', () => {
      it('should have a postConfigStaticassignmentVendorSpecificInformationClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticassignmentVendorSpecificInformationClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticassignmentVendorSpecificInformationClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticassignmentVendorSpecificInformationClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 - errors', () => {
      it('should have a getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 - errors', () => {
      it('should have a patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 - errors', () => {
      it('should have a deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticassignment', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticassignment is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vendorspecificinformation', (done) => {
        try {
          a.deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vendorspecificinformation is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticassignmentVendorSpecificInformationVendorspecificinformation2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressHostServiceHostserviceTransport - errors', () => {
      it('should have a getConfigAddressHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressHostServiceHostserviceTransport - errors', () => {
      it('should have a postConfigAddressHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceTransportClone - errors', () => {
      it('should have a postConfigHostServiceHostserviceTransportClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceTransportClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceTransport - errors', () => {
      it('should have a getConfigHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigHostServiceHostserviceTransport - errors', () => {
      it('should have a patchConfigHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigHostServiceHostserviceTransport - errors', () => {
      it('should have a deleteConfigHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostserviceTransportPortRange - errors', () => {
      it('should have a getConfigHostserviceTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostserviceTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostserviceTransportPortRange - errors', () => {
      it('should have a postConfigHostserviceTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostserviceTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTransportPortRangeClone - errors', () => {
      it('should have a postConfigTransportPortRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTransportPortRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTransportPortRangeClone(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTransportPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTransportPortRangePortrange - errors', () => {
      it('should have a getConfigTransportPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTransportPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTransportPortRangePortrange(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTransportPortRangePortrange - errors', () => {
      it('should have a patchConfigTransportPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTransportPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange(null, null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigTransportPortRangePortrange - errors', () => {
      it('should have a deleteConfigTransportPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigTransportPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing address', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'address is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency - errors', () => {
      it('should have a getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency - errors', () => {
      it('should have a postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceAdjacencyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceAdjacency - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkInterfaceNetworkinterfaceAdjacency - errors', () => {
      it('should have a patchConfigNetworkInterfaceNetworkinterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNetworkInterfaceNetworkinterfaceAdjacency - errors', () => {
      it('should have a deleteConfigNetworkInterfaceNetworkinterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceAdjacencyBfd - errors', () => {
      it('should have a getConfigNetworkinterfaceAdjacencyBfd function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceAdjacencyBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceAdjacencyBfd - errors', () => {
      it('should have a patchConfigNetworkinterfaceAdjacencyBfd function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceAdjacencyBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdjacencyNatKeepAlive - errors', () => {
      it('should have a getConfigAdjacencyNatKeepAlive function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdjacencyNatKeepAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdjacencyNatKeepAlive - errors', () => {
      it('should have a patchConfigAdjacencyNatKeepAlive function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdjacencyNatKeepAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdjacencyNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdjacencyPathMtuDiscovery - errors', () => {
      it('should have a getConfigAdjacencyPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdjacencyPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdjacencyPathMtuDiscovery - errors', () => {
      it('should have a patchConfigAdjacencyPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdjacencyPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdjacencyPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceAdjacencyPortRange - errors', () => {
      it('should have a getConfigNetworkinterfaceAdjacencyPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceAdjacencyPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkinterfaceAdjacencyPortRange - errors', () => {
      it('should have a postConfigNetworkinterfaceAdjacencyPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkinterfaceAdjacencyPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkinterfaceAdjacencyPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceAdjacencyPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAdjacencyPortRangeClone - errors', () => {
      it('should have a postConfigAdjacencyPortRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAdjacencyPortRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAdjacencyPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAdjacencyPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdjacencyPortRangePortrange - errors', () => {
      it('should have a getConfigAdjacencyPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdjacencyPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdjacencyPortRangePortrange - errors', () => {
      it('should have a patchConfigAdjacencyPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdjacencyPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAdjacencyPortRangePortrange - errors', () => {
      it('should have a deleteConfigAdjacencyPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAdjacencyPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigAdjacencyPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAdjacencyPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdjacencyPostEncryptionPadding - errors', () => {
      it('should have a getConfigAdjacencyPostEncryptionPadding function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdjacencyPostEncryptionPadding === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdjacencyPostEncryptionPadding - errors', () => {
      it('should have a patchConfigAdjacencyPostEncryptionPadding function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdjacencyPostEncryptionPadding === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdjacencyPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceAdjacencySessionOptimization - errors', () => {
      it('should have a getConfigNetworkinterfaceAdjacencySessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceAdjacencySessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceAdjacencySessionOptimization - errors', () => {
      it('should have a patchConfigNetworkinterfaceAdjacencySessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceAdjacencySessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencySessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencySessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdjacencyStepPeerPathAdvertisement - errors', () => {
      it('should have a getConfigAdjacencyStepPeerPathAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdjacencyStepPeerPathAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdjacencyStepPeerPathAdvertisement - errors', () => {
      it('should have a patchConfigAdjacencyStepPeerPathAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdjacencyStepPeerPathAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdjacencyStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdjacencyStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPeerPathAdvertisementSlaMetrics - errors', () => {
      it('should have a getConfigPeerPathAdvertisementSlaMetrics function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPeerPathAdvertisementSlaMetrics === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPeerPathAdvertisementSlaMetrics - errors', () => {
      it('should have a patchConfigPeerPathAdvertisementSlaMetrics function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPeerPathAdvertisementSlaMetrics === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSlaMetricsDecreaseReportDelay - errors', () => {
      it('should have a getConfigSlaMetricsDecreaseReportDelay function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSlaMetricsDecreaseReportDelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSlaMetricsDecreaseReportDelay - errors', () => {
      it('should have a postConfigSlaMetricsDecreaseReportDelay function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSlaMetricsDecreaseReportDelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMetricsDecreaseReportDelayClone - errors', () => {
      it('should have a postConfigMetricsDecreaseReportDelayClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMetricsDecreaseReportDelayClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMetricsDecreaseReportDelayDecreasereportdelay - errors', () => {
      it('should have a getConfigMetricsDecreaseReportDelayDecreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMetricsDecreaseReportDelayDecreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigMetricsDecreaseReportDelayDecreasereportdelay - errors', () => {
      it('should have a patchConfigMetricsDecreaseReportDelayDecreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigMetricsDecreaseReportDelayDecreasereportdelay - errors', () => {
      it('should have a deleteConfigMetricsDecreaseReportDelayDecreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSlaMetricsIncreaseReportDelay - errors', () => {
      it('should have a getConfigSlaMetricsIncreaseReportDelay function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSlaMetricsIncreaseReportDelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSlaMetricsIncreaseReportDelay - errors', () => {
      it('should have a postConfigSlaMetricsIncreaseReportDelay function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSlaMetricsIncreaseReportDelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMetricsIncreaseReportDelayClone - errors', () => {
      it('should have a postConfigMetricsIncreaseReportDelayClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMetricsIncreaseReportDelayClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMetricsIncreaseReportDelayIncreasereportdelay - errors', () => {
      it('should have a getConfigMetricsIncreaseReportDelayIncreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMetricsIncreaseReportDelayIncreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigMetricsIncreaseReportDelayIncreasereportdelay - errors', () => {
      it('should have a patchConfigMetricsIncreaseReportDelayIncreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigMetricsIncreaseReportDelayIncreasereportdelay - errors', () => {
      it('should have a deleteConfigMetricsIncreaseReportDelayIncreasereportdelay function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdvertisementSlaMetricsSignificanceThreshold - errors', () => {
      it('should have a getConfigAdvertisementSlaMetricsSignificanceThreshold function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdvertisementSlaMetricsSignificanceThreshold === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdvertisementSlaMetricsSignificanceThreshold - errors', () => {
      it('should have a patchConfigAdvertisementSlaMetricsSignificanceThreshold function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdvertisementSlaMetricsSignificanceThreshold === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceAdjacencyUdpTransform - errors', () => {
      it('should have a getConfigNetworkinterfaceAdjacencyUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceAdjacencyUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.getConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceAdjacencyUdpTransform - errors', () => {
      it('should have a patchConfigNetworkinterfaceAdjacencyUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceAdjacencyUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing adjacency', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'adjacency is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceAdjacencyUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceAdjacencyUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceHostService - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceHostService function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceHostService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceHostService(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceHostService - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceHostService function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceHostService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceHostService('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceHostService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigInterfaceNetworkinterfaceHostServiceClone - errors', () => {
      it('should have a postConfigInterfaceNetworkinterfaceHostServiceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigInterfaceNetworkinterfaceHostServiceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceHostServiceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceHostServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigInterfaceNetworkinterfaceHostServiceHostservice - errors', () => {
      it('should have a getConfigInterfaceNetworkinterfaceHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigInterfaceNetworkinterfaceHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigInterfaceNetworkinterfaceHostServiceHostservice - errors', () => {
      it('should have a patchConfigInterfaceNetworkinterfaceHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigInterfaceNetworkinterfaceHostServiceHostservice - errors', () => {
      it('should have a deleteConfigInterfaceNetworkinterfaceHostServiceHostservice function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceHostServiceHostservice('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceHostServiceHostservice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceAccessPolicy2 - errors', () => {
      it('should have a getConfigHostServiceHostserviceAccessPolicy2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceAccessPolicy2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceAccessPolicy2 - errors', () => {
      it('should have a postConfigHostServiceHostserviceAccessPolicy2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceAccessPolicy2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceAccessPolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceAccessPolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceHostserviceAccessPolicyClone2 - errors', () => {
      it('should have a postConfigServiceHostserviceAccessPolicyClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceHostserviceAccessPolicyClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceHostserviceAccessPolicyClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceHostserviceAccessPolicyClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceHostserviceAccessPolicyAccesspolicy2 - errors', () => {
      it('should have a getConfigServiceHostserviceAccessPolicyAccesspolicy2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceHostserviceAccessPolicyAccesspolicy2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.getConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceHostserviceAccessPolicyAccesspolicy2 - errors', () => {
      it('should have a patchConfigServiceHostserviceAccessPolicyAccesspolicy2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceHostserviceAccessPolicyAccesspolicy2 - errors', () => {
      it('should have a deleteConfigServiceHostserviceAccessPolicyAccesspolicy2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.deleteConfigServiceHostserviceAccessPolicyAccesspolicy2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceHostserviceAccessPolicyAccesspolicy2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceHostServiceHostserviceTransport - errors', () => {
      it('should have a getConfigNetworkinterfaceHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkinterfaceHostServiceHostserviceTransport - errors', () => {
      it('should have a postConfigNetworkinterfaceHostServiceHostserviceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkinterfaceHostServiceHostserviceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkinterfaceHostServiceHostserviceTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceHostServiceHostserviceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostServiceHostserviceTransportClone2 - errors', () => {
      it('should have a postConfigHostServiceHostserviceTransportClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostServiceHostserviceTransportClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostServiceHostserviceTransportClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostServiceHostserviceTransportClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostServiceHostserviceTransport2 - errors', () => {
      it('should have a getConfigHostServiceHostserviceTransport2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostServiceHostserviceTransport2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigHostServiceHostserviceTransport2 - errors', () => {
      it('should have a patchConfigHostServiceHostserviceTransport2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigHostServiceHostserviceTransport2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigHostServiceHostserviceTransport2 - errors', () => {
      it('should have a deleteConfigHostServiceHostserviceTransport2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigHostServiceHostserviceTransport2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigHostServiceHostserviceTransport2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigHostServiceHostserviceTransport2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigHostserviceTransportPortRange2 - errors', () => {
      it('should have a getConfigHostserviceTransportPortRange2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigHostserviceTransportPortRange2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigHostserviceTransportPortRange2 - errors', () => {
      it('should have a postConfigHostserviceTransportPortRange2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigHostserviceTransportPortRange2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigHostserviceTransportPortRange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigHostserviceTransportPortRange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTransportPortRangeClone2 - errors', () => {
      it('should have a postConfigTransportPortRangeClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTransportPortRangeClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTransportPortRangeClone2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTransportPortRangeClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTransportPortRangePortrange2 - errors', () => {
      it('should have a getConfigTransportPortRangePortrange2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTransportPortRangePortrange2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTransportPortRangePortrange2 - errors', () => {
      it('should have a patchConfigTransportPortRangePortrange2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTransportPortRangePortrange2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2(null, null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigTransportPortRangePortrange2 - errors', () => {
      it('should have a deleteConfigTransportPortRangePortrange2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigTransportPortRangePortrange2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing hostservice', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'hostservice is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceIfcfgOption - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceIfcfgOption function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceIfcfgOption - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceIfcfgOption function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceIfcfgOption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceIfcfgOption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigInterfaceNetworkinterfaceIfcfgOptionClone - errors', () => {
      it('should have a postConfigInterfaceNetworkinterfaceIfcfgOptionClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigInterfaceNetworkinterfaceIfcfgOptionClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigInterfaceNetworkinterfaceIfcfgOptionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption - errors', () => {
      it('should have a getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ifcfgoption', (done) => {
        try {
          a.getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ifcfgoption is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption - errors', () => {
      it('should have a patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ifcfgoption', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ifcfgoption is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption - errors', () => {
      it('should have a deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ifcfgoption', (done) => {
        try {
          a.deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ifcfgoption is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigInterfaceNetworkinterfaceIfcfgOptionIfcfgoption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceManagementVector - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceManagementVector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceManagementVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceManagementVector(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkInterfaceNetworkinterfaceManagementVector - errors', () => {
      it('should have a patchConfigNetworkInterfaceNetworkinterfaceManagementVector function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceManagementVector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceManagementVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor - errors', () => {
      it('should have a getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor - errors', () => {
      it('should have a postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceNeighborClone - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceNeighborClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceNeighbor - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkInterfaceNetworkinterfaceNeighbor - errors', () => {
      it('should have a patchConfigNetworkInterfaceNetworkinterfaceNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNetworkInterfaceNetworkinterfaceNeighbor - errors', () => {
      it('should have a deleteConfigNetworkInterfaceNetworkinterfaceNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood - errors', () => {
      it('should have a getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood - errors', () => {
      it('should have a postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigDeviceinterfaceNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove - errors', () => {
      it('should have a postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkInterfaceNetworkinterfaceNeighborhoodMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkInterfaceNetworkinterfaceNeighborhood - errors', () => {
      it('should have a getConfigNetworkInterfaceNetworkinterfaceNeighborhood function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkInterfaceNetworkinterfaceNeighborhood - errors', () => {
      it('should have a patchConfigNetworkInterfaceNetworkinterfaceNeighborhood function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood - errors', () => {
      it('should have a deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNetworkInterfaceNetworkinterfaceNeighborhood', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceNeighborhoodBfd - errors', () => {
      it('should have a getConfigNetworkinterfaceNeighborhoodBfd function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceNeighborhoodBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceNeighborhoodBfd - errors', () => {
      it('should have a patchConfigNetworkinterfaceNeighborhoodBfd function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceNeighborhoodBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodBfd('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborhoodNatKeepAlive - errors', () => {
      it('should have a getConfigNeighborhoodNatKeepAlive function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborhoodNatKeepAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborhoodNatKeepAlive - errors', () => {
      it('should have a patchConfigNeighborhoodNatKeepAlive function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborhoodNatKeepAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborhoodNatKeepAlive('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodNatKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborhoodPathMtuDiscovery - errors', () => {
      it('should have a getConfigNeighborhoodPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborhoodPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborhoodPathMtuDiscovery - errors', () => {
      it('should have a patchConfigNeighborhoodPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborhoodPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborhoodPathMtuDiscovery('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceNeighborhoodPortRange - errors', () => {
      it('should have a getConfigNetworkinterfaceNeighborhoodPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceNeighborhoodPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNetworkinterfaceNeighborhoodPortRange - errors', () => {
      it('should have a postConfigNetworkinterfaceNeighborhoodPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNetworkinterfaceNeighborhoodPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNetworkinterfaceNeighborhoodPortRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNetworkinterfaceNeighborhoodPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNeighborhoodPortRangeClone - errors', () => {
      it('should have a postConfigNeighborhoodPortRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNeighborhoodPortRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNeighborhoodPortRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborhoodPortRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborhoodPortRangePortrange - errors', () => {
      it('should have a getConfigNeighborhoodPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborhoodPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborhoodPortRangePortrange - errors', () => {
      it('should have a patchConfigNeighborhoodPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborhoodPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNeighborhoodPortRangePortrange - errors', () => {
      it('should have a deleteConfigNeighborhoodPortRangePortrange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNeighborhoodPortRangePortrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigNeighborhoodPortRangePortrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborhoodPortRangePortrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborhoodPostEncryptionPadding - errors', () => {
      it('should have a getConfigNeighborhoodPostEncryptionPadding function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborhoodPostEncryptionPadding === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborhoodPostEncryptionPadding - errors', () => {
      it('should have a patchConfigNeighborhoodPostEncryptionPadding function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborhoodPostEncryptionPadding === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborhoodPostEncryptionPadding('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodPostEncryptionPadding', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceNeighborhoodSessionOptimization - errors', () => {
      it('should have a getConfigNetworkinterfaceNeighborhoodSessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceNeighborhoodSessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceNeighborhoodSessionOptimization - errors', () => {
      it('should have a patchConfigNetworkinterfaceNeighborhoodSessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborhoodStepPeerPathAdvertisement - errors', () => {
      it('should have a getConfigNeighborhoodStepPeerPathAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborhoodStepPeerPathAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborhoodStepPeerPathAdvertisement - errors', () => {
      it('should have a patchConfigNeighborhoodStepPeerPathAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborhoodStepPeerPathAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborhoodStepPeerPathAdvertisement('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborhoodStepPeerPathAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPeerPathAdvertisementSlaMetrics2 - errors', () => {
      it('should have a getConfigPeerPathAdvertisementSlaMetrics2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPeerPathAdvertisementSlaMetrics2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPeerPathAdvertisementSlaMetrics2 - errors', () => {
      it('should have a patchConfigPeerPathAdvertisementSlaMetrics2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPeerPathAdvertisementSlaMetrics2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPeerPathAdvertisementSlaMetrics2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPeerPathAdvertisementSlaMetrics2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSlaMetricsDecreaseReportDelay2 - errors', () => {
      it('should have a getConfigSlaMetricsDecreaseReportDelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSlaMetricsDecreaseReportDelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSlaMetricsDecreaseReportDelay2 - errors', () => {
      it('should have a postConfigSlaMetricsDecreaseReportDelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSlaMetricsDecreaseReportDelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSlaMetricsDecreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsDecreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMetricsDecreaseReportDelayClone2 - errors', () => {
      it('should have a postConfigMetricsDecreaseReportDelayClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMetricsDecreaseReportDelayClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMetricsDecreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsDecreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMetricsDecreaseReportDelayDecreasereportdelay2 - errors', () => {
      it('should have a getConfigMetricsDecreaseReportDelayDecreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.getConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigMetricsDecreaseReportDelayDecreasereportdelay2 - errors', () => {
      it('should have a patchConfigMetricsDecreaseReportDelayDecreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2 - errors', () => {
      it('should have a deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing decreasereportdelay', (done) => {
        try {
          a.deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'decreasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsDecreaseReportDelayDecreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSlaMetricsIncreaseReportDelay2 - errors', () => {
      it('should have a getConfigSlaMetricsIncreaseReportDelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSlaMetricsIncreaseReportDelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSlaMetricsIncreaseReportDelay2 - errors', () => {
      it('should have a postConfigSlaMetricsIncreaseReportDelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSlaMetricsIncreaseReportDelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSlaMetricsIncreaseReportDelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSlaMetricsIncreaseReportDelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigMetricsIncreaseReportDelayClone2 - errors', () => {
      it('should have a postConfigMetricsIncreaseReportDelayClone2 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigMetricsIncreaseReportDelayClone2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigMetricsIncreaseReportDelayClone2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigMetricsIncreaseReportDelayClone2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigMetricsIncreaseReportDelayIncreasereportdelay2 - errors', () => {
      it('should have a getConfigMetricsIncreaseReportDelayIncreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.getConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigMetricsIncreaseReportDelayIncreasereportdelay2 - errors', () => {
      it('should have a patchConfigMetricsIncreaseReportDelayIncreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2(null, null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2 - errors', () => {
      it('should have a deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing increasereportdelay', (done) => {
        try {
          a.deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'increasereportdelay is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigMetricsIncreaseReportDelayIncreasereportdelay2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAdvertisementSlaMetricsSignificanceThreshold2 - errors', () => {
      it('should have a getConfigAdvertisementSlaMetricsSignificanceThreshold2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAdvertisementSlaMetricsSignificanceThreshold2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAdvertisementSlaMetricsSignificanceThreshold2 - errors', () => {
      it('should have a patchConfigAdvertisementSlaMetricsSignificanceThreshold2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAdvertisementSlaMetricsSignificanceThreshold2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAdvertisementSlaMetricsSignificanceThreshold2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNetworkinterfaceNeighborhoodUdpTransform - errors', () => {
      it('should have a getConfigNetworkinterfaceNeighborhoodUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNetworkinterfaceNeighborhoodUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.getConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNetworkinterfaceNeighborhoodUdpTransform - errors', () => {
      it('should have a patchConfigNetworkinterfaceNeighborhoodUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNetworkinterfaceNeighborhoodUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkinterface', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'networkinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighborhood', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighborhood is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNetworkinterfaceNeighborhoodUdpTransform('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNetworkinterfaceNeighborhoodUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNodeDeviceInterfaceDeviceinterfacePppoe - errors', () => {
      it('should have a getConfigNodeDeviceInterfaceDeviceinterfacePppoe function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNodeDeviceInterfaceDeviceinterfacePppoe === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfacePppoe(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNodeDeviceInterfaceDeviceinterfacePppoe - errors', () => {
      it('should have a patchConfigNodeDeviceInterfaceDeviceinterfacePppoe function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNodeDeviceInterfaceDeviceinterfacePppoe('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeDeviceInterfaceDeviceinterfacePppoe', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceInterfaceDeviceinterfaceSessionOptimization - errors', () => {
      it('should have a getConfigDeviceInterfaceDeviceinterfaceSessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceInterfaceDeviceinterfaceSessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceSessionOptimization(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization - errors', () => {
      it('should have a patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceSessionOptimization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering - errors', () => {
      it('should have a getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering - errors', () => {
      it('should have a patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceinterface', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigDeviceInterfaceDeviceinterfaceTrafficEngineering', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterNodeReachabilityDetection - errors', () => {
      it('should have a getConfigRouterNodeReachabilityDetection function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterNodeReachabilityDetection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterNodeReachabilityDetection(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterNodeReachabilityDetection('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigRouterNodeReachabilityDetection('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterNodeReachabilityDetection - errors', () => {
      it('should have a patchConfigRouterNodeReachabilityDetection function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterNodeReachabilityDetection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterNodeReachabilityDetection(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterNodeReachabilityDetection('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigRouterNodeReachabilityDetection('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterNodeReachabilityDetection('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterNodeSshKeepalive - errors', () => {
      it('should have a getConfigRouterNodeSshKeepalive function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterNodeSshKeepalive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterNodeSshKeepalive(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterNodeSshKeepalive('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigRouterNodeSshKeepalive('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterNodeSshKeepalive - errors', () => {
      it('should have a patchConfigRouterNodeSshKeepalive function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterNodeSshKeepalive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterNodeSshKeepalive(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterNodeSshKeepalive('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigRouterNodeSshKeepalive('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterNodeSshKeepalive('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterNodeSshKeepalive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigKeepaliveInterConductorRouterServer - errors', () => {
      it('should have a getConfigKeepaliveInterConductorRouterServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigKeepaliveInterConductorRouterServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigKeepaliveInterConductorRouterServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigKeepaliveInterConductorRouterServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigKeepaliveInterConductorRouterServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigKeepaliveInterConductorRouterServer - errors', () => {
      it('should have a patchConfigKeepaliveInterConductorRouterServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigKeepaliveInterConductorRouterServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigKeepaliveInterConductorRouterServer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigKeepaliveInterConductorRouterServer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigKeepaliveInterConductorRouterServer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigKeepaliveInterConductorRouterServer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigKeepaliveInterConductorRouterServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNodeSshKeepaliveInter - errors', () => {
      it('should have a getConfigNodeSshKeepaliveInter function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNodeSshKeepaliveInter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInter(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInter('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInter('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNodeSshKeepaliveInter - errors', () => {
      it('should have a patchConfigNodeSshKeepaliveInter function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNodeSshKeepaliveInter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInter(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInter('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInter('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInter('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSshKeepaliveInterNodeServer - errors', () => {
      it('should have a getConfigSshKeepaliveInterNodeServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSshKeepaliveInterNodeServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSshKeepaliveInterNodeServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSshKeepaliveInterNodeServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigSshKeepaliveInterNodeServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSshKeepaliveInterNodeServer - errors', () => {
      it('should have a patchConfigSshKeepaliveInterNodeServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSshKeepaliveInterNodeServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSshKeepaliveInterNodeServer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigSshKeepaliveInterNodeServer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigSshKeepaliveInterNodeServer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSshKeepaliveInterNodeServer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSshKeepaliveInterNodeServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNodeSshKeepaliveInterRouter - errors', () => {
      it('should have a getConfigNodeSshKeepaliveInterRouter function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNodeSshKeepaliveInterRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInterRouter(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInterRouter('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getConfigNodeSshKeepaliveInterRouter('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNodeSshKeepaliveInterRouter - errors', () => {
      it('should have a patchConfigNodeSshKeepaliveInterRouter function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNodeSshKeepaliveInterRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInterRouter(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInterRouter('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInterRouter('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNodeSshKeepaliveInterRouter('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNodeSshKeepaliveInterRouter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterPathMtuDiscovery - errors', () => {
      it('should have a getConfigRouterPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterPathMtuDiscovery(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterPathMtuDiscovery('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterPathMtuDiscovery - errors', () => {
      it('should have a patchConfigRouterPathMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterPathMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterPathMtuDiscovery(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterPathMtuDiscovery('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterPathMtuDiscovery('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPathMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterPeer - errors', () => {
      it('should have a getConfigAuthorityRouterPeer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterPeer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterPeer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterPeer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterPeer - errors', () => {
      it('should have a postConfigAuthorityRouterPeer function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterPeer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterPeer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterPeer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterPeer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterPeerClone - errors', () => {
      it('should have a postConfigRouterPeerClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterPeerClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterPeerClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterPeerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterPeerClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterPeerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterPeerClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterPeerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterPeer - errors', () => {
      it('should have a getConfigRouterPeer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterPeer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterPeer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterPeer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing peer', (done) => {
        try {
          a.getConfigRouterPeer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'peer is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterPeer - errors', () => {
      it('should have a patchConfigRouterPeer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterPeer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterPeer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterPeer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing peer', (done) => {
        try {
          a.patchConfigRouterPeer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'peer is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterPeer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterPeer - errors', () => {
      it('should have a deleteConfigRouterPeer function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterPeer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterPeer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterPeer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing peer', (done) => {
        try {
          a.deleteConfigRouterPeer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'peer is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterPeer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterPeerBfd - errors', () => {
      it('should have a getConfigRouterPeerBfd function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterPeerBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterPeerBfd(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterPeerBfd('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing peer', (done) => {
        try {
          a.getConfigRouterPeerBfd('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'peer is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterPeerBfd - errors', () => {
      it('should have a patchConfigRouterPeerBfd function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterPeerBfd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterPeerBfd(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterPeerBfd('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing peer', (done) => {
        try {
          a.patchConfigRouterPeerBfd('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'peer is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterPeerBfd('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterPeerBfd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterRedundancyGroup - errors', () => {
      it('should have a getConfigAuthorityRouterRedundancyGroup function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterRedundancyGroup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterRedundancyGroup(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterRedundancyGroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterRedundancyGroup('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterRedundancyGroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterRedundancyGroup - errors', () => {
      it('should have a postConfigAuthorityRouterRedundancyGroup function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterRedundancyGroup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterRedundancyGroup(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRedundancyGroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterRedundancyGroup('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRedundancyGroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterRedundancyGroup('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRedundancyGroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRedundancyGroupClone - errors', () => {
      it('should have a postConfigRouterRedundancyGroupClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRedundancyGroupClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRedundancyGroupClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRedundancyGroupClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRedundancyGroupClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRedundancyGroupRedundancygroup - errors', () => {
      it('should have a getConfigRouterRedundancyGroupRedundancygroup function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRedundancyGroupRedundancygroup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroup(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroup('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroup('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterRedundancyGroupRedundancygroup - errors', () => {
      it('should have a patchConfigRouterRedundancyGroupRedundancygroup function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterRedundancyGroupRedundancygroup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterRedundancyGroupRedundancygroup(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterRedundancyGroupRedundancygroup('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.patchConfigRouterRedundancyGroupRedundancygroup('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterRedundancyGroupRedundancygroup('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterRedundancyGroupRedundancygroup - errors', () => {
      it('should have a deleteConfigRouterRedundancyGroupRedundancygroup function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterRedundancyGroupRedundancygroup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterRedundancyGroupRedundancygroup(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterRedundancyGroupRedundancygroup('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.deleteConfigRouterRedundancyGroupRedundancygroup('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRedundancyGroupRedundancygroup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRedundancyGroupRedundancygroupMember - errors', () => {
      it('should have a getConfigRouterRedundancyGroupRedundancygroupMember function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRedundancyGroupRedundancygroupMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroupMember(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroupMember('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.getConfigRouterRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRedundancyGroupRedundancygroupMember - errors', () => {
      it('should have a postConfigRouterRedundancyGroupRedundancygroupMember function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRedundancyGroupRedundancygroupMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRedundancyGroupRedundancygroupMember(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRedundancyGroupRedundancygroupMember('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.postConfigRouterRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRedundancyGroupRedundancygroupMemberClone - errors', () => {
      it('should have a postConfigRedundancyGroupRedundancygroupMemberClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRedundancyGroupRedundancygroupMemberClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRedundancyGroupRedundancygroupMemberClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRedundancyGroupRedundancygroupMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRedundancyGroupRedundancygroupMemberClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRedundancyGroupRedundancygroupMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.postConfigRedundancyGroupRedundancygroupMemberClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRedundancyGroupRedundancygroupMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRedundancyGroupRedundancygroupMemberClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRedundancyGroupRedundancygroupMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRedundancyGroupRedundancygroupMember - errors', () => {
      it('should have a getConfigRedundancyGroupRedundancygroupMember function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRedundancyGroupRedundancygroupMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRedundancyGroupRedundancygroupMember(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRedundancyGroupRedundancygroupMember('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.getConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.getConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRedundancyGroupRedundancygroupMember - errors', () => {
      it('should have a patchConfigRedundancyGroupRedundancygroupMember function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRedundancyGroupRedundancygroupMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRedundancyGroupRedundancygroupMember(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRedundancyGroupRedundancygroupMember('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.patchConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.patchConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRedundancyGroupRedundancygroupMember - errors', () => {
      it('should have a deleteConfigRedundancyGroupRedundancygroupMember function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRedundancyGroupRedundancygroupMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRedundancyGroupRedundancygroupMember(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRedundancyGroupRedundancygroupMember('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redundancygroup', (done) => {
        try {
          a.deleteConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redundancygroup is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.deleteConfigRedundancyGroupRedundancygroupMember('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRedundancyGroupRedundancygroupMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterRouting - errors', () => {
      it('should have a getConfigAuthorityRouterRouting function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterRouting(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterRouting('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterRouting - errors', () => {
      it('should have a postConfigAuthorityRouterRouting function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterRouting(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterRouting('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterRouting('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRoutingClone - errors', () => {
      it('should have a postConfigRouterRoutingClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRoutingClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRoutingClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRoutingClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRoutingClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRouting - errors', () => {
      it('should have a getConfigRouterRouting function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRouting(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRouting('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouterRouting('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterRouting - errors', () => {
      it('should have a patchConfigRouterRouting function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterRouting(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterRouting('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRouterRouting('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterRouting('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterRouting - errors', () => {
      it('should have a deleteConfigRouterRouting function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterRouting(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterRouting('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRouterRouting('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRoutingInterface - errors', () => {
      it('should have a getConfigRouterRoutingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRoutingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRoutingInterface(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRoutingInterface('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouterRoutingInterface('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRoutingInterface - errors', () => {
      it('should have a postConfigRouterRoutingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRoutingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRoutingInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRoutingInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRouterRoutingInterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRoutingInterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingInterfaceClone - errors', () => {
      it('should have a postConfigRoutingInterfaceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingInterfaceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingInterfaceClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingInterfaceClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingInterfaceClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingInterface - errors', () => {
      it('should have a getConfigRoutingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingInterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.getConfigRoutingInterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingInterface - errors', () => {
      it('should have a patchConfigRoutingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingInterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingInterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingInterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.patchConfigRoutingInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingInterface - errors', () => {
      it('should have a deleteConfigRoutingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingInterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.deleteConfigRoutingInterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRoutingOspf - errors', () => {
      it('should have a getConfigRouterRoutingOspf function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRoutingOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRoutingOspf(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRoutingOspf('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouterRoutingOspf('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRoutingOspf - errors', () => {
      it('should have a postConfigRouterRoutingOspf function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRoutingOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRoutingOspf(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRoutingOspf('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRouterRoutingOspf('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRoutingOspf('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingOspfClone - errors', () => {
      it('should have a postConfigRoutingOspfClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingOspfClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingOspfClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingOspfClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingOspfClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingOspfClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingOspf - errors', () => {
      it('should have a getConfigRoutingOspf function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingOspf(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingOspf('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingOspf('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigRoutingOspf('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingOspf - errors', () => {
      it('should have a patchConfigRoutingOspf function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingOspf(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingOspf('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingOspf('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigRoutingOspf('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingOspf('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingOspf - errors', () => {
      it('should have a deleteConfigRoutingOspf function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingOspf(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingOspf('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingOspf('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.deleteConfigRoutingOspf('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingOspfAdvertiseDefault - errors', () => {
      it('should have a getConfigRoutingOspfAdvertiseDefault function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingOspfAdvertiseDefault === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingOspfAdvertiseDefault(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingOspfAdvertiseDefault('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingOspfAdvertiseDefault('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigRoutingOspfAdvertiseDefault('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingOspfAdvertiseDefault - errors', () => {
      it('should have a patchConfigRoutingOspfAdvertiseDefault function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingOspfAdvertiseDefault === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingOspfAdvertiseDefault(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingOspfAdvertiseDefault('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingOspfAdvertiseDefault('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigRoutingOspfAdvertiseDefault('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingOspfAdvertiseDefault('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingOspfAdvertiseDefault', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingOspfArea - errors', () => {
      it('should have a getConfigRoutingOspfArea function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingOspfArea === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingOspfArea(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingOspfArea('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingOspfArea('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigRoutingOspfArea('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingOspfArea - errors', () => {
      it('should have a postConfigRoutingOspfArea function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingOspfArea === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingOspfArea(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingOspfArea('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingOspfArea('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigRoutingOspfArea('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingOspfArea('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigOspfAreaClone - errors', () => {
      it('should have a postConfigOspfAreaClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigOspfAreaClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigOspfAreaClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigOspfAreaClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigOspfAreaClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigOspfAreaClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigOspfAreaClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigOspfArea - errors', () => {
      it('should have a getConfigOspfArea function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigOspfArea === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigOspfArea(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigOspfArea('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigOspfArea('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.getConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigOspfArea - errors', () => {
      it('should have a patchConfigOspfArea function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigOspfArea === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigOspfArea(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigOspfArea('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigOspfArea('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.patchConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigOspfArea - errors', () => {
      it('should have a deleteConfigOspfArea function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigOspfArea === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigOspfArea(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigOspfArea('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigOspfArea('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.deleteConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.deleteConfigOspfArea('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfArea', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigOspfAreaInterface - errors', () => {
      it('should have a getConfigOspfAreaInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigOspfAreaInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigOspfAreaInterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigOspfAreaInterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigOspfAreaInterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigOspfAreaInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.getConfigOspfAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigOspfAreaInterface - errors', () => {
      it('should have a postConfigOspfAreaInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigOspfAreaInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigOspfAreaInterface(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigOspfAreaInterface('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigOspfAreaInterface('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigOspfAreaInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.postConfigOspfAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigOspfAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAreaInterfaceClone - errors', () => {
      it('should have a postConfigAreaInterfaceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAreaInterfaceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAreaInterfaceClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAreaInterfaceClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigAreaInterfaceClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigAreaInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.postConfigAreaInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAreaInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAreaInterface - errors', () => {
      it('should have a getConfigAreaInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAreaInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAreaInterface(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAreaInterface('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAreaInterface('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.getConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.getConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAreaInterface - errors', () => {
      it('should have a patchConfigAreaInterface function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAreaInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAreaInterface(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAreaInterface - errors', () => {
      it('should have a deleteConfigAreaInterface function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAreaInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAreaInterface(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAreaInterface('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigAreaInterface('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.deleteConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.deleteConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing interfaceParam', (done) => {
        try {
          a.deleteConfigAreaInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'interfaceParam is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigOspfAreaSummaryRange - errors', () => {
      it('should have a getConfigOspfAreaSummaryRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigOspfAreaSummaryRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigOspfAreaSummaryRange(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigOspfAreaSummaryRange('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.getConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigOspfAreaSummaryRange - errors', () => {
      it('should have a postConfigOspfAreaSummaryRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigOspfAreaSummaryRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigOspfAreaSummaryRange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfAreaSummaryRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAreaSummaryRangeClone - errors', () => {
      it('should have a postConfigAreaSummaryRangeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAreaSummaryRangeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAreaSummaryRangeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAreaSummaryRangeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAreaSummaryRangeSummaryrange - errors', () => {
      it('should have a getConfigAreaSummaryRangeSummaryrange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAreaSummaryRangeSummaryrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing summaryrange', (done) => {
        try {
          a.getConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'summaryrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAreaSummaryRangeSummaryrange - errors', () => {
      it('should have a patchConfigAreaSummaryRangeSummaryrange function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAreaSummaryRangeSummaryrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing summaryrange', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'summaryrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAreaSummaryRangeSummaryrange - errors', () => {
      it('should have a deleteConfigAreaSummaryRangeSummaryrange function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAreaSummaryRangeSummaryrange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing area', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'area is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing summaryrange', (done) => {
        try {
          a.deleteConfigAreaSummaryRangeSummaryrange('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'summaryrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAreaSummaryRangeSummaryrange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingOspfRedistribute - errors', () => {
      it('should have a getConfigRoutingOspfRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingOspfRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingOspfRedistribute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingOspfRedistribute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingOspfRedistribute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigRoutingOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingOspfRedistribute - errors', () => {
      it('should have a postConfigRoutingOspfRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingOspfRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingOspfRedistribute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingOspfRedistribute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingOspfRedistribute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigRoutingOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigOspfRedistributeClone - errors', () => {
      it('should have a postConfigOspfRedistributeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigOspfRedistributeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigOspfRedistributeClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigOspfRedistributeClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigOspfRedistributeClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.postConfigOspfRedistributeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigOspfRedistributeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigOspfRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigOspfRedistribute - errors', () => {
      it('should have a getConfigOspfRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigOspfRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigOspfRedistribute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigOspfRedistribute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigOspfRedistribute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.getConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.getConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigOspfRedistribute - errors', () => {
      it('should have a patchConfigOspfRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigOspfRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigOspfRedistribute(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigOspfRedistribute('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigOspfRedistribute('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.patchConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.patchConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigOspfRedistribute - errors', () => {
      it('should have a deleteConfigOspfRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigOspfRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigOspfRedistribute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigOspfRedistribute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigOspfRedistribute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing ospf', (done) => {
        try {
          a.deleteConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'ospf is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.deleteConfigOspfRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigOspfRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRoutingProtocol - errors', () => {
      it('should have a getConfigRouterRoutingProtocol function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRoutingProtocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRoutingProtocol(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRoutingProtocol('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouterRoutingProtocol('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRoutingProtocol - errors', () => {
      it('should have a postConfigRouterRoutingProtocol function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRoutingProtocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRoutingProtocol(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRoutingProtocol('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRouterRoutingProtocol('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRoutingProtocol('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingProtocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolClone - errors', () => {
      it('should have a postConfigRoutingProtocolClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocol - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocol function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocol(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocol('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingProtocolRoutingprotocol - errors', () => {
      it('should have a patchConfigRoutingProtocolRoutingprotocol function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingProtocolRoutingprotocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocol(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocol('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingProtocolRoutingprotocol - errors', () => {
      it('should have a deleteConfigRoutingProtocolRoutingprotocol function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingProtocolRoutingprotocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocol(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocol('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocol('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolAddressFamily - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolAddressFamily function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolAddressFamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolAddressFamily(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolRoutingprotocolAddressFamily - errors', () => {
      it('should have a postConfigRoutingProtocolRoutingprotocolAddressFamily function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolRoutingprotocolAddressFamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolAddressFamily(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolAddressFamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigProtocolRoutingprotocolAddressFamilyClone - errors', () => {
      it('should have a postConfigProtocolRoutingprotocolAddressFamilyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigProtocolRoutingprotocolAddressFamilyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigProtocolRoutingprotocolAddressFamilyClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigProtocolRoutingprotocolAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigProtocolRoutingprotocolAddressFamilyClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigProtocolRoutingprotocolAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigProtocolRoutingprotocolAddressFamilyClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigProtocolRoutingprotocolAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigProtocolRoutingprotocolAddressFamilyClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigProtocolRoutingprotocolAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigProtocolRoutingprotocolAddressFamilyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigProtocolRoutingprotocolAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigProtocolRoutingprotocolAddressFamilyAddressfamily - errors', () => {
      it('should have a getConfigProtocolRoutingprotocolAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily - errors', () => {
      it('should have a patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily - errors', () => {
      it('should have a deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigProtocolRoutingprotocolAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyAggregate - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyAggregate function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyAggregate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyAggregate(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyAggregate('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressFamilyAddressfamilyAggregate - errors', () => {
      it('should have a postConfigAddressFamilyAddressfamilyAggregate function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressFamilyAddressfamilyAggregate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyAggregate('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyAggregate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigFamilyAddressfamilyAggregateAddressClone - errors', () => {
      it('should have a postConfigFamilyAddressfamilyAggregateAddressClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigFamilyAddressfamilyAggregateAddressClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigFamilyAddressfamilyAggregateAddressClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFamilyAddressfamilyAggregateAddressClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigFamilyAddressfamilyAggregateAddressAggregateaddress - errors', () => {
      it('should have a getConfigFamilyAddressfamilyAggregateAddressAggregateaddress function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing aggregateaddress', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'aggregateaddress is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress - errors', () => {
      it('should have a patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing aggregateaddress', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'aggregateaddress is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress - errors', () => {
      it('should have a deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing aggregateaddress', (done) => {
        try {
          a.deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'aggregateaddress is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFamilyAddressfamilyAggregateAddressAggregateaddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigFamilyAddressfamilyDefaultRouteDistance - errors', () => {
      it('should have a getConfigFamilyAddressfamilyDefaultRouteDistance function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigFamilyAddressfamilyDefaultRouteDistance === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigFamilyAddressfamilyDefaultRouteDistance(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigFamilyAddressfamilyDefaultRouteDistance - errors', () => {
      it('should have a patchConfigFamilyAddressfamilyDefaultRouteDistance function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigFamilyAddressfamilyDefaultRouteDistance === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyDefaultRouteDistance('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyDefaultRouteDistance', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyGracefulRestart - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyGracefulRestart function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyGracefulRestart === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyGracefulRestart(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressFamilyAddressfamilyGracefulRestart - errors', () => {
      it('should have a patchConfigAddressFamilyAddressfamilyGracefulRestart function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressFamilyAddressfamilyGracefulRestart === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyGracefulRestart('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyGracefulRestart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork - errors', () => {
      it('should have a getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork - errors', () => {
      it('should have a postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAddressFamilyAddressfamilyNetworkClone - errors', () => {
      it('should have a postConfigAddressFamilyAddressfamilyNetworkClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAddressFamilyAddressfamilyNetworkClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAddressFamilyAddressfamilyNetworkClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAddressFamilyAddressfamilyNetworkClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyNetwork - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing network', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'network is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressFamilyAddressfamilyNetwork - errors', () => {
      it('should have a patchConfigAddressFamilyAddressfamilyNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressFamilyAddressfamilyNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing network', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'network is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAddressFamilyAddressfamilyNetwork - errors', () => {
      it('should have a deleteConfigAddressFamilyAddressfamilyNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAddressFamilyAddressfamilyNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing network', (done) => {
        try {
          a.deleteConfigAddressFamilyAddressfamilyNetwork('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'network is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAddressFamilyAddressfamilyNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyPrefixLimit - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyPrefixLimit function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyPrefixLimit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressFamilyAddressfamilyPrefixLimit - errors', () => {
      it('should have a patchConfigAddressFamilyAddressfamilyPrefixLimit function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressFamilyAddressfamilyPrefixLimit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigFamilyAddressfamilyUseMultiplePaths - errors', () => {
      it('should have a getConfigFamilyAddressfamilyUseMultiplePaths function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigFamilyAddressfamilyUseMultiplePaths === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigFamilyAddressfamilyUseMultiplePaths(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigFamilyAddressfamilyUseMultiplePaths - errors', () => {
      it('should have a patchConfigFamilyAddressfamilyUseMultiplePaths function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigFamilyAddressfamilyUseMultiplePaths === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyUseMultiplePaths('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyUseMultiplePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressfamilyUseMultiplePathsEbgp - errors', () => {
      it('should have a getConfigAddressfamilyUseMultiplePathsEbgp function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressfamilyUseMultiplePathsEbgp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsEbgp(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressfamilyUseMultiplePathsEbgp - errors', () => {
      it('should have a patchConfigAddressfamilyUseMultiplePathsEbgp function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressfamilyUseMultiplePathsEbgp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsEbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsEbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressfamilyUseMultiplePathsIbgp - errors', () => {
      it('should have a getConfigAddressfamilyUseMultiplePathsIbgp function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressfamilyUseMultiplePathsIbgp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsIbgp(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressfamilyUseMultiplePathsIbgp - errors', () => {
      it('should have a patchConfigAddressfamilyUseMultiplePathsIbgp function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressfamilyUseMultiplePathsIbgp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressfamilyUseMultiplePathsIbgp('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressfamilyUseMultiplePathsIbgp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolConfederation - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolConfederation function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolConfederation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolConfederation(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingProtocolRoutingprotocolConfederation - errors', () => {
      it('should have a patchConfigRoutingProtocolRoutingprotocolConfederation function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingProtocolRoutingprotocolConfederation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolConfederation(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolConfederation('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolConfederation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolNeighbor - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolRoutingprotocolNeighbor - errors', () => {
      it('should have a postConfigRoutingProtocolRoutingprotocolNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolRoutingprotocolNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighbor(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolRoutingprotocolNeighborClone - errors', () => {
      it('should have a postConfigRoutingProtocolRoutingprotocolNeighborClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolRoutingprotocolNeighborClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighborClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighborClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighborClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighborClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolNeighborClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolNeighborClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolNeighbor2 - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolNeighbor2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolNeighbor2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor2(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor2('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor2('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor2('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolNeighbor2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolNeighbor2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingProtocolRoutingprotocolNeighbor - errors', () => {
      it('should have a patchConfigRoutingProtocolRoutingprotocolNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingProtocolRoutingprotocolNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingProtocolRoutingprotocolNeighbor - errors', () => {
      it('should have a deleteConfigRoutingProtocolRoutingprotocolNeighbor function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingProtocolRoutingprotocolNeighbor === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolNeighbor(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolNeighbor('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolNeighbor', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolNeighborAddressFamily - errors', () => {
      it('should have a getConfigRoutingprotocolNeighborAddressFamily function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolNeighborAddressFamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborAddressFamily(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborAddressFamily('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingprotocolNeighborAddressFamily - errors', () => {
      it('should have a postConfigRoutingprotocolNeighborAddressFamily function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingprotocolNeighborAddressFamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingprotocolNeighborAddressFamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingprotocolNeighborAddressFamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigNeighborAddressFamilyClone - errors', () => {
      it('should have a postConfigNeighborAddressFamilyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigNeighborAddressFamilyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigNeighborAddressFamilyClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigNeighborAddressFamilyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborAddressFamilyAddressfamily - errors', () => {
      it('should have a getConfigNeighborAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborAddressFamilyAddressfamily - errors', () => {
      it('should have a patchConfigNeighborAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigNeighborAddressFamilyAddressfamily - errors', () => {
      it('should have a deleteConfigNeighborAddressFamilyAddressfamily function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigNeighborAddressFamilyAddressfamily === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.deleteConfigNeighborAddressFamilyAddressfamily('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigNeighborAddressFamilyAddressfamily', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigFamilyAddressfamilyAsPathOptions - errors', () => {
      it('should have a getConfigFamilyAddressfamilyAsPathOptions function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigFamilyAddressfamilyAsPathOptions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigFamilyAddressfamilyAsPathOptions - errors', () => {
      it('should have a patchConfigFamilyAddressfamilyAsPathOptions function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigFamilyAddressfamilyAsPathOptions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigFamilyAddressfamilyAsPathOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFamilyAddressfamilyAsPathOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyPrefixLimit2 - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyPrefixLimit2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyPrefixLimit2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressFamilyAddressfamilyPrefixLimit2 - errors', () => {
      it('should have a patchConfigAddressFamilyAddressfamilyPrefixLimit2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressFamilyAddressfamilyPrefixLimit2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyPrefixLimit2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyPrefixLimit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAddressFamilyAddressfamilyRouteReflector - errors', () => {
      it('should have a getConfigAddressFamilyAddressfamilyRouteReflector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAddressFamilyAddressfamilyRouteReflector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.getConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAddressFamilyAddressfamilyRouteReflector - errors', () => {
      it('should have a patchConfigAddressFamilyAddressfamilyRouteReflector function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAddressFamilyAddressfamilyRouteReflector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector(null, null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing addressfamily', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'addressfamily is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAddressFamilyAddressfamilyRouteReflector('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAddressFamilyAddressfamilyRouteReflector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolNeighborMultihop - errors', () => {
      it('should have a getConfigRoutingprotocolNeighborMultihop function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolNeighborMultihop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborMultihop(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborMultihop('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingprotocolNeighborMultihop - errors', () => {
      it('should have a patchConfigRoutingprotocolNeighborMultihop function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingprotocolNeighborMultihop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborMultihop('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborMultihop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolNeighborPolicy - errors', () => {
      it('should have a getConfigRoutingprotocolNeighborPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolNeighborPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborPolicy(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborPolicy('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingprotocolNeighborPolicy - errors', () => {
      it('should have a patchConfigRoutingprotocolNeighborPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingprotocolNeighborPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborPolicy('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolNeighborTimers - errors', () => {
      it('should have a getConfigRoutingprotocolNeighborTimers function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolNeighborTimers === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTimers(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTimers('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingprotocolNeighborTimers - errors', () => {
      it('should have a patchConfigRoutingprotocolNeighborTimers function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingprotocolNeighborTimers === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTimers('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingprotocolNeighborTransport - errors', () => {
      it('should have a getConfigRoutingprotocolNeighborTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingprotocolNeighborTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTransport(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTransport('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingprotocolNeighborTransport - errors', () => {
      it('should have a patchConfigRoutingprotocolNeighborTransport function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingprotocolNeighborTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingprotocolNeighborTransport('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingprotocolNeighborTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigNeighborTransportLocalAddress - errors', () => {
      it('should have a getConfigNeighborTransportLocalAddress function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigNeighborTransportLocalAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigNeighborTransportLocalAddress(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigNeighborTransportLocalAddress('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.getConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigNeighborTransportLocalAddress - errors', () => {
      it('should have a patchConfigNeighborTransportLocalAddress function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigNeighborTransportLocalAddress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing neighbor', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'neighbor is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigNeighborTransportLocalAddress('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigNeighborTransportLocalAddress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolRedistribute - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolRoutingprotocolRedistribute - errors', () => {
      it('should have a postConfigRoutingProtocolRoutingprotocolRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolRoutingprotocolRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistribute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingProtocolRoutingprotocolRedistributeClone - errors', () => {
      it('should have a postConfigRoutingProtocolRoutingprotocolRedistributeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingProtocolRoutingprotocolRedistributeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistributeClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistributeClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistributeClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistributeClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingProtocolRoutingprotocolRedistributeClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingProtocolRoutingprotocolRedistributeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolRedistribute2 - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolRedistribute2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolRedistribute2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute2(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute2('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute2('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute2('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolRedistribute2('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolRedistribute2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingProtocolRoutingprotocolRedistribute - errors', () => {
      it('should have a patchConfigRoutingProtocolRoutingprotocolRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingProtocolRoutingprotocolRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingProtocolRoutingprotocolRedistribute - errors', () => {
      it('should have a deleteConfigRoutingProtocolRoutingprotocolRedistribute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingProtocolRoutingprotocolRedistribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolRedistribute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing redistribute', (done) => {
        try {
          a.deleteConfigRoutingProtocolRoutingprotocolRedistribute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'redistribute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingProtocolRoutingprotocolRedistribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigProtocolRoutingprotocolRouteSelectionOptions - errors', () => {
      it('should have a getConfigProtocolRoutingprotocolRouteSelectionOptions function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigProtocolRoutingprotocolRouteSelectionOptions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolRouteSelectionOptions(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigProtocolRoutingprotocolRouteSelectionOptions - errors', () => {
      it('should have a patchConfigProtocolRoutingprotocolRouteSelectionOptions function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigProtocolRoutingprotocolRouteSelectionOptions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolRouteSelectionOptions(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigProtocolRoutingprotocolRouteSelectionOptions('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigProtocolRoutingprotocolRouteSelectionOptions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingProtocolRoutingprotocolTimers - errors', () => {
      it('should have a getConfigRoutingProtocolRoutingprotocolTimers function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingProtocolRoutingprotocolTimers === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolTimers(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolTimers('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolTimers('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.getConfigRoutingProtocolRoutingprotocolTimers('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingProtocolRoutingprotocolTimers - errors', () => {
      it('should have a patchConfigRoutingProtocolRoutingprotocolTimers function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingProtocolRoutingprotocolTimers === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolTimers(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolTimers('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolTimers('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routingprotocol', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolTimers('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routingprotocol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingProtocolRoutingprotocolTimers('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingProtocolRoutingprotocolTimers', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterRoutingStaticRoute - errors', () => {
      it('should have a getConfigRouterRoutingStaticRoute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterRoutingStaticRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterRoutingStaticRoute(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterRoutingStaticRoute('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouterRoutingStaticRoute('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterRoutingStaticRoute - errors', () => {
      it('should have a postConfigRouterRoutingStaticRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterRoutingStaticRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterRoutingStaticRoute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterRoutingStaticRoute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRouterRoutingStaticRoute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterRoutingStaticRoute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterRoutingStaticRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingStaticRouteClone - errors', () => {
      it('should have a postConfigRoutingStaticRouteClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingStaticRouteClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingStaticRouteClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRoutingStaticRouteClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRoutingStaticRouteClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingStaticRouteClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingStaticRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingStaticRouteStaticroute - errors', () => {
      it('should have a getConfigRoutingStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingStaticRouteStaticroute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRoutingStaticRouteStaticroute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRoutingStaticRouteStaticroute - errors', () => {
      it('should have a patchConfigRoutingStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRoutingStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRoutingStaticRouteStaticroute(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRoutingStaticRouteStaticroute('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.patchConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRoutingStaticRouteStaticroute - errors', () => {
      it('should have a deleteConfigRoutingStaticRouteStaticroute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRoutingStaticRouteStaticroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRoutingStaticRouteStaticroute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRoutingStaticRouteStaticroute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.deleteConfigRoutingStaticRouteStaticroute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRoutingStaticRouteStaticroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouteStaticrouteNextHopInterface - errors', () => {
      it('should have a getConfigRouteStaticrouteNextHopInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouteStaticrouteNextHopInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouteStaticrouteNextHopInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouteStaticrouteNextHopInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigRouteStaticrouteNextHopInterface('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigRouteStaticrouteNextHopInterface('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouteStaticrouteNextHopInterface - errors', () => {
      it('should have a postConfigRouteStaticrouteNextHopInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouteStaticrouteNextHopInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouteStaticrouteNextHopInterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouteStaticrouteNextHopInterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigRouteStaticrouteNextHopInterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.postConfigRouteStaticrouteNextHopInterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouteStaticrouteNextHopInterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteStaticrouteNextHopInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStaticrouteNextHopInterfaceClone - errors', () => {
      it('should have a postConfigStaticrouteNextHopInterfaceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStaticrouteNextHopInterfaceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStaticrouteNextHopInterfaceClone(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticrouteNextHopInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigStaticrouteNextHopInterfaceClone('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticrouteNextHopInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.postConfigStaticrouteNextHopInterfaceClone('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticrouteNextHopInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.postConfigStaticrouteNextHopInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticrouteNextHopInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStaticrouteNextHopInterfaceClone('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStaticrouteNextHopInterfaceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStaticrouteNextHopInterfaceNexthopinterface - errors', () => {
      it('should have a getConfigStaticrouteNextHopInterfaceNexthopinterface function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStaticrouteNextHopInterfaceNexthopinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStaticrouteNextHopInterfaceNexthopinterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.getConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.getConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthopinterface', (done) => {
        try {
          a.getConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'nexthopinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStaticrouteNextHopInterfaceNexthopinterface - errors', () => {
      it('should have a patchConfigStaticrouteNextHopInterfaceNexthopinterface function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStaticrouteNextHopInterfaceNexthopinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthopinterface', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'nexthopinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStaticrouteNextHopInterfaceNexthopinterface - errors', () => {
      it('should have a deleteConfigStaticrouteNextHopInterfaceNexthopinterface function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing routing', (done) => {
        try {
          a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'routing is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing staticroute', (done) => {
        try {
          a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'staticroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthopinterface', (done) => {
        try {
          a.deleteConfigStaticrouteNextHopInterfaceNexthopinterface('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'nexthopinterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStaticrouteNextHopInterfaceNexthopinterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterServiceRoute - errors', () => {
      it('should have a getConfigAuthorityRouterServiceRoute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterServiceRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterServiceRoute(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterServiceRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterServiceRoute('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterServiceRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRouterServiceRoute - errors', () => {
      it('should have a postConfigAuthorityRouterServiceRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRouterServiceRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRouterServiceRoute(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterServiceRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigAuthorityRouterServiceRoute('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterServiceRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRouterServiceRoute('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRouterServiceRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterServiceRoutePolicy - errors', () => {
      it('should have a getConfigRouterServiceRoutePolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterServiceRoutePolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterServiceRoutePolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterServiceRoutePolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterServiceRoutePolicy - errors', () => {
      it('should have a postConfigRouterServiceRoutePolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterServiceRoutePolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterServiceRoutePolicyClone - errors', () => {
      it('should have a postConfigRouterServiceRoutePolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterServiceRoutePolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicyClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicyClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterServiceRoutePolicyClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRoutePolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterServiceRoutePolicyServiceroutepolicy - errors', () => {
      it('should have a getConfigRouterServiceRoutePolicyServiceroutepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterServiceRoutePolicyServiceroutepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterServiceRoutePolicyServiceroutepolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroutepolicy', (done) => {
        try {
          a.getConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroutepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterServiceRoutePolicyServiceroutepolicy - errors', () => {
      it('should have a patchConfigRouterServiceRoutePolicyServiceroutepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterServiceRoutePolicyServiceroutepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterServiceRoutePolicyServiceroutepolicy(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroutepolicy', (done) => {
        try {
          a.patchConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroutepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterServiceRoutePolicyServiceroutepolicy - errors', () => {
      it('should have a deleteConfigRouterServiceRoutePolicyServiceroutepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterServiceRoutePolicyServiceroutepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterServiceRoutePolicyServiceroutepolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroutepolicy', (done) => {
        try {
          a.deleteConfigRouterServiceRoutePolicyServiceroutepolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroutepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRoutePolicyServiceroutepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterServiceRouteClone - errors', () => {
      it('should have a postConfigRouterServiceRouteClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterServiceRouteClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterServiceRouteClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterServiceRouteClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterServiceRouteClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterServiceRouteServiceroute - errors', () => {
      it('should have a getConfigRouterServiceRouteServiceroute function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterServiceRouteServiceroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterServiceRouteServiceroute(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterServiceRouteServiceroute('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigRouterServiceRouteServiceroute('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterServiceRouteServiceroute - errors', () => {
      it('should have a patchConfigRouterServiceRouteServiceroute function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterServiceRouteServiceroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterServiceRouteServiceroute(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterServiceRouteServiceroute('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.patchConfigRouterServiceRouteServiceroute('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterServiceRouteServiceroute('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterServiceRouteServiceroute - errors', () => {
      it('should have a deleteConfigRouterServiceRouteServiceroute function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterServiceRouteServiceroute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterServiceRouteServiceroute(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterServiceRouteServiceroute('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.deleteConfigRouterServiceRouteServiceroute('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterServiceRouteServiceroute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterServiceRouteServicerouteHost - errors', () => {
      it('should have a getConfigRouterServiceRouteServicerouteHost function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterServiceRouteServicerouteHost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterServiceRouteServicerouteHost(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterServiceRouteServicerouteHost('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigRouterServiceRouteServicerouteHost('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterServiceRouteServicerouteHost - errors', () => {
      it('should have a postConfigRouterServiceRouteServicerouteHost function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterServiceRouteServicerouteHost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterServiceRouteServicerouteHost(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterServiceRouteServicerouteHost('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.postConfigRouterServiceRouteServicerouteHost('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterServiceRouteServicerouteHost('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceRouteServicerouteHostClone - errors', () => {
      it('should have a postConfigServiceRouteServicerouteHostClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceRouteServicerouteHostClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceRouteServicerouteHostClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteHostClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceRouteServicerouteHostClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteHostClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.postConfigServiceRouteServicerouteHostClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteHostClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceRouteServicerouteHostClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteHostClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceRouteServicerouteHost - errors', () => {
      it('should have a getConfigServiceRouteServicerouteHost function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceRouteServicerouteHost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceRouteServicerouteHost(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceRouteServicerouteHost('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing host', (done) => {
        try {
          a.getConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'host is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceRouteServicerouteHost - errors', () => {
      it('should have a patchConfigServiceRouteServicerouteHost function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceRouteServicerouteHost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteHost(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteHost('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing host', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'host is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceRouteServicerouteHost - errors', () => {
      it('should have a deleteConfigServiceRouteServicerouteHost function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceRouteServicerouteHost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceRouteServicerouteHost(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServiceRouteServicerouteHost('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.deleteConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing host', (done) => {
        try {
          a.deleteConfigServiceRouteServicerouteHost('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'host is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceRouteServicerouteHost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceRouteServicerouteNextHop - errors', () => {
      it('should have a getConfigServiceRouteServicerouteNextHop function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceRouteServicerouteNextHop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceRouteServicerouteNextHop(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceRouteServicerouteNextHop('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigServiceRouteServicerouteNextHop('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceRouteServicerouteNextHop - errors', () => {
      it('should have a postConfigServiceRouteServicerouteNextHop function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceRouteServicerouteNextHop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceRouteServicerouteNextHop(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServiceRouteServicerouteNextHop('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.postConfigServiceRouteServicerouteNextHop('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceRouteServicerouteNextHop('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceRouteServicerouteNextHop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouteServicerouteNextHopClone - errors', () => {
      it('should have a postConfigRouteServicerouteNextHopClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouteServicerouteNextHopClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouteServicerouteNextHopClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteServicerouteNextHopClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouteServicerouteNextHopClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteServicerouteNextHopClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.postConfigRouteServicerouteNextHopClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteServicerouteNextHopClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouteServicerouteNextHopClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouteServicerouteNextHopClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouteServicerouteNextHopNexthop - errors', () => {
      it('should have a getConfigRouteServicerouteNextHopNexthop function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouteServicerouteNextHopNexthop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouteServicerouteNextHopNexthop(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouteServicerouteNextHopNexthop('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthop', (done) => {
        try {
          a.getConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'nexthop is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouteServicerouteNextHopNexthop - errors', () => {
      it('should have a patchConfigRouteServicerouteNextHopNexthop function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouteServicerouteNextHopNexthop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouteServicerouteNextHopNexthop(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouteServicerouteNextHopNexthop('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.patchConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthop', (done) => {
        try {
          a.patchConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'nexthop is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouteServicerouteNextHopNexthop - errors', () => {
      it('should have a deleteConfigRouteServicerouteNextHopNexthop function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouteServicerouteNextHopNexthop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouteServicerouteNextHopNexthop(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouteServicerouteNextHopNexthop('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.deleteConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing nexthop', (done) => {
        try {
          a.deleteConfigRouteServicerouteNextHopNexthop('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'nexthop is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouteServicerouteNextHopNexthop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceRouteServicerouteReachabilityDetection - errors', () => {
      it('should have a getConfigServiceRouteServicerouteReachabilityDetection function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceRouteServicerouteReachabilityDetection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceRouteServicerouteReachabilityDetection(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServiceRouteServicerouteReachabilityDetection('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.getConfigServiceRouteServicerouteReachabilityDetection('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceRouteServicerouteReachabilityDetection - errors', () => {
      it('should have a patchConfigServiceRouteServicerouteReachabilityDetection function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceRouteServicerouteReachabilityDetection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteReachabilityDetection(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteReachabilityDetection('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceroute', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteReachabilityDetection('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceroute is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceRouteServicerouteReachabilityDetection('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceRouteServicerouteReachabilityDetection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterSystem - errors', () => {
      it('should have a getConfigAuthorityRouterSystem function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterSystem === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterSystem(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterSystem', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterSystem('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterSystem', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRouterSystem - errors', () => {
      it('should have a patchConfigAuthorityRouterSystem function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRouterSystem === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRouterSystem(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterSystem', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAuthorityRouterSystem('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterSystem', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRouterSystem('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterSystem', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemAudit - errors', () => {
      it('should have a getConfigRouterSystemAudit function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemAudit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemAudit(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAudit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemAudit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAudit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemAudit - errors', () => {
      it('should have a patchConfigRouterSystemAudit function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemAudit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemAudit(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemAudit('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemAudit('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemAuditAdministration - errors', () => {
      it('should have a getConfigRouterSystemAuditAdministration function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemAuditAdministration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemAuditAdministration(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAuditAdministration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemAuditAdministration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAuditAdministration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemAuditAdministration - errors', () => {
      it('should have a patchConfigRouterSystemAuditAdministration function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemAuditAdministration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemAuditAdministration(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditAdministration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemAuditAdministration('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditAdministration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemAuditAdministration('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditAdministration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemAudit2 - errors', () => {
      it('should have a getConfigRouterSystemAudit2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemAudit2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemAudit2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAudit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemAudit2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAudit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemAudit2 - errors', () => {
      it('should have a patchConfigRouterSystemAudit2 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemAudit2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemAudit2(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemAudit2('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemAudit2('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAudit2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemAuditTraffic - errors', () => {
      it('should have a getConfigRouterSystemAuditTraffic function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemAuditTraffic === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemAuditTraffic(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAuditTraffic', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemAuditTraffic('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemAuditTraffic', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemAuditTraffic - errors', () => {
      it('should have a patchConfigRouterSystemAuditTraffic function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemAuditTraffic === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemAuditTraffic(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditTraffic', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemAuditTraffic('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditTraffic', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemAuditTraffic('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemAuditTraffic', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemLocalLogin - errors', () => {
      it('should have a getConfigRouterSystemLocalLogin function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemLocalLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemLocalLogin(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLocalLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemLocalLogin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLocalLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemLocalLogin - errors', () => {
      it('should have a patchConfigRouterSystemLocalLogin function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemLocalLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemLocalLogin(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemLocalLogin('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemLocalLogin('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemLocalLoginNetconf - errors', () => {
      it('should have a getConfigRouterSystemLocalLoginNetconf function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemLocalLoginNetconf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemLocalLoginNetconf(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLocalLoginNetconf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemLocalLoginNetconf('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLocalLoginNetconf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemLocalLoginNetconf - errors', () => {
      it('should have a patchConfigRouterSystemLocalLoginNetconf function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemLocalLoginNetconf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemLocalLoginNetconf(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLoginNetconf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemLocalLoginNetconf('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLoginNetconf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemLocalLoginNetconf('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLocalLoginNetconf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemLogCategory - errors', () => {
      it('should have a getConfigRouterSystemLogCategory function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemLogCategory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemLogCategory(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLogCategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemLogCategory('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLogCategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterSystemLogCategory - errors', () => {
      it('should have a postConfigRouterSystemLogCategory function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterSystemLogCategory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterSystemLogCategory(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterSystemLogCategory('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterSystemLogCategory('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterSystemLogCategoryClone - errors', () => {
      it('should have a postConfigRouterSystemLogCategoryClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterSystemLogCategoryClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterSystemLogCategoryClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategoryClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterSystemLogCategoryClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategoryClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterSystemLogCategoryClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemLogCategoryClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemLogCategoryLogcategory - errors', () => {
      it('should have a getConfigRouterSystemLogCategoryLogcategory function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemLogCategoryLogcategory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemLogCategoryLogcategory(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemLogCategoryLogcategory('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing logcategory', (done) => {
        try {
          a.getConfigRouterSystemLogCategoryLogcategory('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'logcategory is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemLogCategoryLogcategory - errors', () => {
      it('should have a patchConfigRouterSystemLogCategoryLogcategory function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemLogCategoryLogcategory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemLogCategoryLogcategory(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemLogCategoryLogcategory('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing logcategory', (done) => {
        try {
          a.patchConfigRouterSystemLogCategoryLogcategory('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'logcategory is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemLogCategoryLogcategory('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigRouterSystemLogCategoryLogcategory - errors', () => {
      it('should have a deleteConfigRouterSystemLogCategoryLogcategory function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigRouterSystemLogCategoryLogcategory === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigRouterSystemLogCategoryLogcategory(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigRouterSystemLogCategoryLogcategory('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing logcategory', (done) => {
        try {
          a.deleteConfigRouterSystemLogCategoryLogcategory('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'logcategory is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigRouterSystemLogCategoryLogcategory', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemMetrics - errors', () => {
      it('should have a getConfigRouterSystemMetrics function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemMetrics === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemMetrics(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemMetrics('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemMetrics - errors', () => {
      it('should have a patchConfigRouterSystemMetrics function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemMetrics === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemMetrics(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemMetrics('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemMetrics('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemNtp - errors', () => {
      it('should have a getConfigRouterSystemNtp function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemNtp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemNtp(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemNtp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemNtp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemNtp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemNtp - errors', () => {
      it('should have a patchConfigRouterSystemNtp function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemNtp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemNtp(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemNtp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemNtp('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemNtp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemNtp('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemNtp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemNtpServer - errors', () => {
      it('should have a getConfigRouterSystemNtpServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemNtpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemNtpServer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemNtpServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterSystemNtpServer - errors', () => {
      it('should have a postConfigRouterSystemNtpServer function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterSystemNtpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterSystemNtpServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterSystemNtpServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterSystemNtpServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSystemNtpServerClone - errors', () => {
      it('should have a postConfigSystemNtpServerClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSystemNtpServerClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSystemNtpServerClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemNtpServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSystemNtpServerClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemNtpServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSystemNtpServerClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemNtpServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSystemNtpServer - errors', () => {
      it('should have a getConfigSystemNtpServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSystemNtpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSystemNtpServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSystemNtpServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.getConfigSystemNtpServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSystemNtpServer - errors', () => {
      it('should have a patchConfigSystemNtpServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSystemNtpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSystemNtpServer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigSystemNtpServer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.patchConfigSystemNtpServer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSystemNtpServer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigSystemNtpServer - errors', () => {
      it('should have a deleteConfigSystemNtpServer function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigSystemNtpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigSystemNtpServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigSystemNtpServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.deleteConfigSystemNtpServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemNtpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemRemoteLogin - errors', () => {
      it('should have a getConfigRouterSystemRemoteLogin function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemRemoteLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemRemoteLogin(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemRemoteLogin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemRemoteLogin - errors', () => {
      it('should have a patchConfigRouterSystemRemoteLogin function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemRemoteLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemRemoteLogin(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemRemoteLogin('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemRemoteLogin('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemRemoteLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemServices - errors', () => {
      it('should have a getConfigRouterSystemServices function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemServices === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemServices(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServices', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemServices('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServices', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemServices - errors', () => {
      it('should have a patchConfigRouterSystemServices function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemServices === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemServices(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServices', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemServices('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServices', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemServices('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServices', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemServicesSnmpServer - errors', () => {
      it('should have a getConfigRouterSystemServicesSnmpServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemServicesSnmpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemServicesSnmpServer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServicesSnmpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemServicesSnmpServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServicesSnmpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemServicesSnmpServer - errors', () => {
      it('should have a patchConfigRouterSystemServicesSnmpServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemServicesSnmpServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemServicesSnmpServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesSnmpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemServicesSnmpServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesSnmpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemServicesSnmpServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesSnmpServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServicesSnmpServerAccessControl - errors', () => {
      it('should have a getConfigServicesSnmpServerAccessControl function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServicesSnmpServerAccessControl === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServicesSnmpServerAccessControl(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesSnmpServerAccessControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServicesSnmpServerAccessControl('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesSnmpServerAccessControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicesSnmpServerAccessControl - errors', () => {
      it('should have a postConfigServicesSnmpServerAccessControl function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicesSnmpServerAccessControl === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicesSnmpServerAccessControl(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerAccessControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServicesSnmpServerAccessControl('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerAccessControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicesSnmpServerAccessControl('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerAccessControl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSnmpServerAccessControlClone - errors', () => {
      it('should have a postConfigSnmpServerAccessControlClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSnmpServerAccessControlClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSnmpServerAccessControlClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerAccessControlClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSnmpServerAccessControlClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerAccessControlClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSnmpServerAccessControlClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerAccessControlClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSnmpServerAccessControlAccesscontrol - errors', () => {
      it('should have a getConfigSnmpServerAccessControlAccesscontrol function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSnmpServerAccessControlAccesscontrol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSnmpServerAccessControlAccesscontrol(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSnmpServerAccessControlAccesscontrol('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesscontrol', (done) => {
        try {
          a.getConfigSnmpServerAccessControlAccesscontrol('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesscontrol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSnmpServerAccessControlAccesscontrol - errors', () => {
      it('should have a patchConfigSnmpServerAccessControlAccesscontrol function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSnmpServerAccessControlAccesscontrol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSnmpServerAccessControlAccesscontrol(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigSnmpServerAccessControlAccesscontrol('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesscontrol', (done) => {
        try {
          a.patchConfigSnmpServerAccessControlAccesscontrol('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'accesscontrol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSnmpServerAccessControlAccesscontrol('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigSnmpServerAccessControlAccesscontrol - errors', () => {
      it('should have a deleteConfigSnmpServerAccessControlAccesscontrol function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigSnmpServerAccessControlAccesscontrol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigSnmpServerAccessControlAccesscontrol(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigSnmpServerAccessControlAccesscontrol('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesscontrol', (done) => {
        try {
          a.deleteConfigSnmpServerAccessControlAccesscontrol('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesscontrol is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerAccessControlAccesscontrol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServicesSnmpServerNotificationReceiver - errors', () => {
      it('should have a getConfigServicesSnmpServerNotificationReceiver function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServicesSnmpServerNotificationReceiver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServicesSnmpServerNotificationReceiver(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesSnmpServerNotificationReceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServicesSnmpServerNotificationReceiver('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesSnmpServerNotificationReceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicesSnmpServerNotificationReceiver - errors', () => {
      it('should have a postConfigServicesSnmpServerNotificationReceiver function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicesSnmpServerNotificationReceiver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicesSnmpServerNotificationReceiver(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerNotificationReceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServicesSnmpServerNotificationReceiver('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerNotificationReceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicesSnmpServerNotificationReceiver('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesSnmpServerNotificationReceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSnmpServerNotificationReceiverClone - errors', () => {
      it('should have a postConfigSnmpServerNotificationReceiverClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSnmpServerNotificationReceiverClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSnmpServerNotificationReceiverClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerNotificationReceiverClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSnmpServerNotificationReceiverClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerNotificationReceiverClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSnmpServerNotificationReceiverClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSnmpServerNotificationReceiverClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSnmpServerNotificationReceiverNotificationreceiver - errors', () => {
      it('should have a getConfigSnmpServerNotificationReceiverNotificationreceiver function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSnmpServerNotificationReceiverNotificationreceiver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSnmpServerNotificationReceiverNotificationreceiver(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing notificationreceiver', (done) => {
        try {
          a.getConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'notificationreceiver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSnmpServerNotificationReceiverNotificationreceiver - errors', () => {
      it('should have a patchConfigSnmpServerNotificationReceiverNotificationreceiver function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSnmpServerNotificationReceiverNotificationreceiver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSnmpServerNotificationReceiverNotificationreceiver(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing notificationreceiver', (done) => {
        try {
          a.patchConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'notificationreceiver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigSnmpServerNotificationReceiverNotificationreceiver - errors', () => {
      it('should have a deleteConfigSnmpServerNotificationReceiverNotificationreceiver function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigSnmpServerNotificationReceiverNotificationreceiver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigSnmpServerNotificationReceiverNotificationreceiver(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing notificationreceiver', (done) => {
        try {
          a.deleteConfigSnmpServerNotificationReceiverNotificationreceiver('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'notificationreceiver is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSnmpServerNotificationReceiverNotificationreceiver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemServicesWebserver - errors', () => {
      it('should have a getConfigRouterSystemServicesWebserver function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemServicesWebserver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemServicesWebserver(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServicesWebserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemServicesWebserver('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemServicesWebserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemServicesWebserver - errors', () => {
      it('should have a patchConfigRouterSystemServicesWebserver function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemServicesWebserver === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemServicesWebserver(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesWebserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemServicesWebserver('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesWebserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemServicesWebserver('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemServicesWebserver', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSystemServicesWebserverServer - errors', () => {
      it('should have a getConfigSystemServicesWebserverServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSystemServicesWebserverServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSystemServicesWebserverServer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSystemServicesWebserverServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSystemServicesWebserverServer - errors', () => {
      it('should have a postConfigSystemServicesWebserverServer function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSystemServicesWebserverServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSystemServicesWebserverServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSystemServicesWebserverServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSystemServicesWebserverServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicesWebserverServerClone - errors', () => {
      it('should have a postConfigServicesWebserverServerClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicesWebserverServerClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicesWebserverServerClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesWebserverServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigServicesWebserverServerClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesWebserverServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicesWebserverServerClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicesWebserverServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServicesWebserverServer - errors', () => {
      it('should have a getConfigServicesWebserverServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServicesWebserverServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServicesWebserverServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigServicesWebserverServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.getConfigServicesWebserverServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServicesWebserverServer - errors', () => {
      it('should have a patchConfigServicesWebserverServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServicesWebserverServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServicesWebserverServer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigServicesWebserverServer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.patchConfigServicesWebserverServer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServicesWebserverServer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServicesWebserverServer - errors', () => {
      it('should have a deleteConfigServicesWebserverServer function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServicesWebserverServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServicesWebserverServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigServicesWebserverServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.deleteConfigServicesWebserverServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicesWebserverServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemSoftwareUpdate - errors', () => {
      it('should have a getConfigRouterSystemSoftwareUpdate function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemSoftwareUpdate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemSoftwareUpdate(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSoftwareUpdate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemSoftwareUpdate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSoftwareUpdate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemSoftwareUpdate - errors', () => {
      it('should have a patchConfigRouterSystemSoftwareUpdate function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemSoftwareUpdate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdate(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdate('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdate('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemSoftwareUpdateRepository - errors', () => {
      it('should have a getConfigRouterSystemSoftwareUpdateRepository function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemSoftwareUpdateRepository === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemSoftwareUpdateRepository(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSoftwareUpdateRepository', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemSoftwareUpdateRepository('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSoftwareUpdateRepository', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemSoftwareUpdateRepository - errors', () => {
      it('should have a patchConfigRouterSystemSoftwareUpdateRepository function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemSoftwareUpdateRepository === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdateRepository(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdateRepository', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdateRepository('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdateRepository', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemSoftwareUpdateRepository('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSoftwareUpdateRepository', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemSyslog - errors', () => {
      it('should have a getConfigRouterSystemSyslog function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemSyslog === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemSyslog(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSyslog', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemSyslog('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSyslog', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigRouterSystemSyslog - errors', () => {
      it('should have a patchConfigRouterSystemSyslog function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigRouterSystemSyslog === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigRouterSystemSyslog(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSyslog', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigRouterSystemSyslog('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSyslog', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigRouterSystemSyslog('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigRouterSystemSyslog', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRouterSystemSyslogServer - errors', () => {
      it('should have a getConfigRouterSystemSyslogServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRouterSystemSyslogServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRouterSystemSyslogServer(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigRouterSystemSyslogServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRouterSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRouterSystemSyslogServer - errors', () => {
      it('should have a postConfigRouterSystemSyslogServer function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRouterSystemSyslogServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRouterSystemSyslogServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigRouterSystemSyslogServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRouterSystemSyslogServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRouterSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSystemSyslogServerClone - errors', () => {
      it('should have a postConfigSystemSyslogServerClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSystemSyslogServerClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSystemSyslogServerClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemSyslogServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postConfigSystemSyslogServerClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemSyslogServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSystemSyslogServerClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSystemSyslogServerClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSystemSyslogServer - errors', () => {
      it('should have a getConfigSystemSyslogServer function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSystemSyslogServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSystemSyslogServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigSystemSyslogServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.getConfigSystemSyslogServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSystemSyslogServer - errors', () => {
      it('should have a patchConfigSystemSyslogServer function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSystemSyslogServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSystemSyslogServer(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigSystemSyslogServer('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.patchConfigSystemSyslogServer('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSystemSyslogServer('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigSystemSyslogServer - errors', () => {
      it('should have a deleteConfigSystemSyslogServer function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigSystemSyslogServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigSystemSyslogServer(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.deleteConfigSystemSyslogServer('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing server', (done) => {
        try {
          a.deleteConfigSystemSyslogServer('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'server is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSystemSyslogServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRouterUdpTransform - errors', () => {
      it('should have a getConfigAuthorityRouterUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRouterUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRouterUdpTransform(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getConfigAuthorityRouterUdpTransform('fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRouterUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRouterUdpTransform - errors', () => {
      it('should have a patchConfigAuthorityRouterUdpTransform function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRouterUdpTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRouterUdpTransform(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.patchConfigAuthorityRouterUdpTransform('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRouterUdpTransform('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRouterUdpTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRouting - errors', () => {
      it('should have a getConfigStoreAuthorityRouting function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRouting(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityRouting - errors', () => {
      it('should have a patchConfigStoreAuthorityRouting function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityRouting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityRouting(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityRouting('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityRouting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRoutingFilter - errors', () => {
      it('should have a getConfigStoreAuthorityRoutingFilter function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRoutingFilter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRoutingFilter(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityRoutingFilter - errors', () => {
      it('should have a postConfigStoreAuthorityRoutingFilter function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityRoutingFilter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityRoutingFilter(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityRoutingFilter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRoutingFilterClone - errors', () => {
      it('should have a postConfigAuthorityRoutingFilterClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRoutingFilterClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRoutingFilterClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRoutingFilterClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRoutingFilterClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRoutingFilterClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRoutingFilter - errors', () => {
      it('should have a getConfigAuthorityRoutingFilter function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRoutingFilter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRoutingFilter(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.getConfigAuthorityRoutingFilter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRoutingFilter - errors', () => {
      it('should have a patchConfigAuthorityRoutingFilter function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRoutingFilter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRoutingFilter(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.patchConfigAuthorityRoutingFilter('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRoutingFilter('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAuthorityRoutingFilter - errors', () => {
      it('should have a deleteConfigAuthorityRoutingFilter function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAuthorityRoutingFilter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAuthorityRoutingFilter(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.deleteConfigAuthorityRoutingFilter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAuthorityRoutingFilter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingFilterRule - errors', () => {
      it('should have a getConfigRoutingFilterRule function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingFilterRule === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingFilterRule(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.getConfigRoutingFilterRule('fakeparam', null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingFilterRule - errors', () => {
      it('should have a postConfigRoutingFilterRule function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingFilterRule === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingFilterRule(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.postConfigRoutingFilterRule('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingFilterRule('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigFilterRuleClone - errors', () => {
      it('should have a postConfigFilterRuleClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigFilterRuleClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigFilterRuleClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.postConfigFilterRuleClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigFilterRuleClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigFilterRuleMove - errors', () => {
      it('should have a postConfigFilterRuleMove function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigFilterRuleMove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigFilterRuleMove(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.postConfigFilterRuleMove('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigFilterRuleMove('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigFilterRuleMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigFilterRule - errors', () => {
      it('should have a getConfigFilterRule function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigFilterRule === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigFilterRule(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.getConfigFilterRule('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing rule', (done) => {
        try {
          a.getConfigFilterRule('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'rule is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigFilterRule - errors', () => {
      it('should have a patchConfigFilterRule function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigFilterRule === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigFilterRule(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.patchConfigFilterRule('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing rule', (done) => {
        try {
          a.patchConfigFilterRule('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'rule is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigFilterRule('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigFilterRule - errors', () => {
      it('should have a deleteConfigFilterRule function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigFilterRule === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigFilterRule(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing filter', (done) => {
        try {
          a.deleteConfigFilterRule('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'filter is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing rule', (done) => {
        try {
          a.deleteConfigFilterRule('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'rule is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigFilterRule', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityRoutingPolicy - errors', () => {
      it('should have a getConfigStoreAuthorityRoutingPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityRoutingPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityRoutingPolicy(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityRoutingPolicy - errors', () => {
      it('should have a postConfigStoreAuthorityRoutingPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityRoutingPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityRoutingPolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityRoutingPolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityRoutingPolicyClone - errors', () => {
      it('should have a postConfigAuthorityRoutingPolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityRoutingPolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityRoutingPolicyClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRoutingPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityRoutingPolicyClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityRoutingPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityRoutingPolicy - errors', () => {
      it('should have a getConfigAuthorityRoutingPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityRoutingPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityRoutingPolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigAuthorityRoutingPolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityRoutingPolicy - errors', () => {
      it('should have a patchConfigAuthorityRoutingPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityRoutingPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityRoutingPolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.patchConfigAuthorityRoutingPolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityRoutingPolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigAuthorityRoutingPolicy - errors', () => {
      it('should have a deleteConfigAuthorityRoutingPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigAuthorityRoutingPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigAuthorityRoutingPolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.deleteConfigAuthorityRoutingPolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigAuthorityRoutingPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigRoutingPolicyStatement - errors', () => {
      it('should have a getConfigRoutingPolicyStatement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigRoutingPolicyStatement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigRoutingPolicyStatement(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigRoutingPolicyStatement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigRoutingPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigRoutingPolicyStatement - errors', () => {
      it('should have a postConfigRoutingPolicyStatement function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigRoutingPolicyStatement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigRoutingPolicyStatement(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigRoutingPolicyStatement('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigRoutingPolicyStatement('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigRoutingPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPolicyStatementClone - errors', () => {
      it('should have a postConfigPolicyStatementClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPolicyStatementClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPolicyStatementClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigPolicyStatementClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPolicyStatementClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPolicyStatementMove - errors', () => {
      it('should have a postConfigPolicyStatementMove function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPolicyStatementMove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPolicyStatementMove(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigPolicyStatementMove('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPolicyStatementMove('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPolicyStatement - errors', () => {
      it('should have a getConfigPolicyStatement function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPolicyStatement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPolicyStatement(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigPolicyStatement('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.getConfigPolicyStatement('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPolicyStatement - errors', () => {
      it('should have a patchConfigPolicyStatement function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPolicyStatement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPolicyStatement(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.patchConfigPolicyStatement('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.patchConfigPolicyStatement('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPolicyStatement('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPolicyStatement - errors', () => {
      it('should have a deleteConfigPolicyStatement function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPolicyStatement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPolicyStatement(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.deleteConfigPolicyStatement('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.deleteConfigPolicyStatement('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyStatement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPolicyStatementAction - errors', () => {
      it('should have a getConfigPolicyStatementAction function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPolicyStatementAction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPolicyStatementAction(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigPolicyStatementAction('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.getConfigPolicyStatementAction('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPolicyStatementAction - errors', () => {
      it('should have a postConfigPolicyStatementAction function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPolicyStatementAction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPolicyStatementAction(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigPolicyStatementAction('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.postConfigPolicyStatementAction('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPolicyStatementAction('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStatementActionClone - errors', () => {
      it('should have a postConfigStatementActionClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStatementActionClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStatementActionClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementActionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigStatementActionClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementActionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.postConfigStatementActionClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementActionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStatementActionClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementActionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStatementAction - errors', () => {
      it('should have a getConfigStatementAction function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStatementAction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStatementAction(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigStatementAction('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.getConfigStatementAction('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing action', (done) => {
        try {
          a.getConfigStatementAction('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'action is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStatementAction - errors', () => {
      it('should have a patchConfigStatementAction function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStatementAction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStatementAction(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.patchConfigStatementAction('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.patchConfigStatementAction('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing action', (done) => {
        try {
          a.patchConfigStatementAction('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'action is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStatementAction('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStatementAction - errors', () => {
      it('should have a deleteConfigStatementAction function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStatementAction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStatementAction(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.deleteConfigStatementAction('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.deleteConfigStatementAction('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing action', (done) => {
        try {
          a.deleteConfigStatementAction('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'action is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementAction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPolicyStatementCondition - errors', () => {
      it('should have a getConfigPolicyStatementCondition function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPolicyStatementCondition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPolicyStatementCondition(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigPolicyStatementCondition('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.getConfigPolicyStatementCondition('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPolicyStatementCondition - errors', () => {
      it('should have a postConfigPolicyStatementCondition function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPolicyStatementCondition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPolicyStatementCondition(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigPolicyStatementCondition('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.postConfigPolicyStatementCondition('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPolicyStatementCondition('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStatementConditionClone - errors', () => {
      it('should have a postConfigStatementConditionClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStatementConditionClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStatementConditionClone(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementConditionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.postConfigStatementConditionClone('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementConditionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.postConfigStatementConditionClone('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementConditionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStatementConditionClone('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStatementConditionClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStatementCondition - errors', () => {
      it('should have a getConfigStatementCondition function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStatementCondition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStatementCondition(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.getConfigStatementCondition('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.getConfigStatementCondition('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing condition', (done) => {
        try {
          a.getConfigStatementCondition('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'condition is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStatementCondition - errors', () => {
      it('should have a patchConfigStatementCondition function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStatementCondition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStatementCondition(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.patchConfigStatementCondition('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.patchConfigStatementCondition('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing condition', (done) => {
        try {
          a.patchConfigStatementCondition('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'condition is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStatementCondition('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStatementCondition - errors', () => {
      it('should have a deleteConfigStatementCondition function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStatementCondition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStatementCondition(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing policy', (done) => {
        try {
          a.deleteConfigStatementCondition('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'policy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing statement', (done) => {
        try {
          a.deleteConfigStatementCondition('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'statement is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing condition', (done) => {
        try {
          a.deleteConfigStatementCondition('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'condition is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStatementCondition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthoritySecurity - errors', () => {
      it('should have a getConfigStoreAuthoritySecurity function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthoritySecurity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthoritySecurity(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthoritySecurity - errors', () => {
      it('should have a postConfigStoreAuthoritySecurity function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthoritySecurity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthoritySecurity(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthoritySecurity('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthoritySecurityClone - errors', () => {
      it('should have a postConfigStoreAuthoritySecurityClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthoritySecurityClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthoritySecurityClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySecurityClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthoritySecurityClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySecurityClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthoritySecurity2 - errors', () => {
      it('should have a getConfigStoreAuthoritySecurity2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthoritySecurity2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthoritySecurity2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySecurity2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing security', (done) => {
        try {
          a.getConfigStoreAuthoritySecurity2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'security is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySecurity2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthoritySecurity - errors', () => {
      it('should have a patchConfigStoreAuthoritySecurity function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthoritySecurity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthoritySecurity(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing security', (done) => {
        try {
          a.patchConfigStoreAuthoritySecurity('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'security is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthoritySecurity('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthoritySecurity - errors', () => {
      it('should have a deleteConfigStoreAuthoritySecurity function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthoritySecurity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthoritySecurity(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing security', (done) => {
        try {
          a.deleteConfigStoreAuthoritySecurity('fakeparam', null, (data, error) => {
            try {
              const displayE = 'security is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthoritySecurity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityService - errors', () => {
      it('should have a getConfigStoreAuthorityService function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityService(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityService - errors', () => {
      it('should have a postConfigStoreAuthorityService function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityService(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityService('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityServiceClass - errors', () => {
      it('should have a getConfigStoreAuthorityServiceClass function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityServiceClass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityServiceClass(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServiceClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityServiceClass - errors', () => {
      it('should have a postConfigStoreAuthorityServiceClass function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityServiceClass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClass(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClass('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityServiceClassClone - errors', () => {
      it('should have a postConfigStoreAuthorityServiceClassClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityServiceClassClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClassClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClassClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClassClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClassClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityServiceClassServiceclass - errors', () => {
      it('should have a getConfigStoreAuthorityServiceClassServiceclass function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityServiceClassServiceclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityServiceClassServiceclass(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceclass', (done) => {
        try {
          a.getConfigStoreAuthorityServiceClassServiceclass('fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityServiceClassServiceclass - errors', () => {
      it('should have a patchConfigStoreAuthorityServiceClassServiceclass function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityServiceClassServiceclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityServiceClassServiceclass(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceclass', (done) => {
        try {
          a.patchConfigStoreAuthorityServiceClassServiceclass('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'serviceclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityServiceClassServiceclass('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityServiceClassServiceclass - errors', () => {
      it('should have a deleteConfigStoreAuthorityServiceClassServiceclass function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityServiceClassServiceclass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityServiceClassServiceclass(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing serviceclass', (done) => {
        try {
          a.deleteConfigStoreAuthorityServiceClassServiceclass('fakeparam', null, (data, error) => {
            try {
              const displayE = 'serviceclass is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityServiceClassServiceclass', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityServicePolicy - errors', () => {
      it('should have a getConfigStoreAuthorityServicePolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityServicePolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityServicePolicy(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServicePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityServicePolicy - errors', () => {
      it('should have a postConfigStoreAuthorityServicePolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityServicePolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityServicePolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServicePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityServicePolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServicePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityServicePolicyClone - errors', () => {
      it('should have a postConfigStoreAuthorityServicePolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityServicePolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityServicePolicyClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServicePolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityServicePolicyClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServicePolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityServicePolicyServicepolicy - errors', () => {
      it('should have a getConfigStoreAuthorityServicePolicyServicepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityServicePolicyServicepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityServicePolicyServicepolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.getConfigStoreAuthorityServicePolicyServicepolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityServicePolicyServicepolicy - errors', () => {
      it('should have a patchConfigStoreAuthorityServicePolicyServicepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityServicePolicyServicepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityServicePolicyServicepolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.patchConfigStoreAuthorityServicePolicyServicepolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityServicePolicyServicepolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityServicePolicyServicepolicy - errors', () => {
      it('should have a deleteConfigStoreAuthorityServicePolicyServicepolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityServicePolicyServicepolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityServicePolicyServicepolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.deleteConfigStoreAuthorityServicePolicyServicepolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityServicePolicyServicepolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServicePolicyServicepolicyAppliesTo - errors', () => {
      it('should have a getConfigServicePolicyServicepolicyAppliesTo function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServicePolicyServicepolicyAppliesTo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServicePolicyServicepolicyAppliesTo(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicePolicyServicepolicyAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.getConfigServicePolicyServicepolicyAppliesTo('fakeparam', null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicePolicyServicepolicyAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicePolicyServicepolicyAppliesTo - errors', () => {
      it('should have a postConfigServicePolicyServicepolicyAppliesTo function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicePolicyServicepolicyAppliesTo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyAppliesTo(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyAppliesTo('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyAppliesTo('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigPolicyServicepolicyAppliesToClone - errors', () => {
      it('should have a postConfigPolicyServicepolicyAppliesToClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigPolicyServicepolicyAppliesToClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigPolicyServicepolicyAppliesToClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyServicepolicyAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.postConfigPolicyServicepolicyAppliesToClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyServicepolicyAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigPolicyServicepolicyAppliesToClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigPolicyServicepolicyAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigPolicyServicepolicyAppliesToAppliesto - errors', () => {
      it('should have a getConfigPolicyServicepolicyAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigPolicyServicepolicyAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigPolicyServicepolicyAppliesToAppliesto(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.getConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.getConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigPolicyServicepolicyAppliesToAppliesto - errors', () => {
      it('should have a patchConfigPolicyServicepolicyAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigPolicyServicepolicyAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigPolicyServicepolicyAppliesToAppliesto(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.patchConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.patchConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigPolicyServicepolicyAppliesToAppliesto - errors', () => {
      it('should have a deleteConfigPolicyServicepolicyAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigPolicyServicepolicyAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigPolicyServicepolicyAppliesToAppliesto(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.deleteConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.deleteConfigPolicyServicepolicyAppliesToAppliesto('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigPolicyServicepolicyAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityServicePolicyServicepolicyVector - errors', () => {
      it('should have a getConfigAuthorityServicePolicyServicepolicyVector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityServicePolicyServicepolicyVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityServicePolicyServicepolicyVector(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.getConfigAuthorityServicePolicyServicepolicyVector('fakeparam', null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityServicePolicyServicepolicyVector - errors', () => {
      it('should have a postConfigAuthorityServicePolicyServicepolicyVector function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityServicePolicyServicepolicyVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityServicePolicyServicepolicyVector(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.postConfigAuthorityServicePolicyServicepolicyVector('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityServicePolicyServicepolicyVector('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicePolicyServicepolicyVectorClone - errors', () => {
      it('should have a postConfigServicePolicyServicepolicyVectorClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicePolicyServicepolicyVectorClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServicePolicyServicepolicyVectorMove - errors', () => {
      it('should have a postConfigServicePolicyServicepolicyVectorMove function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServicePolicyServicepolicyVectorMove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorMove(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorMove('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServicePolicyServicepolicyVectorMove('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServicePolicyServicepolicyVectorMove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServicePolicyServicepolicyVector - errors', () => {
      it('should have a getConfigServicePolicyServicepolicyVector function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServicePolicyServicepolicyVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServicePolicyServicepolicyVector(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.getConfigServicePolicyServicepolicyVector('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vector', (done) => {
        try {
          a.getConfigServicePolicyServicepolicyVector('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServicePolicyServicepolicyVector - errors', () => {
      it('should have a patchConfigServicePolicyServicepolicyVector function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServicePolicyServicepolicyVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServicePolicyServicepolicyVector(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.patchConfigServicePolicyServicepolicyVector('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vector', (done) => {
        try {
          a.patchConfigServicePolicyServicepolicyVector('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'vector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServicePolicyServicepolicyVector('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServicePolicyServicepolicyVector - errors', () => {
      it('should have a deleteConfigServicePolicyServicepolicyVector function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServicePolicyServicepolicyVector === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServicePolicyServicepolicyVector(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing servicepolicy', (done) => {
        try {
          a.deleteConfigServicePolicyServicepolicyVector('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'servicepolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing vector', (done) => {
        try {
          a.deleteConfigServicePolicyServicepolicyVector('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'vector is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServicePolicyServicepolicyVector', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityServiceClone - errors', () => {
      it('should have a postConfigStoreAuthorityServiceClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityServiceClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityServiceClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityServiceClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityService2 - errors', () => {
      it('should have a getConfigStoreAuthorityService2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityService2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityService2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityService2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigStoreAuthorityService2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityService2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityService - errors', () => {
      it('should have a patchConfigStoreAuthorityService function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityService(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigStoreAuthorityService('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityService('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityService - errors', () => {
      it('should have a deleteConfigStoreAuthorityService function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityService(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigStoreAuthorityService('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityServiceAccessPolicy - errors', () => {
      it('should have a getConfigAuthorityServiceAccessPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityServiceAccessPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityServiceAccessPolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigAuthorityServiceAccessPolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityServiceAccessPolicy - errors', () => {
      it('should have a postConfigAuthorityServiceAccessPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityServiceAccessPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityServiceAccessPolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigAuthorityServiceAccessPolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityServiceAccessPolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAccessPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceAccessPolicyClone - errors', () => {
      it('should have a postConfigServiceAccessPolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceAccessPolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceAccessPolicyClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceAccessPolicyClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceAccessPolicyClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAccessPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceAccessPolicyAccesspolicy - errors', () => {
      it('should have a getConfigServiceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceAccessPolicyAccesspolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceAccessPolicyAccesspolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.getConfigServiceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceAccessPolicyAccesspolicy - errors', () => {
      it('should have a patchConfigServiceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceAccessPolicyAccesspolicy(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigServiceAccessPolicyAccesspolicy('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.patchConfigServiceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceAccessPolicyAccesspolicy - errors', () => {
      it('should have a deleteConfigServiceAccessPolicyAccesspolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceAccessPolicyAccesspolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceAccessPolicyAccesspolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigServiceAccessPolicyAccesspolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing accesspolicy', (done) => {
        try {
          a.deleteConfigServiceAccessPolicyAccesspolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'accesspolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAccessPolicyAccesspolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityServiceAppliesTo - errors', () => {
      it('should have a getConfigAuthorityServiceAppliesTo function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityServiceAppliesTo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityServiceAppliesTo(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigAuthorityServiceAppliesTo('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityServiceAppliesTo - errors', () => {
      it('should have a postConfigAuthorityServiceAppliesTo function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityServiceAppliesTo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityServiceAppliesTo(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigAuthorityServiceAppliesTo('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityServiceAppliesTo('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceAppliesTo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceAppliesToClone - errors', () => {
      it('should have a postConfigServiceAppliesToClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceAppliesToClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceAppliesToClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceAppliesToClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceAppliesToClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceAppliesToClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceAppliesToAppliesto - errors', () => {
      it('should have a getConfigServiceAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceAppliesToAppliesto(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceAppliesToAppliesto('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.getConfigServiceAppliesToAppliesto('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceAppliesToAppliesto - errors', () => {
      it('should have a patchConfigServiceAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceAppliesToAppliesto(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigServiceAppliesToAppliesto('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.patchConfigServiceAppliesToAppliesto('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceAppliesToAppliesto('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceAppliesToAppliesto - errors', () => {
      it('should have a deleteConfigServiceAppliesToAppliesto function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceAppliesToAppliesto === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceAppliesToAppliesto(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigServiceAppliesToAppliesto('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing appliesto', (done) => {
        try {
          a.deleteConfigServiceAppliesToAppliesto('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'appliesto is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceAppliesToAppliesto', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceMulticastSenderPolicy - errors', () => {
      it('should have a getConfigServiceMulticastSenderPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceMulticastSenderPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceMulticastSenderPolicy(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceMulticastSenderPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceMulticastSenderPolicy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceMulticastSenderPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceMulticastSenderPolicy - errors', () => {
      it('should have a postConfigServiceMulticastSenderPolicy function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceMulticastSenderPolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceMulticastSenderPolicyClone - errors', () => {
      it('should have a postConfigServiceMulticastSenderPolicyClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceMulticastSenderPolicyClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicyClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicyClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceMulticastSenderPolicyClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceMulticastSenderPolicyClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceMulticastSenderPolicyMulticastsenderpolicy - errors', () => {
      it('should have a getConfigServiceMulticastSenderPolicyMulticastsenderpolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceMulticastSenderPolicyMulticastsenderpolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceMulticastSenderPolicyMulticastsenderpolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing multicastsenderpolicy', (done) => {
        try {
          a.getConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'multicastsenderpolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy - errors', () => {
      it('should have a patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing multicastsenderpolicy', (done) => {
        try {
          a.patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'multicastsenderpolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy - errors', () => {
      it('should have a deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing multicastsenderpolicy', (done) => {
        try {
          a.deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'multicastsenderpolicy is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceMulticastSenderPolicyMulticastsenderpolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityServiceTransport - errors', () => {
      it('should have a getConfigAuthorityServiceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityServiceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityServiceTransport(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigAuthorityServiceTransport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityServiceTransport - errors', () => {
      it('should have a postConfigAuthorityServiceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityServiceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityServiceTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigAuthorityServiceTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityServiceTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceTransportClone - errors', () => {
      it('should have a postConfigServiceTransportClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceTransportClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceTransportClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceTransportClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceTransportClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceTransport - errors', () => {
      it('should have a getConfigServiceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigServiceTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigServiceTransport - errors', () => {
      it('should have a patchConfigServiceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigServiceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigServiceTransport(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigServiceTransport('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigServiceTransport('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigServiceTransport('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigServiceTransport - errors', () => {
      it('should have a deleteConfigServiceTransport function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigServiceTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigServiceTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigServiceTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigServiceTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigServiceTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigServiceTransportPortRange - errors', () => {
      it('should have a getConfigServiceTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigServiceTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigServiceTransportPortRange(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigServiceTransportPortRange('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigServiceTransportPortRange('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigServiceTransportPortRange - errors', () => {
      it('should have a postConfigServiceTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigServiceTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigServiceTransportPortRange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigServiceTransportPortRange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigServiceTransportPortRange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigServiceTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigServiceTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTransportPortRangeClone3 - errors', () => {
      it('should have a postConfigTransportPortRangeClone3 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTransportPortRangeClone3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTransportPortRangeClone3(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.postConfigTransportPortRangeClone3('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigTransportPortRangeClone3('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTransportPortRangeClone3('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTransportPortRangePortrange3 - errors', () => {
      it('should have a getConfigTransportPortRangePortrange3 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTransportPortRangePortrange3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTransportPortRangePortrange3(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getConfigTransportPortRangePortrange3('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTransportPortRangePortrange3 - errors', () => {
      it('should have a patchConfigTransportPortRangePortrange3 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTransportPortRangePortrange3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange3(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange3('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigTransportPortRangePortrange3 - errors', () => {
      it('should have a deleteConfigTransportPortRangePortrange3 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigTransportPortRangePortrange3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange3(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange3('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange3('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthoritySessionType - errors', () => {
      it('should have a getConfigStoreAuthoritySessionType function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthoritySessionType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthoritySessionType(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySessionType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthoritySessionType - errors', () => {
      it('should have a postConfigStoreAuthoritySessionType function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthoritySessionType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthoritySessionType(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySessionType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthoritySessionType('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySessionType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthoritySessionTypeClone - errors', () => {
      it('should have a postConfigStoreAuthoritySessionTypeClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthoritySessionTypeClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthoritySessionTypeClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySessionTypeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthoritySessionTypeClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthoritySessionTypeClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthoritySessionTypeSessiontype - errors', () => {
      it('should have a getConfigStoreAuthoritySessionTypeSessiontype function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthoritySessionTypeSessiontype === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthoritySessionTypeSessiontype(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.getConfigStoreAuthoritySessionTypeSessiontype('fakeparam', null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthoritySessionTypeSessiontype - errors', () => {
      it('should have a patchConfigStoreAuthoritySessionTypeSessiontype function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthoritySessionTypeSessiontype === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthoritySessionTypeSessiontype(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.patchConfigStoreAuthoritySessionTypeSessiontype('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthoritySessionTypeSessiontype('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthoritySessionTypeSessiontype - errors', () => {
      it('should have a deleteConfigStoreAuthoritySessionTypeSessiontype function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthoritySessionTypeSessiontype === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthoritySessionTypeSessiontype(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.deleteConfigStoreAuthoritySessionTypeSessiontype('fakeparam', null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthoritySessionTypeSessiontype', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthoritySessionTypeSessiontypeTransport - errors', () => {
      it('should have a getConfigAuthoritySessionTypeSessiontypeTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthoritySessionTypeSessiontypeTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthoritySessionTypeSessiontypeTransport(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthoritySessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.getConfigAuthoritySessionTypeSessiontypeTransport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthoritySessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthoritySessionTypeSessiontypeTransport - errors', () => {
      it('should have a postConfigAuthoritySessionTypeSessiontypeTransport function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthoritySessionTypeSessiontypeTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthoritySessionTypeSessiontypeTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthoritySessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.postConfigAuthoritySessionTypeSessiontypeTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthoritySessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthoritySessionTypeSessiontypeTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthoritySessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSessionTypeSessiontypeTransportClone - errors', () => {
      it('should have a postConfigSessionTypeSessiontypeTransportClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSessionTypeSessiontypeTransportClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSessionTypeSessiontypeTransportClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessionTypeSessiontypeTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.postConfigSessionTypeSessiontypeTransportClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessionTypeSessiontypeTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSessionTypeSessiontypeTransportClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessionTypeSessiontypeTransportClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSessionTypeSessiontypeTransport - errors', () => {
      it('should have a getConfigSessionTypeSessiontypeTransport function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSessionTypeSessiontypeTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSessionTypeSessiontypeTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.getConfigSessionTypeSessiontypeTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigSessionTypeSessiontypeTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigSessionTypeSessiontypeTransport - errors', () => {
      it('should have a patchConfigSessionTypeSessiontypeTransport function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigSessionTypeSessiontypeTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigSessionTypeSessiontypeTransport(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.patchConfigSessionTypeSessiontypeTransport('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigSessionTypeSessiontypeTransport('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigSessionTypeSessiontypeTransport('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigSessionTypeSessiontypeTransport - errors', () => {
      it('should have a deleteConfigSessionTypeSessiontypeTransport function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigSessionTypeSessiontypeTransport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigSessionTypeSessiontypeTransport(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.deleteConfigSessionTypeSessiontypeTransport('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigSessionTypeSessiontypeTransport('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigSessionTypeSessiontypeTransport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigSessiontypeTransportPortRange - errors', () => {
      it('should have a getConfigSessiontypeTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigSessiontypeTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigSessiontypeTransportPortRange(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.getConfigSessiontypeTransportPortRange('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigSessiontypeTransportPortRange('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigSessiontypeTransportPortRange - errors', () => {
      it('should have a postConfigSessiontypeTransportPortRange function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigSessiontypeTransportPortRange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigSessiontypeTransportPortRange(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.postConfigSessiontypeTransportPortRange('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigSessiontypeTransportPortRange('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigSessiontypeTransportPortRange('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigSessiontypeTransportPortRange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTransportPortRangeClone4 - errors', () => {
      it('should have a postConfigTransportPortRangeClone4 function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTransportPortRangeClone4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTransportPortRangeClone4(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.postConfigTransportPortRangeClone4('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.postConfigTransportPortRangeClone4('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTransportPortRangeClone4('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTransportPortRangeClone4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTransportPortRangePortrange4 - errors', () => {
      it('should have a getConfigTransportPortRangePortrange4 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTransportPortRangePortrange4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTransportPortRangePortrange4(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.getConfigTransportPortRangePortrange4('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.getConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.getConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTransportPortRangePortrange4 - errors', () => {
      it('should have a patchConfigTransportPortRangePortrange4 function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTransportPortRangePortrange4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange4(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange4('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigTransportPortRangePortrange4 - errors', () => {
      it('should have a deleteConfigTransportPortRangePortrange4 function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigTransportPortRangePortrange4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange4(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing sessiontype', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange4('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'sessiontype is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing transport', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'transport is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing portrange', (done) => {
        try {
          a.deleteConfigTransportPortRangePortrange4('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'portrange is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTransportPortRangePortrange4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityStepRepo - errors', () => {
      it('should have a getConfigStoreAuthorityStepRepo function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityStepRepo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityStepRepo(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityStepRepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityStepRepo - errors', () => {
      it('should have a postConfigStoreAuthorityStepRepo function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityStepRepo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityStepRepo(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityStepRepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityStepRepo('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityStepRepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityStepRepoClone - errors', () => {
      it('should have a postConfigStoreAuthorityStepRepoClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityStepRepoClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityStepRepoClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityStepRepoClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityStepRepoClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityStepRepoClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityStepRepoSteprepo - errors', () => {
      it('should have a getConfigStoreAuthorityStepRepoSteprepo function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityStepRepoSteprepo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityStepRepoSteprepo(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing steprepo', (done) => {
        try {
          a.getConfigStoreAuthorityStepRepoSteprepo('fakeparam', null, (data, error) => {
            try {
              const displayE = 'steprepo is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityStepRepoSteprepo - errors', () => {
      it('should have a patchConfigStoreAuthorityStepRepoSteprepo function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityStepRepoSteprepo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityStepRepoSteprepo(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing steprepo', (done) => {
        try {
          a.patchConfigStoreAuthorityStepRepoSteprepo('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'steprepo is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityStepRepoSteprepo('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityStepRepoSteprepo - errors', () => {
      it('should have a deleteConfigStoreAuthorityStepRepoSteprepo function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityStepRepoSteprepo === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityStepRepoSteprepo(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing steprepo', (done) => {
        try {
          a.deleteConfigStoreAuthorityStepRepoSteprepo('fakeparam', null, (data, error) => {
            try {
              const displayE = 'steprepo is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityStepRepoSteprepo', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityTenant - errors', () => {
      it('should have a getConfigStoreAuthorityTenant function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityTenant(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityTenant - errors', () => {
      it('should have a postConfigStoreAuthorityTenant function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityTenant(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityTenant('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityTenantClone - errors', () => {
      it('should have a postConfigStoreAuthorityTenantClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityTenantClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityTenantClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTenantClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityTenantClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTenantClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityTenant2 - errors', () => {
      it('should have a getConfigStoreAuthorityTenant2 function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityTenant2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityTenant2(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTenant2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.getConfigStoreAuthorityTenant2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTenant2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityTenant - errors', () => {
      it('should have a patchConfigStoreAuthorityTenant function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityTenant(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.patchConfigStoreAuthorityTenant('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityTenant('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityTenant - errors', () => {
      it('should have a deleteConfigStoreAuthorityTenant function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityTenant(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.deleteConfigStoreAuthorityTenant('fakeparam', null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityTenantMember - errors', () => {
      it('should have a getConfigAuthorityTenantMember function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityTenantMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityTenantMember(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.getConfigAuthorityTenantMember('fakeparam', null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigAuthorityTenantMember - errors', () => {
      it('should have a postConfigAuthorityTenantMember function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigAuthorityTenantMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigAuthorityTenantMember(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.postConfigAuthorityTenantMember('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigAuthorityTenantMember('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigAuthorityTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigTenantMemberClone - errors', () => {
      it('should have a postConfigTenantMemberClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigTenantMemberClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigTenantMemberClone(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTenantMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.postConfigTenantMemberClone('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTenantMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigTenantMemberClone('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigTenantMemberClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTenantMember - errors', () => {
      it('should have a getConfigTenantMember function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTenantMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTenantMember(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.getConfigTenantMember('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.getConfigTenantMember('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTenantMember - errors', () => {
      it('should have a patchConfigTenantMember function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTenantMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTenantMember(null, null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.patchConfigTenantMember('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.patchConfigTenantMember('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTenantMember('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigTenantMember - errors', () => {
      it('should have a deleteConfigTenantMember function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigTenantMember === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigTenantMember(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.deleteConfigTenantMember('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing member', (done) => {
        try {
          a.deleteConfigTenantMember('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'member is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigTenantMember', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityTrafficProfile - errors', () => {
      it('should have a getConfigStoreAuthorityTrafficProfile function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityTrafficProfile === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityTrafficProfile(null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTrafficProfile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityTrafficProfile - errors', () => {
      it('should have a postConfigStoreAuthorityTrafficProfile function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityTrafficProfile === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityTrafficProfile(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTrafficProfile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityTrafficProfile('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTrafficProfile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postConfigStoreAuthorityTrafficProfileClone - errors', () => {
      it('should have a postConfigStoreAuthorityTrafficProfileClone function', (done) => {
        try {
          assert.equal(true, typeof a.postConfigStoreAuthorityTrafficProfileClone === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.postConfigStoreAuthorityTrafficProfileClone(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTrafficProfileClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postConfigStoreAuthorityTrafficProfileClone('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postConfigStoreAuthorityTrafficProfileClone', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigStoreAuthorityTrafficProfileTrafficprofile - errors', () => {
      it('should have a getConfigStoreAuthorityTrafficProfileTrafficprofile function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigStoreAuthorityTrafficProfileTrafficprofile === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigStoreAuthorityTrafficProfileTrafficprofile(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.getConfigStoreAuthorityTrafficProfileTrafficprofile('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigStoreAuthorityTrafficProfileTrafficprofile - errors', () => {
      it('should have a patchConfigStoreAuthorityTrafficProfileTrafficprofile function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigStoreAuthorityTrafficProfileTrafficprofile === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigStoreAuthorityTrafficProfileTrafficprofile(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.patchConfigStoreAuthorityTrafficProfileTrafficprofile('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigStoreAuthorityTrafficProfileTrafficprofile('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#deleteConfigStoreAuthorityTrafficProfileTrafficprofile - errors', () => {
      it('should have a deleteConfigStoreAuthorityTrafficProfileTrafficprofile function', (done) => {
        try {
          assert.equal(true, typeof a.deleteConfigStoreAuthorityTrafficProfileTrafficprofile === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.deleteConfigStoreAuthorityTrafficProfileTrafficprofile(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.deleteConfigStoreAuthorityTrafficProfileTrafficprofile('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-deleteConfigStoreAuthorityTrafficProfileTrafficprofile', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigTrafficProfileTrafficprofileBestEffort - errors', () => {
      it('should have a getConfigTrafficProfileTrafficprofileBestEffort function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigTrafficProfileTrafficprofileBestEffort === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigTrafficProfileTrafficprofileBestEffort(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTrafficProfileTrafficprofileBestEffort', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.getConfigTrafficProfileTrafficprofileBestEffort('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigTrafficProfileTrafficprofileBestEffort', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigTrafficProfileTrafficprofileBestEffort - errors', () => {
      it('should have a patchConfigTrafficProfileTrafficprofileBestEffort function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigTrafficProfileTrafficprofileBestEffort === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigTrafficProfileTrafficprofileBestEffort(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTrafficProfileTrafficprofileBestEffort', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.patchConfigTrafficProfileTrafficprofileBestEffort('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTrafficProfileTrafficprofileBestEffort', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigTrafficProfileTrafficprofileBestEffort('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigTrafficProfileTrafficprofileBestEffort', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityTrafficProfileTrafficprofileHigh - errors', () => {
      it('should have a getConfigAuthorityTrafficProfileTrafficprofileHigh function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityTrafficProfileTrafficprofileHigh === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileHigh(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileHigh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileHigh('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileHigh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityTrafficProfileTrafficprofileHigh - errors', () => {
      it('should have a patchConfigAuthorityTrafficProfileTrafficprofileHigh function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityTrafficProfileTrafficprofileHigh === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileHigh(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileHigh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileHigh('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileHigh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileHigh('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileHigh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityTrafficProfileTrafficprofileLow - errors', () => {
      it('should have a getConfigAuthorityTrafficProfileTrafficprofileLow function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityTrafficProfileTrafficprofileLow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileLow(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileLow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileLow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileLow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityTrafficProfileTrafficprofileLow - errors', () => {
      it('should have a patchConfigAuthorityTrafficProfileTrafficprofileLow function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityTrafficProfileTrafficprofileLow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileLow(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileLow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileLow('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileLow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileLow('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileLow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getConfigAuthorityTrafficProfileTrafficprofileMedium - errors', () => {
      it('should have a getConfigAuthorityTrafficProfileTrafficprofileMedium function', (done) => {
        try {
          assert.equal(true, typeof a.getConfigAuthorityTrafficProfileTrafficprofileMedium === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileMedium(null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileMedium', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.getConfigAuthorityTrafficProfileTrafficprofileMedium('fakeparam', null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getConfigAuthorityTrafficProfileTrafficprofileMedium', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#patchConfigAuthorityTrafficProfileTrafficprofileMedium - errors', () => {
      it('should have a patchConfigAuthorityTrafficProfileTrafficprofileMedium function', (done) => {
        try {
          assert.equal(true, typeof a.patchConfigAuthorityTrafficProfileTrafficprofileMedium === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing configStore', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileMedium(null, null, null, (data, error) => {
            try {
              const displayE = 'configStore is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileMedium', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing trafficprofile', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileMedium('fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'trafficprofile is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileMedium', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.patchConfigAuthorityTrafficProfileTrafficprofileMedium('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-patchConfigAuthorityTrafficProfileTrafficprofileMedium', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postLogin - errors', () => {
      it('should have a postLogin function', (done) => {
        try {
          assert.equal(true, typeof a.postLogin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postLogin(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postLogin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getPluginsAvailable - errors', () => {
      it('should have a getPluginsAvailable function', (done) => {
        try {
          assert.equal(true, typeof a.getPluginsAvailable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postPluginsInstall - errors', () => {
      it('should have a postPluginsInstall function', (done) => {
        try {
          assert.equal(true, typeof a.postPluginsInstall === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postPluginsInstall(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postPluginsInstall', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getPluginsInstalled - errors', () => {
      it('should have a getPluginsInstalled function', (done) => {
        try {
          assert.equal(true, typeof a.getPluginsInstalled === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postPluginsRemove - errors', () => {
      it('should have a postPluginsRemove function', (done) => {
        try {
          assert.equal(true, typeof a.postPluginsRemove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postPluginsRemove(null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postPluginsRemove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouter - errors', () => {
      it('should have a getRouter function', (done) => {
        try {
          assert.equal(true, typeof a.getRouter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouter2 - errors', () => {
      it('should have a getRouter2 function', (done) => {
        try {
          assert.equal(true, typeof a.getRouter2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouter2(null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouter2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetrics - errors', () => {
      it('should have a postRouterMetrics function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetrics === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetrics(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetrics('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetrics', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterTopSessions - errors', () => {
      it('should have a getRouterTopSessions function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterTopSessions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterTopSessions(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterTopSessions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterTopSources - errors', () => {
      it('should have a getRouterTopSources function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterTopSources === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterTopSources(null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterTopSources', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNode - errors', () => {
      it('should have a getRouterNode function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNode(null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNode2 - errors', () => {
      it('should have a getRouterNode2 function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNode2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNode2(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNode2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNode2('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNode2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeAdjacency - errors', () => {
      it('should have a getRouterNodeAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeAdjacency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeAdjacency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNetworkInterfaceAdjacency - errors', () => {
      it('should have a getRouterNetworkInterfaceAdjacency function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNetworkInterfaceAdjacency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNetworkInterfaceAdjacency(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNetworkInterfaceAdjacency('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceInterface', (done) => {
        try {
          a.getRouterNetworkInterfaceAdjacency('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'deviceInterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing networkInterface', (done) => {
        try {
          a.getRouterNetworkInterfaceAdjacency('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'networkInterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceAdjacency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeDeviceInterface - errors', () => {
      it('should have a getRouterNodeDeviceInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeDeviceInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeDeviceInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeDeviceInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeDeviceInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterInterfaceDeviceNetwork - errors', () => {
      it('should have a getRouterInterfaceDeviceNetwork function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterInterfaceDeviceNetwork === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterInterfaceDeviceNetwork(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterInterfaceDeviceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterInterfaceDeviceNetwork('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterInterfaceDeviceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing deviceInterface', (done) => {
        try {
          a.getRouterInterfaceDeviceNetwork('fakeparam', 'fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'deviceInterface is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterInterfaceDeviceNetwork', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeNetworkInterface - errors', () => {
      it('should have a getRouterNodeNetworkInterface function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeNetworkInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeNetworkInterface(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeNetworkInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeNetworkInterface('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeNetworkInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNetworkInterfaceByDevice - errors', () => {
      it('should have a getRouterNetworkInterfaceByDevice function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNetworkInterfaceByDevice === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNetworkInterfaceByDevice(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceByDevice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNetworkInterfaceByDevice('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNetworkInterfaceByDevice', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeStatus - errors', () => {
      it('should have a getRouterNodeStatus function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeStatus === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeStatus(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeStatus', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeStatus('fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeStatus', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeVersion - errors', () => {
      it('should have a getRouterNodeVersion function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeVersion === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeVersion(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeVersion('fakeparam', null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getSystem - errors', () => {
      it('should have a getSystem function', (done) => {
        try {
          assert.equal(true, typeof a.getSystem === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeTrafficFib - errors', () => {
      it('should have a getRouterNodeTrafficFib function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeTrafficFib === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeTrafficFib(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeTrafficFib('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeTrafficFlows - errors', () => {
      it('should have a getRouterNodeTrafficFlows function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeTrafficFlows === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeTrafficFlows(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficFlows', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeTrafficFlows('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficFlows', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeTrafficRib - errors', () => {
      it('should have a getRouterNodeTrafficRib function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeTrafficRib === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeTrafficRib(null, null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficRib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeTrafficRib('fakeparam', null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficRib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterNodeTrafficSourceTenant - errors', () => {
      it('should have a getRouterNodeTrafficSourceTenant function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterNodeTrafficSourceTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterNodeTrafficSourceTenant(null, null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficSourceTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing node', (done) => {
        try {
          a.getRouterNodeTrafficSourceTenant('fakeparam', null, null, null, null, (data, error) => {
            try {
              const displayE = 'node is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterNodeTrafficSourceTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getRouterServiceRoutePolicy - errors', () => {
      it('should have a getRouterServiceRoutePolicy function', (done) => {
        try {
          assert.equal(true, typeof a.getRouterServiceRoutePolicy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getRouterServiceRoutePolicy(null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getRouterServiceRoutePolicy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAccessPolicyMeter - errors', () => {
      it('should have a postRouterStatsAccessPolicyMeter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAccessPolicyMeter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAccessPolicyMeter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAccessPolicyMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAccessPolicyMeter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAccessPolicyMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsActiveSources - errors', () => {
      it('should have a postRouterStatsActiveSources function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsActiveSources === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsActiveSources(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsActiveSources', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsActiveSources('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsActiveSources', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionDeviceInterfaceBandwidth - errors', () => {
      it('should have a postRouterAggregateSessionDeviceInterfaceBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionDeviceInterfaceBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionDeviceInterfaceBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionDeviceInterfaceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionDeviceInterfaceBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionDeviceInterfaceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceBandwidthReceived - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterDeviceInterfaceBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterDeviceInterfaceBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceBandwidthTcp - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceBandwidthTransmitted - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterDeviceInterfaceBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterDeviceInterfaceBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceBandwidthUdp - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionDeviceInterfacePackets - errors', () => {
      it('should have a postRouterAggregateSessionDeviceInterfacePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionDeviceInterfacePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionDeviceInterfacePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionDeviceInterfacePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionDeviceInterfacePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionDeviceInterfacePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfacePacketsReceived - errors', () => {
      it('should have a postRouterSessionDeviceInterfacePacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfacePacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfacePacketsReceivedTcp - errors', () => {
      it('should have a postRouterDeviceInterfacePacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfacePacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfacePacketsReceivedUdp - errors', () => {
      it('should have a postRouterDeviceInterfacePacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfacePacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfacePacketsTcp - errors', () => {
      it('should have a postRouterSessionDeviceInterfacePacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfacePacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfacePacketsTransmitted - errors', () => {
      it('should have a postRouterSessionDeviceInterfacePacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfacePacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfacePacketsTransmittedTcp - errors', () => {
      it('should have a postRouterDeviceInterfacePacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfacePacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfacePacketsTransmittedUdp - errors', () => {
      it('should have a postRouterDeviceInterfacePacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfacePacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfacePacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfacePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfacePacketsUdp - errors', () => {
      it('should have a postRouterSessionDeviceInterfacePacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfacePacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfacePacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfacePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceSessionArrivalRate - errors', () => {
      it('should have a postRouterDeviceInterfaceSessionArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceSessionArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceSessionArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceSessionArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceCount - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceSessionDepartureRate - errors', () => {
      it('should have a postRouterDeviceInterfaceSessionDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceSessionDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceSessionDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceSessionDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionDeviceInterfaceTcpRetransmissions - errors', () => {
      it('should have a postRouterSessionDeviceInterfaceTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionDeviceInterfaceTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionDeviceInterfaceTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionDeviceInterfaceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterDeviceInterfaceTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDeviceInterfaceTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterDeviceInterfaceTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDeviceInterfaceTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDeviceInterfaceTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDeviceInterfaceTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDeviceInterfaceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNetworkInterfaceBandwidth - errors', () => {
      it('should have a postRouterAggregateSessionNetworkInterfaceBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNetworkInterfaceBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNetworkInterfaceBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNetworkInterfaceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNetworkInterfaceBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNetworkInterfaceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceBandwidthReceived - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterNetworkInterfaceBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterNetworkInterfaceBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceBandwidthTcp - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceBandwidthTransmitted - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterNetworkInterfaceBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterNetworkInterfaceBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceBandwidthUdp - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNetworkInterfacePackets - errors', () => {
      it('should have a postRouterAggregateSessionNetworkInterfacePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNetworkInterfacePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNetworkInterfacePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNetworkInterfacePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNetworkInterfacePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNetworkInterfacePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfacePacketsReceived - errors', () => {
      it('should have a postRouterSessionNetworkInterfacePacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfacePacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfacePacketsReceivedTcp - errors', () => {
      it('should have a postRouterNetworkInterfacePacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfacePacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfacePacketsReceivedUdp - errors', () => {
      it('should have a postRouterNetworkInterfacePacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfacePacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfacePacketsTcp - errors', () => {
      it('should have a postRouterSessionNetworkInterfacePacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfacePacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfacePacketsTransmitted - errors', () => {
      it('should have a postRouterSessionNetworkInterfacePacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfacePacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfacePacketsTransmittedTcp - errors', () => {
      it('should have a postRouterNetworkInterfacePacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfacePacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfacePacketsTransmittedUdp - errors', () => {
      it('should have a postRouterNetworkInterfacePacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfacePacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfacePacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfacePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfacePacketsUdp - errors', () => {
      it('should have a postRouterSessionNetworkInterfacePacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfacePacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfacePacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfacePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceSessionArrivalRate - errors', () => {
      it('should have a postRouterNetworkInterfaceSessionArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceSessionArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceSessionArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceSessionArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceCount - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceSessionDepartureRate - errors', () => {
      it('should have a postRouterNetworkInterfaceSessionDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceSessionDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceSessionDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceSessionDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNetworkInterfaceTcpRetransmissions - errors', () => {
      it('should have a postRouterSessionNetworkInterfaceTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNetworkInterfaceTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNetworkInterfaceTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNetworkInterfaceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterNetworkInterfaceTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNetworkInterfaceTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterNetworkInterfaceTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNetworkInterfaceTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNetworkInterfaceTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNetworkInterfaceTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNetworkInterfaceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionNodeBandwidth - errors', () => {
      it('should have a postRouterStatsAggregateSessionNodeBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionNodeBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionNodeBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionNodeBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionNodeBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionNodeBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeBandwidthReceived - errors', () => {
      it('should have a postRouterAggregateSessionNodeBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterSessionNodeBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterSessionNodeBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeBandwidthTcp - errors', () => {
      it('should have a postRouterAggregateSessionNodeBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeBandwidthTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionNodeBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterSessionNodeBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterSessionNodeBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeBandwidthUdp - errors', () => {
      it('should have a postRouterAggregateSessionNodeBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeGrossEntitlement - errors', () => {
      it('should have a postRouterAggregateSessionNodeGrossEntitlement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeGrossEntitlement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeGrossEntitlement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeGrossEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeGrossEntitlement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeGrossEntitlement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionNodePackets - errors', () => {
      it('should have a postRouterStatsAggregateSessionNodePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionNodePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionNodePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionNodePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionNodePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionNodePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodePacketsReceived - errors', () => {
      it('should have a postRouterAggregateSessionNodePacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodePacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodePacketsReceivedTcp - errors', () => {
      it('should have a postRouterSessionNodePacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodePacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodePacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodePacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodePacketsReceivedUdp - errors', () => {
      it('should have a postRouterSessionNodePacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodePacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodePacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodePacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodePacketsTcp - errors', () => {
      it('should have a postRouterAggregateSessionNodePacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodePacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodePacketsTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionNodePacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodePacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodePacketsTransmittedTcp - errors', () => {
      it('should have a postRouterSessionNodePacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodePacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodePacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodePacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodePacketsTransmittedUdp - errors', () => {
      it('should have a postRouterSessionNodePacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodePacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodePacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodePacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodePacketsUdp - errors', () => {
      it('should have a postRouterAggregateSessionNodePacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodePacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodePacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeArrivalRate - errors', () => {
      it('should have a postRouterSessionNodeArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeCount - errors', () => {
      it('should have a postRouterAggregateSessionNodeCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeDepartureRate - errors', () => {
      it('should have a postRouterSessionNodeDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionNodeTcpRetransmissions - errors', () => {
      it('should have a postRouterAggregateSessionNodeTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionNodeTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionNodeTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionNodeTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionNodeTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterSessionNodeTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionNodeTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterSessionNodeTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionNodeTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionNodeTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionNodeTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionNodeTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceClassBandwidth - errors', () => {
      it('should have a postRouterAggregateSessionServiceClassBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceClassBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceClassBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceClassBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceClassBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceClassBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassBandwidthReceived - errors', () => {
      it('should have a postRouterSessionServiceClassBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterServiceClassBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterServiceClassBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassBandwidthTcp - errors', () => {
      it('should have a postRouterSessionServiceClassBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassBandwidthTransmitted - errors', () => {
      it('should have a postRouterSessionServiceClassBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterServiceClassBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterServiceClassBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassBandwidthUdp - errors', () => {
      it('should have a postRouterSessionServiceClassBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceClassPackets - errors', () => {
      it('should have a postRouterAggregateSessionServiceClassPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceClassPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceClassPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceClassPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceClassPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceClassPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassPacketsReceived - errors', () => {
      it('should have a postRouterSessionServiceClassPacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassPacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassPacketsReceivedTcp - errors', () => {
      it('should have a postRouterServiceClassPacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassPacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassPacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassPacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassPacketsReceivedUdp - errors', () => {
      it('should have a postRouterServiceClassPacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassPacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassPacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassPacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassPacketsTcp - errors', () => {
      it('should have a postRouterSessionServiceClassPacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassPacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassPacketsTransmitted - errors', () => {
      it('should have a postRouterSessionServiceClassPacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassPacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassPacketsTransmittedTcp - errors', () => {
      it('should have a postRouterServiceClassPacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassPacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassPacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassPacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassPacketsTransmittedUdp - errors', () => {
      it('should have a postRouterServiceClassPacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassPacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassPacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassPacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassPacketsUdp - errors', () => {
      it('should have a postRouterSessionServiceClassPacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassPacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassPacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassSessionArrivalRate - errors', () => {
      it('should have a postRouterServiceClassSessionArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassSessionArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassSessionArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassSessionArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassCount - errors', () => {
      it('should have a postRouterSessionServiceClassCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassSessionDepartureRate - errors', () => {
      it('should have a postRouterServiceClassSessionDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassSessionDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassSessionDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassSessionDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceClassTcpRetransmissions - errors', () => {
      it('should have a postRouterSessionServiceClassTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceClassTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceClassTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceClassTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceClassTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterServiceClassTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceClassTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterServiceClassTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceClassTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceClassTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceClassTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceClassTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceGroupBandwidth - errors', () => {
      it('should have a postRouterAggregateSessionServiceGroupBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceGroupBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceGroupBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceGroupBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceGroupBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceGroupBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupBandwidthReceived - errors', () => {
      it('should have a postRouterSessionServiceGroupBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterServiceGroupBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterServiceGroupBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupBandwidthTcp - errors', () => {
      it('should have a postRouterSessionServiceGroupBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupBandwidthTransmitted - errors', () => {
      it('should have a postRouterSessionServiceGroupBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterServiceGroupBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterServiceGroupBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupBandwidthUdp - errors', () => {
      it('should have a postRouterSessionServiceGroupBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceGroupPackets - errors', () => {
      it('should have a postRouterAggregateSessionServiceGroupPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceGroupPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceGroupPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceGroupPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceGroupPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceGroupPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupPacketsReceived - errors', () => {
      it('should have a postRouterSessionServiceGroupPacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupPacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupPacketsReceivedTcp - errors', () => {
      it('should have a postRouterServiceGroupPacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupPacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupPacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupPacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupPacketsReceivedUdp - errors', () => {
      it('should have a postRouterServiceGroupPacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupPacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupPacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupPacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupPacketsTcp - errors', () => {
      it('should have a postRouterSessionServiceGroupPacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupPacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupPacketsTransmitted - errors', () => {
      it('should have a postRouterSessionServiceGroupPacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupPacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupPacketsTransmittedTcp - errors', () => {
      it('should have a postRouterServiceGroupPacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupPacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupPacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupPacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupPacketsTransmittedUdp - errors', () => {
      it('should have a postRouterServiceGroupPacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupPacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupPacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupPacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupPacketsUdp - errors', () => {
      it('should have a postRouterSessionServiceGroupPacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupPacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupPacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupSessionArrivalRate - errors', () => {
      it('should have a postRouterServiceGroupSessionArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupSessionArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupSessionArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupSessionArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupCount - errors', () => {
      it('should have a postRouterSessionServiceGroupCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupSessionDepartureRate - errors', () => {
      it('should have a postRouterServiceGroupSessionDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupSessionDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupSessionDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupSessionDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceGroupTcpRetransmissions - errors', () => {
      it('should have a postRouterSessionServiceGroupTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceGroupTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceGroupTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceGroupTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceGroupTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterServiceGroupTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceGroupTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterServiceGroupTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceGroupTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceGroupTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceGroupTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceGroupTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceRouteBandwidth - errors', () => {
      it('should have a postRouterAggregateSessionServiceRouteBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceRouteBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceRouteBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceRouteBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceRouteBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceRouteBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteBandwidthReceived - errors', () => {
      it('should have a postRouterSessionServiceRouteBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterServiceRouteBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterServiceRouteBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteBandwidthTcp - errors', () => {
      it('should have a postRouterSessionServiceRouteBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteBandwidthTransmitted - errors', () => {
      it('should have a postRouterSessionServiceRouteBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterServiceRouteBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterServiceRouteBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteBandwidthUdp - errors', () => {
      it('should have a postRouterSessionServiceRouteBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceRoutePackets - errors', () => {
      it('should have a postRouterAggregateSessionServiceRoutePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceRoutePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceRoutePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceRoutePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceRoutePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceRoutePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRoutePacketsReceived - errors', () => {
      it('should have a postRouterSessionServiceRoutePacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRoutePacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRoutePacketsReceivedTcp - errors', () => {
      it('should have a postRouterServiceRoutePacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRoutePacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRoutePacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRoutePacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRoutePacketsReceivedUdp - errors', () => {
      it('should have a postRouterServiceRoutePacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRoutePacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRoutePacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRoutePacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRoutePacketsTcp - errors', () => {
      it('should have a postRouterSessionServiceRoutePacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRoutePacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRoutePacketsTransmitted - errors', () => {
      it('should have a postRouterSessionServiceRoutePacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRoutePacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRoutePacketsTransmittedTcp - errors', () => {
      it('should have a postRouterServiceRoutePacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRoutePacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRoutePacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRoutePacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRoutePacketsTransmittedUdp - errors', () => {
      it('should have a postRouterServiceRoutePacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRoutePacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRoutePacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRoutePacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRoutePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRoutePacketsUdp - errors', () => {
      it('should have a postRouterSessionServiceRoutePacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRoutePacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRoutePacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRoutePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteSessionArrivalRate - errors', () => {
      it('should have a postRouterServiceRouteSessionArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteSessionArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteSessionArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteSessionArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteSessionArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteCount - errors', () => {
      it('should have a postRouterSessionServiceRouteCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteSessionDepartureRate - errors', () => {
      it('should have a postRouterServiceRouteSessionDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteSessionDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteSessionDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteSessionDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteSessionDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceRouteTcpRetransmissions - errors', () => {
      it('should have a postRouterSessionServiceRouteTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceRouteTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceRouteTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceRouteTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceRouteTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterServiceRouteTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceRouteTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterServiceRouteTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceRouteTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceRouteTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceRouteTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceRouteTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionServiceBandwidth - errors', () => {
      it('should have a postRouterStatsAggregateSessionServiceBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionServiceBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionServiceBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionServiceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionServiceBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionServiceBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceBandwidthReceived - errors', () => {
      it('should have a postRouterAggregateSessionServiceBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterSessionServiceBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterSessionServiceBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceBandwidthTcp - errors', () => {
      it('should have a postRouterAggregateSessionServiceBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceBandwidthTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionServiceBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterSessionServiceBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterSessionServiceBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceBandwidthUdp - errors', () => {
      it('should have a postRouterAggregateSessionServiceBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionServicePackets - errors', () => {
      it('should have a postRouterStatsAggregateSessionServicePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionServicePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionServicePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionServicePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionServicePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionServicePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServicePacketsReceived - errors', () => {
      it('should have a postRouterAggregateSessionServicePacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServicePacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServicePacketsReceivedTcp - errors', () => {
      it('should have a postRouterSessionServicePacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServicePacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServicePacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServicePacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServicePacketsReceivedUdp - errors', () => {
      it('should have a postRouterSessionServicePacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServicePacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServicePacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServicePacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServicePacketsTcp - errors', () => {
      it('should have a postRouterAggregateSessionServicePacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServicePacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServicePacketsTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionServicePacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServicePacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServicePacketsTransmittedTcp - errors', () => {
      it('should have a postRouterSessionServicePacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServicePacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServicePacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServicePacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServicePacketsTransmittedUdp - errors', () => {
      it('should have a postRouterSessionServicePacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServicePacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServicePacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServicePacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServicePacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServicePacketsUdp - errors', () => {
      it('should have a postRouterAggregateSessionServicePacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServicePacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServicePacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServicePacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceArrivalRate - errors', () => {
      it('should have a postRouterSessionServiceArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceCount - errors', () => {
      it('should have a postRouterAggregateSessionServiceCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceDepartureRate - errors', () => {
      it('should have a postRouterSessionServiceDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionServiceTcpRetransmissions - errors', () => {
      it('should have a postRouterAggregateSessionServiceTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionServiceTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionServiceTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionServiceTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionServiceTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterSessionServiceTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionServiceTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterSessionServiceTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionServiceTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionServiceTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionServiceTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionServiceTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionTenantBandwidth - errors', () => {
      it('should have a postRouterStatsAggregateSessionTenantBandwidth function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionTenantBandwidth === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionTenantBandwidth(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionTenantBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionTenantBandwidth('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionTenantBandwidth', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantBandwidthReceived - errors', () => {
      it('should have a postRouterAggregateSessionTenantBandwidthReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantBandwidthReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantBandwidthReceivedTcp - errors', () => {
      it('should have a postRouterSessionTenantBandwidthReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantBandwidthReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantBandwidthReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantBandwidthReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantBandwidthReceivedUdp - errors', () => {
      it('should have a postRouterSessionTenantBandwidthReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantBandwidthReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantBandwidthReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantBandwidthReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantBandwidthTcp - errors', () => {
      it('should have a postRouterAggregateSessionTenantBandwidthTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantBandwidthTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantBandwidthTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionTenantBandwidthTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantBandwidthTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantBandwidthTransmittedTcp - errors', () => {
      it('should have a postRouterSessionTenantBandwidthTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantBandwidthTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantBandwidthTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantBandwidthTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantBandwidthTransmittedUdp - errors', () => {
      it('should have a postRouterSessionTenantBandwidthTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantBandwidthTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantBandwidthTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantBandwidthTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantBandwidthTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantBandwidthUdp - errors', () => {
      it('should have a postRouterAggregateSessionTenantBandwidthUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantBandwidthUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantBandwidthUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantBandwidthUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAggregateSessionTenantPackets - errors', () => {
      it('should have a postRouterStatsAggregateSessionTenantPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAggregateSessionTenantPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAggregateSessionTenantPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionTenantPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAggregateSessionTenantPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAggregateSessionTenantPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantPacketsReceived - errors', () => {
      it('should have a postRouterAggregateSessionTenantPacketsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantPacketsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantPacketsReceivedTcp - errors', () => {
      it('should have a postRouterSessionTenantPacketsReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantPacketsReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantPacketsReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantPacketsReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantPacketsReceivedUdp - errors', () => {
      it('should have a postRouterSessionTenantPacketsReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantPacketsReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantPacketsReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantPacketsReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantPacketsTcp - errors', () => {
      it('should have a postRouterAggregateSessionTenantPacketsTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantPacketsTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantPacketsTransmitted - errors', () => {
      it('should have a postRouterAggregateSessionTenantPacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantPacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantPacketsTransmittedTcp - errors', () => {
      it('should have a postRouterSessionTenantPacketsTransmittedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantPacketsTransmittedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantPacketsTransmittedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantPacketsTransmittedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsTransmittedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantPacketsTransmittedUdp - errors', () => {
      it('should have a postRouterSessionTenantPacketsTransmittedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantPacketsTransmittedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantPacketsTransmittedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantPacketsTransmittedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantPacketsTransmittedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantPacketsUdp - errors', () => {
      it('should have a postRouterAggregateSessionTenantPacketsUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantPacketsUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantPacketsUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantPacketsUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantArrivalRate - errors', () => {
      it('should have a postRouterSessionTenantArrivalRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantArrivalRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantArrivalRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantArrivalRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantArrivalRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantCount - errors', () => {
      it('should have a postRouterAggregateSessionTenantCount function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantCount === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantCount(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantCount('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantCount', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantDepartureRate - errors', () => {
      it('should have a postRouterSessionTenantDepartureRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantDepartureRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantDepartureRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantDepartureRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantDepartureRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAggregateSessionTenantTcpRetransmissions - errors', () => {
      it('should have a postRouterAggregateSessionTenantTcpRetransmissions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAggregateSessionTenantTcpRetransmissions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAggregateSessionTenantTcpRetransmissions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAggregateSessionTenantTcpRetransmissions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAggregateSessionTenantTcpRetransmissions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantTcpRetransmissionsReceived - errors', () => {
      it('should have a postRouterSessionTenantTcpRetransmissionsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantTcpRetransmissionsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantTcpRetransmissionsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantTcpRetransmissionsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantTcpRetransmissionsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionTenantTcpRetransmissionsTransmitted - errors', () => {
      it('should have a postRouterSessionTenantTcpRetransmissionsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionTenantTcpRetransmissionsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionTenantTcpRetransmissionsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionTenantTcpRetransmissionsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionTenantTcpRetransmissionsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAppIdRefreshes - errors', () => {
      it('should have a postRouterStatsAppIdRefreshes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAppIdRefreshes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAppIdRefreshes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAppIdRefreshes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAppIdRefreshes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAppIdRefreshes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsAppIdRetries - errors', () => {
      it('should have a postRouterStatsAppIdRetries function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsAppIdRetries === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsAppIdRetries(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAppIdRetries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsAppIdRetries('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsAppIdRetries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsApplicationFrameworkInit - errors', () => {
      it('should have a postRouterStatsApplicationFrameworkInit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsApplicationFrameworkInit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkInit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkInit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkInit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkInit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsApplicationFrameworkShutdown - errors', () => {
      it('should have a postRouterStatsApplicationFrameworkShutdown function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsApplicationFrameworkShutdown === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkShutdown(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkShutdown', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkShutdown('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkShutdown', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsApplicationFrameworkStart - errors', () => {
      it('should have a postRouterStatsApplicationFrameworkStart function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsApplicationFrameworkStart === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkStart(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkStart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkStart('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkStart', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsApplicationFrameworkState - errors', () => {
      it('should have a postRouterStatsApplicationFrameworkState function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsApplicationFrameworkState === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkState(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkState', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkState('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkState', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsApplicationFrameworkStop - errors', () => {
      it('should have a postRouterStatsApplicationFrameworkStop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsApplicationFrameworkStop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkStop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkStop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsApplicationFrameworkStop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsApplicationFrameworkStop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpEncapsulationSentFailure - errors', () => {
      it('should have a postRouterArpEncapsulationSentFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpEncapsulationSentFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpEncapsulationSentFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpEncapsulationSentFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpEncapsulationSentFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpEncapsulationSentFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpEncapsulationSentSuccess - errors', () => {
      it('should have a postRouterArpEncapsulationSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpEncapsulationSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpEncapsulationSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpEncapsulationSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpEncapsulationSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpEncapsulationSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpEntries - errors', () => {
      it('should have a postRouterStatsArpEntries function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpEntries === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpEntries(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpEntries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpEntries('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpEntries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpQueuedFailureDrop - errors', () => {
      it('should have a postRouterArpQueuedFailureDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpQueuedFailureDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpQueuedFailureDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpQueuedFailureDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpQueuedFailureDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpQueuedFailureDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpQueuedFailureQueueFull - errors', () => {
      it('should have a postRouterArpQueuedFailureQueueFull function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpQueuedFailureQueueFull === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpQueuedFailureQueueFull(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpQueuedFailureQueueFull', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpQueuedFailureQueueFull('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpQueuedFailureQueueFull', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpQueuedPackets - errors', () => {
      it('should have a postRouterStatsArpQueuedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpQueuedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpQueuedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpQueuedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpQueuedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpQueuedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpReceivedReply - errors', () => {
      it('should have a postRouterStatsArpReceivedReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpReceivedReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpReceivedReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpReceivedReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpReceivedRequest - errors', () => {
      it('should have a postRouterStatsArpReceivedRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpReceivedRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpReceivedRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpReceivedRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsReply - errors', () => {
      it('should have a postRouterArpReceivedErrorsReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsRequest - errors', () => {
      it('should have a postRouterArpReceivedErrorsRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsNeighborAdvertisement - errors', () => {
      it('should have a postRouterArpReceivedErrorsNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsNeighborSolicit - errors', () => {
      it('should have a postRouterArpReceivedErrorsNeighborSolicit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsNeighborSolicit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsNeighborSolicit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsNeighborSolicit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsProcessing - errors', () => {
      it('should have a postRouterArpReceivedErrorsProcessing function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsProcessing === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsProcessing(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsProcessing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsProcessing('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsProcessing', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedErrorsUnknownType - errors', () => {
      it('should have a postRouterArpReceivedErrorsUnknownType function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedErrorsUnknownType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedErrorsUnknownType(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsUnknownType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedErrorsUnknownType('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedErrorsUnknownType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpReceivedGratuitous - errors', () => {
      it('should have a postRouterStatsArpReceivedGratuitous function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpReceivedGratuitous === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpReceivedGratuitous(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpReceivedGratuitous('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpReceivedNeighborAdvertisement - errors', () => {
      it('should have a postRouterStatsArpReceivedNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpReceivedNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpReceivedNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpReceivedNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsArpReceivedNeighborSolicit - errors', () => {
      it('should have a postRouterStatsArpReceivedNeighborSolicit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsArpReceivedNeighborSolicit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsArpReceivedNeighborSolicit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsArpReceivedNeighborSolicit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsArpReceivedNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedOffSubnetArpRequest - errors', () => {
      it('should have a postRouterReceivedOffSubnetArpRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedOffSubnetArpRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedOffSubnetArpRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedOffSubnetArpRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedOffSubnetArpRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedOffSubnetArpRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedOffSubnetNeighborSolicit - errors', () => {
      it('should have a postRouterReceivedOffSubnetNeighborSolicit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedOffSubnetNeighborSolicit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedOffSubnetNeighborSolicit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedOffSubnetNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedOffSubnetNeighborSolicit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedOffSubnetNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpReceivedUnsolicitedNeighborAdvertisement - errors', () => {
      it('should have a postRouterArpReceivedUnsolicitedNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpReceivedUnsolicitedNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpReceivedUnsolicitedNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpReceivedUnsolicitedNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpReceivedUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureAllocation - errors', () => {
      it('should have a postRouterArpSentFailureAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureReply - errors', () => {
      it('should have a postRouterArpSentFailureReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureRequest - errors', () => {
      it('should have a postRouterArpSentFailureRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureGratuitous - errors', () => {
      it('should have a postRouterArpSentFailureGratuitous function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureGratuitous === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureGratuitous(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureGratuitous('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentFailureMulticastListenerReport - errors', () => {
      it('should have a postRouterSentFailureMulticastListenerReport function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentFailureMulticastListenerReport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentFailureMulticastListenerReport(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureMulticastListenerReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentFailureMulticastListenerReport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureMulticastListenerReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureNeighborAdvertisement - errors', () => {
      it('should have a postRouterArpSentFailureNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureNeighborSolicit - errors', () => {
      it('should have a postRouterArpSentFailureNeighborSolicit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureNeighborSolicit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureNeighborSolicit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureNeighborSolicit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentFailureStandby - errors', () => {
      it('should have a postRouterArpSentFailureStandby function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentFailureStandby === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentFailureStandby(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentFailureStandby('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentFailureStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentFailureUnsolicitedNeighborAdvertisement - errors', () => {
      it('should have a postRouterSentFailureUnsolicitedNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentFailureUnsolicitedNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentFailureUnsolicitedNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentFailureUnsolicitedNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentSuccessReply - errors', () => {
      it('should have a postRouterArpSentSuccessReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentSuccessReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentSuccessReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentSuccessReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentSuccessRequest - errors', () => {
      it('should have a postRouterArpSentSuccessRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentSuccessRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentSuccessRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentSuccessRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentSuccessGratuitous - errors', () => {
      it('should have a postRouterArpSentSuccessGratuitous function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentSuccessGratuitous === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentSuccessGratuitous(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentSuccessGratuitous('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessGratuitous', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentSuccessInSubnetProxy - errors', () => {
      it('should have a postRouterSentSuccessInSubnetProxy function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentSuccessInSubnetProxy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentSuccessInSubnetProxy(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessInSubnetProxy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentSuccessInSubnetProxy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessInSubnetProxy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentSuccessMulticastListenerReport - errors', () => {
      it('should have a postRouterSentSuccessMulticastListenerReport function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentSuccessMulticastListenerReport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentSuccessMulticastListenerReport(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessMulticastListenerReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentSuccessMulticastListenerReport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessMulticastListenerReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentSuccessNeighborAdvertisement - errors', () => {
      it('should have a postRouterArpSentSuccessNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentSuccessNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentSuccessNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentSuccessNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterArpSentSuccessNeighborSolicit - errors', () => {
      it('should have a postRouterArpSentSuccessNeighborSolicit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterArpSentSuccessNeighborSolicit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterArpSentSuccessNeighborSolicit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterArpSentSuccessNeighborSolicit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterArpSentSuccessNeighborSolicit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentSuccessOffSubnetProxy - errors', () => {
      it('should have a postRouterSentSuccessOffSubnetProxy function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentSuccessOffSubnetProxy === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentSuccessOffSubnetProxy(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessOffSubnetProxy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentSuccessOffSubnetProxy('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessOffSubnetProxy', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentSuccessUnsolicitedNeighborAdvertisement - errors', () => {
      it('should have a postRouterSentSuccessUnsolicitedNeighborAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentSuccessUnsolicitedNeighborAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentSuccessUnsolicitedNeighborAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentSuccessUnsolicitedNeighborAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentSuccessUnsolicitedNeighborAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdAsyncReceivedMiss - errors', () => {
      it('should have a postRouterBfdAsyncReceivedMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdAsyncReceivedMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdAsyncReceivedMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncReceivedMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdAsyncReceivedMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncReceivedMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdAsyncReceivedSuccess - errors', () => {
      it('should have a postRouterBfdAsyncReceivedSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdAsyncReceivedSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdAsyncReceivedSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdAsyncReceivedSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdAsyncSentArpFailure - errors', () => {
      it('should have a postRouterBfdAsyncSentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdAsyncSentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdAsyncSentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdAsyncSentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAsyncSentBufferAllocationFailure - errors', () => {
      it('should have a postRouterAsyncSentBufferAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAsyncSentBufferAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAsyncSentBufferAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAsyncSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAsyncSentBufferAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAsyncSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdAsyncSentSuccess - errors', () => {
      it('should have a postRouterBfdAsyncSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdAsyncSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdAsyncSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdAsyncSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdAsyncSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdEchoReceived - errors', () => {
      it('should have a postRouterStatsBfdEchoReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdEchoReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdEchoReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdEchoReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdEchoReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdEchoReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdEchoSentArpFailure - errors', () => {
      it('should have a postRouterBfdEchoSentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdEchoSentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdEchoSentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdEchoSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdEchoSentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdEchoSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEchoSentBufferAllocationFailure - errors', () => {
      it('should have a postRouterEchoSentBufferAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEchoSentBufferAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEchoSentBufferAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEchoSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEchoSentBufferAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEchoSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdEchoSentSuccess - errors', () => {
      it('should have a postRouterBfdEchoSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdEchoSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdEchoSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdEchoSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdEchoSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdEchoSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLinkDownLocalOper - errors', () => {
      it('should have a postRouterLinkDownLocalOper function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLinkDownLocalOper === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLinkDownLocalOper(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownLocalOper', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLinkDownLocalOper('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownLocalOper', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLinkDownRemoteAdmin - errors', () => {
      it('should have a postRouterLinkDownRemoteAdmin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLinkDownRemoteAdmin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLinkDownRemoteAdmin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownRemoteAdmin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLinkDownRemoteAdmin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownRemoteAdmin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdLinkDownRemote - errors', () => {
      it('should have a postRouterBfdLinkDownRemote function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdLinkDownRemote === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdLinkDownRemote(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLinkDownRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdLinkDownRemote('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLinkDownRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdLinkDownTimerExpiry - errors', () => {
      it('should have a postRouterBfdLinkDownTimerExpiry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdLinkDownTimerExpiry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdLinkDownTimerExpiry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLinkDownTimerExpiry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdLinkDownTimerExpiry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLinkDownTimerExpiry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdLinkUp - errors', () => {
      it('should have a postRouterStatsBfdLinkUp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdLinkUp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdLinkUp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdLinkUp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdLinkUp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdLinkUp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdLocalSourceNatChange - errors', () => {
      it('should have a postRouterBfdLocalSourceNatChange function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdLocalSourceNatChange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdLocalSourceNatChange(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLocalSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdLocalSourceNatChange('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdLocalSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdNeighborFailover - errors', () => {
      it('should have a postRouterStatsBfdNeighborFailover function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdNeighborFailover === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdNeighborFailover(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdNeighborFailover', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdNeighborFailover('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdNeighborFailover', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdNeighborSourceNatChange - errors', () => {
      it('should have a postRouterBfdNeighborSourceNatChange function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdNeighborSourceNatChange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdNeighborSourceNatChange(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdNeighborSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdNeighborSourceNatChange('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdNeighborSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerPathAsyncReceivedMiss - errors', () => {
      it('should have a postRouterPeerPathAsyncReceivedMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerPathAsyncReceivedMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerPathAsyncReceivedMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncReceivedMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerPathAsyncReceivedMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncReceivedMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerPathAsyncReceivedSuccess - errors', () => {
      it('should have a postRouterPeerPathAsyncReceivedSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerPathAsyncReceivedSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerPathAsyncReceivedSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerPathAsyncReceivedSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathAsyncSentArpFailure - errors', () => {
      it('should have a postRouterPathAsyncSentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathAsyncSentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathAsyncSentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathAsyncSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathAsyncSentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathAsyncSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAsyncSentBufferAllocationFailure2 - errors', () => {
      it('should have a postRouterAsyncSentBufferAllocationFailure2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAsyncSentBufferAllocationFailure2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAsyncSentBufferAllocationFailure2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAsyncSentBufferAllocationFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAsyncSentBufferAllocationFailure2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAsyncSentBufferAllocationFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerPathAsyncSentSuccess - errors', () => {
      it('should have a postRouterPeerPathAsyncSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerPathAsyncSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerPathAsyncSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerPathAsyncSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathAsyncSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdPeerPathEchoReceived - errors', () => {
      it('should have a postRouterBfdPeerPathEchoReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdPeerPathEchoReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdPeerPathEchoReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathEchoReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdPeerPathEchoReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathEchoReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathEchoSentArpFailure - errors', () => {
      it('should have a postRouterPathEchoSentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathEchoSentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathEchoSentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathEchoSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathEchoSentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathEchoSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEchoSentBufferAllocationFailure2 - errors', () => {
      it('should have a postRouterEchoSentBufferAllocationFailure2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEchoSentBufferAllocationFailure2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEchoSentBufferAllocationFailure2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEchoSentBufferAllocationFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEchoSentBufferAllocationFailure2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEchoSentBufferAllocationFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerPathEchoSentSuccess - errors', () => {
      it('should have a postRouterPeerPathEchoSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerPathEchoSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerPathEchoSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathEchoSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerPathEchoSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathEchoSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdPeerPathJitter - errors', () => {
      it('should have a postRouterStatsBfdPeerPathJitter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdPeerPathJitter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdPeerPathJitter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathJitter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdPeerPathJitter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathJitter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdPeerPathLatency - errors', () => {
      it('should have a postRouterStatsBfdPeerPathLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdPeerPathLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdPeerPathLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdPeerPathLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLinkDownLocalOper2 - errors', () => {
      it('should have a postRouterLinkDownLocalOper2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLinkDownLocalOper2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLinkDownLocalOper2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownLocalOper2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLinkDownLocalOper2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownLocalOper2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLinkDownRemoteAdmin2 - errors', () => {
      it('should have a postRouterLinkDownRemoteAdmin2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLinkDownRemoteAdmin2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLinkDownRemoteAdmin2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownRemoteAdmin2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLinkDownRemoteAdmin2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLinkDownRemoteAdmin2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathLinkDownRemote - errors', () => {
      it('should have a postRouterPathLinkDownRemote function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathLinkDownRemote === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathLinkDownRemote(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLinkDownRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathLinkDownRemote('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLinkDownRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathLinkDownTimerExpiry - errors', () => {
      it('should have a postRouterPathLinkDownTimerExpiry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathLinkDownTimerExpiry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathLinkDownTimerExpiry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLinkDownTimerExpiry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathLinkDownTimerExpiry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLinkDownTimerExpiry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdPeerPathLinkUp - errors', () => {
      it('should have a postRouterBfdPeerPathLinkUp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdPeerPathLinkUp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdPeerPathLinkUp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathLinkUp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdPeerPathLinkUp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathLinkUp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathLocalSourceNatChange - errors', () => {
      it('should have a postRouterPathLocalSourceNatChange function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathLocalSourceNatChange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathLocalSourceNatChange(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLocalSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathLocalSourceNatChange('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathLocalSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdPeerPathLoss - errors', () => {
      it('should have a postRouterStatsBfdPeerPathLoss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdPeerPathLoss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdPeerPathLoss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathLoss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdPeerPathLoss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathLoss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdPeerPathMos - errors', () => {
      it('should have a postRouterStatsBfdPeerPathMos function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdPeerPathMos === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdPeerPathMos(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathMos', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdPeerPathMos('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdPeerPathMos', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdPeerPathNeighborFailover - errors', () => {
      it('should have a postRouterBfdPeerPathNeighborFailover function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdPeerPathNeighborFailover === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdPeerPathNeighborFailover(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathNeighborFailover', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdPeerPathNeighborFailover('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdPeerPathNeighborFailover', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathNeighborSourceNatChange - errors', () => {
      it('should have a postRouterPathNeighborSourceNatChange function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathNeighborSourceNatChange === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathNeighborSourceNatChange(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathNeighborSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathNeighborSourceNatChange('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathNeighborSourceNatChange', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidAuthenticationField - errors', () => {
      it('should have a postRouterBfdReceivedInvalidAuthenticationField function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidAuthenticationField === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidAuthenticationField(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidAuthenticationField', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidAuthenticationField('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidAuthenticationField', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidDetectMultiple - errors', () => {
      it('should have a postRouterBfdReceivedInvalidDetectMultiple function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidDetectMultiple === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidDetectMultiple(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidDetectMultiple', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidDetectMultiple('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidDetectMultiple', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdReceivedInvalidDiscriminator - errors', () => {
      it('should have a postRouterStatsBfdReceivedInvalidDiscriminator function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdReceivedInvalidDiscriminator === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidDiscriminator(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidDiscriminator', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidDiscriminator('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidDiscriminator', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidHeaderLarge - errors', () => {
      it('should have a postRouterBfdReceivedInvalidHeaderLarge function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidHeaderLarge === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderLarge(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderLarge', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderLarge('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderLarge', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidHeaderSmall - errors', () => {
      it('should have a postRouterBfdReceivedInvalidHeaderSmall function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidHeaderSmall === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderSmall(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderSmall', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderSmall('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderSmall', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidHeaderVersion - errors', () => {
      it('should have a postRouterBfdReceivedInvalidHeaderVersion function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidHeaderVersion === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderVersion(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidHeaderVersion('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidHeaderVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidLocalDiscriminator - errors', () => {
      it('should have a postRouterBfdReceivedInvalidLocalDiscriminator function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidLocalDiscriminator === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidLocalDiscriminator(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidLocalDiscriminator', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidLocalDiscriminator('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidLocalDiscriminator', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdReceivedInvalidMetadata - errors', () => {
      it('should have a postRouterStatsBfdReceivedInvalidMetadata function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdReceivedInvalidMetadata === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidMetadata(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidMetadata('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidPacketLength - errors', () => {
      it('should have a postRouterBfdReceivedInvalidPacketLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidPacketLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidPacketLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidPacketLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterBfdReceivedInvalidPayloadSmall - errors', () => {
      it('should have a postRouterBfdReceivedInvalidPayloadSmall function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterBfdReceivedInvalidPayloadSmall === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterBfdReceivedInvalidPayloadSmall(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidPayloadSmall', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterBfdReceivedInvalidPayloadSmall('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterBfdReceivedInvalidPayloadSmall', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsBfdReceivedInvalidSession - errors', () => {
      it('should have a postRouterStatsBfdReceivedInvalidSession function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsBfdReceivedInvalidSession === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidSession(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsBfdReceivedInvalidSession('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsBfdReceivedInvalidSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsConfigCacheHits - errors', () => {
      it('should have a postRouterStatsConfigCacheHits function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsConfigCacheHits === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsConfigCacheHits(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfigCacheHits', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsConfigCacheHits('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfigCacheHits', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsConfigCacheMisses - errors', () => {
      it('should have a postRouterStatsConfigCacheMisses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsConfigCacheMisses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsConfigCacheMisses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfigCacheMisses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsConfigCacheMisses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfigCacheMisses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsConfluxBatchesLatency - errors', () => {
      it('should have a postRouterStatsConfluxBatchesLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsConfluxBatchesLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsConfluxBatchesLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxBatchesLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsConfluxBatchesLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxBatchesLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxBatchesWriteDuration - errors', () => {
      it('should have a postRouterConfluxBatchesWriteDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxBatchesWriteDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxBatchesWriteDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxBatchesWriteDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxBatchesWriteFailure - errors', () => {
      it('should have a postRouterConfluxBatchesWriteFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxBatchesWriteFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxBatchesWriteFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxBatchesWriteFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxBatchesWriteSuccess - errors', () => {
      it('should have a postRouterConfluxBatchesWriteSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxBatchesWriteSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxBatchesWriteSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxBatchesWriteSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxBatchesWriteSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsConfluxGapsDetected - errors', () => {
      it('should have a postRouterStatsConfluxGapsDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsConfluxGapsDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsConfluxGapsDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxGapsDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsConfluxGapsDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxGapsDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxGapsResolutionRetry - errors', () => {
      it('should have a postRouterConfluxGapsResolutionRetry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxGapsResolutionRetry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxGapsResolutionRetry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxGapsResolutionRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxGapsResolutionRetry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxGapsResolutionRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxGapsResolutionSuccess - errors', () => {
      it('should have a postRouterConfluxGapsResolutionSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxGapsResolutionSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxGapsResolutionSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxGapsResolutionSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxGapsResolutionSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxGapsResolutionSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsConfluxMessagesRead - errors', () => {
      it('should have a postRouterStatsConfluxMessagesRead function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsConfluxMessagesRead === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsConfluxMessagesRead(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxMessagesRead', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsConfluxMessagesRead('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsConfluxMessagesRead', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxMessagesWriteFailure - errors', () => {
      it('should have a postRouterConfluxMessagesWriteFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxMessagesWriteFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxMessagesWriteFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxMessagesWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxMessagesWriteFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxMessagesWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterConfluxMessagesWriteSuccess - errors', () => {
      it('should have a postRouterConfluxMessagesWriteSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterConfluxMessagesWriteSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterConfluxMessagesWriteSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxMessagesWriteSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterConfluxMessagesWriteSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterConfluxMessagesWriteSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsCpuUtilization - errors', () => {
      it('should have a postRouterStatsCpuUtilization function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsCpuUtilization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsCpuUtilization(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsCpuUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsCpuUtilization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsCpuUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDatabaseMemoryConsumed - errors', () => {
      it('should have a postRouterStatsDatabaseMemoryConsumed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDatabaseMemoryConsumed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDatabaseMemoryConsumed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDatabaseMemoryConsumed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDatabaseMemoryConsumed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDatabaseMemoryConsumed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDeviceInterfaceMessageFailure - errors', () => {
      it('should have a postRouterStatsDeviceInterfaceMessageFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDeviceInterfaceMessageFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDeviceInterfaceMessageFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDeviceInterfaceMessageFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDeviceInterfaceMessageFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDeviceInterfaceMessageFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDeviceInterfaceMessageSuccess - errors', () => {
      it('should have a postRouterStatsDeviceInterfaceMessageSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDeviceInterfaceMessageSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDeviceInterfaceMessageSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDeviceInterfaceMessageSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDeviceInterfaceMessageSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDeviceInterfaceMessageSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDiskCapacity - errors', () => {
      it('should have a postRouterStatsDiskCapacity function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDiskCapacity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDiskCapacity(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskCapacity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDiskCapacity('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskCapacity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDiskComponent - errors', () => {
      it('should have a postRouterStatsDiskComponent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDiskComponent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDiskComponent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskComponent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDiskComponent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskComponent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsDiskUsed - errors', () => {
      it('should have a postRouterStatsDiskUsed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsDiskUsed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsDiskUsed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskUsed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsDiskUsed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsDiskUsed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiParsedFailureTls - errors', () => {
      it('should have a postRouterDpiParsedFailureTls function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiParsedFailureTls === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiParsedFailureTls(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiParsedFailureTls', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiParsedFailureTls('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiParsedFailureTls', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiParsedSuccessTls - errors', () => {
      it('should have a postRouterDpiParsedSuccessTls function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiParsedSuccessTls === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiParsedSuccessTls(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiParsedSuccessTls', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiParsedSuccessTls('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiParsedSuccessTls', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiReceivedFailureCapacityExceeded - errors', () => {
      it('should have a postRouterDpiReceivedFailureCapacityExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiReceivedFailureCapacityExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiReceivedFailureCapacityExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedFailureCapacityExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiReceivedFailureCapacityExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedFailureCapacityExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedFailureTooManySegments - errors', () => {
      it('should have a postRouterReceivedFailureTooManySegments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedFailureTooManySegments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedFailureTooManySegments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedFailureTooManySegments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedFailureTooManySegments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedFailureTooManySegments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiReceivedSuccessAdd - errors', () => {
      it('should have a postRouterDpiReceivedSuccessAdd function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiReceivedSuccessAdd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiReceivedSuccessAdd(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedSuccessAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiReceivedSuccessAdd('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedSuccessAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiReceivedSuccessDuplicate - errors', () => {
      it('should have a postRouterDpiReceivedSuccessDuplicate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiReceivedSuccessDuplicate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiReceivedSuccessDuplicate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedSuccessDuplicate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiReceivedSuccessDuplicate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiReceivedSuccessDuplicate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedSuccessReadyToParse - errors', () => {
      it('should have a postRouterReceivedSuccessReadyToParse function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedSuccessReadyToParse === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedSuccessReadyToParse(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSuccessReadyToParse', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedSuccessReadyToParse('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSuccessReadyToParse', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateConnectionFirstConductorConnect - errors', () => {
      it('should have a postRouterUpdateConnectionFirstConductorConnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateConnectionFirstConductorConnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateConnectionFirstConductorConnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionFirstConductorConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateConnectionFirstConductorConnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionFirstConductorConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateConnectionFirstServerConnect - errors', () => {
      it('should have a postRouterUpdateConnectionFirstServerConnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateConnectionFirstServerConnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateConnectionFirstServerConnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionFirstServerConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateConnectionFirstServerConnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionFirstServerConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateConnectionLastConductorDisconnect - errors', () => {
      it('should have a postRouterUpdateConnectionLastConductorDisconnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateConnectionLastConductorDisconnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateConnectionLastConductorDisconnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionLastConductorDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateConnectionLastConductorDisconnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateConnectionLastConductorDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDynamicPeerUpdateLeadershipLost - errors', () => {
      it('should have a postRouterDynamicPeerUpdateLeadershipLost function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDynamicPeerUpdateLeadershipLost === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDynamicPeerUpdateLeadershipLost(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDynamicPeerUpdateLeadershipLost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDynamicPeerUpdateLeadershipLost('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDynamicPeerUpdateLeadershipLost', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDynamicPeerUpdateLeadershipWon - errors', () => {
      it('should have a postRouterDynamicPeerUpdateLeadershipWon function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDynamicPeerUpdateLeadershipWon === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDynamicPeerUpdateLeadershipWon(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDynamicPeerUpdateLeadershipWon', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDynamicPeerUpdateLeadershipWon('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDynamicPeerUpdateLeadershipWon', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerUpdateRequestsReceivedPush - errors', () => {
      it('should have a postRouterPeerUpdateRequestsReceivedPush function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerUpdateRequestsReceivedPush === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerUpdateRequestsReceivedPush(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsReceivedPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerUpdateRequestsReceivedPush('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsReceivedPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerUpdateRequestsReceivedSync - errors', () => {
      it('should have a postRouterPeerUpdateRequestsReceivedSync function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerUpdateRequestsReceivedSync === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerUpdateRequestsReceivedSync(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsReceivedSync', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerUpdateRequestsReceivedSync('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsReceivedSync', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRequestsReceivedSyncPeerAddresses - errors', () => {
      it('should have a postRouterRequestsReceivedSyncPeerAddresses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRequestsReceivedSyncPeerAddresses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRequestsReceivedSyncPeerAddresses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRequestsReceivedSyncPeerAddresses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRequestsReceivedSyncPeerAddresses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRequestsReceivedSyncPeerAddresses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerUpdateRequestsSentPush - errors', () => {
      it('should have a postRouterPeerUpdateRequestsSentPush function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerUpdateRequestsSentPush === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerUpdateRequestsSentPush(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsSentPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerUpdateRequestsSentPush('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsSentPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerUpdateRequestsSentSync - errors', () => {
      it('should have a postRouterPeerUpdateRequestsSentSync function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerUpdateRequestsSentSync === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerUpdateRequestsSentSync(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsSentSync', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerUpdateRequestsSentSync('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateRequestsSentSync', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateResponsesReceivedNotFound - errors', () => {
      it('should have a postRouterUpdateResponsesReceivedNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateResponsesReceivedNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateResponsesReceivedOtherFailure - errors', () => {
      it('should have a postRouterUpdateResponsesReceivedOtherFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateResponsesReceivedOtherFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedOtherFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedOtherFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedOtherFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedOtherFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerUpdateResponsesReceivedSuccess - errors', () => {
      it('should have a postRouterPeerUpdateResponsesReceivedSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerUpdateResponsesReceivedSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerUpdateResponsesReceivedSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateResponsesReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerUpdateResponsesReceivedSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerUpdateResponsesReceivedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateResponsesReceivedTimeOut - errors', () => {
      it('should have a postRouterUpdateResponsesReceivedTimeOut function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateResponsesReceivedTimeOut === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedTimeOut(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedTimeOut', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateResponsesReceivedTimeOut('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateResponsesReceivedTimeOut', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentChannelMessagesReceived - errors', () => {
      it('should have a postRouterProtocolsAgentChannelMessagesReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentChannelMessagesReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentChannelMessagesReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentChannelMessagesReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentChannelMessagesReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentChannelMessagesReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentChannelMessagesSent - errors', () => {
      it('should have a postRouterProtocolsAgentChannelMessagesSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentChannelMessagesSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentChannelMessagesSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentChannelMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentChannelMessagesSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentChannelMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterErrorsBgpOverSvrDrops - errors', () => {
      it('should have a postRouterErrorsBgpOverSvrDrops function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterErrorsBgpOverSvrDrops === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterErrorsBgpOverSvrDrops(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsBgpOverSvrDrops', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterErrorsBgpOverSvrDrops('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsBgpOverSvrDrops', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsClassifyDrop - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsClassifyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsClassifyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsClassifyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsClassifyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsClassifyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsClassifyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsEarlyInbound - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsEarlyInbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsEarlyInbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsEarlyInbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsEarlyInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsEarlyInbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsEarlyInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsEarlyOutbound - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsEarlyOutbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsEarlyOutbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsEarlyOutbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsEarlyOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsEarlyOutbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsEarlyOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterErrorsFastlaneNotReadyDrop - errors', () => {
      it('should have a postRouterErrorsFastlaneNotReadyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterErrorsFastlaneNotReadyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterErrorsFastlaneNotReadyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsFastlaneNotReadyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterErrorsFastlaneNotReadyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsFastlaneNotReadyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterErrorsGlobalInterfaceLookupDrop - errors', () => {
      it('should have a postRouterErrorsGlobalInterfaceLookupDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterErrorsGlobalInterfaceLookupDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterErrorsGlobalInterfaceLookupDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsGlobalInterfaceLookupDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterErrorsGlobalInterfaceLookupDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsGlobalInterfaceLookupDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAgentErrorsIllegalEthernetDrop - errors', () => {
      it('should have a postRouterAgentErrorsIllegalEthernetDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAgentErrorsIllegalEthernetDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAgentErrorsIllegalEthernetDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentErrorsIllegalEthernetDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAgentErrorsIllegalEthernetDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentErrorsIllegalEthernetDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIllegalExternalProtocolsChannelMessage - errors', () => {
      it('should have a postRouterIllegalExternalProtocolsChannelMessage function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIllegalExternalProtocolsChannelMessage === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIllegalExternalProtocolsChannelMessage(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIllegalExternalProtocolsChannelMessage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIllegalExternalProtocolsChannelMessage('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIllegalExternalProtocolsChannelMessage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAgentErrorsIllegalIpDrop - errors', () => {
      it('should have a postRouterAgentErrorsIllegalIpDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAgentErrorsIllegalIpDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAgentErrorsIllegalIpDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentErrorsIllegalIpDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAgentErrorsIllegalIpDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentErrorsIllegalIpDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsInboundDrop - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsInboundDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsInboundDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsInboundDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsInboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsInboundDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsInboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInboundFromInternalInterfaceDrop - errors', () => {
      it('should have a postRouterInboundFromInternalInterfaceDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInboundFromInternalInterfaceDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInboundFromInternalInterfaceDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInboundFromInternalInterfaceDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInboundFromInternalInterfaceDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInboundFromInternalInterfaceDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsOutboundDrop - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsOutboundDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsOutboundDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsOutboundDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsOutboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsOutboundDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsOutboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterOutboundL2ResolutionReplyDrop - errors', () => {
      it('should have a postRouterOutboundL2ResolutionReplyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterOutboundL2ResolutionReplyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterOutboundL2ResolutionReplyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterOutboundL2ResolutionReplyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterOutboundL2ResolutionReplyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterOutboundL2ResolutionReplyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsAgentErrorsSessionCollision - errors', () => {
      it('should have a postRouterProtocolsAgentErrorsSessionCollision function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsAgentErrorsSessionCollision === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsSessionCollision(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsSessionCollision', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsAgentErrorsSessionCollision('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsAgentErrorsSessionCollision', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterExternalProtocolsAgentPacketsInbound - errors', () => {
      it('should have a postRouterExternalProtocolsAgentPacketsInbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterExternalProtocolsAgentPacketsInbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterExternalProtocolsAgentPacketsInbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsAgentPacketsInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterExternalProtocolsAgentPacketsInbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsAgentPacketsInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterExternalProtocolsAgentPacketsOutbound - errors', () => {
      it('should have a postRouterExternalProtocolsAgentPacketsOutbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterExternalProtocolsAgentPacketsOutbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterExternalProtocolsAgentPacketsOutbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsAgentPacketsOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterExternalProtocolsAgentPacketsOutbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsAgentPacketsOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAgentPacketsSecureVectorRouted - errors', () => {
      it('should have a postRouterAgentPacketsSecureVectorRouted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAgentPacketsSecureVectorRouted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAgentPacketsSecureVectorRouted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentPacketsSecureVectorRouted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAgentPacketsSecureVectorRouted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAgentPacketsSecureVectorRouted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerChannelMessagesReceived - errors', () => {
      it('should have a postRouterProtocolsManagerChannelMessagesReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerChannelMessagesReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerChannelMessagesReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerChannelMessagesReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerChannelMessagesReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerChannelMessagesReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerChannelMessagesSent - errors', () => {
      it('should have a postRouterProtocolsManagerChannelMessagesSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerChannelMessagesSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerChannelMessagesSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerChannelMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerChannelMessagesSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerChannelMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerErrorsEarlyInbound - errors', () => {
      it('should have a postRouterProtocolsManagerErrorsEarlyInbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerErrorsEarlyInbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsEarlyInbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsEarlyInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsEarlyInbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsEarlyInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerErrorsEarlyOutbound - errors', () => {
      it('should have a postRouterProtocolsManagerErrorsEarlyOutbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerErrorsEarlyOutbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsEarlyOutbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsEarlyOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsEarlyOutbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsEarlyOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIllegalExternalProtocolsChannelMessage2 - errors', () => {
      it('should have a postRouterIllegalExternalProtocolsChannelMessage2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIllegalExternalProtocolsChannelMessage2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIllegalExternalProtocolsChannelMessage2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIllegalExternalProtocolsChannelMessage2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIllegalExternalProtocolsChannelMessage2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIllegalExternalProtocolsChannelMessage2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerErrorsInboundDrop - errors', () => {
      it('should have a postRouterProtocolsManagerErrorsInboundDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerErrorsInboundDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsInboundDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsInboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsInboundDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsInboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProtocolsManagerErrorsOutboundDrop - errors', () => {
      it('should have a postRouterProtocolsManagerErrorsOutboundDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProtocolsManagerErrorsOutboundDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsOutboundDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsOutboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProtocolsManagerErrorsOutboundDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProtocolsManagerErrorsOutboundDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterExternalProtocolsManagerPacketsInbound - errors', () => {
      it('should have a postRouterExternalProtocolsManagerPacketsInbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterExternalProtocolsManagerPacketsInbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterExternalProtocolsManagerPacketsInbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsManagerPacketsInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterExternalProtocolsManagerPacketsInbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsManagerPacketsInbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterExternalProtocolsManagerPacketsOutbound - errors', () => {
      it('should have a postRouterExternalProtocolsManagerPacketsOutbound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterExternalProtocolsManagerPacketsOutbound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterExternalProtocolsManagerPacketsOutbound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsManagerPacketsOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterExternalProtocolsManagerPacketsOutbound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExternalProtocolsManagerPacketsOutbound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayActionEventSendSuccess - errors', () => {
      it('should have a postRouterHighwayActionEventSendSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayActionEventSendSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayActionEventSendSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayActionEventSendSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayActionEventSendSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayActionEventSendSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayActionEventSendTimeout - errors', () => {
      it('should have a postRouterHighwayActionEventSendTimeout function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayActionEventSendTimeout === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayActionEventSendTimeout(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayActionEventSendTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayActionEventSendTimeout('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayActionEventSendTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsHighwayActionsActionMeter - errors', () => {
      it('should have a postRouterStatsHighwayActionsActionMeter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsHighwayActionsActionMeter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsHighwayActionsActionMeter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsHighwayActionsActionMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsHighwayActionsActionMeter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsHighwayActionsActionMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityIcmpEstablished - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityIcmpEstablished function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityIcmpEstablished === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityIcmpEstablished(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityIcmpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityIcmpEstablished('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityIcmpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpTimeToEstablishmentMax - errors', () => {
      it('should have a postRouterIcmpTimeToEstablishmentMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpTimeToEstablishmentMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpTimeToEstablishmentMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpTimeToEstablishmentMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpTimeToEstablishmentMin - errors', () => {
      it('should have a postRouterIcmpTimeToEstablishmentMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpTimeToEstablishmentMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpTimeToEstablishmentMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpTimeToEstablishmentMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityIcmpTimeoutBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityIcmpTimeoutBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityIcmpTimeoutBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityIcmpTimeoutBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityIcmpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityIcmpTimeoutBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityIcmpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityIcmpUnreachable - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityIcmpUnreachable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityIcmpUnreachable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityIcmpUnreachable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityIcmpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityIcmpUnreachable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityIcmpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityTcpCloseBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityTcpCloseBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityTcpCloseBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityTcpCloseBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTcpCloseBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityTcpCloseBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTcpCloseBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityTcpEstablished - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityTcpEstablished function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityTcpEstablished === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTcpEstablished(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTcpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTcpEstablished('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTcpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpTimeToEstablishmentMax - errors', () => {
      it('should have a postRouterTcpTimeToEstablishmentMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpTimeToEstablishmentMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpTimeToEstablishmentMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpTimeToEstablishmentMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpTimeToEstablishmentMin - errors', () => {
      it('should have a postRouterTcpTimeToEstablishmentMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpTimeToEstablishmentMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpTimeToEstablishmentMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpTimeToEstablishmentMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityTcpTimeoutBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityTcpTimeoutBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityTcpTimeoutBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityTcpTimeoutBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTcpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityTcpTimeoutBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTcpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityTcpUnreachable - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityTcpUnreachable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityTcpUnreachable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTcpUnreachable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTcpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTcpUnreachable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTcpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityTlsCloseBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityTlsCloseBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityTlsCloseBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityTlsCloseBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTlsCloseBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityTlsCloseBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTlsCloseBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityTlsEstablished - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityTlsEstablished function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityTlsEstablished === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTlsEstablished(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTlsEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityTlsEstablished('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityTlsEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTlsTimeToEstablishmentMax - errors', () => {
      it('should have a postRouterTlsTimeToEstablishmentMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTlsTimeToEstablishmentMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTlsTimeToEstablishmentMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTlsTimeToEstablishmentMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTlsTimeToEstablishmentMin - errors', () => {
      it('should have a postRouterTlsTimeToEstablishmentMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTlsTimeToEstablishmentMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTlsTimeToEstablishmentMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTlsTimeToEstablishmentMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityTlsTimeoutBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityTlsTimeoutBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityTlsTimeoutBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityTlsTimeoutBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTlsTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityTlsTimeoutBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityTlsTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityUdpEstablished - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityUdpEstablished function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityUdpEstablished === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityUdpEstablished(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityUdpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityUdpEstablished('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityUdpEstablished', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUdpTimeToEstablishmentMax - errors', () => {
      it('should have a postRouterUdpTimeToEstablishmentMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUdpTimeToEstablishmentMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUdpTimeToEstablishmentMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUdpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUdpTimeToEstablishmentMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUdpTimeToEstablishmentMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUdpTimeToEstablishmentMin - errors', () => {
      it('should have a postRouterUdpTimeToEstablishmentMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUdpTimeToEstablishmentMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUdpTimeToEstablishmentMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUdpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUdpTimeToEstablishmentMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUdpTimeToEstablishmentMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReachabilityUdpTimeoutBeforeEstablishment - errors', () => {
      it('should have a postRouterReachabilityUdpTimeoutBeforeEstablishment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReachabilityUdpTimeoutBeforeEstablishment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReachabilityUdpTimeoutBeforeEstablishment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityUdpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReachabilityUdpTimeoutBeforeEstablishment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReachabilityUdpTimeoutBeforeEstablishment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayDestinationReachabilityUdpUnreachable - errors', () => {
      it('should have a postRouterHighwayDestinationReachabilityUdpUnreachable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayDestinationReachabilityUdpUnreachable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityUdpUnreachable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityUdpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayDestinationReachabilityUdpUnreachable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayDestinationReachabilityUdpUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFirewallDetectorDiscoveryPacketDropped - errors', () => {
      it('should have a postRouterFirewallDetectorDiscoveryPacketDropped function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFirewallDetectorDiscoveryPacketDropped === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFirewallDetectorDiscoveryPacketDropped(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorDiscoveryPacketDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFirewallDetectorDiscoveryPacketDropped('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorDiscoveryPacketDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorDiscoveryTimeout - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorDiscoveryTimeout function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorDiscoveryTimeout === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorDiscoveryTimeout(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorDiscoveryTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorDiscoveryTimeout('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorDiscoveryTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFirewallDetectorDuplicateContextDetected - errors', () => {
      it('should have a postRouterFirewallDetectorDuplicateContextDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFirewallDetectorDuplicateContextDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFirewallDetectorDuplicateContextDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorDuplicateContextDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFirewallDetectorDuplicateContextDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorDuplicateContextDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorReplyReceived - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorReplyReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorReplyReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorReplyReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorReplyReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorReplyReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorReplyReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorReplySent - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorReplySent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorReplySent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorReplySent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorReplySent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorReplySent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorReplySent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorRequestReceived - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorRequestReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorRequestReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorRequestReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorRequestReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorRequestReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorRequestReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorRequestSent - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorRequestSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorRequestSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorRequestSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorRequestSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorRequestSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorRequestSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFirewallDetectorTcpResetReceived - errors', () => {
      it('should have a postRouterFirewallDetectorTcpResetReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFirewallDetectorTcpResetReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFirewallDetectorTcpResetReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorTcpResetReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFirewallDetectorTcpResetReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFirewallDetectorTcpResetReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHighwayFirewallDetectorUnknownReceived - errors', () => {
      it('should have a postRouterHighwayFirewallDetectorUnknownReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHighwayFirewallDetectorUnknownReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorUnknownReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorUnknownReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHighwayFirewallDetectorUnknownReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHighwayFirewallDetectorUnknownReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpReceivedFailureDestinationUnreachable - errors', () => {
      it('should have a postRouterIcmpReceivedFailureDestinationUnreachable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpReceivedFailureDestinationUnreachable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpReceivedFailureDestinationUnreachable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpReceivedFailureDestinationUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpReceivedFailureDestinationUnreachable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpReceivedFailureDestinationUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpReceivedSuccessDestinationUnreachable - errors', () => {
      it('should have a postRouterIcmpReceivedSuccessDestinationUnreachable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpReceivedSuccessDestinationUnreachable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpReceivedSuccessDestinationUnreachable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpReceivedSuccessDestinationUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpReceivedSuccessDestinationUnreachable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpReceivedSuccessDestinationUnreachable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDestinationUnreachablePacketTooBig - errors', () => {
      it('should have a postRouterDestinationUnreachablePacketTooBig function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDestinationUnreachablePacketTooBig === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDestinationUnreachablePacketTooBig(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachablePacketTooBig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDestinationUnreachablePacketTooBig('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachablePacketTooBig', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpSentFailureEchoReply - errors', () => {
      it('should have a postRouterIcmpSentFailureEchoReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpSentFailureEchoReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpSentFailureEchoReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpSentFailureEchoReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpSentFailureEchoReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpSentFailureEchoReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentFailureEchoReplyDrop - errors', () => {
      it('should have a postRouterSentFailureEchoReplyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentFailureEchoReplyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentFailureEchoReplyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureEchoReplyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentFailureEchoReplyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentFailureEchoReplyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDestinationUnreachableNet - errors', () => {
      it('should have a postRouterSuccessDestinationUnreachableNet function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDestinationUnreachableNet === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableNet(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableNet', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableNet('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableNet', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDestinationUnreachableNotRequired - errors', () => {
      it('should have a postRouterSuccessDestinationUnreachableNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDestinationUnreachableNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDestinationUnreachablePacketTooBig2 - errors', () => {
      it('should have a postRouterDestinationUnreachablePacketTooBig2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDestinationUnreachablePacketTooBig2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDestinationUnreachablePacketTooBig2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachablePacketTooBig2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDestinationUnreachablePacketTooBig2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachablePacketTooBig2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDestinationUnreachableReassemblyTimeExceeded - errors', () => {
      it('should have a postRouterDestinationUnreachableReassemblyTimeExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDestinationUnreachableReassemblyTimeExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDestinationUnreachableReassemblyTimeExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachableReassemblyTimeExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDestinationUnreachableReassemblyTimeExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDestinationUnreachableReassemblyTimeExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDestinationUnreachableTimeExceeded - errors', () => {
      it('should have a postRouterSuccessDestinationUnreachableTimeExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDestinationUnreachableTimeExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableTimeExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableTimeExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDestinationUnreachableTimeExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDestinationUnreachableTimeExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpSentSuccessEchoReply - errors', () => {
      it('should have a postRouterIcmpSentSuccessEchoReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpSentSuccessEchoReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpSentSuccessEchoReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpSentSuccessEchoReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpSentSuccessEchoReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpSentSuccessEchoReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInterfaceReceivedBufferAllocationFailure - errors', () => {
      it('should have a postRouterInterfaceReceivedBufferAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInterfaceReceivedBufferAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInterfaceReceivedBufferAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInterfaceReceivedBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInterfaceReceivedBufferAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInterfaceReceivedBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceReceivedBytes - errors', () => {
      it('should have a postRouterStatsInterfaceReceivedBytes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceReceivedBytes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedBytes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedBytes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedBytes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedBytes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceReceivedError - errors', () => {
      it('should have a postRouterStatsInterfaceReceivedError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceReceivedError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceReceivedMissed - errors', () => {
      it('should have a postRouterStatsInterfaceReceivedMissed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceReceivedMissed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedMissed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedMissed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedMissed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedMissed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceReceivedPackets - errors', () => {
      it('should have a postRouterStatsInterfaceReceivedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceReceivedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceReceivedUtilization - errors', () => {
      it('should have a postRouterStatsInterfaceReceivedUtilization function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceReceivedUtilization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedUtilization(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceReceivedUtilization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceReceivedUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceSentBytes - errors', () => {
      it('should have a postRouterStatsInterfaceSentBytes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceSentBytes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceSentBytes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentBytes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceSentBytes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentBytes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceSentError - errors', () => {
      it('should have a postRouterStatsInterfaceSentError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceSentError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceSentError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceSentError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceSentPackets - errors', () => {
      it('should have a postRouterStatsInterfaceSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsInterfaceSentUtilization - errors', () => {
      it('should have a postRouterStatsInterfaceSentUtilization function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsInterfaceSentUtilization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsInterfaceSentUtilization(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsInterfaceSentUtilization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsInterfaceSentUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsIpfixRecordExportRate - errors', () => {
      it('should have a postRouterStatsIpfixRecordExportRate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsIpfixRecordExportRate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsIpfixRecordExportRate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixRecordExportRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsIpfixRecordExportRate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixRecordExportRate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsIpfixTimePerExport - errors', () => {
      it('should have a postRouterStatsIpfixTimePerExport function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsIpfixTimePerExport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsIpfixTimePerExport(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTimePerExport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsIpfixTimePerExport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTimePerExport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsIpfixTimePerSession - errors', () => {
      it('should have a postRouterStatsIpfixTimePerSession function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsIpfixTimePerSession === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsIpfixTimePerSession(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTimePerSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsIpfixTimePerSession('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTimePerSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsIpfixTotalGenerationTime - errors', () => {
      it('should have a postRouterStatsIpfixTotalGenerationTime function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsIpfixTotalGenerationTime === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsIpfixTotalGenerationTime(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTotalGenerationTime', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsIpfixTotalGenerationTime('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTotalGenerationTime', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsIpfixTotalRecords - errors', () => {
      it('should have a postRouterStatsIpfixTotalRecords function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsIpfixTotalRecords === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsIpfixTotalRecords(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTotalRecords', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsIpfixTotalRecords('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsIpfixTotalRecords', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMemoryCapacity - errors', () => {
      it('should have a postRouterStatsMemoryCapacity function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMemoryCapacity === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMemoryCapacity(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMemoryCapacity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMemoryCapacity('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMemoryCapacity', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMemoryUsed - errors', () => {
      it('should have a postRouterStatsMemoryUsed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMemoryUsed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMemoryUsed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMemoryUsed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMemoryUsed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMemoryUsed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsActive - errors', () => {
      it('should have a postRouterStatsMetricsActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsCollectionsPending - errors', () => {
      it('should have a postRouterStatsMetricsCollectionsPending function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsCollectionsPending === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsCollectionsPending(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsCollectionsPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsCollectionsPending('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsCollectionsPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsCollectionsSkipped - errors', () => {
      it('should have a postRouterStatsMetricsCollectionsSkipped function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsCollectionsSkipped === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsCollectionsSkipped(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsCollectionsSkipped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsCollectionsSkipped('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsCollectionsSkipped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsPerCollectionLatency - errors', () => {
      it('should have a postRouterStatsMetricsPerCollectionLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsPerCollectionLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsPerCollectionLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsPerCollectionLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsPerCollectionLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsPerCollectionLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsPerMetricLatency - errors', () => {
      it('should have a postRouterStatsMetricsPerMetricLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsPerMetricLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsPerMetricLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsPerMetricLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsPerMetricLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsPerMetricLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMetricsReadMetricLatency - errors', () => {
      it('should have a postRouterStatsMetricsReadMetricLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMetricsReadMetricLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMetricsReadMetricLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsReadMetricLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMetricsReadMetricLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMetricsReadMetricLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMtuDiscoveryPeerPathReceived - errors', () => {
      it('should have a postRouterMtuDiscoveryPeerPathReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMtuDiscoveryPeerPathReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMtuDiscoveryPeerPathReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMtuDiscoveryPeerPathReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMtuDiscoveryPeerPathReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMtuDiscoveryPeerPathReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPeerPathSentArpFailure - errors', () => {
      it('should have a postRouterPeerPathSentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPeerPathSentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPeerPathSentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPeerPathSentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPeerPathSentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathSentBufferAllocationFailure - errors', () => {
      it('should have a postRouterPathSentBufferAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathSentBufferAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathSentBufferAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathSentBufferAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathSentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDiscoveryPeerPathSentSuccess - errors', () => {
      it('should have a postRouterDiscoveryPeerPathSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDiscoveryPeerPathSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDiscoveryPeerPathSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDiscoveryPeerPathSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDiscoveryPeerPathSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDiscoveryPeerPathSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMtuDiscoveryReceived - errors', () => {
      it('should have a postRouterStatsMtuDiscoveryReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMtuDiscoveryReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMtuDiscoveryReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMtuDiscoveryReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMtuDiscoveryReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMtuDiscoveryReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMtuDiscoverySentArpFailure - errors', () => {
      it('should have a postRouterMtuDiscoverySentArpFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMtuDiscoverySentArpFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMtuDiscoverySentArpFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMtuDiscoverySentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMtuDiscoverySentArpFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMtuDiscoverySentArpFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDiscoverySentBufferAllocationFailure - errors', () => {
      it('should have a postRouterDiscoverySentBufferAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDiscoverySentBufferAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDiscoverySentBufferAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDiscoverySentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDiscoverySentBufferAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDiscoverySentBufferAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsMtuDiscoverySentSuccess - errors', () => {
      it('should have a postRouterStatsMtuDiscoverySentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsMtuDiscoverySentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsMtuDiscoverySentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMtuDiscoverySentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsMtuDiscoverySentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsMtuDiscoverySentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsPacketCaptureSuccess - errors', () => {
      it('should have a postRouterStatsPacketCaptureSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsPacketCaptureSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsPacketCaptureSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketCaptureSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsPacketCaptureSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketCaptureSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsPacketCaptureWriteFailure - errors', () => {
      it('should have a postRouterStatsPacketCaptureWriteFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsPacketCaptureWriteFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsPacketCaptureWriteFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketCaptureWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsPacketCaptureWriteFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketCaptureWriteFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesDataNormalization - errors', () => {
      it('should have a postRouterActionFailureAesDataNormalization function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesDataNormalization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesDataNormalization(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesDataNormalization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesDataNormalization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesDataNormalization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesDecryptionException - errors', () => {
      it('should have a postRouterActionFailureAesDecryptionException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesDecryptionException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesDecryptionException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesDecryptionException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesDecryptionException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesDecryptionException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesEncryptionException - errors', () => {
      it('should have a postRouterActionFailureAesEncryptionException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesEncryptionException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesEncryptionException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesEncryptionException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesEncryptionException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesEncryptionException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureAesGetDataLength - errors', () => {
      it('should have a postRouterFailureAesGetDataLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureAesGetDataLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureAesGetDataLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesGetDataLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureAesGetDataLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesGetDataLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesIvAppend - errors', () => {
      it('should have a postRouterActionFailureAesIvAppend function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesIvAppend === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesIvAppend(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvAppend', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesIvAppend('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvAppend', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesIvGeneration - errors', () => {
      it('should have a postRouterActionFailureAesIvGeneration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesIvGeneration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesIvGeneration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesIvGeneration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvGeneration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesIvSeed - errors', () => {
      it('should have a postRouterActionFailureAesIvSeed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesIvSeed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesIvSeed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvSeed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesIvSeed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesIvSeed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesMetadataDecryption - errors', () => {
      it('should have a postRouterActionFailureAesMetadataDecryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesMetadataDecryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesMetadataDecryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesMetadataDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesMetadataDecryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesMetadataDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesMetadataEncryption - errors', () => {
      it('should have a postRouterActionFailureAesMetadataEncryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesMetadataEncryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesMetadataEncryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesMetadataEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesMetadataEncryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesMetadataEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureAesNoContextFound - errors', () => {
      it('should have a postRouterFailureAesNoContextFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureAesNoContextFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureAesNoContextFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesNoContextFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureAesNoContextFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesNoContextFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesPayloadDecryption - errors', () => {
      it('should have a postRouterActionFailureAesPayloadDecryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesPayloadDecryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesPayloadDecryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesPayloadDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesPayloadDecryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesPayloadDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureAesPayloadEncryption - errors', () => {
      it('should have a postRouterActionFailureAesPayloadEncryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureAesPayloadEncryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureAesPayloadEncryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesPayloadEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureAesPayloadEncryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureAesPayloadEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureAesSetDataLength - errors', () => {
      it('should have a postRouterFailureAesSetDataLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureAesSetDataLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureAesSetDataLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesSetDataLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureAesSetDataLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureAesSetDataLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdDecode - errors', () => {
      it('should have a postRouterActionFailureBfdDecode function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdDecode === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdDecode(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdDecode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdDecode('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdDecode', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdEchoInit - errors', () => {
      it('should have a postRouterActionFailureBfdEchoInit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdEchoInit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdEchoInit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEchoInit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdEchoInit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEchoInit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdEmptyMetadata - errors', () => {
      it('should have a postRouterActionFailureBfdEmptyMetadata function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdEmptyMetadata === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdEmptyMetadata(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEmptyMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdEmptyMetadata('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEmptyMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdEmptyPayload - errors', () => {
      it('should have a postRouterActionFailureBfdEmptyPayload function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdEmptyPayload === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdEmptyPayload(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEmptyPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdEmptyPayload('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdEmptyPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureBfdFragmentedMtuDiscovery - errors', () => {
      it('should have a postRouterFailureBfdFragmentedMtuDiscovery function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureBfdFragmentedMtuDiscovery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureBfdFragmentedMtuDiscovery(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureBfdFragmentedMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureBfdFragmentedMtuDiscovery('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureBfdFragmentedMtuDiscovery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdInvalidHeader - errors', () => {
      it('should have a postRouterActionFailureBfdInvalidHeader function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdInvalidHeader === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdInvalidHeader(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdInvalidHeader', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdInvalidHeader('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdInvalidHeader', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureBfdMetadataParse - errors', () => {
      it('should have a postRouterActionFailureBfdMetadataParse function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureBfdMetadataParse === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureBfdMetadataParse(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdMetadataParse', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureBfdMetadataParse('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureBfdMetadataParse', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureDpiFtpParseFailed - errors', () => {
      it('should have a postRouterFailureDpiFtpParseFailed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureDpiFtpParseFailed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureDpiFtpParseFailed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpParseFailed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureDpiFtpParseFailed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpParseFailed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureDpiFtpPinholeMiss - errors', () => {
      it('should have a postRouterFailureDpiFtpPinholeMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureDpiFtpPinholeMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureDpiFtpPinholeMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpPinholeMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureDpiFtpPinholeMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpPinholeMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureDpiFtpPinholeTimeout - errors', () => {
      it('should have a postRouterFailureDpiFtpPinholeTimeout function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureDpiFtpPinholeTimeout === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureDpiFtpPinholeTimeout(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpPinholeTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureDpiFtpPinholeTimeout('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureDpiFtpPinholeTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHeaderTransformArpTableMisses - errors', () => {
      it('should have a postRouterHeaderTransformArpTableMisses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHeaderTransformArpTableMisses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHeaderTransformArpTableMisses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHeaderTransformArpTableMisses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHeaderTransformArpTableMisses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHeaderTransformArpTableMisses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEthernetHeaderTransformPacketExpansion - errors', () => {
      it('should have a postRouterEthernetHeaderTransformPacketExpansion function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEthernetHeaderTransformPacketExpansion === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEthernetHeaderTransformPacketExpansion(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEthernetHeaderTransformPacketExpansion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEthernetHeaderTransformPacketExpansion('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEthernetHeaderTransformPacketExpansion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMoveDisableForwardMetadataDiscards - errors', () => {
      it('should have a postRouterMoveDisableForwardMetadataDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMoveDisableForwardMetadataDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMoveDisableForwardMetadataDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMoveDisableForwardMetadataDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMoveDisableForwardMetadataDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMoveDisableForwardMetadataDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureHmacAllocation - errors', () => {
      it('should have a postRouterActionFailureHmacAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureHmacAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureHmacAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureHmacAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureHmacCannotFindDigest - errors', () => {
      it('should have a postRouterFailureHmacCannotFindDigest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureHmacCannotFindDigest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureHmacCannotFindDigest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureHmacCannotFindDigest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureHmacCannotFindDigest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureHmacCannotFindDigest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureHmacContextNotFound - errors', () => {
      it('should have a postRouterFailureHmacContextNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureHmacContextNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureHmacContextNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureHmacContextNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureHmacContextNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureHmacContextNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureHmacDigestCalculation - errors', () => {
      it('should have a postRouterActionFailureHmacDigestCalculation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureHmacDigestCalculation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureHmacDigestCalculation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacDigestCalculation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureHmacDigestCalculation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacDigestCalculation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureHmacDigestCompare - errors', () => {
      it('should have a postRouterActionFailureHmacDigestCompare function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureHmacDigestCompare === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureHmacDigestCompare(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacDigestCompare', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureHmacDigestCompare('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureHmacDigestCompare', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterHmacOppositeFlowNotFound - errors', () => {
      it('should have a postRouterHmacOppositeFlowNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterHmacOppositeFlowNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterHmacOppositeFlowNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHmacOppositeFlowNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterHmacOppositeFlowNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterHmacOppositeFlowNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIpHeaderTransformIcmpTranslation - errors', () => {
      it('should have a postRouterIpHeaderTransformIcmpTranslation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIpHeaderTransformIcmpTranslation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIpHeaderTransformIcmpTranslation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIpHeaderTransformIcmpTranslation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIpHeaderTransformIcmpTranslation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIpHeaderTransformIcmpTranslation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTransformIpv6ToIpv4Mapping - errors', () => {
      it('should have a postRouterTransformIpv6ToIpv4Mapping function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTransformIpv6ToIpv4Mapping === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTransformIpv6ToIpv4Mapping(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTransformIpv6ToIpv4Mapping', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTransformIpv6ToIpv4Mapping('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTransformIpv6ToIpv4Mapping', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIpHeaderTransformL4Translation - errors', () => {
      it('should have a postRouterIpHeaderTransformL4Translation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIpHeaderTransformL4Translation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIpHeaderTransformL4Translation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIpHeaderTransformL4Translation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIpHeaderTransformL4Translation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIpHeaderTransformL4Translation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureKeepAliveBufferAllocation - errors', () => {
      it('should have a postRouterFailureKeepAliveBufferAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureKeepAliveBufferAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureKeepAliveBufferAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureKeepAliveBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureKeepAliveBufferAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureKeepAliveBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureMetadataAdd - errors', () => {
      it('should have a postRouterActionFailureMetadataAdd function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureMetadataAdd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureMetadataAdd(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureMetadataAdd('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataCachedAttributesNotFound - errors', () => {
      it('should have a postRouterMetadataCachedAttributesNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataCachedAttributesNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataCachedAttributesNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataCachedAttributesNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataCachedAttributesNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataCachedAttributesNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataEnableFlowMiss - errors', () => {
      it('should have a postRouterMetadataEnableFlowMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataEnableFlowMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataEnableFlowMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataEnableFlowMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataEnableFlowMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataEnableFlowMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureMetadataGetLength - errors', () => {
      it('should have a postRouterActionFailureMetadataGetLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureMetadataGetLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureMetadataGetLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataGetLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureMetadataGetLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataGetLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureMetadataInvalidType - errors', () => {
      it('should have a postRouterActionFailureMetadataInvalidType function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureMetadataInvalidType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureMetadataInvalidType(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataInvalidType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureMetadataInvalidType('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataInvalidType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureMetadataNotPresent - errors', () => {
      it('should have a postRouterActionFailureMetadataNotPresent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureMetadataNotPresent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureMetadataNotPresent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataNotPresent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureMetadataNotPresent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMetadataNotPresent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedUnknownForcedDropReason - errors', () => {
      it('should have a postRouterReceivedUnknownForcedDropReason function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedUnknownForcedDropReason === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedUnknownForcedDropReason(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedUnknownForcedDropReason', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedUnknownForcedDropReason('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedUnknownForcedDropReason', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureMetadataReverseBufferAllocation - errors', () => {
      it('should have a postRouterFailureMetadataReverseBufferAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureMetadataReverseBufferAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureMetadataReverseBufferAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureMetadataReverseBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureMetadataReverseBufferAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureMetadataReverseBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataTurnOffBufferAllocation - errors', () => {
      it('should have a postRouterMetadataTurnOffBufferAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataTurnOffBufferAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataTurnOffBufferAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataTurnOffBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataTurnOffBufferAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataTurnOffBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureMultiplexorUntagDiscards - errors', () => {
      it('should have a postRouterActionFailureMultiplexorUntagDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureMultiplexorUntagDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureMultiplexorUntagDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMultiplexorUntagDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureMultiplexorUntagDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureMultiplexorUntagDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailurePaddingInvalidPacketLength - errors', () => {
      it('should have a postRouterFailurePaddingInvalidPacketLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailurePaddingInvalidPacketLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailurePaddingInvalidPacketLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailurePaddingInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailurePaddingInvalidPacketLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailurePaddingInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsSentCorrelatedData - errors', () => {
      it('should have a postRouterPathMetricsSentCorrelatedData function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsSentCorrelatedData === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsSentCorrelatedData(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentCorrelatedData', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsSentCorrelatedData('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentCorrelatedData', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsSentUnidirectionalReply - errors', () => {
      it('should have a postRouterPathMetricsSentUnidirectionalReply function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsSentUnidirectionalReply === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsSentUnidirectionalReply(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentUnidirectionalReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsSentUnidirectionalReply('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentUnidirectionalReply', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureRateLimitCheck - errors', () => {
      it('should have a postRouterActionFailureRateLimitCheck function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureRateLimitCheck === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureRateLimitCheck(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureRateLimitCheck', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureRateLimitCheck('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureRateLimitCheck', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureRetransmissionHandlerBufferAllocation - errors', () => {
      it('should have a postRouterFailureRetransmissionHandlerBufferAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureRetransmissionHandlerBufferAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerBufferAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerBufferAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureRetransmissionHandlerCapacityExceeded - errors', () => {
      it('should have a postRouterFailureRetransmissionHandlerCapacityExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureRetransmissionHandlerCapacityExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerCapacityExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerCapacityExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerCapacityExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerCapacityExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRetransmissionHandlerDeletedFlowDiscards - errors', () => {
      it('should have a postRouterRetransmissionHandlerDeletedFlowDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRetransmissionHandlerDeletedFlowDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRetransmissionHandlerDeletedFlowDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerDeletedFlowDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRetransmissionHandlerDeletedFlowDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerDeletedFlowDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRetransmissionHandlerInvalidAcknowledgementAttribute - errors', () => {
      it('should have a postRouterRetransmissionHandlerInvalidAcknowledgementAttribute function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRetransmissionHandlerInvalidAcknowledgementAttribute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRetransmissionHandlerInvalidAcknowledgementAttribute(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerInvalidAcknowledgementAttribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRetransmissionHandlerInvalidAcknowledgementAttribute('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerInvalidAcknowledgementAttribute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRetransmissionHandlerInvalidPacketLength - errors', () => {
      it('should have a postRouterRetransmissionHandlerInvalidPacketLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRetransmissionHandlerInvalidPacketLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRetransmissionHandlerInvalidPacketLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRetransmissionHandlerInvalidPacketLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerInvalidPacketLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionFailureRetransmissionHandlerReclassify - errors', () => {
      it('should have a postRouterActionFailureRetransmissionHandlerReclassify function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionFailureRetransmissionHandlerReclassify === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionFailureRetransmissionHandlerReclassify(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureRetransmissionHandlerReclassify', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionFailureRetransmissionHandlerReclassify('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionFailureRetransmissionHandlerReclassify', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureRetransmissionHandlerTimeoutDiscards - errors', () => {
      it('should have a postRouterFailureRetransmissionHandlerTimeoutDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureRetransmissionHandlerTimeoutDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerTimeoutDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerTimeoutDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureRetransmissionHandlerTimeoutDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureRetransmissionHandlerTimeoutDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureTcpProxyBufferAllocation - errors', () => {
      it('should have a postRouterFailureTcpProxyBufferAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureTcpProxyBufferAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureTcpProxyBufferAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureTcpProxyBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureTcpProxyBufferAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureTcpProxyBufferAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFailureTcpProxyInboundDiscards - errors', () => {
      it('should have a postRouterFailureTcpProxyInboundDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFailureTcpProxyInboundDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFailureTcpProxyInboundDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureTcpProxyInboundDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFailureTcpProxyInboundDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFailureTcpProxyInboundDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpProxySessionsTimedOut - errors', () => {
      it('should have a postRouterTcpProxySessionsTimedOut function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpProxySessionsTimedOut === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpProxySessionsTimedOut(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxySessionsTimedOut', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpProxySessionsTimedOut('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxySessionsTimedOut', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpStateIllegalFlagCombination - errors', () => {
      it('should have a postRouterTcpStateIllegalFlagCombination function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpStateIllegalFlagCombination === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpStateIllegalFlagCombination(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpStateIllegalFlagCombination', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpStateIllegalFlagCombination('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpStateIllegalFlagCombination', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpStateInvalidTransition - errors', () => {
      it('should have a postRouterTcpStateInvalidTransition function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpStateInvalidTransition === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpStateInvalidTransition(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpStateInvalidTransition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpStateInvalidTransition('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpStateInvalidTransition', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingActionFailureTtlValidate - errors', () => {
      it('should have a postRouterProcessingActionFailureTtlValidate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingActionFailureTtlValidate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingActionFailureTtlValidate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionFailureTtlValidate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingActionFailureTtlValidate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionFailureTtlValidate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesMetadataDecryption - errors', () => {
      it('should have a postRouterActionSuccessAesMetadataDecryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesMetadataDecryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesMetadataDecryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesMetadataDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesMetadataDecryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesMetadataDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesMetadataEncryption - errors', () => {
      it('should have a postRouterActionSuccessAesMetadataEncryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesMetadataEncryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesMetadataEncryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesMetadataEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesMetadataEncryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesMetadataEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAesMetadataFalsePositiveDetected - errors', () => {
      it('should have a postRouterAesMetadataFalsePositiveDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAesMetadataFalsePositiveDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAesMetadataFalsePositiveDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesMetadataFalsePositiveDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAesMetadataFalsePositiveDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesMetadataFalsePositiveDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesNoMetadata - errors', () => {
      it('should have a postRouterActionSuccessAesNoMetadata function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesNoMetadata === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesNoMetadata(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesNoMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesNoMetadata('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesNoMetadata', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesNoPayload - errors', () => {
      it('should have a postRouterActionSuccessAesNoPayload function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesNoPayload === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesNoPayload(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesNoPayload('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesPayloadDecryption - errors', () => {
      it('should have a postRouterActionSuccessAesPayloadDecryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesPayloadDecryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesPayloadDecryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesPayloadDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesPayloadDecryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesPayloadDecryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessAesPayloadEncryption - errors', () => {
      it('should have a postRouterActionSuccessAesPayloadEncryption function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessAesPayloadEncryption === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessAesPayloadEncryption(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesPayloadEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessAesPayloadEncryption('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessAesPayloadEncryption', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessBfdAsyncDiscarded - errors', () => {
      it('should have a postRouterActionSuccessBfdAsyncDiscarded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessBfdAsyncDiscarded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessBfdAsyncDiscarded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessBfdAsyncDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessBfdAsyncDiscarded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessBfdAsyncDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessBfdAsyncProcessed - errors', () => {
      it('should have a postRouterActionSuccessBfdAsyncProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessBfdAsyncProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessBfdAsyncProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessBfdAsyncProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessBfdAsyncProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessBfdAsyncProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessBfdEchoRequestProcessed - errors', () => {
      it('should have a postRouterSuccessBfdEchoRequestProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessBfdEchoRequestProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessBfdEchoRequestProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdEchoRequestProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessBfdEchoRequestProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdEchoRequestProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessBfdEchoResponseProcessed - errors', () => {
      it('should have a postRouterSuccessBfdEchoResponseProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessBfdEchoResponseProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessBfdEchoResponseProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdEchoResponseProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessBfdEchoResponseProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdEchoResponseProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessBfdMtuDiscoveryRequests - errors', () => {
      it('should have a postRouterSuccessBfdMtuDiscoveryRequests function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessBfdMtuDiscoveryRequests === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessBfdMtuDiscoveryRequests(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdMtuDiscoveryRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessBfdMtuDiscoveryRequests('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdMtuDiscoveryRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessBfdRemoteSessionRequested - errors', () => {
      it('should have a postRouterSuccessBfdRemoteSessionRequested function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessBfdRemoteSessionRequested === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessBfdRemoteSessionRequested(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdRemoteSessionRequested', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessBfdRemoteSessionRequested('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessBfdRemoteSessionRequested', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingActionSuccessDetour - errors', () => {
      it('should have a postRouterPacketProcessingActionSuccessDetour function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingActionSuccessDetour === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingActionSuccessDetour(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingActionSuccessDetour', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingActionSuccessDetour('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingActionSuccessDetour', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiFtpDataPortDetected - errors', () => {
      it('should have a postRouterDpiFtpDataPortDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiFtpDataPortDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiFtpDataPortDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiFtpDataPortDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiFtpDataPortDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiFtpDataPortDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDpiFtpPinholeClosed - errors', () => {
      it('should have a postRouterSuccessDpiFtpPinholeClosed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDpiFtpPinholeClosed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeClosed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeClosed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDpiFtpPinholeHit - errors', () => {
      it('should have a postRouterSuccessDpiFtpPinholeHit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDpiFtpPinholeHit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeHit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeHit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessDpiFtpPinholeOpened - errors', () => {
      it('should have a postRouterSuccessDpiFtpPinholeOpened function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessDpiFtpPinholeOpened === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeOpened(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessDpiFtpPinholeOpened('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessDpiFtpPinholeOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiTlsClientDetected - errors', () => {
      it('should have a postRouterDpiTlsClientDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiTlsClientDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiTlsClientDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiTlsClientDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiTlsClientDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiTlsClientDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDpiTlsServerHandshakeDetected - errors', () => {
      it('should have a postRouterDpiTlsServerHandshakeDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDpiTlsServerHandshakeDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDpiTlsServerHandshakeDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiTlsServerHandshakeDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDpiTlsServerHandshakeDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDpiTlsServerHandshakeDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTlsServerSentForInspection - errors', () => {
      it('should have a postRouterTlsServerSentForInspection function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTlsServerSentForInspection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTlsServerSentForInspection(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsServerSentForInspection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTlsServerSentForInspection('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTlsServerSentForInspection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessDpiUnencrypted - errors', () => {
      it('should have a postRouterActionSuccessDpiUnencrypted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessDpiUnencrypted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessDpiUnencrypted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDpiUnencrypted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessDpiUnencrypted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDpiUnencrypted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingActionSuccessDrop - errors', () => {
      it('should have a postRouterPacketProcessingActionSuccessDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingActionSuccessDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingActionSuccessDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingActionSuccessDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingActionSuccessDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingActionSuccessDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessDuplicateDetectorAllow - errors', () => {
      it('should have a postRouterActionSuccessDuplicateDetectorAllow function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessDuplicateDetectorAllow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessDuplicateDetectorAllow(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDuplicateDetectorAllow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessDuplicateDetectorAllow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDuplicateDetectorAllow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessDuplicateDetectorDeny - errors', () => {
      it('should have a postRouterActionSuccessDuplicateDetectorDeny function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessDuplicateDetectorDeny === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessDuplicateDetectorDeny(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDuplicateDetectorDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessDuplicateDetectorDeny('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessDuplicateDetectorDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessEthernetHeaderTransform - errors', () => {
      it('should have a postRouterActionSuccessEthernetHeaderTransform function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessEthernetHeaderTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessEthernetHeaderTransform(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessEthernetHeaderTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessEthernetHeaderTransform('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessEthernetHeaderTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessFlowMoveDetected - errors', () => {
      it('should have a postRouterActionSuccessFlowMoveDetected function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessFlowMoveDetected === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessFlowMoveDetected(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessFlowMoveDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessFlowMoveDetected('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessFlowMoveDetected', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessFlowMovePacketsEnqueued - errors', () => {
      it('should have a postRouterSuccessFlowMovePacketsEnqueued function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessFlowMovePacketsEnqueued === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessFlowMovePacketsEnqueued(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessFlowMovePacketsEnqueued', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessFlowMovePacketsEnqueued('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessFlowMovePacketsEnqueued', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessForwardToWire - errors', () => {
      it('should have a postRouterActionSuccessForwardToWire function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessForwardToWire === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessForwardToWire(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessForwardToWire', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessForwardToWire('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessForwardToWire', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessHmacAppendNoPayload - errors', () => {
      it('should have a postRouterSuccessHmacAppendNoPayload function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessHmacAppendNoPayload === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessHmacAppendNoPayload(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessHmacAppendNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessHmacAppendNoPayload('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessHmacAppendNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessHmacDigestAdded - errors', () => {
      it('should have a postRouterActionSuccessHmacDigestAdded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessHmacDigestAdded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessHmacDigestAdded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacDigestAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessHmacDigestAdded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacDigestAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessHmacDigestRemoved - errors', () => {
      it('should have a postRouterActionSuccessHmacDigestRemoved function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessHmacDigestRemoved === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessHmacDigestRemoved(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacDigestRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessHmacDigestRemoved('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacDigestRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessHmacNotRequired - errors', () => {
      it('should have a postRouterActionSuccessHmacNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessHmacNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessHmacNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessHmacNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessHmacNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessHmacValidateNoPayload - errors', () => {
      it('should have a postRouterSuccessHmacValidateNoPayload function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessHmacValidateNoPayload === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessHmacValidateNoPayload(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessHmacValidateNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessHmacValidateNoPayload('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessHmacValidateNoPayload', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessIpHeaderTransform - errors', () => {
      it('should have a postRouterActionSuccessIpHeaderTransform function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessIpHeaderTransform === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessIpHeaderTransform(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessIpHeaderTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessIpHeaderTransform('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessIpHeaderTransform', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessKeepAliveSentTcp - errors', () => {
      it('should have a postRouterSuccessKeepAliveSentTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessKeepAliveSentTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessKeepAliveSentTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSentTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessKeepAliveSentTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSentTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessKeepAliveSentUdp - errors', () => {
      it('should have a postRouterSuccessKeepAliveSentUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessKeepAliveSentUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessKeepAliveSentUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSentUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessKeepAliveSentUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSentUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessKeepAliveSessionsClosed - errors', () => {
      it('should have a postRouterSuccessKeepAliveSessionsClosed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessKeepAliveSessionsClosed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessKeepAliveSessionsClosed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessKeepAliveSessionsClosed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessKeepAliveSessionsOpened - errors', () => {
      it('should have a postRouterSuccessKeepAliveSessionsOpened function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessKeepAliveSessionsOpened === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessKeepAliveSessionsOpened(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessKeepAliveSessionsOpened('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessKeepAliveSessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataAddFalsePositiveCorrection - errors', () => {
      it('should have a postRouterMetadataAddFalsePositiveCorrection function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataAddFalsePositiveCorrection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataAddFalsePositiveCorrection(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataAddFalsePositiveCorrection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataAddFalsePositiveCorrection('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataAddFalsePositiveCorrection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataAddNotRequired - errors', () => {
      it('should have a postRouterSuccessMetadataAddNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataAddNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataAddNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataAddNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataAddNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataAddNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessMetadataAdded - errors', () => {
      it('should have a postRouterActionSuccessMetadataAdded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessMetadataAdded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessMetadataAdded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessMetadataAdded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataPropagateNotRequired - errors', () => {
      it('should have a postRouterSuccessMetadataPropagateNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataPropagateNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataPropagateNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataPropagateNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataPropagateNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataPropagateNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataReceivedDisableForward - errors', () => {
      it('should have a postRouterMetadataReceivedDisableForward function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataReceivedDisableForward === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataReceivedDisableForward(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataReceivedDisableForward', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataReceivedDisableForward('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataReceivedDisableForward', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataReceivedEnable - errors', () => {
      it('should have a postRouterSuccessMetadataReceivedEnable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataReceivedEnable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataReceivedEnable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataReceivedEnable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataReceivedEnable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataReceivedEnable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataReceivedKeepAlive - errors', () => {
      it('should have a postRouterSuccessMetadataReceivedKeepAlive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataReceivedKeepAlive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataReceivedKeepAlive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataReceivedKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataReceivedKeepAlive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataReceivedKeepAlive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataRemoveFalsePositiveCorrection - errors', () => {
      it('should have a postRouterMetadataRemoveFalsePositiveCorrection function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataRemoveFalsePositiveCorrection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataRemoveFalsePositiveCorrection(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataRemoveFalsePositiveCorrection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataRemoveFalsePositiveCorrection('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataRemoveFalsePositiveCorrection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataRemoveNotRequired - errors', () => {
      it('should have a postRouterSuccessMetadataRemoveNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataRemoveNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataRemoveNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataRemoveNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataRemoveNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataRemoveNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessMetadataRemoved - errors', () => {
      it('should have a postRouterActionSuccessMetadataRemoved function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessMetadataRemoved === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessMetadataRemoved(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessMetadataRemoved('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessMetadataRightLaneSent - errors', () => {
      it('should have a postRouterSuccessMetadataRightLaneSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessMetadataRightLaneSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessMetadataRightLaneSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataRightLaneSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessMetadataRightLaneSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessMetadataRightLaneSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterMetadataSentDisableForward - errors', () => {
      it('should have a postRouterMetadataSentDisableForward function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterMetadataSentDisableForward === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterMetadataSentDisableForward(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataSentDisableForward', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterMetadataSentDisableForward('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterMetadataSentDisableForward', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessMetadataUpdated - errors', () => {
      it('should have a postRouterActionSuccessMetadataUpdated function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessMetadataUpdated === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessMetadataUpdated(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataUpdated', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessMetadataUpdated('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMetadataUpdated', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessMultiplexorTagged - errors', () => {
      it('should have a postRouterActionSuccessMultiplexorTagged function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessMultiplexorTagged === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessMultiplexorTagged(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMultiplexorTagged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessMultiplexorTagged('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMultiplexorTagged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessMultiplexorUntagged - errors', () => {
      it('should have a postRouterActionSuccessMultiplexorUntagged function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessMultiplexorUntagged === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessMultiplexorUntagged(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMultiplexorUntagged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessMultiplexorUntagged('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessMultiplexorUntagged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessPaddingAdded - errors', () => {
      it('should have a postRouterActionSuccessPaddingAdded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessPaddingAdded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessPaddingAdded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPaddingAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessPaddingAdded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPaddingAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessPaddingRemoved - errors', () => {
      it('should have a postRouterActionSuccessPaddingRemoved function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessPaddingRemoved === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessPaddingRemoved(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPaddingRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessPaddingRemoved('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPaddingRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsAddNotRequired - errors', () => {
      it('should have a postRouterPathMetricsAddNotRequired function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsAddNotRequired === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsAddNotRequired(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsAddNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsAddNotRequired('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsAddNotRequired', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessPathMetricsRemoveDropped - errors', () => {
      it('should have a postRouterSuccessPathMetricsRemoveDropped function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessPathMetricsRemoveDropped === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessPathMetricsRemoveDropped(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessPathMetricsRemoveDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessPathMetricsRemoveDropped('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessPathMetricsRemoveDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsRemoveNotPresent - errors', () => {
      it('should have a postRouterPathMetricsRemoveNotPresent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsRemoveNotPresent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsRemoveNotPresent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsRemoveNotPresent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsRemoveNotPresent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsRemoveNotPresent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessPathMetricsRemoved - errors', () => {
      it('should have a postRouterActionSuccessPathMetricsRemoved function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessPathMetricsRemoved === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessPathMetricsRemoved(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPathMetricsRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessPathMetricsRemoved('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessPathMetricsRemoved', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsSentCorrelatedData2 - errors', () => {
      it('should have a postRouterPathMetricsSentCorrelatedData2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsSentCorrelatedData2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsSentCorrelatedData2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentCorrelatedData2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsSentCorrelatedData2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentCorrelatedData2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsSentUnidirectionalReply2 - errors', () => {
      it('should have a postRouterPathMetricsSentUnidirectionalReply2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsSentUnidirectionalReply2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsSentUnidirectionalReply2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentUnidirectionalReply2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsSentUnidirectionalReply2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsSentUnidirectionalReply2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsWithMetadataAdded - errors', () => {
      it('should have a postRouterPathMetricsWithMetadataAdded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsWithMetadataAdded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsWithMetadataAdded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsWithMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsWithMetadataAdded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsWithMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPathMetricsWithoutMetadataAdded - errors', () => {
      it('should have a postRouterPathMetricsWithoutMetadataAdded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPathMetricsWithoutMetadataAdded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPathMetricsWithoutMetadataAdded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsWithoutMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPathMetricsWithoutMetadataAdded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPathMetricsWithoutMetadataAdded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessRateLimitCheck - errors', () => {
      it('should have a postRouterActionSuccessRateLimitCheck function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessRateLimitCheck === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessRateLimitCheck(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRateLimitCheck', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessRateLimitCheck('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRateLimitCheck', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessRetransmissionHandlerAccepted - errors', () => {
      it('should have a postRouterActionSuccessRetransmissionHandlerAccepted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessRetransmissionHandlerAccepted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerAccepted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerAccepted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerAccepted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerAccepted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessRetransmissionHandlerAcknowledged - errors', () => {
      it('should have a postRouterActionSuccessRetransmissionHandlerAcknowledged function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessRetransmissionHandlerAcknowledged === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerAcknowledged(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerAcknowledged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerAcknowledged('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerAcknowledged', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerAcknowledgementReceived - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerAcknowledgementReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerAcknowledgementReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerAcknowledgementReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerAcknowledgementReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerAcknowledgementReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerAcknowledgementReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerAcknowledgementSent - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerAcknowledgementSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerAcknowledgementSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerAcknowledgementSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerAcknowledgementSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerAcknowledgementSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerAcknowledgementSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessRetransmissionHandlerBypassed - errors', () => {
      it('should have a postRouterActionSuccessRetransmissionHandlerBypassed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessRetransmissionHandlerBypassed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerBypassed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerBypassed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerBypassed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerBypassed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterActionSuccessRetransmissionHandlerCached - errors', () => {
      it('should have a postRouterActionSuccessRetransmissionHandlerCached function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterActionSuccessRetransmissionHandlerCached === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerCached(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerCached', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterActionSuccessRetransmissionHandlerCached('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterActionSuccessRetransmissionHandlerCached', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerDuplicateDiscards - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerDuplicateDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerDuplicateDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerDuplicateDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerDuplicateDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerDuplicateDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerDuplicateDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRetransmissionHandlerOutstandingPackets - errors', () => {
      it('should have a postRouterRetransmissionHandlerOutstandingPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRetransmissionHandlerOutstandingPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRetransmissionHandlerOutstandingPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerOutstandingPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRetransmissionHandlerOutstandingPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRetransmissionHandlerOutstandingPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerRetransmissionsSent - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerRetransmissionsSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerRetransmissionsSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerRetransmissionsSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerRetransmissionsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerRetransmissionsSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerRetransmissionsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerSessionsClosed - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerSessionsClosed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerSessionsClosed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerSessionsClosed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerSessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerSessionsClosed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerSessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessRetransmissionHandlerSessionsOpened - errors', () => {
      it('should have a postRouterSuccessRetransmissionHandlerSessionsOpened function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessRetransmissionHandlerSessionsOpened === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerSessionsOpened(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerSessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessRetransmissionHandlerSessionsOpened('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessRetransmissionHandlerSessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxyAcknowledgementReceived - errors', () => {
      it('should have a postRouterSuccessTcpProxyAcknowledgementReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxyAcknowledgementReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxyAcknowledgementReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyAcknowledgementReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxyAcknowledgementReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyAcknowledgementReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxyAcknowledgementSent - errors', () => {
      it('should have a postRouterSuccessTcpProxyAcknowledgementSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxyAcknowledgementSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxyAcknowledgementSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyAcknowledgementSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxyAcknowledgementSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyAcknowledgementSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpProxyDataPacketsTransmitted - errors', () => {
      it('should have a postRouterTcpProxyDataPacketsTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpProxyDataPacketsTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpProxyDataPacketsTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyDataPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpProxyDataPacketsTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyDataPacketsTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxyDataTransmitted - errors', () => {
      it('should have a postRouterSuccessTcpProxyDataTransmitted function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxyDataTransmitted === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxyDataTransmitted(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyDataTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxyDataTransmitted('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyDataTransmitted', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpProxyOutstandingPackets - errors', () => {
      it('should have a postRouterTcpProxyOutstandingPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpProxyOutstandingPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpProxyOutstandingPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyOutstandingPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpProxyOutstandingPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyOutstandingPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxyRetransmissionsSent - errors', () => {
      it('should have a postRouterSuccessTcpProxyRetransmissionsSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxyRetransmissionsSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxyRetransmissionsSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyRetransmissionsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxyRetransmissionsSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxyRetransmissionsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTcpProxyRetransmittedDataSent - errors', () => {
      it('should have a postRouterTcpProxyRetransmittedDataSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTcpProxyRetransmittedDataSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTcpProxyRetransmittedDataSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyRetransmittedDataSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTcpProxyRetransmittedDataSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTcpProxyRetransmittedDataSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxySessionsClosed - errors', () => {
      it('should have a postRouterSuccessTcpProxySessionsClosed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxySessionsClosed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsClosed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsClosed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsClosed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxySessionsOpened - errors', () => {
      it('should have a postRouterSuccessTcpProxySessionsOpened function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxySessionsOpened === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsOpened(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsOpened('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsOpened', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSuccessTcpProxySessionsReset - errors', () => {
      it('should have a postRouterSuccessTcpProxySessionsReset function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSuccessTcpProxySessionsReset === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsReset(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsReset', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSuccessTcpProxySessionsReset('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSuccessTcpProxySessionsReset', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingActionSuccessTcpState - errors', () => {
      it('should have a postRouterProcessingActionSuccessTcpState function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingActionSuccessTcpState === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingActionSuccessTcpState(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionSuccessTcpState', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingActionSuccessTcpState('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionSuccessTcpState', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingActionSuccessTtlValidate - errors', () => {
      it('should have a postRouterProcessingActionSuccessTtlValidate function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingActionSuccessTtlValidate === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingActionSuccessTtlValidate(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionSuccessTtlValidate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingActionSuccessTtlValidate('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingActionSuccessTtlValidate', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedArp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedArp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedArp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedArp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedArp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedArp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedArp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedBroadcastMacDiscards - errors', () => {
      it('should have a postRouterClassifierReceivedBroadcastMacDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedBroadcastMacDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedBroadcastMacDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedBroadcastMacDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedBroadcastMacDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedBroadcastMacDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedDhcp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedDhcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedDhcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDhcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDhcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDhcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDhcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedDhcpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedDhcpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedDhcpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDhcpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDhcpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDhcpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDhcpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedDiscards - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedException - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedFragmentedIcmpDiscards - errors', () => {
      it('should have a postRouterClassifierReceivedFragmentedIcmpDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedFragmentedIcmpDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedFragmentedIcmpDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedFragmentedIcmpDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedFragmentedIcmpDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedFragmentedIcmpDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedFragmentedIcmpv6Discards - errors', () => {
      it('should have a postRouterClassifierReceivedFragmentedIcmpv6Discards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedFragmentedIcmpv6Discards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedFragmentedIcmpv6Discards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedFragmentedIcmpv6Discards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedFragmentedIcmpv6Discards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedFragmentedIcmpv6Discards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedIcmp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedIcmp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedIcmp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIcmp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIcmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIcmp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIcmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIcmpTtlDiscards - errors', () => {
      it('should have a postRouterClassifierReceivedIcmpTtlDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIcmpTtlDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIcmpTtlDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIcmpTtlDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIcmpTtlDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIcmpTtlDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedIcmpUnknownTypeDiscards - errors', () => {
      it('should have a postRouterReceivedIcmpUnknownTypeDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedIcmpUnknownTypeDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedIcmpUnknownTypeDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpUnknownTypeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedIcmpUnknownTypeDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpUnknownTypeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedIcmpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedIcmpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedIcmpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIcmpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIcmpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIcmpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIcmpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedIcmpv6HopLimitDiscards - errors', () => {
      it('should have a postRouterReceivedIcmpv6HopLimitDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedIcmpv6HopLimitDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedIcmpv6HopLimitDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6HopLimitDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedIcmpv6HopLimitDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6HopLimitDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedIcmpv6LinkLocalDiscards - errors', () => {
      it('should have a postRouterReceivedIcmpv6LinkLocalDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedIcmpv6LinkLocalDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedIcmpv6LinkLocalDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6LinkLocalDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedIcmpv6LinkLocalDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6LinkLocalDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedIcmpv6UnknownTypeDiscards - errors', () => {
      it('should have a postRouterReceivedIcmpv6UnknownTypeDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedIcmpv6UnknownTypeDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedIcmpv6UnknownTypeDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6UnknownTypeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedIcmpv6UnknownTypeDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedIcmpv6UnknownTypeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpInvalid - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpInvalid function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpInvalid === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpInvalid(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpInvalid', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpInvalid('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpInvalid', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpLeave - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpLeave function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpLeave === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpLeave(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpLeave', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpLeave('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpLeave', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpQuery - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpQuery function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpQuery === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpQuery(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpQuery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpQuery('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpQuery', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpReport - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpReport function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpReport === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpReport(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpReport('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpReport', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpReportV3 - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpReportV3 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpReportV3 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpReportV3(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpReportV3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpReportV3('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpReportV3', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIgmpTotal - errors', () => {
      it('should have a postRouterClassifierReceivedIgmpTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIgmpTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIgmpTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIgmpTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedInvalidHeaderLength - errors', () => {
      it('should have a postRouterClassifierReceivedInvalidHeaderLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedInvalidHeaderLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedInvalidHeaderLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedInvalidHeaderLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedInvalidHeaderLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedInvalidHeaderLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedInvalidIpv6 - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedInvalidIpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedInvalidIpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedInvalidIpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedInvalidIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedInvalidIpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedInvalidIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedInvalidLength - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedInvalidLength function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedInvalidLength === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedInvalidLength(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedInvalidLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedInvalidLength('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedInvalidLength', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedIpv4 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedIpv4 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedIpv4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIpv4(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIpv4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIpv4('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIpv4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIpv4FabricFragmented - errors', () => {
      it('should have a postRouterClassifierReceivedIpv4FabricFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIpv4FabricFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIpv4FabricFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIpv4FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIpv4FabricFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIpv4FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedIpv4Fragmented - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedIpv4Fragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedIpv4Fragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIpv4Fragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIpv4Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIpv4Fragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIpv4Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedIpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedIpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedIpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedIpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedIpv6FabricFragmented - errors', () => {
      it('should have a postRouterClassifierReceivedIpv6FabricFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedIpv6FabricFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedIpv6FabricFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIpv6FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedIpv6FabricFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedIpv6FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedIpv6Fragmented - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedIpv6Fragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedIpv6Fragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIpv6Fragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIpv6Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIpv6Fragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIpv6Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedIs - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedIs function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedIs === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIs(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIs', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedIs('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedIs', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedLacp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedLacp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedLacp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedLacp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedLacp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedLacp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedLacp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedNdp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedNdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedNdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedNdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedNdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedNdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedNdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedOspf - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedOspf function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedOspf(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedOspf('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedOtherLayer4Protocol - errors', () => {
      it('should have a postRouterClassifierReceivedOtherLayer4Protocol function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedOtherLayer4Protocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedOtherLayer4Protocol(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedOtherLayer4Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedOtherLayer4Protocol('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedOtherLayer4Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierReceivedAdvertisement - errors', () => {
      it('should have a postRouterProcessingClassifierReceivedAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierReceivedAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierReceivedAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierReceivedAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedSctp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedSctp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedSctp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedSctp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedSctp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedSctp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedSctp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierReceivedSpanningTreeDiscards - errors', () => {
      it('should have a postRouterClassifierReceivedSpanningTreeDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierReceivedSpanningTreeDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierReceivedSpanningTreeDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedSpanningTreeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierReceivedSpanningTreeDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierReceivedSpanningTreeDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedTcp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedTotal - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedTunnel - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedTunnel function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedTunnel === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTunnel(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTunnel', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedTunnel('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedTunnel', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedUdp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedUnknownLayer3ProtocolDiscards - errors', () => {
      it('should have a postRouterReceivedUnknownLayer3ProtocolDiscards function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedUnknownLayer3ProtocolDiscards === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedUnknownLayer3ProtocolDiscards(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedUnknownLayer3ProtocolDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedUnknownLayer3ProtocolDiscards('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedUnknownLayer3ProtocolDiscards', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedVlan - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedVlan function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedVlan === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedVlan(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedVlan', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedVlan('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedVlan', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierReceivedVrrp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierReceivedVrrp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierReceivedVrrp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedVrrp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedVrrp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierReceivedVrrp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierReceivedVrrp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentArp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentArp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentArp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentArp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentArp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentArp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentArp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentDhcp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentDhcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentDhcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentDhcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentDhcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentDhcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentDhcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentDhcpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentDhcpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentDhcpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentDhcpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentDhcpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentDhcpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentDhcpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentIcmp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentIcmp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentIcmp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIcmp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIcmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIcmp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIcmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentIcmpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentIcmpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentIcmpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIcmpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIcmpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIcmpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIcmpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentIgmp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentIgmp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentIgmp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIgmp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIgmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIgmp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIgmp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentIpv4 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentIpv4 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentIpv4 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIpv4(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIpv4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIpv4('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIpv4', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierSentIpv4FabricFragmented - errors', () => {
      it('should have a postRouterClassifierSentIpv4FabricFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierSentIpv4FabricFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierSentIpv4FabricFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentIpv4FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierSentIpv4FabricFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentIpv4FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierSentIpv4Fragmented - errors', () => {
      it('should have a postRouterProcessingClassifierSentIpv4Fragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierSentIpv4Fragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierSentIpv4Fragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIpv4Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierSentIpv4Fragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIpv4Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentIpv6 - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentIpv6 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentIpv6 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIpv6(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentIpv6('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentIpv6', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierSentIpv6FabricFragmented - errors', () => {
      it('should have a postRouterClassifierSentIpv6FabricFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierSentIpv6FabricFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierSentIpv6FabricFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentIpv6FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierSentIpv6FabricFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentIpv6FabricFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierSentIpv6Fragmented - errors', () => {
      it('should have a postRouterProcessingClassifierSentIpv6Fragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierSentIpv6Fragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierSentIpv6Fragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIpv6Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierSentIpv6Fragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIpv6Fragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierSentIs - errors', () => {
      it('should have a postRouterProcessingClassifierSentIs function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierSentIs === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierSentIs(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIs', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierSentIs('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentIs', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentLacp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentLacp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentLacp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentLacp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentLacp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentLacp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentLacp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentNdp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentNdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentNdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentNdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentNdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentNdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentNdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentOspf - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentOspf function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentOspf === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentOspf(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentOspf('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentOspf', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierSentOtherLayer3Protocol - errors', () => {
      it('should have a postRouterClassifierSentOtherLayer3Protocol function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierSentOtherLayer3Protocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierSentOtherLayer3Protocol(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentOtherLayer3Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierSentOtherLayer3Protocol('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentOtherLayer3Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClassifierSentOtherLayer4Protocol - errors', () => {
      it('should have a postRouterClassifierSentOtherLayer4Protocol function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClassifierSentOtherLayer4Protocol === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClassifierSentOtherLayer4Protocol(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentOtherLayer4Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClassifierSentOtherLayer4Protocol('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClassifierSentOtherLayer4Protocol', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingClassifierSentAdvertisement - errors', () => {
      it('should have a postRouterProcessingClassifierSentAdvertisement function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingClassifierSentAdvertisement === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingClassifierSentAdvertisement(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingClassifierSentAdvertisement('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingClassifierSentAdvertisement', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentSctp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentSctp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentSctp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentSctp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentSctp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentSctp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentSctp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentTcp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentTcp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentTcp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTcp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTcp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTcp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentTotal - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentTunnel - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentTunnel function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentTunnel === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTunnel(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTunnel', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentTunnel('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentTunnel', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentUdp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentUdp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentUdp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentUdp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentUdp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentUdp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentVlan - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentVlan function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentVlan === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentVlan(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentVlan', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentVlan('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentVlan', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingClassifierSentVrrp - errors', () => {
      it('should have a postRouterPacketProcessingClassifierSentVrrp function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingClassifierSentVrrp === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentVrrp(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentVrrp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingClassifierSentVrrp('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingClassifierSentVrrp', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsPacketProcessingDhcpDiscarded - errors', () => {
      it('should have a postRouterStatsPacketProcessingDhcpDiscarded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsPacketProcessingDhcpDiscarded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsPacketProcessingDhcpDiscarded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketProcessingDhcpDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsPacketProcessingDhcpDiscarded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsPacketProcessingDhcpDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpForwardToDerivedInterface - errors', () => {
      it('should have a postRouterDhcpForwardToDerivedInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpForwardToDerivedInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpForwardToDerivedInterface(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpForwardToDerivedInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpForwardToDerivedInterface('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpForwardToDerivedInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpForwardToOriginatingInterface - errors', () => {
      it('should have a postRouterDhcpForwardToOriginatingInterface function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpForwardToOriginatingInterface === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpForwardToOriginatingInterface(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpForwardToOriginatingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpForwardToOriginatingInterface('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpForwardToOriginatingInterface', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpReceivedForInternalApplication - errors', () => {
      it('should have a postRouterDhcpReceivedForInternalApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpReceivedForInternalApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpReceivedForInternalApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpReceivedForInternalApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpReceivedForInternalApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpReceivedForInternalApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEnqueueToDeferredRingFailure - errors', () => {
      it('should have a postRouterEnqueueToDeferredRingFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEnqueueToDeferredRingFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEnqueueToDeferredRingFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToDeferredRingFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEnqueueToDeferredRingFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToDeferredRingFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEnqueueToDeferredRingSuccess - errors', () => {
      it('should have a postRouterEnqueueToDeferredRingSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEnqueueToDeferredRingSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEnqueueToDeferredRingSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToDeferredRingSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEnqueueToDeferredRingSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToDeferredRingSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEnqueueToWorkerCoreFailure - errors', () => {
      it('should have a postRouterEnqueueToWorkerCoreFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEnqueueToWorkerCoreFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEnqueueToWorkerCoreFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToWorkerCoreFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEnqueueToWorkerCoreFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToWorkerCoreFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEnqueueToWorkerCoreSuccess - errors', () => {
      it('should have a postRouterEnqueueToWorkerCoreSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEnqueueToWorkerCoreSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEnqueueToWorkerCoreSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToWorkerCoreSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEnqueueToWorkerCoreSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEnqueueToWorkerCoreSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFibActionSummaryException - errors', () => {
      it('should have a postRouterProcessingFibActionSummaryException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFibActionSummaryException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFibActionSummaryException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummaryException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFibActionSummaryException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummaryException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFibActionSummaryFailure - errors', () => {
      it('should have a postRouterProcessingFibActionSummaryFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFibActionSummaryFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFibActionSummaryFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummaryFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFibActionSummaryFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummaryFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFibActionSummarySuccess - errors', () => {
      it('should have a postRouterProcessingFibActionSummarySuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFibActionSummarySuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFibActionSummarySuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummarySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFibActionSummarySuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFibActionSummarySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFlowActionSummaryException - errors', () => {
      it('should have a postRouterProcessingFlowActionSummaryException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFlowActionSummaryException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFlowActionSummaryException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummaryException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFlowActionSummaryException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummaryException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFlowActionSummaryFailure - errors', () => {
      it('should have a postRouterProcessingFlowActionSummaryFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFlowActionSummaryFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFlowActionSummaryFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummaryFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFlowActionSummaryFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummaryFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFlowActionSummarySuccess - errors', () => {
      it('should have a postRouterProcessingFlowActionSummarySuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFlowActionSummarySuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFlowActionSummarySuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummarySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFlowActionSummarySuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFlowActionSummarySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedDuplicateFirstFragment - errors', () => {
      it('should have a postRouterFragmentationReceivedDuplicateFirstFragment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedDuplicateFirstFragment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedDuplicateFirstFragment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedDuplicateFirstFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedDuplicateFirstFragment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedDuplicateFirstFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedDuplicateLastFragment - errors', () => {
      it('should have a postRouterFragmentationReceivedDuplicateLastFragment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedDuplicateLastFragment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedDuplicateLastFragment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedDuplicateLastFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedDuplicateLastFragment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedDuplicateLastFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedFailureToReassemble - errors', () => {
      it('should have a postRouterFragmentationReceivedFailureToReassemble function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedFailureToReassemble === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedFailureToReassemble(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFailureToReassemble', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedFailureToReassemble('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFailureToReassemble', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedFragmentChainsDiscarded - errors', () => {
      it('should have a postRouterFragmentationReceivedFragmentChainsDiscarded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedFragmentChainsDiscarded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsDiscarded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsDiscarded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsDiscarded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedFragmentChainsExceeded - errors', () => {
      it('should have a postRouterFragmentationReceivedFragmentChainsExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedFragmentChainsExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationReceivedFragmentChainsTimeout - errors', () => {
      it('should have a postRouterFragmentationReceivedFragmentChainsTimeout function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationReceivedFragmentChainsTimeout === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsTimeout(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationReceivedFragmentChainsTimeout('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationReceivedFragmentChainsTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFragmentationReceivedIncompleteFragments - errors', () => {
      it('should have a postRouterProcessingFragmentationReceivedIncompleteFragments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFragmentationReceivedIncompleteFragments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFragmentationReceivedIncompleteFragments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFragmentationReceivedIncompleteFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFragmentationReceivedIncompleteFragments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFragmentationReceivedIncompleteFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedInvalidLengthFirstFragment - errors', () => {
      it('should have a postRouterReceivedInvalidLengthFirstFragment function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedInvalidLengthFirstFragment === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedInvalidLengthFirstFragment(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidLengthFirstFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedInvalidLengthFirstFragment('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidLengthFirstFragment', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingFragmentationReceivedSuccessfullyReassembled - errors', () => {
      it('should have a postRouterProcessingFragmentationReceivedSuccessfullyReassembled function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingFragmentationReceivedSuccessfullyReassembled === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingFragmentationReceivedSuccessfullyReassembled(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFragmentationReceivedSuccessfullyReassembled', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingFragmentationReceivedSuccessfullyReassembled('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingFragmentationReceivedSuccessfullyReassembled', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentFragmentCreationFailure - errors', () => {
      it('should have a postRouterFragmentationSentFragmentCreationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentFragmentCreationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentFragmentCreationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentFragmentCreationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentFragmentCreationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentFragmentCreationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentIpv4DontFragmentDrop - errors', () => {
      it('should have a postRouterSentIpv4DontFragmentDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentIpv4DontFragmentDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentIpv4DontFragmentDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv4DontFragmentDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentIpv4DontFragmentDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv4DontFragmentDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentIpv4FabricFragments - errors', () => {
      it('should have a postRouterFragmentationSentIpv4FabricFragments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentIpv4FabricFragments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentIpv4FabricFragments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv4FabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentIpv4FabricFragments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv4FabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentIpv4NonFabricFragments - errors', () => {
      it('should have a postRouterSentIpv4NonFabricFragments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentIpv4NonFabricFragments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentIpv4NonFabricFragments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv4NonFabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentIpv4NonFabricFragments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv4NonFabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentIpv4PacketsFragmented - errors', () => {
      it('should have a postRouterFragmentationSentIpv4PacketsFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentIpv4PacketsFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentIpv4PacketsFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv4PacketsFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentIpv4PacketsFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv4PacketsFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentIpv6FabricFragments - errors', () => {
      it('should have a postRouterFragmentationSentIpv6FabricFragments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentIpv6FabricFragments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentIpv6FabricFragments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv6FabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentIpv6FabricFragments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv6FabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentIpv6MtuExceededDrop - errors', () => {
      it('should have a postRouterSentIpv6MtuExceededDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentIpv6MtuExceededDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentIpv6MtuExceededDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv6MtuExceededDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentIpv6MtuExceededDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv6MtuExceededDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentIpv6NonFabricFragments - errors', () => {
      it('should have a postRouterSentIpv6NonFabricFragments function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentIpv6NonFabricFragments === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentIpv6NonFabricFragments(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv6NonFabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentIpv6NonFabricFragments('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentIpv6NonFabricFragments', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentIpv6PacketsFragmented - errors', () => {
      it('should have a postRouterFragmentationSentIpv6PacketsFragmented function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentIpv6PacketsFragmented === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentIpv6PacketsFragmented(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv6PacketsFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentIpv6PacketsFragmented('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentIpv6PacketsFragmented', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFragmentationSentUnknownL2Version - errors', () => {
      it('should have a postRouterFragmentationSentUnknownL2Version function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFragmentationSentUnknownL2Version === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFragmentationSentUnknownL2Version(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentUnknownL2Version', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFragmentationSentUnknownL2Version('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFragmentationSentUnknownL2Version', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupAccessPolicyTableAllow - errors', () => {
      it('should have a postRouterLookupAccessPolicyTableAllow function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupAccessPolicyTableAllow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableAllow(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableAllow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableAllow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableAllow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupAccessPolicyTableDeny - errors', () => {
      it('should have a postRouterLookupAccessPolicyTableDeny function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupAccessPolicyTableDeny === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableDeny(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableDeny('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupAccessPolicyTableFailure - errors', () => {
      it('should have a postRouterLookupAccessPolicyTableFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupAccessPolicyTableFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupAccessPolicyTableMiss - errors', () => {
      it('should have a postRouterLookupAccessPolicyTableMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupAccessPolicyTableMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupAccessPolicyTableMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupAccessPolicyTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFibTableDeny - errors', () => {
      it('should have a postRouterProcessingLookupFibTableDeny function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFibTableDeny === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFibTableDeny(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFibTableDeny('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableDeny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFibTableFailure - errors', () => {
      it('should have a postRouterProcessingLookupFibTableFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFibTableFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFibTableFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFibTableFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFibTableHit - errors', () => {
      it('should have a postRouterProcessingLookupFibTableHit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFibTableHit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFibTableHit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFibTableHit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFibTableMiss - errors', () => {
      it('should have a postRouterProcessingLookupFibTableMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFibTableMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFibTableMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFibTableMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFibTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFlowTableHit - errors', () => {
      it('should have a postRouterProcessingLookupFlowTableHit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFlowTableHit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableHit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableHit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFlowTableInvalidated - errors', () => {
      it('should have a postRouterProcessingLookupFlowTableInvalidated function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFlowTableInvalidated === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableInvalidated(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableInvalidated', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableInvalidated('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableInvalidated', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupFlowTableMiss - errors', () => {
      it('should have a postRouterProcessingLookupFlowTableMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupFlowTableMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupFlowTableMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupFlowTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupIcmpTableIpv4Allow - errors', () => {
      it('should have a postRouterLookupIcmpTableIpv4Allow function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupIcmpTableIpv4Allow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv4Allow(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv4Allow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv4Allow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv4Allow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupIcmpTableIpv4Deny - errors', () => {
      it('should have a postRouterLookupIcmpTableIpv4Deny function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupIcmpTableIpv4Deny === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv4Deny(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv4Deny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv4Deny('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv4Deny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupIcmpTableIpv6Allow - errors', () => {
      it('should have a postRouterLookupIcmpTableIpv6Allow function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupIcmpTableIpv6Allow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv6Allow(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv6Allow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv6Allow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv6Allow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupIcmpTableIpv6Deny - errors', () => {
      it('should have a postRouterLookupIcmpTableIpv6Deny function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupIcmpTableIpv6Deny === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv6Deny(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv6Deny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupIcmpTableIpv6Deny('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupIcmpTableIpv6Deny', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentEnableMetadataAllocationFailure - errors', () => {
      it('should have a postRouterSentEnableMetadataAllocationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentEnableMetadataAllocationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentEnableMetadataAllocationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentEnableMetadataAllocationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataAllocationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentEnableMetadataCapacityFailure - errors', () => {
      it('should have a postRouterSentEnableMetadataCapacityFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentEnableMetadataCapacityFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentEnableMetadataCapacityFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataCapacityFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentEnableMetadataCapacityFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataCapacityFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSentEnableMetadataHmacFailure - errors', () => {
      it('should have a postRouterSentEnableMetadataHmacFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSentEnableMetadataHmacFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSentEnableMetadataHmacFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataHmacFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSentEnableMetadataHmacFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSentEnableMetadataHmacFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupSentEnableMetadataSuccess - errors', () => {
      it('should have a postRouterLookupSentEnableMetadataSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupSentEnableMetadataSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupSentEnableMetadataSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupSentEnableMetadataSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupSentEnableMetadataSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupSentEnableMetadataSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupTenantTableFailure - errors', () => {
      it('should have a postRouterProcessingLookupTenantTableFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupTenantTableFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupTenantTableHit - errors', () => {
      it('should have a postRouterProcessingLookupTenantTableHit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupTenantTableHit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableHit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableHit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingLookupTenantTableMiss - errors', () => {
      it('should have a postRouterProcessingLookupTenantTableMiss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingLookupTenantTableMiss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableMiss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingLookupTenantTableMiss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingLookupTenantTableMiss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingReceivedControlSuccess - errors', () => {
      it('should have a postRouterPacketProcessingReceivedControlSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingReceivedControlSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingReceivedControlSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedControlSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingReceivedControlSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedControlSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingReceivedInjectedSuccess - errors', () => {
      it('should have a postRouterPacketProcessingReceivedInjectedSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingReceivedInjectedSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInjectedSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInjectedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInjectedSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInjectedSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingReceivedInterfaceStandby - errors', () => {
      it('should have a postRouterPacketProcessingReceivedInterfaceStandby function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingReceivedInterfaceStandby === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInterfaceStandby(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInterfaceStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInterfaceStandby('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInterfaceStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingReceivedInterfaceStandbyDrop - errors', () => {
      it('should have a postRouterProcessingReceivedInterfaceStandbyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingReceivedInterfaceStandbyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingReceivedInterfaceStandbyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingReceivedInterfaceStandbyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingReceivedInterfaceStandbyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingReceivedInterfaceStandbyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingReceivedInterfaceSuccess - errors', () => {
      it('should have a postRouterPacketProcessingReceivedInterfaceSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingReceivedInterfaceSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInterfaceSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInterfaceSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingReceivedInterfaceSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingReceivedInterfaceSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingSentInterfaceFailure - errors', () => {
      it('should have a postRouterPacketProcessingSentInterfaceFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingSentInterfaceFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingSentInterfaceRetry - errors', () => {
      it('should have a postRouterPacketProcessingSentInterfaceRetry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingSentInterfaceRetry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceRetry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceRetry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingSentInterfaceStandby - errors', () => {
      it('should have a postRouterPacketProcessingSentInterfaceStandby function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingSentInterfaceStandby === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceStandby(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceStandby('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingSentInterfaceStandbyDrop - errors', () => {
      it('should have a postRouterProcessingSentInterfaceStandbyDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingSentInterfaceStandbyDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingSentInterfaceStandbyDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingSentInterfaceStandbyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingSentInterfaceStandbyDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingSentInterfaceStandbyDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketProcessingSentInterfaceSuccess - errors', () => {
      it('should have a postRouterPacketProcessingSentInterfaceSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketProcessingSentInterfaceSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketProcessingSentInterfaceSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketProcessingSentInterfaceSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterProcessingSentInterfaceUnconfiguredDrop - errors', () => {
      it('should have a postRouterProcessingSentInterfaceUnconfiguredDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterProcessingSentInterfaceUnconfiguredDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterProcessingSentInterfaceUnconfiguredDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingSentInterfaceUnconfiguredDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterProcessingSentInterfaceUnconfiguredDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterProcessingSentInterfaceUnconfiguredDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedNotificationsDuration - errors', () => {
      it('should have a postRouterClientReceivedNotificationsDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedNotificationsDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedNotificationsDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedNotificationsDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedNotificationsError - errors', () => {
      it('should have a postRouterClientReceivedNotificationsError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedNotificationsError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedNotificationsError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedNotificationsError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedNotificationsTotal - errors', () => {
      it('should have a postRouterClientReceivedNotificationsTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedNotificationsTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedNotificationsTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedNotificationsTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedNotificationsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedResponsesDuration - errors', () => {
      it('should have a postRouterClientReceivedResponsesDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedResponsesDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedResponsesDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedResponsesDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedResponsesError - errors', () => {
      it('should have a postRouterClientReceivedResponsesError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedResponsesError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedResponsesError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedResponsesError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedResponsesTotal - errors', () => {
      it('should have a postRouterClientReceivedResponsesTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedResponsesTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedResponsesTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedResponsesTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedResponsesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientReceivedUnknownTotal - errors', () => {
      it('should have a postRouterClientReceivedUnknownTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientReceivedUnknownTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientReceivedUnknownTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedUnknownTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientReceivedUnknownTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientReceivedUnknownTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientSentRequestsError - errors', () => {
      it('should have a postRouterClientSentRequestsError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientSentRequestsError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientSentRequestsError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientSentRequestsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientSentRequestsError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientSentRequestsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientSentRequestsTotal - errors', () => {
      it('should have a postRouterClientSentRequestsTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientSentRequestsTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientSentRequestsTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientSentRequestsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientSentRequestsTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientSentRequestsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTreeClientTransactionsServerLatency - errors', () => {
      it('should have a postRouterTreeClientTransactionsServerLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTreeClientTransactionsServerLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTreeClientTransactionsServerLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTreeClientTransactionsServerLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTreeClientTransactionsServerLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTreeClientTransactionsServerLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReceivedRequestsAverage - errors', () => {
      it('should have a postRouterServerReceivedRequestsAverage function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReceivedRequestsAverage === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReceivedRequestsAverage(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsAverage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReceivedRequestsAverage('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsAverage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReceivedRequestsError - errors', () => {
      it('should have a postRouterServerReceivedRequestsError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReceivedRequestsError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReceivedRequestsError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReceivedRequestsError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReceivedRequestsIgnored - errors', () => {
      it('should have a postRouterServerReceivedRequestsIgnored function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReceivedRequestsIgnored === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReceivedRequestsIgnored(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsIgnored', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReceivedRequestsIgnored('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsIgnored', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReceivedRequestsTotal - errors', () => {
      it('should have a postRouterServerReceivedRequestsTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReceivedRequestsTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReceivedRequestsTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReceivedRequestsTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReceivedRequestsTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerSentResponsesError - errors', () => {
      it('should have a postRouterServerSentResponsesError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerSentResponsesError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerSentResponsesError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerSentResponsesError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerSentResponsesError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerSentResponsesError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerSentResponsesTotal - errors', () => {
      it('should have a postRouterServerSentResponsesTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerSentResponsesTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerSentResponsesTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerSentResponsesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerSentResponsesTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerSentResponsesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsConnectDuration - errors', () => {
      it('should have a postRouterServerTransactionsConnectDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsConnectDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsConnectDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsConnectDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsConnectError - errors', () => {
      it('should have a postRouterServerTransactionsConnectError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsConnectError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsConnectError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsConnectError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsConnectProcessed - errors', () => {
      it('should have a postRouterServerTransactionsConnectProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsConnectProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsConnectProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsConnectProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsConnectProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsDisconnectDuration - errors', () => {
      it('should have a postRouterServerTransactionsDisconnectDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsDisconnectDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsDisconnectError - errors', () => {
      it('should have a postRouterServerTransactionsDisconnectError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsDisconnectError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsDisconnectProcessed - errors', () => {
      it('should have a postRouterServerTransactionsDisconnectProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsDisconnectProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsDisconnectProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsDisconnectProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsListDuration - errors', () => {
      it('should have a postRouterServerTransactionsListDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsListDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsListDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsListDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsListError - errors', () => {
      it('should have a postRouterServerTransactionsListError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsListError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsListError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsListError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsListProcessed - errors', () => {
      it('should have a postRouterServerTransactionsListProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsListProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsListProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsListProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsListProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPingDuration - errors', () => {
      it('should have a postRouterServerTransactionsPingDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPingDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPingDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPingDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPingError - errors', () => {
      it('should have a postRouterServerTransactionsPingError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPingError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPingError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPingError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPingProcessed - errors', () => {
      it('should have a postRouterServerTransactionsPingProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPingProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPingProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPingProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPingProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPublishDuration - errors', () => {
      it('should have a postRouterServerTransactionsPublishDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPublishDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPublishDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPublishDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPublishError - errors', () => {
      it('should have a postRouterServerTransactionsPublishError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPublishError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPublishError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPublishError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsPublishProcessed - errors', () => {
      it('should have a postRouterServerTransactionsPublishProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsPublishProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsPublishProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsPublishProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsPublishProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsReadDuration - errors', () => {
      it('should have a postRouterServerTransactionsReadDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsReadDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsReadDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsReadDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsReadError - errors', () => {
      it('should have a postRouterServerTransactionsReadError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsReadError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsReadError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsReadError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsReadProcessed - errors', () => {
      it('should have a postRouterServerTransactionsReadProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsReadProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsReadProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsReadProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsReadProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsRemoveDuration - errors', () => {
      it('should have a postRouterServerTransactionsRemoveDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsRemoveDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsRemoveDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsRemoveDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsRemoveError - errors', () => {
      it('should have a postRouterServerTransactionsRemoveError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsRemoveError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsRemoveError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsRemoveError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsRemoveProcessed - errors', () => {
      it('should have a postRouterServerTransactionsRemoveProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsRemoveProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsRemoveProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsRemoveProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsRemoveProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUnsubscribeDuration - errors', () => {
      it('should have a postRouterServerTransactionsUnsubscribeDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUnsubscribeDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUnsubscribeError - errors', () => {
      it('should have a postRouterServerTransactionsUnsubscribeError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUnsubscribeError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUnsubscribeProcessed - errors', () => {
      it('should have a postRouterServerTransactionsUnsubscribeProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUnsubscribeProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUnsubscribeProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUnsubscribeProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUpdateDuration - errors', () => {
      it('should have a postRouterServerTransactionsUpdateDuration function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUpdateDuration === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUpdateDuration(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUpdateDuration('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateDuration', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUpdateError - errors', () => {
      it('should have a postRouterServerTransactionsUpdateError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUpdateError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUpdateError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUpdateError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerTransactionsUpdateProcessed - errors', () => {
      it('should have a postRouterServerTransactionsUpdateProcessed function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerTransactionsUpdateProcessed === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerTransactionsUpdateProcessed(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerTransactionsUpdateProcessed('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerTransactionsUpdateProcessed', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPerformanceMonitoringPeerPathJitter - errors', () => {
      it('should have a postRouterPerformanceMonitoringPeerPathJitter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPerformanceMonitoringPeerPathJitter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathJitter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathJitter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathJitter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathJitter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPerformanceMonitoringPeerPathLatency - errors', () => {
      it('should have a postRouterPerformanceMonitoringPeerPathLatency function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPerformanceMonitoringPeerPathLatency === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathLatency(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathLatency('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathLatency', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPerformanceMonitoringPeerPathLoss - errors', () => {
      it('should have a postRouterPerformanceMonitoringPeerPathLoss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPerformanceMonitoringPeerPathLoss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathLoss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathLoss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathLoss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathLoss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPerformanceMonitoringPeerPathMos - errors', () => {
      it('should have a postRouterPerformanceMonitoringPeerPathMos function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPerformanceMonitoringPeerPathMos === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathMos(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathMos', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPerformanceMonitoringPeerPathMos('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPerformanceMonitoringPeerPathMos', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsProcessCpuUsage - errors', () => {
      it('should have a postRouterStatsProcessCpuUsage function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsProcessCpuUsage === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsProcessCpuUsage(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessCpuUsage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsProcessCpuUsage('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessCpuUsage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsProcessMemoryRss - errors', () => {
      it('should have a postRouterStatsProcessMemoryRss function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsProcessMemoryRss === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsProcessMemoryRss(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessMemoryRss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsProcessMemoryRss('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessMemoryRss', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsProcessMemoryVsz - errors', () => {
      it('should have a postRouterStatsProcessMemoryVsz function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsProcessMemoryVsz === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsProcessMemoryVsz(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessMemoryVsz', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsProcessMemoryVsz('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsProcessMemoryVsz', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerManagerRedisInactives - errors', () => {
      it('should have a postRouterServerManagerRedisInactives function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerManagerRedisInactives === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerManagerRedisInactives(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerManagerRedisInactives', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerManagerRedisInactives('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerManagerRedisInactives', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterManagerRedisServerNetworkFlaps - errors', () => {
      it('should have a postRouterManagerRedisServerNetworkFlaps function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterManagerRedisServerNetworkFlaps === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterManagerRedisServerNetworkFlaps(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterManagerRedisServerNetworkFlaps', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterManagerRedisServerNetworkFlaps('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterManagerRedisServerNetworkFlaps', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerManagerRedisSessionExpirations - errors', () => {
      it('should have a postRouterServerManagerRedisSessionExpirations function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerManagerRedisSessionExpirations === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerManagerRedisSessionExpirations(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerManagerRedisSessionExpirations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerManagerRedisSessionExpirations('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerManagerRedisSessionExpirations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyDatabaseConnectionActiveConnections - errors', () => {
      it('should have a postRouterRedundancyDatabaseConnectionActiveConnections function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyDatabaseConnectionActiveConnections === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionActiveConnections(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionActiveConnections', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionActiveConnections('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionActiveConnections', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyDatabaseConnectionClientDisconnect - errors', () => {
      it('should have a postRouterRedundancyDatabaseConnectionClientDisconnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyDatabaseConnectionClientDisconnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionClientDisconnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionClientDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionClientDisconnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionClientDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyDatabaseConnectionConnect - errors', () => {
      it('should have a postRouterStatsRedundancyDatabaseConnectionConnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyDatabaseConnectionConnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyDatabaseConnectionConnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyDatabaseConnectionConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyDatabaseConnectionConnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyDatabaseConnectionConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyDatabaseConnectionErrorDisconnect - errors', () => {
      it('should have a postRouterRedundancyDatabaseConnectionErrorDisconnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyDatabaseConnectionErrorDisconnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionErrorDisconnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionErrorDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionErrorDisconnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionErrorDisconnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyDatabaseConnectionErrors - errors', () => {
      it('should have a postRouterStatsRedundancyDatabaseConnectionErrors function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyDatabaseConnectionErrors === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyDatabaseConnectionErrors(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyDatabaseConnectionErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyDatabaseConnectionErrors('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyDatabaseConnectionErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyDatabaseConnectionFailedConnect - errors', () => {
      it('should have a postRouterRedundancyDatabaseConnectionFailedConnect function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyDatabaseConnectionFailedConnect === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionFailedConnect(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionFailedConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyDatabaseConnectionFailedConnect('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyDatabaseConnectionFailedConnect', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPortInitializationMarkerQueryPending - errors', () => {
      it('should have a postRouterPortInitializationMarkerQueryPending function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPortInitializationMarkerQueryPending === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPortInitializationMarkerQueryPending(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerQueryPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPortInitializationMarkerQueryPending('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerQueryPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPortInitializationMarkerRecordFound - errors', () => {
      it('should have a postRouterPortInitializationMarkerRecordFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPortInitializationMarkerRecordFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPortInitializationMarkerRecordFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerRecordFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPortInitializationMarkerRecordFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerRecordFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInitializationMarkerRecordNotFound - errors', () => {
      it('should have a postRouterInitializationMarkerRecordNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInitializationMarkerRecordNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInitializationMarkerRecordNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInitializationMarkerRecordNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInitializationMarkerRecordNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInitializationMarkerRecordNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInitializationMarkerRecordReadError - errors', () => {
      it('should have a postRouterInitializationMarkerRecordReadError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInitializationMarkerRecordReadError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInitializationMarkerRecordReadError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInitializationMarkerRecordReadError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInitializationMarkerRecordReadError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInitializationMarkerRecordReadError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPortInitializationMarkerRecordWrite - errors', () => {
      it('should have a postRouterPortInitializationMarkerRecordWrite function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPortInitializationMarkerRecordWrite === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPortInitializationMarkerRecordWrite(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerRecordWrite', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPortInitializationMarkerRecordWrite('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortInitializationMarkerRecordWrite', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyPortInitializationPendingQueries - errors', () => {
      it('should have a postRouterRedundancyPortInitializationPendingQueries function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyPortInitializationPendingQueries === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyPortInitializationPendingQueries(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyPortInitializationPendingQueries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyPortInitializationPendingQueries('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyPortInitializationPendingQueries', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterListRecordsRecordFound - errors', () => {
      it('should have a postRouterListRecordsRecordFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterListRecordsRecordFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterListRecordsRecordFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterListRecordsRecordFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterListRecordsRecordFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterListRecordsRecordFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordNotFound - errors', () => {
      it('should have a postRouterRecordsListRecordNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordParseError - errors', () => {
      it('should have a postRouterRecordsListRecordParseError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordParseError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordParseError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordParseError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordParseError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordParseError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordPopErrors - errors', () => {
      it('should have a postRouterRecordsListRecordPopErrors function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordPopErrors === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordPopErrors(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordPopErrors('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordPopFailures - errors', () => {
      it('should have a postRouterRecordsListRecordPopFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordPopFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordPopFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordPopFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordPopPending - errors', () => {
      it('should have a postRouterRecordsListRecordPopPending function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordPopPending === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordPopPending(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordPopPending('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRecordsListRecordPopSuccess - errors', () => {
      it('should have a postRouterRecordsListRecordPopSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRecordsListRecordPopSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRecordsListRecordPopSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRecordsListRecordPopSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRecordsListRecordPopSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterListRecordsRecordPush - errors', () => {
      it('should have a postRouterListRecordsRecordPush function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterListRecordsRecordPush === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterListRecordsRecordPush(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterListRecordsRecordPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterListRecordsRecordPush('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterListRecordsRecordPush', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancySessionDelete - errors', () => {
      it('should have a postRouterStatsRedundancySessionDelete function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancySessionDelete === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancySessionDelete(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancySessionDelete', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancySessionDelete('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancySessionDelete', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancySessionErrorsCacheTimeouts - errors', () => {
      it('should have a postRouterRedundancySessionErrorsCacheTimeouts function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancySessionErrorsCacheTimeouts === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancySessionErrorsCacheTimeouts(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsCacheTimeouts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancySessionErrorsCacheTimeouts('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsCacheTimeouts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancySessionErrorsDecodeFailures - errors', () => {
      it('should have a postRouterRedundancySessionErrorsDecodeFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancySessionErrorsDecodeFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancySessionErrorsDecodeFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsDecodeFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancySessionErrorsDecodeFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsDecodeFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsFibLookupError - errors', () => {
      it('should have a postRouterSessionErrorsFibLookupError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsFibLookupError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsFibLookupError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsFibLookupError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsFibLookupError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsFibLookupError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsInvalidBufferReceived - errors', () => {
      it('should have a postRouterSessionErrorsInvalidBufferReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsInvalidBufferReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsInvalidBufferReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsInvalidBufferReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsInvalidBufferReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsInvalidBufferReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsInvalidKey - errors', () => {
      it('should have a postRouterSessionErrorsInvalidKey function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsInvalidKey === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsInvalidKey(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsInvalidKey', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsInvalidKey('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsInvalidKey', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterErrorsNewSessionCreationFailure - errors', () => {
      it('should have a postRouterErrorsNewSessionCreationFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterErrorsNewSessionCreationFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterErrorsNewSessionCreationFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsNewSessionCreationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterErrorsNewSessionCreationFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterErrorsNewSessionCreationFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsNotFound - errors', () => {
      it('should have a postRouterSessionErrorsNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsUpdateFailures - errors', () => {
      it('should have a postRouterSessionErrorsUpdateFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsUpdateFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsUpdateFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsUpdateFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsUpdateFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsUpdateFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionErrorsSourceLookupError - errors', () => {
      it('should have a postRouterSessionErrorsSourceLookupError function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionErrorsSourceLookupError === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionErrorsSourceLookupError(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsSourceLookupError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionErrorsSourceLookupError('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionErrorsSourceLookupError', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancySessionErrorsUnsupportedFeature - errors', () => {
      it('should have a postRouterRedundancySessionErrorsUnsupportedFeature function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancySessionErrorsUnsupportedFeature === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancySessionErrorsUnsupportedFeature(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsUnsupportedFeature', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancySessionErrorsUnsupportedFeature('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionErrorsUnsupportedFeature', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionReadsCreateNew - errors', () => {
      it('should have a postRouterSessionReadsCreateNew function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionReadsCreateNew === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionReadsCreateNew(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsCreateNew', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionReadsCreateNew('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsCreateNew', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionReadsNetUnreachableSent - errors', () => {
      it('should have a postRouterSessionReadsNetUnreachableSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionReadsNetUnreachableSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionReadsNetUnreachableSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsNetUnreachableSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionReadsNetUnreachableSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsNetUnreachableSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancySessionReadsQueryPending - errors', () => {
      it('should have a postRouterRedundancySessionReadsQueryPending function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancySessionReadsQueryPending === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancySessionReadsQueryPending(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionReadsQueryPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancySessionReadsQueryPending('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancySessionReadsQueryPending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReadsQueryResultNotFound - errors', () => {
      it('should have a postRouterReadsQueryResultNotFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReadsQueryResultNotFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReadsQueryResultNotFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReadsQueryResultNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReadsQueryResultNotFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReadsQueryResultNotFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionReadsQueryResultSuccess - errors', () => {
      it('should have a postRouterSessionReadsQueryResultSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionReadsQueryResultSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionReadsQueryResultSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsQueryResultSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionReadsQueryResultSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsQueryResultSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionReadsQueryResultTimeout - errors', () => {
      it('should have a postRouterSessionReadsQueryResultTimeout function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionReadsQueryResultTimeout === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionReadsQueryResultTimeout(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsQueryResultTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionReadsQueryResultTimeout('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsQueryResultTimeout', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionReadsTcpResetSent - errors', () => {
      it('should have a postRouterSessionReadsTcpResetSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionReadsTcpResetSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionReadsTcpResetSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsTcpResetSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionReadsTcpResetSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionReadsTcpResetSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterCreatedInterNodeServicePaths - errors', () => {
      it('should have a postRouterCreatedInterNodeServicePaths function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterCreatedInterNodeServicePaths === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterCreatedInterNodeServicePaths(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterCreatedInterNodeServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterCreatedInterNodeServicePaths('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterCreatedInterNodeServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterCreatedInterServicePaths - errors', () => {
      it('should have a postRouterCreatedInterServicePaths function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterCreatedInterServicePaths === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterCreatedInterServicePaths(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterCreatedInterServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterCreatedInterServicePaths('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterCreatedInterServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReconstructionCreatedLocalServicePaths - errors', () => {
      it('should have a postRouterReconstructionCreatedLocalServicePaths function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReconstructionCreatedLocalServicePaths === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReconstructionCreatedLocalServicePaths(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReconstructionCreatedLocalServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReconstructionCreatedLocalServicePaths('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReconstructionCreatedLocalServicePaths', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDivertedToExternalProtocolAgent - errors', () => {
      it('should have a postRouterDivertedToExternalProtocolAgent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDivertedToExternalProtocolAgent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDivertedToExternalProtocolAgent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDivertedToExternalProtocolAgent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDivertedToExternalProtocolAgent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDivertedToExternalProtocolAgent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancySessionRefresh - errors', () => {
      it('should have a postRouterStatsRedundancySessionRefresh function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancySessionRefresh === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancySessionRefresh(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancySessionRefresh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancySessionRefresh('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancySessionRefresh', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionWritesErrorsEncodeFailures - errors', () => {
      it('should have a postRouterSessionWritesErrorsEncodeFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionWritesErrorsEncodeFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionWritesErrorsEncodeFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesErrorsEncodeFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionWritesErrorsEncodeFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesErrorsEncodeFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWritesErrorsFromDatabase - errors', () => {
      it('should have a postRouterWritesErrorsFromDatabase function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWritesErrorsFromDatabase === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWritesErrorsFromDatabase(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWritesErrorsFromDatabase', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWritesErrorsFromDatabase('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWritesErrorsFromDatabase', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionWritesWriteRequestSent - errors', () => {
      it('should have a postRouterSessionWritesWriteRequestSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionWritesWriteRequestSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionWritesWriteRequestSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesWriteRequestSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionWritesWriteRequestSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesWriteRequestSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionWritesWriteRequestSuccess - errors', () => {
      it('should have a postRouterSessionWritesWriteRequestSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionWritesWriteRequestSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionWritesWriteRequestSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesWriteRequestSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionWritesWriteRequestSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionWritesWriteRequestSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpAdvertisementsDropped - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpAdvertisementsDropped function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpAdvertisementsDropped === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsDropped(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsDropped('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsDropped', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpAdvertisementsReceived - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpAdvertisementsReceived function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpAdvertisementsReceived === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsReceived(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsReceived('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsReceived', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpAdvertisementsSent - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpAdvertisementsSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpAdvertisementsSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpAdvertisementsSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpAdvertisementsSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpBecomeBackup - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpBecomeBackup function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpBecomeBackup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpBecomeBackup(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpBecomeBackup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpBecomeBackup('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpBecomeBackup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpBecomeMaster - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpBecomeMaster function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpBecomeMaster === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpBecomeMaster(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpBecomeMaster', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpBecomeMaster('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpBecomeMaster', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyVrrpBecomeMasterFailure - errors', () => {
      it('should have a postRouterRedundancyVrrpBecomeMasterFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyVrrpBecomeMasterFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyVrrpBecomeMasterFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpBecomeMasterFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyVrrpBecomeMasterFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpBecomeMasterFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRedundancyVrrpReceivedInvalid - errors', () => {
      it('should have a postRouterStatsRedundancyVrrpReceivedInvalid function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRedundancyVrrpReceivedInvalid === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpReceivedInvalid(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpReceivedInvalid', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRedundancyVrrpReceivedInvalid('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRedundancyVrrpReceivedInvalid', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyVrrpReceivedInvalidTtl - errors', () => {
      it('should have a postRouterRedundancyVrrpReceivedInvalidTtl function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyVrrpReceivedInvalidTtl === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidTtl(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidTtl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidTtl('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidTtl', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyVrrpReceivedInvalidType - errors', () => {
      it('should have a postRouterRedundancyVrrpReceivedInvalidType function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyVrrpReceivedInvalidType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidType(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidType('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRedundancyVrrpReceivedInvalidVersion - errors', () => {
      it('should have a postRouterRedundancyVrrpReceivedInvalidVersion function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRedundancyVrrpReceivedInvalidVersion === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidVersion(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRedundancyVrrpReceivedInvalidVersion('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRedundancyVrrpReceivedInvalidVersion', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRegisteredServicesLocal - errors', () => {
      it('should have a postRouterStatsRegisteredServicesLocal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRegisteredServicesLocal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRegisteredServicesLocal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesLocal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRegisteredServicesLocal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesLocal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRegisteredServicesRemote - errors', () => {
      it('should have a postRouterStatsRegisteredServicesRemote function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRegisteredServicesRemote === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRegisteredServicesRemote(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRegisteredServicesRemote('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesRemote', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRegisteredServicesTotal - errors', () => {
      it('should have a postRouterStatsRegisteredServicesTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRegisteredServicesTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRegisteredServicesTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRegisteredServicesTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRegisteredServicesTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingAgentAddRoute - errors', () => {
      it('should have a postRouterStatsRoutingAgentAddRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingAgentAddRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingAgentAddRoute(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentAddRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingAgentAddRoute('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentAddRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingAgentDeleteRoute - errors', () => {
      it('should have a postRouterStatsRoutingAgentDeleteRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingAgentDeleteRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingAgentDeleteRoute(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentDeleteRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingAgentDeleteRoute('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentDeleteRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRoutingAgentFibInstallErrors - errors', () => {
      it('should have a postRouterRoutingAgentFibInstallErrors function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRoutingAgentFibInstallErrors === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRoutingAgentFibInstallErrors(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingAgentFibInstallErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRoutingAgentFibInstallErrors('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingAgentFibInstallErrors', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingAgentFibMeter - errors', () => {
      it('should have a postRouterStatsRoutingAgentFibMeter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingAgentFibMeter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingAgentFibMeter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentFibMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingAgentFibMeter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentFibMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingAgentInstallRoute - errors', () => {
      it('should have a postRouterStatsRoutingAgentInstallRoute function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingAgentInstallRoute === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingAgentInstallRoute(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentInstallRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingAgentInstallRoute('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingAgentInstallRoute', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingErrorsConnection - errors', () => {
      it('should have a postRouterStatsRoutingErrorsConnection function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingErrorsConnection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingErrorsConnection(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingErrorsConnection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingErrorsConnection('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingErrorsConnection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRoutingRouteUpdatesReceivedAdd - errors', () => {
      it('should have a postRouterRoutingRouteUpdatesReceivedAdd function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRoutingRouteUpdatesReceivedAdd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedAdd(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedAdd('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRouteUpdatesReceivedClearFib - errors', () => {
      it('should have a postRouterRouteUpdatesReceivedClearFib function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRouteUpdatesReceivedClearFib === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedClearFib(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedClearFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedClearFib('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedClearFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRoutingRouteUpdatesReceivedDelete - errors', () => {
      it('should have a postRouterRoutingRouteUpdatesReceivedDelete function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRoutingRouteUpdatesReceivedDelete === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedDelete(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedDelete', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedDelete('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedDelete', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdatesReceivedEndOfFib - errors', () => {
      it('should have a postRouterUpdatesReceivedEndOfFib function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdatesReceivedEndOfFib === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdatesReceivedEndOfFib(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdatesReceivedEndOfFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdatesReceivedEndOfFib('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdatesReceivedEndOfFib', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRouteUpdatesReceivedIamActive - errors', () => {
      it('should have a postRouterRouteUpdatesReceivedIamActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRouteUpdatesReceivedIamActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedIamActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedIamActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedIamActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedIamActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRouteUpdatesReceivedIamStandby - errors', () => {
      it('should have a postRouterRouteUpdatesReceivedIamStandby function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRouteUpdatesReceivedIamStandby === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedIamStandby(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedIamStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRouteUpdatesReceivedIamStandby('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRouteUpdatesReceivedIamStandby', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRoutingRouteUpdatesReceivedTotal - errors', () => {
      it('should have a postRouterRoutingRouteUpdatesReceivedTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRoutingRouteUpdatesReceivedTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesReceivedTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesReceivedTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRoutingRouteUpdatesSentTotal - errors', () => {
      it('should have a postRouterRoutingRouteUpdatesSentTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRoutingRouteUpdatesSentTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesSentTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesSentTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRoutingRouteUpdatesSentTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRoutingRouteUpdatesSentTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsRoutingRoutes - errors', () => {
      it('should have a postRouterStatsRoutingRoutes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsRoutingRoutes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsRoutingRoutes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingRoutes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsRoutingRoutes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsRoutingRoutes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaDhcpUnknownDroppedPackets - errors', () => {
      it('should have a postRouterAreaDhcpUnknownDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaDhcpUnknownDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaDhcpUnknownDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaDhcpUnknownDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaDhcpUnknownDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaDhcpUnknownDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientAckDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientAckDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientAckDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientAckDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientAckDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientAckProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientAckProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientAckProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientAckProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientAckProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientAckSentPackets - errors', () => {
      it('should have a postRouterV4ClientAckSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientAckSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientAckSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientAckSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientAckSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDeclineDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientDeclineDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDeclineDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDeclineDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDeclineDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDeclineProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientDeclineProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDeclineProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDeclineProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDeclineProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDeclineSentPackets - errors', () => {
      it('should have a postRouterV4ClientDeclineSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDeclineSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDeclineSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDeclineSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDeclineSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDiscoverDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientDiscoverDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDiscoverDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDiscoverDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDiscoverDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDiscoverProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientDiscoverProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDiscoverProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDiscoverProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDiscoverProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientDiscoverSentPackets - errors', () => {
      it('should have a postRouterV4ClientDiscoverSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientDiscoverSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientDiscoverSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientDiscoverSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientDiscoverSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpV4ClientDroppedPackets - errors', () => {
      it('should have a postRouterDhcpV4ClientDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpV4ClientDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpV4ClientDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpV4ClientDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientNackDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientNackDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientNackDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientNackDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientNackDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientNackProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientNackProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientNackProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientNackProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientNackProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientNackSentPackets - errors', () => {
      it('should have a postRouterV4ClientNackSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientNackSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientNackSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientNackSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientNackSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientOfferDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientOfferDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientOfferDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientOfferDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientOfferDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientOfferProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientOfferProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientOfferProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientOfferProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientOfferProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientOfferSentPackets - errors', () => {
      it('should have a postRouterV4ClientOfferSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientOfferSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientOfferSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientOfferSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientOfferSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpV4ClientProcessedPackets - errors', () => {
      it('should have a postRouterDhcpV4ClientProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpV4ClientProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpV4ClientProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpV4ClientProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientReleaseDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientReleaseDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientReleaseDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientReleaseDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientReleaseDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientReleaseProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientReleaseProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientReleaseProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientReleaseProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientReleaseProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientReleaseSentPackets - errors', () => {
      it('should have a postRouterV4ClientReleaseSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientReleaseSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientReleaseSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientReleaseSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientReleaseSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientRequestDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientRequestDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientRequestDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientRequestDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientRequestDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientRequestProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientRequestProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientRequestProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientRequestProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientRequestProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientRequestSentPackets - errors', () => {
      it('should have a postRouterV4ClientRequestSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientRequestSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientRequestSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientRequestSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientRequestSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDhcpV4ClientSentPackets - errors', () => {
      it('should have a postRouterDhcpV4ClientSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDhcpV4ClientSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDhcpV4ClientSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDhcpV4ClientSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDhcpV4ClientSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientUnknownDroppedPackets - errors', () => {
      it('should have a postRouterV4ClientUnknownDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientUnknownDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientUnknownDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientUnknownDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientUnknownProcessedPackets - errors', () => {
      it('should have a postRouterV4ClientUnknownProcessedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientUnknownProcessedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientUnknownProcessedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientUnknownProcessedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownProcessedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4ClientUnknownSentPackets - errors', () => {
      it('should have a postRouterV4ClientUnknownSentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4ClientUnknownSentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4ClientUnknownSentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4ClientUnknownSentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4ClientUnknownSentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4RelayReceivedInvalidPackets - errors', () => {
      it('should have a postRouterV4RelayReceivedInvalidPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4RelayReceivedInvalidPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4RelayReceivedInvalidPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedInvalidPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4RelayReceivedInvalidPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedInvalidPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRelayReceivedInvalidRequestPackets - errors', () => {
      it('should have a postRouterRelayReceivedInvalidRequestPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRelayReceivedInvalidRequestPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRelayReceivedInvalidRequestPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRelayReceivedInvalidRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRelayReceivedInvalidRequestPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRelayReceivedInvalidRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRelayReceivedInvalidResponsePackets - errors', () => {
      it('should have a postRouterRelayReceivedInvalidResponsePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRelayReceivedInvalidResponsePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRelayReceivedInvalidResponsePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRelayReceivedInvalidResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRelayReceivedInvalidResponsePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRelayReceivedInvalidResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4RelayReceivedRequestPackets - errors', () => {
      it('should have a postRouterV4RelayReceivedRequestPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4RelayReceivedRequestPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4RelayReceivedRequestPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4RelayReceivedRequestPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4RelayReceivedResponsePackets - errors', () => {
      it('should have a postRouterV4RelayReceivedResponsePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4RelayReceivedResponsePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4RelayReceivedResponsePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4RelayReceivedResponsePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayReceivedResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4RelayTransmittedRequestPackets - errors', () => {
      it('should have a postRouterV4RelayTransmittedRequestPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4RelayTransmittedRequestPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4RelayTransmittedRequestPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayTransmittedRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4RelayTransmittedRequestPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayTransmittedRequestPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterV4RelayTransmittedResponsePackets - errors', () => {
      it('should have a postRouterV4RelayTransmittedResponsePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterV4RelayTransmittedResponsePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterV4RelayTransmittedResponsePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayTransmittedResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterV4RelayTransmittedResponsePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterV4RelayTransmittedResponsePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientRequestReceivedFromApplication - errors', () => {
      it('should have a postRouterClientRequestReceivedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientRequestReceivedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientRequestReceivedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientRequestReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientRequestReceivedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientRequestReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterRequestSentToFastLane - errors', () => {
      it('should have a postRouterRequestSentToFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterRequestSentToFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterRequestSentToFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRequestSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterRequestSentToFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterRequestSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAdvertisementDroppedFromApplication - errors', () => {
      it('should have a postRouterAdvertisementDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAdvertisementDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAdvertisementDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAdvertisementDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAdvertisementReceivedFromFastLane - errors', () => {
      it('should have a postRouterAdvertisementReceivedFromFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAdvertisementReceivedFromFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAdvertisementReceivedFromFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAdvertisementReceivedFromFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAdvertisementSentToApplication - errors', () => {
      it('should have a postRouterAdvertisementSentToApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAdvertisementSentToApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAdvertisementSentToApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementSentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAdvertisementSentToApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementSentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSolicitationReceivedFromApplication - errors', () => {
      it('should have a postRouterSolicitationReceivedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSolicitationReceivedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSolicitationReceivedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSolicitationReceivedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSolicitationSentToFastLane - errors', () => {
      it('should have a postRouterSolicitationSentToFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSolicitationSentToFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSolicitationSentToFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSolicitationSentToFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReplyDroppedFromApplication - errors', () => {
      it('should have a postRouterServerReplyDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReplyDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReplyDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplyDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReplyDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplyDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReplyReceivedFromFastLane - errors', () => {
      it('should have a postRouterReplyReceivedFromFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReplyReceivedFromFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReplyReceivedFromFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReplyReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReplyReceivedFromFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReplyReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReplySentToApplication - errors', () => {
      it('should have a postRouterServerReplySentToApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReplySentToApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReplySentToApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplySentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReplySentToApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplySentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientUnknownDroppedFromApplication - errors', () => {
      it('should have a postRouterClientUnknownDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientUnknownDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientUnknownDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientUnknownDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientUnknownDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientUnknownDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterClientRequestDroppedFromApplication - errors', () => {
      it('should have a postRouterClientRequestDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterClientRequestDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterClientRequestDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientRequestDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterClientRequestDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterClientRequestDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAdvertisementReceivedFromApplication - errors', () => {
      it('should have a postRouterAdvertisementReceivedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAdvertisementReceivedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAdvertisementReceivedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAdvertisementReceivedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementReceivedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAdvertisementSentToFastLane - errors', () => {
      it('should have a postRouterAdvertisementSentToFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAdvertisementSentToFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAdvertisementSentToFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAdvertisementSentToFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAdvertisementSentToFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSolicitationDroppedFromApplication - errors', () => {
      it('should have a postRouterSolicitationDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSolicitationDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSolicitationDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSolicitationDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSolicitationReceivedFromFastLane - errors', () => {
      it('should have a postRouterSolicitationReceivedFromFastLane function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSolicitationReceivedFromFastLane === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSolicitationReceivedFromFastLane(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSolicitationReceivedFromFastLane('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationReceivedFromFastLane', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSolicitationSentToApplication - errors', () => {
      it('should have a postRouterSolicitationSentToApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSolicitationSentToApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSolicitationSentToApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationSentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSolicitationSentToApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSolicitationSentToApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerReplyDroppedFromApplication2 - errors', () => {
      it('should have a postRouterServerReplyDroppedFromApplication2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerReplyDroppedFromApplication2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerReplyDroppedFromApplication2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplyDroppedFromApplication2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerReplyDroppedFromApplication2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerReplyDroppedFromApplication2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServerUnknownDroppedFromApplication - errors', () => {
      it('should have a postRouterServerUnknownDroppedFromApplication function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServerUnknownDroppedFromApplication === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServerUnknownDroppedFromApplication(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerUnknownDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServerUnknownDroppedFromApplication('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServerUnknownDroppedFromApplication', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedAdaptiveEncryptionModifyPackets - errors', () => {
      it('should have a postRouterReceivedAdaptiveEncryptionModifyPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedAdaptiveEncryptionModifyPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedAdaptiveEncryptionModifyPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedAdaptiveEncryptionModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedAdaptiveEncryptionModifyPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedAdaptiveEncryptionModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaReceivedCollisionModifyPackets - errors', () => {
      it('should have a postRouterAreaReceivedCollisionModifyPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaReceivedCollisionModifyPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaReceivedCollisionModifyPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedCollisionModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaReceivedCollisionModifyPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedCollisionModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDroppedIcmpRequestPacketBlackhole - errors', () => {
      it('should have a postRouterDroppedIcmpRequestPacketBlackhole function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDroppedIcmpRequestPacketBlackhole === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDroppedIcmpRequestPacketBlackhole(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDroppedIcmpRequestPacketBlackhole', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDroppedIcmpRequestPacketBlackhole('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDroppedIcmpRequestPacketBlackhole', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterIcmpRequestPacketNoEgress - errors', () => {
      it('should have a postRouterIcmpRequestPacketNoEgress function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterIcmpRequestPacketNoEgress === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterIcmpRequestPacketNoEgress(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpRequestPacketNoEgress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterIcmpRequestPacketNoEgress('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterIcmpRequestPacketNoEgress', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceAreaReceivedDroppedPackets - errors', () => {
      it('should have a postRouterServiceAreaReceivedDroppedPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceAreaReceivedDroppedPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceAreaReceivedDroppedPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceAreaReceivedDroppedPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedDroppedPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDroppedTapMultiplexRecoveryPackets - errors', () => {
      it('should have a postRouterDroppedTapMultiplexRecoveryPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDroppedTapMultiplexRecoveryPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDroppedTapMultiplexRecoveryPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDroppedTapMultiplexRecoveryPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDroppedTapMultiplexRecoveryPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDroppedTapMultiplexRecoveryPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedDuplicateReverseMetadataPackets - errors', () => {
      it('should have a postRouterReceivedDuplicateReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedDuplicateReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedDuplicateReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedDuplicateReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedDuplicateReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedDuplicateReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedEmptyReverseMetadataPackets - errors', () => {
      it('should have a postRouterReceivedEmptyReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedEmptyReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedEmptyReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedEmptyReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedEmptyReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedEmptyReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceAreaReceivedExistingFlows - errors', () => {
      it('should have a postRouterServiceAreaReceivedExistingFlows function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceAreaReceivedExistingFlows === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceAreaReceivedExistingFlows(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedExistingFlows', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceAreaReceivedExistingFlows('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedExistingFlows', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceAreaReceivedExistingSession - errors', () => {
      it('should have a postRouterServiceAreaReceivedExistingSession function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceAreaReceivedExistingSession === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceAreaReceivedExistingSession(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedExistingSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceAreaReceivedExistingSession('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedExistingSession', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterExistingSessionReverseMetadataPackets - errors', () => {
      it('should have a postRouterExistingSessionReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterExistingSessionReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterExistingSessionReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExistingSessionReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterExistingSessionReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterExistingSessionReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedExternalProtocolAgentPackets - errors', () => {
      it('should have a postRouterReceivedExternalProtocolAgentPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedExternalProtocolAgentPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedExternalProtocolAgentPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedExternalProtocolAgentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedExternalProtocolAgentPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedExternalProtocolAgentPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterServiceAreaReceivedFabricPackets - errors', () => {
      it('should have a postRouterServiceAreaReceivedFabricPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterServiceAreaReceivedFabricPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterServiceAreaReceivedFabricPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedFabricPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterServiceAreaReceivedFabricPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterServiceAreaReceivedFabricPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFibBadSharedMemEntry - errors', () => {
      it('should have a postRouterFibBadSharedMemEntry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFibBadSharedMemEntry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFibBadSharedMemEntry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFibBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFibBadSharedMemEntry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFibBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFlowBadSharedMemEntry - errors', () => {
      it('should have a postRouterFlowBadSharedMemEntry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFlowBadSharedMemEntry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFlowBadSharedMemEntry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFlowBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFlowBadSharedMemEntry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFlowBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterFlowExpiredReverseMetadataPackets - errors', () => {
      it('should have a postRouterFlowExpiredReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterFlowExpiredReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterFlowExpiredReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFlowExpiredReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterFlowExpiredReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterFlowExpiredReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaReceivedFlowMovePackets - errors', () => {
      it('should have a postRouterAreaReceivedFlowMovePackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaReceivedFlowMovePackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaReceivedFlowMovePackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedFlowMovePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaReceivedFlowMovePackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedFlowMovePackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedInvalidReverseFlowModify - errors', () => {
      it('should have a postRouterReceivedInvalidReverseFlowModify function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedInvalidReverseFlowModify === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedInvalidReverseFlowModify(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidReverseFlowModify', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedInvalidReverseFlowModify('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidReverseFlowModify', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedInvalidReverseMetadataPackets - errors', () => {
      it('should have a postRouterReceivedInvalidReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedInvalidReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedInvalidReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedInvalidReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedInvalidReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedMidFlowModifyPackets - errors', () => {
      it('should have a postRouterReceivedMidFlowModifyPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedMidFlowModifyPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedMidFlowModifyPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedMidFlowModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedMidFlowModifyPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedMidFlowModifyPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaReceivedNoFibEntry - errors', () => {
      it('should have a postRouterAreaReceivedNoFibEntry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaReceivedNoFibEntry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaReceivedNoFibEntry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedNoFibEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaReceivedNoFibEntry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedNoFibEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaReceivedNonFabricPackets - errors', () => {
      it('should have a postRouterAreaReceivedNonFabricPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaReceivedNonFabricPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaReceivedNonFabricPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedNonFabricPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaReceivedNonFabricPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedNonFabricPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPacketsWithStaleFlowHit - errors', () => {
      it('should have a postRouterPacketsWithStaleFlowHit function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPacketsWithStaleFlowHit === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPacketsWithStaleFlowHit(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketsWithStaleFlowHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPacketsWithStaleFlowHit('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPacketsWithStaleFlowHit', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedReverseMetadataProcessFailure - errors', () => {
      it('should have a postRouterReceivedReverseMetadataProcessFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedReverseMetadataProcessFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedReverseMetadataProcessFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedReverseMetadataProcessFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedReverseMetadataProcessFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedReverseMetadataProcessFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedSessionCollisionChangeDirection - errors', () => {
      it('should have a postRouterReceivedSessionCollisionChangeDirection function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedSessionCollisionChangeDirection === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedSessionCollisionChangeDirection(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSessionCollisionChangeDirection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedSessionCollisionChangeDirection('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSessionCollisionChangeDirection', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedSessionCollisionIgnoreDrop - errors', () => {
      it('should have a postRouterReceivedSessionCollisionIgnoreDrop function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedSessionCollisionIgnoreDrop === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedSessionCollisionIgnoreDrop(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSessionCollisionIgnoreDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedSessionCollisionIgnoreDrop('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedSessionCollisionIgnoreDrop', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaReceivedSessionCollisionIgnored - errors', () => {
      it('should have a postRouterAreaReceivedSessionCollisionIgnored function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaReceivedSessionCollisionIgnored === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaReceivedSessionCollisionIgnored(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedSessionCollisionIgnored', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaReceivedSessionCollisionIgnored('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaReceivedSessionCollisionIgnored', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionIgnoredReverseMetadataPackets - errors', () => {
      it('should have a postRouterSessionIgnoredReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionIgnoredReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionIgnoredReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionIgnoredReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionIgnoredReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionIgnoredReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionUpdatedReverseMetadataPackets - errors', () => {
      it('should have a postRouterSessionUpdatedReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionUpdatedReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionUpdatedReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionUpdatedReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionUpdatedReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionUpdatedReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterLookupBadSharedMemEntry - errors', () => {
      it('should have a postRouterLookupBadSharedMemEntry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterLookupBadSharedMemEntry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterLookupBadSharedMemEntry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterLookupBadSharedMemEntry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterLookupBadSharedMemEntry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateFlowWithPeerFailure - errors', () => {
      it('should have a postRouterUpdateFlowWithPeerFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateFlowWithPeerFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateFlowWithPeerFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateFlowWithPeerFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateFlowWithPeerFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateFlowWithPeerFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateFlowWithPeerSuccess - errors', () => {
      it('should have a postRouterUpdateFlowWithPeerSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateFlowWithPeerSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateFlowWithPeerSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateFlowWithPeerSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateFlowWithPeerSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateFlowWithPeerSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateSessionWithPeerFailure - errors', () => {
      it('should have a postRouterUpdateSessionWithPeerFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateSessionWithPeerFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateSessionWithPeerFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateSessionWithPeerFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateSessionWithPeerFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateSessionWithPeerFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterUpdateSessionWithPeerSuccess - errors', () => {
      it('should have a postRouterUpdateSessionWithPeerSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterUpdateSessionWithPeerSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterUpdateSessionWithPeerSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateSessionWithPeerSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterUpdateSessionWithPeerSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterUpdateSessionWithPeerSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedValidReverseMetadataPackets - errors', () => {
      it('should have a postRouterReceivedValidReverseMetadataPackets function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedValidReverseMetadataPackets === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedValidReverseMetadataPackets(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedValidReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedValidReverseMetadataPackets('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedValidReverseMetadataPackets', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaSentArpResolveFailure - errors', () => {
      it('should have a postRouterAreaSentArpResolveFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaSentArpResolveFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaSentArpResolveFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolveFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaSentArpResolveFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolveFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaSentArpResolvePending - errors', () => {
      it('should have a postRouterAreaSentArpResolvePending function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaSentArpResolvePending === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaSentArpResolvePending(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolvePending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaSentArpResolvePending('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolvePending', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAreaSentArpResolveSuccess - errors', () => {
      it('should have a postRouterAreaSentArpResolveSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAreaSentArpResolveSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAreaSentArpResolveSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolveSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAreaSentArpResolveSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAreaSentArpResolveSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsServiceAreaSentSuccess - errors', () => {
      it('should have a postRouterStatsServiceAreaSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsServiceAreaSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsServiceAreaSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsServiceAreaSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsServiceAreaSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsServiceAreaSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterResetForAdaptiveEncryptionFailure - errors', () => {
      it('should have a postRouterResetForAdaptiveEncryptionFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterResetForAdaptiveEncryptionFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterResetForAdaptiveEncryptionFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterResetForAdaptiveEncryptionFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterResetForAdaptiveEncryptionFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterResetForAdaptiveEncryptionFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInstallErrorsDuplicateReverseFlow - errors', () => {
      it('should have a postRouterInstallErrorsDuplicateReverseFlow function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInstallErrorsDuplicateReverseFlow === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInstallErrorsDuplicateReverseFlow(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsDuplicateReverseFlow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInstallErrorsDuplicateReverseFlow('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsDuplicateReverseFlow', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionInstallErrorsFibLookup - errors', () => {
      it('should have a postRouterSessionInstallErrorsFibLookup function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionInstallErrorsFibLookup === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionInstallErrorsFibLookup(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsFibLookup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionInstallErrorsFibLookup('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsFibLookup', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInstallErrorsGatewayLookupFailure - errors', () => {
      it('should have a postRouterInstallErrorsGatewayLookupFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInstallErrorsGatewayLookupFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInstallErrorsGatewayLookupFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsGatewayLookupFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInstallErrorsGatewayLookupFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsGatewayLookupFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionInstallErrorsInvalidPacket - errors', () => {
      it('should have a postRouterSessionInstallErrorsInvalidPacket function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionInstallErrorsInvalidPacket === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionInstallErrorsInvalidPacket(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsInvalidPacket', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionInstallErrorsInvalidPacket('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsInvalidPacket', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInstallErrorsServicePathsUnavailable - errors', () => {
      it('should have a postRouterInstallErrorsServicePathsUnavailable function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInstallErrorsServicePathsUnavailable === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInstallErrorsServicePathsUnavailable(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsServicePathsUnavailable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInstallErrorsServicePathsUnavailable('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsServicePathsUnavailable', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInstallErrorsUrpfCheckFailure - errors', () => {
      it('should have a postRouterInstallErrorsUrpfCheckFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInstallErrorsUrpfCheckFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInstallErrorsUrpfCheckFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsUrpfCheckFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInstallErrorsUrpfCheckFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInstallErrorsUrpfCheckFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionInstallErrorsWaypointAllocation - errors', () => {
      it('should have a postRouterSessionInstallErrorsWaypointAllocation function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionInstallErrorsWaypointAllocation === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionInstallErrorsWaypointAllocation(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsWaypointAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionInstallErrorsWaypointAllocation('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionInstallErrorsWaypointAllocation', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionActive - errors', () => {
      it('should have a postRouterStatsSessionActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionAdd - errors', () => {
      it('should have a postRouterStatsSessionAdd function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionAdd === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionAdd(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionAdd('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionAdd', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionAddedExisting - errors', () => {
      it('should have a postRouterStatsSessionAddedExisting function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionAddedExisting === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionAddedExisting(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionAddedExisting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionAddedExisting('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionAddedExisting', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionDuplicateId - errors', () => {
      it('should have a postRouterStatsSessionDuplicateId function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionDuplicateId === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionDuplicateId(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionDuplicateId', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionDuplicateId('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionDuplicateId', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionFlowActive - errors', () => {
      it('should have a postRouterStatsSessionFlowActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionFlowActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionFlowActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionFlowActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionFlowActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionFlowActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionFlowAddFailure - errors', () => {
      it('should have a postRouterSessionFlowAddFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionFlowAddFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionFlowAddFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionFlowAddFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionFlowAddFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionFlowAddFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionFlowRemoveFailure - errors', () => {
      it('should have a postRouterSessionFlowRemoveFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionFlowRemoveFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionFlowRemoveFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionFlowRemoveFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionFlowRemoveFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionFlowRemoveFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSessionMaxRemoveTimeoutExceeded - errors', () => {
      it('should have a postRouterSessionMaxRemoveTimeoutExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSessionMaxRemoveTimeoutExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSessionMaxRemoveTimeoutExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionMaxRemoveTimeoutExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSessionMaxRemoveTimeoutExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSessionMaxRemoveTimeoutExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSessionRemove - errors', () => {
      it('should have a postRouterStatsSessionRemove function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSessionRemove === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSessionRemove(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionRemove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSessionRemove('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSessionRemove', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSourceNatActiveTables - errors', () => {
      it('should have a postRouterStatsSourceNatActiveTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSourceNatActiveTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSourceNatActiveTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatActiveTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSourceNatActiveTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatActiveTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNatAllocatePortsForDb - errors', () => {
      it('should have a postRouterNatAllocatePortsForDb function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNatAllocatePortsForDb === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNatAllocatePortsForDb(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatAllocatePortsForDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNatAllocatePortsForDb('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatAllocatePortsForDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNatGiidTransitionToActive - errors', () => {
      it('should have a postRouterNatGiidTransitionToActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNatGiidTransitionToActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNatGiidTransitionToActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatGiidTransitionToActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNatGiidTransitionToActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatGiidTransitionToActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNatGiidTransitionToInactive - errors', () => {
      it('should have a postRouterNatGiidTransitionToInactive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNatGiidTransitionToInactive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNatGiidTransitionToInactive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatGiidTransitionToInactive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNatGiidTransitionToInactive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatGiidTransitionToInactive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSourceNatInactiveTables - errors', () => {
      it('should have a postRouterStatsSourceNatInactiveTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSourceNatInactiveTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSourceNatInactiveTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatInactiveTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSourceNatInactiveTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatInactiveTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterNatReinitializePortsInDb - errors', () => {
      it('should have a postRouterNatReinitializePortsInDb function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterNatReinitializePortsInDb === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterNatReinitializePortsInDb(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatReinitializePortsInDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterNatReinitializePortsInDb('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterNatReinitializePortsInDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReleasePortsToDbFailure - errors', () => {
      it('should have a postRouterReleasePortsToDbFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReleasePortsToDbFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReleasePortsToDbFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReleasePortsToDbFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReleasePortsToDbSuccess - errors', () => {
      it('should have a postRouterReleasePortsToDbSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReleasePortsToDbSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReleasePortsToDbSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReleasePortsToDbSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSourceNatReservedPorts - errors', () => {
      it('should have a postRouterStatsSourceNatReservedPorts function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSourceNatReservedPorts === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSourceNatReservedPorts(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatReservedPorts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSourceNatReservedPorts('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceNatReservedPorts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSourceTenantMeter - errors', () => {
      it('should have a postRouterStatsSourceTenantMeter function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSourceTenantMeter === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSourceTenantMeter(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceTenantMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSourceTenantMeter('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSourceTenantMeter', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscClientsConnects - errors', () => {
      it('should have a postRouterStatsSscClientsConnects function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscClientsConnects === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscClientsConnects(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscClientsConnects', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscClientsConnects('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscClientsConnects', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscClientsDisconnects - errors', () => {
      it('should have a postRouterStatsSscClientsDisconnects function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscClientsDisconnects === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscClientsDisconnects(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscClientsDisconnects', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscClientsDisconnects('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscClientsDisconnects', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscOperationsCompressions - errors', () => {
      it('should have a postRouterStatsSscOperationsCompressions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscOperationsCompressions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscOperationsCompressions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscOperationsCompressions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscOperationsCompressions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscOperationsCompressions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscOperationsDecompressions - errors', () => {
      it('should have a postRouterStatsSscOperationsDecompressions function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscOperationsDecompressions === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscOperationsDecompressions(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscOperationsDecompressions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscOperationsDecompressions('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscOperationsDecompressions', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedByServiceDeregistrations - errors', () => {
      it('should have a postRouterSscReceivedByServiceDeregistrations function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedByServiceDeregistrations === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedByServiceDeregistrations(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceDeregistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedByServiceDeregistrations('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceDeregistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedByServicePublishes - errors', () => {
      it('should have a postRouterSscReceivedByServicePublishes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedByServicePublishes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedByServicePublishes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServicePublishes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedByServicePublishes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServicePublishes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedByServiceRegistrationUpdates - errors', () => {
      it('should have a postRouterReceivedByServiceRegistrationUpdates function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedByServiceRegistrationUpdates === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedByServiceRegistrationUpdates(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedByServiceRegistrationUpdates', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedByServiceRegistrationUpdates('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedByServiceRegistrationUpdates', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedByServiceRegistrations - errors', () => {
      it('should have a postRouterSscReceivedByServiceRegistrations function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedByServiceRegistrations === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedByServiceRegistrations(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceRegistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedByServiceRegistrations('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceRegistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedByServiceRequests - errors', () => {
      it('should have a postRouterSscReceivedByServiceRequests function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedByServiceRequests === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedByServiceRequests(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedByServiceRequests('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedByServiceResponses - errors', () => {
      it('should have a postRouterSscReceivedByServiceResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedByServiceResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedByServiceResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedByServiceResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedByServiceResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsBadRequestResponses - errors', () => {
      it('should have a postRouterReceivedErrorsBadRequestResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsBadRequestResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsBadRequestResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsBadRequestResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsBadRequestResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsBadRequestResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedErrorsInvalidMessage - errors', () => {
      it('should have a postRouterSscReceivedErrorsInvalidMessage function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedErrorsInvalidMessage === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedErrorsInvalidMessage(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedErrorsInvalidMessage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedErrorsInvalidMessage('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedErrorsInvalidMessage', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscReceivedErrorsInvalidRequest - errors', () => {
      it('should have a postRouterSscReceivedErrorsInvalidRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscReceivedErrorsInvalidRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscReceivedErrorsInvalidRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedErrorsInvalidRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscReceivedErrorsInvalidRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscReceivedErrorsInvalidRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsNoClientTransaction - errors', () => {
      it('should have a postRouterReceivedErrorsNoClientTransaction function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsNoClientTransaction === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsNoClientTransaction(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsNoClientTransaction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsNoClientTransaction('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsNoClientTransaction', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsNoEndpointsFound - errors', () => {
      it('should have a postRouterReceivedErrorsNoEndpointsFound function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsNoEndpointsFound === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsNoEndpointsFound(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsNoEndpointsFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsNoEndpointsFound('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsNoEndpointsFound', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsResponseNotReady - errors', () => {
      it('should have a postRouterReceivedErrorsResponseNotReady function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsResponseNotReady === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsResponseNotReady(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsResponseNotReady', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsResponseNotReady('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsResponseNotReady', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsServerErrorResponses - errors', () => {
      it('should have a postRouterReceivedErrorsServerErrorResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsServerErrorResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsServerErrorResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsServerErrorResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsServerErrorResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsServerErrorResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsUnknownMessageType - errors', () => {
      it('should have a postRouterReceivedErrorsUnknownMessageType function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsUnknownMessageType === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsUnknownMessageType(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsUnknownMessageType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsUnknownMessageType('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsUnknownMessageType', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReceivedErrorsUnknownSscRequest - errors', () => {
      it('should have a postRouterReceivedErrorsUnknownSscRequest function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReceivedErrorsUnknownSscRequest === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReceivedErrorsUnknownSscRequest(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsUnknownSscRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReceivedErrorsUnknownSscRequest('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReceivedErrorsUnknownSscRequest', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscSessionsActive - errors', () => {
      it('should have a postRouterStatsSscSessionsActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscSessionsActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscSessionsActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscSessionsActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscSessionsActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscSessionsActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscTransactionsClient - errors', () => {
      it('should have a postRouterStatsSscTransactionsClient function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscTransactionsClient === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscTransactionsClient(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransactionsClient', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscTransactionsClient('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransactionsClient', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscTransactionsErrorsInsertionFailures - errors', () => {
      it('should have a postRouterSscTransactionsErrorsInsertionFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscTransactionsErrorsInsertionFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscTransactionsErrorsInsertionFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransactionsErrorsInsertionFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscTransactionsErrorsInsertionFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransactionsErrorsInsertionFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscTransactionsServer - errors', () => {
      it('should have a postRouterStatsSscTransactionsServer function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscTransactionsServer === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscTransactionsServer(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransactionsServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscTransactionsServer('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransactionsServer', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscTransmittedByServicePublishes - errors', () => {
      it('should have a postRouterSscTransmittedByServicePublishes function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscTransmittedByServicePublishes === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscTransmittedByServicePublishes(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedByServicePublishes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscTransmittedByServicePublishes('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedByServicePublishes', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscTransmittedByServiceRequests - errors', () => {
      it('should have a postRouterSscTransmittedByServiceRequests function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscTransmittedByServiceRequests === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscTransmittedByServiceRequests(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedByServiceRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscTransmittedByServiceRequests('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedByServiceRequests', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscTransmittedCompletedResponses - errors', () => {
      it('should have a postRouterStatsSscTransmittedCompletedResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscTransmittedCompletedResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscTransmittedCompletedResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedCompletedResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscTransmittedCompletedResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedCompletedResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscTransmittedErrorsWriteFailures - errors', () => {
      it('should have a postRouterSscTransmittedErrorsWriteFailures function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscTransmittedErrorsWriteFailures === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscTransmittedErrorsWriteFailures(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedErrorsWriteFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscTransmittedErrorsWriteFailures('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedErrorsWriteFailures', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscTransmittedLocalResponses - errors', () => {
      it('should have a postRouterStatsSscTransmittedLocalResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscTransmittedLocalResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscTransmittedLocalResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedLocalResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscTransmittedLocalResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedLocalResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterSscTransmittedPublishOnRegistrations - errors', () => {
      it('should have a postRouterSscTransmittedPublishOnRegistrations function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterSscTransmittedPublishOnRegistrations === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterSscTransmittedPublishOnRegistrations(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedPublishOnRegistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterSscTransmittedPublishOnRegistrations('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterSscTransmittedPublishOnRegistrations', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsSscTransmittedRetransmittedResponses - errors', () => {
      it('should have a postRouterStatsSscTransmittedRetransmittedResponses function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsSscTransmittedRetransmittedResponses === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsSscTransmittedRetransmittedResponses(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedRetransmittedResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsSscTransmittedRetransmittedResponses('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsSscTransmittedRetransmittedResponses', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDevicePortQueueDepthAvg - errors', () => {
      it('should have a postRouterDevicePortQueueDepthAvg function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDevicePortQueueDepthAvg === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDevicePortQueueDepthAvg(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthAvg', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDevicePortQueueDepthAvg('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthAvg', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDevicePortQueueDepthMax - errors', () => {
      it('should have a postRouterDevicePortQueueDepthMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDevicePortQueueDepthMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDevicePortQueueDepthMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDevicePortQueueDepthMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDevicePortQueueDepthMin - errors', () => {
      it('should have a postRouterDevicePortQueueDepthMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDevicePortQueueDepthMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDevicePortQueueDepthMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDevicePortQueueDepthMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterDevicePortQueueDepthTotal - errors', () => {
      it('should have a postRouterDevicePortQueueDepthTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterDevicePortQueueDepthTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterDevicePortQueueDepthTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterDevicePortQueueDepthTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterDevicePortQueueDepthTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngDevicePortScheduleFailure - errors', () => {
      it('should have a postRouterEngDevicePortScheduleFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngDevicePortScheduleFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngDevicePortScheduleFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngDevicePortScheduleFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngDevicePortScheduleFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngDevicePortScheduleFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngDevicePortScheduleSuccess - errors', () => {
      it('should have a postRouterEngDevicePortScheduleSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngDevicePortScheduleSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngDevicePortScheduleSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngDevicePortScheduleSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngDevicePortScheduleSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngDevicePortScheduleSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInternalApplicationQueueDepthAvg - errors', () => {
      it('should have a postRouterInternalApplicationQueueDepthAvg function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInternalApplicationQueueDepthAvg === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthAvg(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthAvg', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthAvg('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthAvg', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInternalApplicationQueueDepthMax - errors', () => {
      it('should have a postRouterInternalApplicationQueueDepthMax function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInternalApplicationQueueDepthMax === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthMax(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthMax('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthMax', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInternalApplicationQueueDepthMin - errors', () => {
      it('should have a postRouterInternalApplicationQueueDepthMin function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInternalApplicationQueueDepthMin === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthMin(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthMin('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthMin', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterInternalApplicationQueueDepthTotal - errors', () => {
      it('should have a postRouterInternalApplicationQueueDepthTotal function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterInternalApplicationQueueDepthTotal === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthTotal(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterInternalApplicationQueueDepthTotal('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterInternalApplicationQueueDepthTotal', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngInternalApplicationScheduleFailure - errors', () => {
      it('should have a postRouterEngInternalApplicationScheduleFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngInternalApplicationScheduleFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngInternalApplicationScheduleFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationScheduleFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngInternalApplicationScheduleFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationScheduleFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngInternalApplicationScheduleSuccess - errors', () => {
      it('should have a postRouterEngInternalApplicationScheduleSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngInternalApplicationScheduleSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngInternalApplicationScheduleSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationScheduleSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngInternalApplicationScheduleSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationScheduleSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngInternalApplicationSentRetry - errors', () => {
      it('should have a postRouterEngInternalApplicationSentRetry function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngInternalApplicationSentRetry === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngInternalApplicationSentRetry(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationSentRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngInternalApplicationSentRetry('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationSentRetry', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterEngInternalApplicationSentSuccess - errors', () => {
      it('should have a postRouterEngInternalApplicationSentSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterEngInternalApplicationSentSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterEngInternalApplicationSentSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterEngInternalApplicationSentSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterEngInternalApplicationSentSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsTrafficExportEnqueueFailure - errors', () => {
      it('should have a postRouterStatsTrafficExportEnqueueFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsTrafficExportEnqueueFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsTrafficExportEnqueueFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportEnqueueFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsTrafficExportEnqueueFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportEnqueueFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterTrafficExportRateLimitExceeded - errors', () => {
      it('should have a postRouterTrafficExportRateLimitExceeded function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterTrafficExportRateLimitExceeded === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterTrafficExportRateLimitExceeded(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTrafficExportRateLimitExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterTrafficExportRateLimitExceeded('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterTrafficExportRateLimitExceeded', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsTrafficExportTransmitBatches - errors', () => {
      it('should have a postRouterStatsTrafficExportTransmitBatches function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsTrafficExportTransmitBatches === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitBatches(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitBatches', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitBatches('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitBatches', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsTrafficExportTransmitFailure - errors', () => {
      it('should have a postRouterStatsTrafficExportTransmitFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsTrafficExportTransmitFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsTrafficExportTransmitSuccess - errors', () => {
      it('should have a postRouterStatsTrafficExportTransmitSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsTrafficExportTransmitSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsTrafficExportTransmitSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsTrafficExportTransmitSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWaypointActiveInterTables - errors', () => {
      it('should have a postRouterWaypointActiveInterTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWaypointActiveInterTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWaypointActiveInterTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointActiveInterTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWaypointActiveInterTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointActiveInterTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPortsForDefaultRangeTables - errors', () => {
      it('should have a postRouterPortsForDefaultRangeTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPortsForDefaultRangeTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPortsForDefaultRangeTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortsForDefaultRangeTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPortsForDefaultRangeTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortsForDefaultRangeTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterPortsForRangeBasedTables - errors', () => {
      it('should have a postRouterPortsForRangeBasedTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterPortsForRangeBasedTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterPortsForRangeBasedTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortsForRangeBasedTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterPortsForRangeBasedTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterPortsForRangeBasedTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWaypointGiidTransitionToActive - errors', () => {
      it('should have a postRouterWaypointGiidTransitionToActive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWaypointGiidTransitionToActive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWaypointGiidTransitionToActive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointGiidTransitionToActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWaypointGiidTransitionToActive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointGiidTransitionToActive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWaypointGiidTransitionToInactive - errors', () => {
      it('should have a postRouterWaypointGiidTransitionToInactive function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWaypointGiidTransitionToInactive === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWaypointGiidTransitionToInactive(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointGiidTransitionToInactive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWaypointGiidTransitionToInactive('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointGiidTransitionToInactive', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWaypointInactiveInterTables - errors', () => {
      it('should have a postRouterWaypointInactiveInterTables function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWaypointInactiveInterTables === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWaypointInactiveInterTables(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointInactiveInterTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWaypointInactiveInterTables('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointInactiveInterTables', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWaypointReinitializePortsInDb - errors', () => {
      it('should have a postRouterWaypointReinitializePortsInDb function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWaypointReinitializePortsInDb === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWaypointReinitializePortsInDb(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointReinitializePortsInDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWaypointReinitializePortsInDb('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWaypointReinitializePortsInDb', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReleasePortsToDbFailure2 - errors', () => {
      it('should have a postRouterReleasePortsToDbFailure2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReleasePortsToDbFailure2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReleasePortsToDbFailure2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReleasePortsToDbFailure2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbFailure2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterReleasePortsToDbSuccess2 - errors', () => {
      it('should have a postRouterReleasePortsToDbSuccess2 function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterReleasePortsToDbSuccess2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterReleasePortsToDbSuccess2(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbSuccess2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterReleasePortsToDbSuccess2('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterReleasePortsToDbSuccess2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterStatsWaypointReservedPorts - errors', () => {
      it('should have a postRouterStatsWaypointReservedPorts function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterStatsWaypointReservedPorts === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterStatsWaypointReservedPorts(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsWaypointReservedPorts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterStatsWaypointReservedPorts('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterStatsWaypointReservedPorts', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAesContextCreateFailure - errors', () => {
      it('should have a postRouterAesContextCreateFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAesContextCreateFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAesContextCreateFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextCreateFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAesContextCreateFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextCreateFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAesContextCreateSuccess - errors', () => {
      it('should have a postRouterAesContextCreateSuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAesContextCreateSuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAesContextCreateSuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextCreateSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAesContextCreateSuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextCreateSuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAesContextDestroyFailure - errors', () => {
      it('should have a postRouterAesContextDestroyFailure function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAesContextDestroyFailure === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAesContextDestroyFailure(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextDestroyFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAesContextDestroyFailure('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextDestroyFailure', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterAesContextDestroySuccess - errors', () => {
      it('should have a postRouterAesContextDestroySuccess function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterAesContextDestroySuccess === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterAesContextDestroySuccess(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextDestroySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterAesContextDestroySuccess('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterAesContextDestroySuccess', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWorkerCoreCachingLoopException - errors', () => {
      it('should have a postRouterWorkerCoreCachingLoopException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWorkerCoreCachingLoopException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWorkerCoreCachingLoopException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreCachingLoopException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWorkerCoreCachingLoopException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreCachingLoopException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWorkerCoreControlMessagesSent - errors', () => {
      it('should have a postRouterWorkerCoreControlMessagesSent function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWorkerCoreControlMessagesSent === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWorkerCoreControlMessagesSent(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreControlMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWorkerCoreControlMessagesSent('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreControlMessagesSent', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWorkerCorePacketProcessingUtilization - errors', () => {
      it('should have a postRouterWorkerCorePacketProcessingUtilization function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWorkerCorePacketProcessingUtilization === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWorkerCorePacketProcessingUtilization(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCorePacketProcessingUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWorkerCorePacketProcessingUtilization('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCorePacketProcessingUtilization', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#postRouterWorkerCoreProcessLoopException - errors', () => {
      it('should have a postRouterWorkerCoreProcessLoopException function', (done) => {
        try {
          assert.equal(true, typeof a.postRouterWorkerCoreProcessLoopException === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.postRouterWorkerCoreProcessLoopException(null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreProcessLoopException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing body', (done) => {
        try {
          a.postRouterWorkerCoreProcessLoopException('fakeparam', null, (data, error) => {
            try {
              const displayE = 'body is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-postRouterWorkerCoreProcessLoopException', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getService - errors', () => {
      it('should have a getService function', (done) => {
        try {
          assert.equal(true, typeof a.getService === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getService('fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getService', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getService2 - errors', () => {
      it('should have a getService2 function', (done) => {
        try {
          assert.equal(true, typeof a.getService2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getService2('fakeparam', 'fakeparam', null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getService2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing service', (done) => {
        try {
          a.getService2('fakeparam', 'fakeparam', 'fakeparam', null, (data, error) => {
            try {
              const displayE = 'service is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getService2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getServiceClass - errors', () => {
      it('should have a getServiceClass function', (done) => {
        try {
          assert.equal(true, typeof a.getServiceClass === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getTenant - errors', () => {
      it('should have a getTenant function', (done) => {
        try {
          assert.equal(true, typeof a.getTenant === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getTenant(null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getTenant', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });

    describe('#getTenant2 - errors', () => {
      it('should have a getTenant2 function', (done) => {
        try {
          assert.equal(true, typeof a.getTenant2 === 'function');
          done();
        } catch (error) {
          log.error(`Test Failure: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing router', (done) => {
        try {
          a.getTenant2(null, null, null, null, (data, error) => {
            try {
              const displayE = 'router is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getTenant2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
      it('should error if - missing tenant', (done) => {
        try {
          a.getTenant2('fakeparam', null, null, null, (data, error) => {
            try {
              const displayE = 'tenant is required';
              runErrorAsserts(data, error, 'AD.300', 'Test-128technology-adapter-getTenant2', displayE);
              done();
            } catch (err) {
              log.error(`Test Failure: ${err}`);
              done(err);
            }
          });
        } catch (error) {
          log.error(`Adapter Exception: ${error}`);
          done(error);
        }
      }).timeout(attemptTimeout);
    });
  });
});
